C     Main program to solve linearly constrained NLP problems using MINOS.

C     Last update: June 17th, 2008.

C     Compile with: 
C
C     g77 -O -fno-globals -Wno-globals minos.f minosma-nlp.f -o minosma-nlp

      program minosma

      implicit none

C     PARAMETERS
      integer mmax,nmax,annzmax,nwcore
      parameter ( mmax    =         1000 )
      parameter ( nmax    =         1000 )
      parameter ( annzmax =  nmax * mmax )
      parameter ( nwcore  =        50000 )

C     There is another nmax declaration within subroutine coo2csc below.

C     LOCAL SCALARS
      logical nullc
      integer annz,i,inform,ispecs,iobj,iprint,isumm,m,mincor,n,nb,ninf,
     +        nname,nncon,nnobj,nnjac,ns
      double precision obj,objadd,sinf

C     LOCAL ARRAYS
      character * 8 names(5)
      integer alin(annzmax),acol(annzmax),asta(nmax),hs(nmax+mmax),
     +        name1(1),name2(1)
      double precision aval(annzmax),blow(mmax),bupp(mmax),c(nmax),
     +        x(nmax),l(nmax),u(nmax),bl(nmax+mmax),bu(nmax+mmax),
     +        pi(mmax),xn(nmax+mmax),rc(nmax+mmax),z(nwcore)

C     ===================================================
C     P R O B L E M   D E F I N I T I O N
C     ===================================================

C     EXAMPLE 1:
C     ==========

C     Min - ( x1 - 0.01 ) ** 2 - ( x2 - 0.01 ) ** 2 
C
C     subject to 
C       x1 + 2 x2 <= 3,
C     2 x1 +   x2 <= 3,   
C              x1 >= -1,
C              x2 >= -1.

C     Sol.: (x1,x2) = (-1,-1).

      n      = 2
      m      = 2

      blow(1) = - 1.0d+20
      blow(2) = - 1.0d+20

      bupp(1) = 3.0d0
      bupp(2) = 3.0d0

      annz = 4

      alin(1) = 1
      acol(1) = 1
      aval(1) = 1.0d0
      
      alin(2) = 1
      acol(2) = 2
      aval(2) = 2.0d0
      
      alin(3) = 2
      acol(3) = 1
      aval(3) = 2.0d0
      
      alin(4) = 2
      acol(4) = 2
      aval(4) = 1.0d0
      
      do i = 1,n
          l(i) = - 1.0d0
          u(i) =   1.0d+20
      end do

C     ===================================================
C     E N D   O F   P R O B L E M   D E F I N I T I O N
C     ===================================================

C     Set problem related data

      names(1) = 'MyTest  '
      names(2) = 'funobj  '
      names(3) = 'rhs     '
      names(4) = 'ranges  '
      names(5) = 'bounds  '

      ispecs = 0
      iprint = 9
      isumm  = 6

      call mispec(ispecs,iprint,isumm,nwcore,inform)
      call miopt ('Jacobian    Sparse',iprint,isumm,inform)

      nname  = 0

      nncon  = 0
      nnjac  = 0

      nnobj  = n
      iobj   = 0

      objadd = 0.0d0

      nb = n + m

C     Add slacks coefficients

      do i = 1,m
          annz = annz + 1

          alin(annz) = i
          acol(annz) = n + i
          aval(annz) = 1.0d0
      end do

C     Covert to compressed sparse column format

      call coo2csc(nb,annz,alin,acol,aval,asta)

C     Set bounds on x

      do i = 1,n
          bl(i) = l(i)
          bu(i) = u(i)
      end do

C     Set slacks bounds

      do i = 1,m
          bl(n+i) = - bupp(i)
          bu(n+i) = - blow(i)
      end do

C     Call MINOS

      call minoss('Cold',m,n,nb,annz,nname,nncon,nnobj,nnjac,iobj,
     +            objadd,names,aval,alin,asta,bl,bu,name1,name2,
     +            hs,xn,pi,rc,inform,mincor,ns,ninf,sinf,obj,z,nwcore)

C     Write final values

      write(*,*) ' inform = ', inform
      write(*,*) ' ninf   = ', ninf
      write(*,*) ' sinf   = ', sinf
      write(*,*) ' obj    = ', obj

      write(*,*) ' Solution: ',(xn(i),i=1,n)

      stop

      end

C     ******************************************************************
C     ******************************************************************

      subroutine coo2csc(n,nnz,alin,acol,aval,asta)

      implicit none

C     SCALAR ARGUMENTS
      integer n,nnz

C     ARRAY ARGUMENTS
      integer acol(nnz),alin(nnz),asta(n)
      double precision aval(nnz)

C     This subroutines converts a matrix from coordinate format to
C     compressed sparse column format.

C     PARAMETERS
      integer nmax
      parameter ( nmax = 1000 )

C     LOCAL SCALARS
      integer i,j,col,coltmp,lin,lintmp
      double precision val,valtmp

C     LOCAL ARRAY
      integer alen(nmax)

      do i = 1,n
          alen(i) = 0
      end do

      do i = 1,nnz
          col = acol(i)
          alen(col) = alen(col) + 1
      end do

      asta(1) = 1
      do i = 2,n
          asta(i) = asta(i-1) + alen(i-1)
      end do

      do i = 1,nnz

          val = aval(i)
          col = acol(i)
          lin = alin(i)

          acol(i) = - 1

 10       if ( col .ge. 0 ) then

              j = asta(col)
              asta(col) = j + 1

              valtmp = aval(j)
              coltmp = acol(j)
              lintmp = alin(j)

              aval(j) = val
              alin(j) = lin
              acol(j) = - 1

              val = valtmp
              col = coltmp
              lin = lintmp

              go to 10

          end if

      end do

      do i = 1,n
          asta(i) = asta(i) - alen(i)
      end do

      asta(n+1) = nnz + 1

      end

C     ******************************************************************
C     ******************************************************************

      SUBROUTINE FUNOBJ( MODE, N, X, F, G, NSTATE, NPROB, Z, NWCORE )
      IMPLICIT           REAL*8(A-H,O-Z)
      DOUBLE PRECISION   X(N), G(N), Z(NWCORE)

C     LOCAL SCALARS
      integer i

      f = 0.0d0
      do i = 1,n
          f = f - ( x(i) - 0.01d0 ) ** 2
      end do

      do i = 1,n
          g(i) = - 2.0d0 * ( x(i) - 0.01d0 )
      end do

      END

C     ******************************************************************
C     ******************************************************************

      SUBROUTINE FUNCON( MODE, M, N, NJAC, X, F, G,
     *                   NSTATE, NPROB, Z, NWCORE )
      IMPLICIT           REAL*8(A-H,O-Z)
      DOUBLE PRECISION   X(N), F(M), G(NJAC), Z(NWCORE)

      END

C     ******************************************************************
C     ******************************************************************

      SUBROUTINE MATMOD( NCYCLE, NPROB, FINISH,
     *                   M, N, NB, NE, NKA, NS, NSCL,
     *                   A, HA, KA, BL, BU,
     *                   ASCALE, HS, ID1, ID2,
     *                   X, PI, Z, NWCORE )

      IMPLICIT           REAL*8(A-H,O-Z)
      INTEGER*2          HA(NE), HS(NB)
      INTEGER            KA(NKA), ID1(NB), ID2(NB)
      DOUBLE PRECISION   A(NE), ASCALE(NSCL), BL(NB), BU(NB)
      DOUBLE PRECISION   X(NB), PI(M), Z(NWCORE)
      LOGICAL            FINISH

      END
