************************************************************************
*
*     File  mi10unix  fortran.
*-->  Unix version of  mi10mach  fortran
*
*     minoss   minos1   minos2   minos3
*     mifile   mispec   misolv
*     m1clos   m1envt   m1init
*     m1open   m1page   m1time   m1timp   m1cpu
*
*     mi10vms and mi10unix are the same except for
*     minos2   m1cpu
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine minoss( start, m, n, nb, ne, nname,
     $                   nncon, nnobj, nnjac, 
     $                   iobj, objadd, names,
     $                   a, ha, ka, bl, bu, name1, name2,
     $                   hs, xn, pi, rc, 
     $                   inform, mincor, ns, ninf, sinf, obj,
     $                   z, nwcore )

      implicit           double precision (a-h,o-z)
      character*(*)      start
      character*8        names(5)
      integer*4          ha(ne), hs(nb)
      integer            ka(n+1), name1(nname), name2(nname)
      double precision   a(ne), bl(nb), bu(nb)
      double precision   xn(nb), pi(m), rc(nb), z(nwcore)

*     ------------------------------------------------------------------
*     minoss (pronounced minos-s) is the subroutine version of MINOS.
*     It has all the data passed to it as parameters instead of reading
*     it from an MPS file.
*
*     ON ENTRY:
*
*     start   specifies how a starting basis (and certain other items)
*             are to be obtained.
*             start = 'Cold' means that Crash should be used to choose
*                      an initial basis, unless a basis file is given
*                      via Old basis, Insert or Load in the Specs file.
*             start = 'Basis file' means the same (but is more
*                      meaningful in the latter case).
*             start = 'Warm' means that a basis is already defined in hs
*                      (probably from an earlier call).
*             start = 'Hot' or 'Hot FHS' implies a hot start.
*                      hs defines a basis and an earlier call has
*                      defined certain other things that should also be
*                      kept.  The problem dimensions and the array z(*)
*                      must not have changed.
*                      F refers to the LU factors of the basis.
*                      H refers to the approximate reduced Hessian R.
*                      S refers to column and row scales.
*             start = 'Hot H' (for example) means that only the Hessian
*                      is defined.
*
*     m       is the number of general constraints.  For LP problems
*             this means the number of rows in the constraint matrix A.
*             m > 0 in principle, though sometimes m = 0 may be ok.
*             (Strictly speaking, Fortran declarations of the form
*                double precision   pi(m)
*             require m > 0.  In debug mode, compilers will probably
*             enforce m > 0, but optimized code may run ok with m = 0.)
*
*     n       is the number of variables, excluding slacks.
*             For LP problems, this is the number of columns in A.
*             n > 0.
*
*     nb      is n + m.
*
*     ne      is the number of nonzero entries in A (including the
*             Jacobian for any nonlinear constraints).
*             ne > 0 in principle, though again m = 0, ne = 0 may work
*             with some compilers.
*
*     nname   is the number of column and row names provided in the
*             arrays name1 and name2.  If nname = 1, there are NO names.
*             Generic names will be used in the printed solution.
*             Otherwise, nname = nb and all names must be provided.
*
*     nncon   is the number of nonlinear constraints.
*             nncon ge 0.
*
*     nnobj   is the number of nonlinear objective variables.
*             nnobj ge 0.
*
*     nnjac   is the number of nonlinear Jacobian variables.
*             If nncon = 0, nnjac = 0.
*             if nncon > 0, nnjac > 0.
*
*     iobj    says which row of A is a free row containing a linear
*             objective vector  c  (iobj = 0 if none).
*             iobj = 0  or  nncon < iobj le m.
*
*     objadd  is a constant that will be added to the objective.
*             Typically objadd = 0.0d+0.
*
*     names(5)is a set of 8-character names for the problem, the linear
*             objective, the rhs, the ranges and bounds.  (This is a
*             hangover from MPS files.  The names are used in the
*             printed solution and in some of the basis files.)
*
*     a(ne)   is the constraint matrix (Jacobian), stored column-wise.
*
*     ha(ne)  is the list of row indices for each nonzero in a(*).
*
*     ka(n+1) is a set of pointers to the beginning of each column of
*             the constraint matrix within a(*) and ha(*).
*             Must have ka(1) = 1 and ka(n+1) = ne+1.
*
*  NOTES:  1. If the problem has a nonlinear objective,
*             the first nnobj columns of a and ha belong to the
*             nonlinear objective variables.
*             Subroutine funobj deals with these variables.
*          
*          2. If the problem has nonlinear constraints,
*             the first nnjac columns of a and ha belong to the
*             nonlinear Jacobian variables, and
*             the first nncon rows of a and ha belong to the
*             nonlinear constraints.
*             Subroutine funcon deals with these variables and
*             constraints.
*          
*          3. If nnobj > 0 and nnjac > 0, the two sets of
*             nonlinear variables overlap.  The total number of
*             nonlinear variables is nn = max( nnobj, nnjac ).
*          
*          4. The Jacobian forms the top left corner of a and ha.
*             If a Jacobian column j (1 le j le nnjac) contains
*             any entries a(k), ha(k) associated with nonlinear
*             constraints (1 le ha(k) le nncon), those entries must
*             come before any other (linear) entries.
*          
*          5. The row indices ha(k) for a column may be in any order
*             (subject to Jacobian entries appearing first).
*             Subroutine funcon must define Jacobian entries in the
*             same order.
*          
*          6. If column j contains no entries, perhaps
*             ka(j) = ka(j+1) is acceptable.  (Must check this.
*             When MINOS reads an MPS with empty columns,
*             it inserts a dummy entry a(k) = 0.0d+0, ha(k) = 1.
*             This may not be necessary.)
*
*          7. To allocate storage, MINOS needs to know if the Jacobian
*             is dense or sparse.  The default is dense.  If this is
*             not appropriate, define
*                              Jacobian    Sparse
*             in the Specs file, or
*                call miopt ( 'Jacobian    Sparse', 0, 0, inform )
*             before calling minoss.
*                
*     bl(nb)  is the lower bounds on the variables and slacks (x, s).
*
*     bu(nb)  is the upper bounds on (x, s).
*
*     BEWARE: MINOS represents general constraints as Ax + s = 0.
*             Constraints of the form   l <= Ax <= u  
*             therefore mean            l <= -s <= u,
*             so that                  -u <=  s <= -l.
*             The last m components of bl and bu are -u and -l.
*
*     name1(nname), name2(nname) are two integer arrays.
*             If nname =  1, name1 and name2 are not used.  The printed
*             solution will use generic names for the columns and row.
*             If nname = nb, name1(j) and name2(j) should contain the
*             name of the j-th variable in 2a4 format (j = 1, nb).
*             If j = n+i, the j-th variable is the i-th row.
*
*     hs(nb)  sometimes contains a set of initial states for each
*             variable x or for each column and slack (x, s).
*             See the following NOTES.
*
*     xn(nb)  sometimes contains a set of initial values for each
*             variable x or for each column and slack (x, s).
*             See the following NOTES.
*
*  NOTES:  1. If start = 'Cold' or 'Basis file' and a BASIS file
*             of some sort is to be input
*             (an OLD BASIS file, INSERT file or LOAD file),
*             hs and xn need not be set at all.
*
*          2. Otherwise, hs(j) and xn(j), j=1:n, must be defined for a
*             Cold start.  (The values for j=n+1:nb need not be set.)
*             If nothing special is known about the problem, or if
*             there is no wish to provide special information,
*             you may set hs(j) = 0, xn(j) = 0.0d+0 for all j=1:n.
*             All variables will be eligible for the initial basis.
*        
*             Less trivially, to say that variable j will probably
*             be equal to one of its bounds,
*             set hs(j) = 4 and xn(j) = bl(j)
*             or  hs(j) = 5 and xn(j) = bu(j) as appropriate.
*        
*          3. For Cold starts with no basis file, a Crash procedure
*             is used to select an initial basis.  The initial basis
*             matrix will be triangular (ignoring certain small
*             entries in each column).
*             The values hs(j) = 0, 1, 2, 3, 4, 5 have the following
*             meaning:
*                
*             hs(j)    State of variable j during Crash
*        
*             0, 1, 3  Eligible for the basis.  3 is given preference.
*             2, 4, 5  Ignored.
*        
*             After Crash, hs(j) = 2 entries are made superbasic.
*             Other entries not selected for the basis are made
*             nonbasic at the value xn(j) if bl(j) <= xn(j) <= bu(j),
*             or at the value bl(j) or bu(j) closest to xn(j).
*
*          4. For Warm or Hot starts, all of hs(1:nb) is assumed to be
*             set to the values 0, 1, 2 or 3 (probably from some
*             previous call) and all of xn(1:nb) must have values.
*        
*     pi(m)   contains an estimate of the vector of Lagrange multipliers
*             (shadow prices) for the NONLINEAR constraints.  The first
*             nncon components must be defined.  They will be used as
*             lambda in the subproblem objective function for the first
*             major iteration.  If nothing is known about lambda,
*             set pi(i) = 0.0d+0, i = 1 to nncon.
*
*     ns      need not be specified for Cold starts,
*             but should retain its value from a previous call
*             when a Warm or Hot start is used.
*
*
*     ON EXIT:
*
*     hs(nb)  is the final state vector:
*
*                hs(j)    State of variable j    Normal value of xn(j)
*
*                  0      nonbasic               bl(j)
*                  1      nonbasic               bu(j)
*                  2      superbasic             Between bl(j) and bu(j)
*                  3      basic                  ditto
*
*             Very occasionally there may be nonbasic variables for
*             which xn(j) lies strictly between its bounds.
*             If ninf = 0, basic and superbasic variables may be outside
*             their bounds by as much as the Feasibility tolerance.
*             Note that if Scale is specified, the Feasibility tolerance
*             applies to the variables of the SCALED problem. 
*             In this case, the variables of the original problem may be
*             as much as 0.1 outside their bounds, but this is unlikely
*             unless the problem is very badly scaled.
*
*     xn(nb)  is the final variables and slacks (x, s).
*
*     pi(m)   is the vector of Lagrange multipliers (shadow prices)
*             for the general constraints.
*
*     rc(nb)  is a vector of reduced costs: rc = g - (A I)'pi, where g
*             is the gradient of the objective function if xn is feasible
*             (or the gradient of the Phase-1 objective otherwise).
*             If ninf = 0, the last m entries are -pi (negative pi).
*
*     inform  says what happened; see Chapter 6.3 of the User's Guide.
*             A summary of possible values follows:
*
*             inform   Meaning
*
*                0     Optimal solution found.
*                1     The problem is infeasible.
*                2     The problem is unbounded (or badly scaled).
*                3     Too many iterations.
*                4     Apparent stall.  The solution has not changed
*                      for a large number of iterations (e.g. 1000).
*                5     The Superbasics limit is too small.
*                6     Subroutine funobj or funcon requested termination
*                      by returning mode < 0.
*                7     Subroutine funobj seems to be giving incorrect
*                      gradients.
*                8     Subroutine funcon seems to be giving incorrect
*                      gradients.
*                9     The current point cannot be improved.
*               10     Numerical error in trying to satisfy the linear
*                      constraints (or the linearized nonlinear
*                      constraints).  The basis is very ill-conditioned.
*               11     Cannot find a superbasic to replace a basic
*                      variable.
*               12     Basis factorization requested twice in a row.
*                      Should probably be treated as inform = 9.
*               13     Near-optimal solution found.
*                      Should probably be treated as inform = 9.
*
*               20     Not enough storage for the basis factorization.
*               21     Error in basis package.
*               22     The basis is singular after several attempts to
*                      factorize it (and add slacks where necessary).
*
*               30     An OLD BASIS file had dimensions that did not
*                      match the current problem.
*               32     System error.  Wrong number of basic variables.
*
*               40     Fatal errors in the MPS file.
*               41     Not enough storage to read the MPS file.
*               42     Not enough storage to solve the problem.
*
*     mincor  says how much storage is needed to solve the problem.
*             If inform = 42, the work array z(nwcore) was too small.
*             minoss may be called again with nwcore suitably larger
*             than mincor.  (The bigger the better, since it is
*             not certain how much storage the basis factors need.)
*
*     ns      is the final number of superbasics.
*
*     ninf    is the number of infeasibilities.
*
*     sinf    is the sum    of infeasibilities.
*
*     obj     is the value  of the objective function.
*             If ninf = 0, obj includes the nonlinear objective if any.
*             If ninf > 0, obj is just the linear objective if any.
*
*     30 Sep 1991: First version.
*     06 Dec 1991: A few more output parameters.
*     10 Apr 1992: Parameter  objadd added.  Parameters reordered.
*     20 Apr 1992: Parameters nname, name1, name2 added.
*     27 Apr 1992: Parameter  mincor added to allow reentry with more
*                  storage.
*     27 Jun 1992: Parameter  start  implemented.  Passed to misolv.
*     ------------------------------------------------------------------
*     NOTE:
*     In /m7len / and /m8len /, nnobjx, nnconx, nnjacx are normally
*                               nnobj , nncon , nnjac .
*     Here it is better to save those names for the minoss parameters.

      common    /m2len / mrows,mcols,melms
      common    /m2mapz/ maxw  ,maxz
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m7len / fobj  ,fobj2 ,nnobjx,nnobj0
      common    /m8len / njac  ,nnconx,nncon0,nnjacx
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /cycle2/ objtru,suminf,numinf

      character*5        f1
      data               f1 /'(2a4)'/
*     ------------------------------------------------------------------

*     Initialize timers.

      call m1time( 0,0 )

*     Load the Common variables with various problem dimensions.

      mrows  = m
      mcols  = n
      melms  = ne
      nnconx = nncon
      nnobjx = nnobj
      nnjacx = nnjac

*     Say how much z(*) we've got, in case mispec wasn't called,
*     or nwcore has been altered since mispec was called.
*     This means the Specs file can't set  Workspace (TOTAL) .

      maxz   = nwcore

*     The Specs file has been read (or the options have been
*     otherwise defined).  Check that the options have sensible values.

      call m3dflt( 2 )

*     ------------------------------------------------------------------
*     Determine storage requirements using the
*     following Common variables:
*        (m2len )   mrows, mcols, melms
*        (m3len )   nscl  (determined by lscale)
*        (m5len )   maxr, maxs, nn
*        (m7len )   nnobj
*        (m8len )   njac, nncon, nnjac
*     All have to be known exactly before calling m2core( 4, ... ).
*     The only one in doubt is njac, the number of Jacobian elements.
*     If Jacobian = dense  (nden = 1), m2core sets njac = nncon*nnjac.
*     If Jacobian = sparse (nden = 2), we have to set njac here.
*     ------------------------------------------------------------------

      njac = 0
      if (nncon .gt. 0  .and.  nden .eq. 2) then
         last = ka(nnjac+1) - 1
         if (nncon .eq. m) then
            njac = last
         else
            do 100 k = 1, last
               i     = ha(k)
               if (i .le. nncon) njac = njac + 1
  100       continue         
         end if
      end if

      call m2core( 4, mincor )

      if (mincor .gt. nwcore) then
         inform = 42
         return
      end if

*     ------------------------------------------------------------------
*     Open files needed for this problem.
*     Print the options if iprint > 0, Print level > 0 and iparm(3) > 0.
*     ------------------------------------------------------------------
      call mifile( 2 )
      call m3dflt( 3 )

*     ------------------------------------------------------------------
*     Load names into the MINOS arrays.
*     ------------------------------------------------------------------
      read (names(1), f1) name
      read (names(2), f1) mobj
      read (names(3), f1) mrhs
      read (names(4), f1) mrng
      read (names(5), f1) mbnd
      
*     ------------------------------------------------------------------
*     Solve the problem.
*     ------------------------------------------------------------------
      mimode = 2
      nka    = n + 1
      call misolv( mimode, start, m, n, nb, ne, nka, nname,
     $             iobj, objadd, 
     $             a, ha, ka, bl, bu, name1, name2,
     $             hs, xn, pi, rc, 
     $             inform, ns, z, nwcore )

      ninf   = numinf
      sinf   = suminf
      obj    = objtru

*     Print times for all clocks (if ltime > 0).

      call m1time( 0,2 )

*     end of minoss
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine minos1( z, nwcore )

      implicit           double precision (a-h,o-z)
      double precision   z(nwcore)

*     ------------------------------------------------------------------
*     minos1 is used for the stand-alone version of MINOS.
*     It is called by the main program (or equivalent driver).
*     It repeatedly looks for a new problem in the SPECS file
*     and asks for it to be solved, until opfile returns inform gt 1,
*     which means an ENDRUN card was found in the SPECS file,
*     or end-of-file was encountered.
*
*     06 Oct 1985: minos1 calls minos2 to allow reallocation of z(*),
*                  following suggestions from David Gay,
*                  AT&T Bell Laboratories (for Unix).
*     22 Dec 1987: For DEC VAX VMS, newz tells minos2 whether or not to
*                  re-allocate z(*).
*     03 Mar 1988: f77 version calls opfile.
*     01 Oct 1991: mispec and misolv implemented.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m1savz/ nbytes,newz
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m2mapz/ maxw  ,maxz
      common    /m8len / njac  ,nncon ,nncon0,nnjac

      external           m3key
      character*30       title

      newz   = 0

*     ------------------------------------------------------------------
*     Define global files (reader, printer, etc.)
*     ------------------------------------------------------------------
      ispecs = 4
      iprint = 9
      isumm  = 6
      call mifile( 1 )

*     ==================================================================
*     Loop through each problem in the SPECS file.
*     ==================================================================
      do 100 loop = 1, 100000
         ncalls = loop
         maxw   = 0
         maxz   = nwcore

*        Initialize timers.
      
         call m1time( 0,0 )
      
*        ---------------------------------------------------------------
*        Define the MINOS title and read the Specs file.
*        ---------------------------------------------------------------
         call m1init( title )
         call opfile( ncalls, ispecs, m3key,
     $                title , iprint, isumm, inform )
      
         if (inform .ge. 2) then
            inform = 100 + inform 
            return
         end if
      
*        ---------------------------------------------------------------
*        Check options.
*        Open files needed for this problem.
*        ---------------------------------------------------------------
         call m3dflt( 2 )
         call mifile( 2 )
      
*        ---------------------------------------------------------------
*        Estimate storage requirements using the
*        following Common variables:
*           (m2len )   mrows, mcols, melms
*           (m3len )   nscl
*           (m5len )   maxr, maxs
*           (m7len )   nnobj
*           (m8len )   njac, nncon, nnjac
*        All except njac have been set by default or by the SPECS file.
*        We haven't read the MPS file yet, so m2core estimates njac.
*        ---------------------------------------------------------------
         call m2core( 1, mincor )
      
*        ---------------------------------------------------------------
*        Solve the problem.
*        ---------------------------------------------------------------
         call minos2( z, nwcore, mincor, inform )
  100 continue
*     ==================================================================
*     End of loop through SPECS file.
*     ==================================================================

*     end of minos1
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine minos2( z, nwcore, mincor, inform )

      implicit           double precision (a-h,o-z)
      double precision   z(nwcore)

*     ------------------------------------------------------------------
*     minos2 asks minos3 to solve the problem just found in the SPECS
*     file.  For standard Fortran installations, that is all.
*
*     For some installations, this is an appropriate place to
*     increase the size of the workspace array  z,  to allow
*     solution of arbitrarily large problems without recompiling
*     the main program, in which z(nwcore) is originally declared.
*
*     z  might have to be in blank common if it is to be expanded.
*     The default size  nwcore  should be large enough to solve
*     reasonably big problems without change (e.g., nwcore = 100000).
*
*     At this stage, the SPECS file has been read and values are known
*     for maxw, maxz and mincor.  The default values for the first two
*     are maxw = 0 and maxz = nwcore, but we allow the user to alter
*     these by means of two cards in the SPECS file of the following
*     form...
*
*        Workspace (user)      10000 words  (This sets  maxw = 10000)
*        Workspace (total)     90000 words  (This sets  maxz = 90000)
*
*     MINOS will use only  Z(maxw+1), ..., Z(maxz).  Hence,
*     z(1), ..., z(maxw)    and possibly   z(maxz+1), ..., z(nwcore)
*     may be used as workspace by the user during solution of this
*     particular problem (e.g., within funobj or funcon).
*
*     If maxz is set to a value less than nwcore, it may serve to
*     reduce paging activity on a machine with virtual memory, by
*     confining MINOS (in particular the basis-factorization routines)
*     to an area of core that is sensible for the current problem.
*     On some systems (e.g., Burroughs), this will allow z(nwcore)
*     to be declared arbitrarily large at compile time.
*
*     mincor contains an estimate of the minimum core requirements,
*     allowing for maxw but ignoring maxz.  z is already large enough
*     if nwcore .ge. max( maxz, mincor ).  Systems that allow z to be
*     re-allocated at run-time should make appropriate use of the
*     logical variable enough.
*
*     22 Dec 1987: VAX dynamic storage added by Steve White, DSIR.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m1savz/ nbytes,newz
      common    /m2mapz/ maxw  ,maxz

      intrinsic          max, min
      logical            enough

      if (iprint .gt. 0) write(iprint, 1100) maxw, mincor
      nwcor2 = max( maxz, mincor )
*-->  enough = nwcore .ge. nwcor2
      enough = .true.

      if ( enough ) then

*        Use the storage already available.

         maxz   = min( maxz, nwcore )
         if (iprint .gt. 0) write(iprint, 1200) maxw, maxz, nwcore
         call minos3( z, nwcore, inform )
      else
*-->
*        Re-allocate  z  (using some non-standard Fortran).
*        newz says if we have to free z from an earlier call.
*        The following is for DEC VAX/VMS.
*
*        nwcor2 = max( nwcore, maxz, mincor )
*        if (newz .ne. 0) then
*           istat = lib$free_vm( nbytes, newz )
*           if (.not. istat) then
*              call lib$signal( %val(istat) )
*           end if
*        end if
*        maxz   = nwcor2
*        nbytes = nwcor2*8
*        istat  = lib$get_vm( nbytes, newz )
*        if (.not. istat) then
*           call lib$signal( %val(istat) )
*        else
*           if (iprint .gt. 0) write(iprint, 1200) maxw, maxz, nwcor2
*           call minos3( %val(newz), nwcor2, inform )
*        end if
      end if
      return

 1100 format(/ ' Reasonable Workspace limits are', i10, ' ...', i8)
 1200 format(  ' Actual     Workspace limits are', i10, ' ...',
     $   i8, ' ...', i8, '  words of  z.')

*     end of minos2
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine minos3( z, nwcore, inform )

      implicit           double precision (a-h,o-z)
      double precision   z(nwcore)

*     ------------------------------------------------------------------
*     minos3 prints the options (sometimes),
*     then reads a problem from an MPS file
*     and asks misolv to solve it.
*
*     01 Oct 1991: misolv implemented.
*     20 Apr 1992: name1, name2, nname added as parameters to misolv.
*     26 Apr 1992: call m3dflt( 3 ) added.
*     ------------------------------------------------------------------

      common    /m2mapa/ ne    ,nka   ,la    ,lha   ,lka
      common    /m3len / m     ,n     ,nb    ,nscl
      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m3mps1/ lname1,lname2,lkeynm,nname
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr  ,lprint
*     ------------------------------------------------------------------

      call m3dflt( 3 )
      call m1time( 1,0 )
      call m3inpt( objadd, z, nwcore )
      call m1time(-1,0 )

      if (ierr .ne. 0) return

      mimode = 1
      nname  = nb
      lrc    = lpi + m
      
      call misolv( mimode, 'Cold', m, n, nb, ne, nka, nname,
     $             iobj  , objadd,
     $             z(la),  z(lha), z(lka), z(lbl), z(lbu),
     $             z(lname1),   z(lname2),
     $             z(lhs), z(lxn), z(lpi), z(lrc), 
     $             inform, ns    , z     , nwcore )

*     Print times for all clocks (if ltime > 0).

      call m1time( 0,2 )

*     end of minos3
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine mifile( mode )

*     ------------------------------------------------------------------
*     mifile  is a (possibly) machine-dependent routine for opening
*     various files.
*     Some systems (e.g. DEC VAX/VMS) open files automatically.
*     Other systems may require explicit OPEN statements.
*
*     19 Jun 1989: Modified to call m1open.
*     11 Nov 1991: mode 1 now assumes that ispecs, iprint, isumm
*                  have been set by minos1 (for MINOS)
*                             or by mispec (for minoss).
*
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt

      integer            iprinx, isummx
      save               iprinx, isummx

      if (mode .eq. 1) then
*        ---------------------------------------------------------------
*        Mode 1: Open default files (if opens are necessary).
*
*        iread and iprint have the following use:
*        Input  files (MPS, INSERT, LOAD, OLD BASIS)     are rewound
*        after being read,    but not if they are the same as  iread.
*        Output files (DUMP, NEW BASIS, PUNCH, SOLUTION) are rewound
*        after being written, but not if they are the same as iprint.
*     
*        iread  = (conceptually) the CARD READER that can't be rewound.
*                 If there are no such units, use 0.
*                 MINOS does not use this file directly, so there is
*                 no 'open'.
*        iprint = the PRINT file.
*        isumm  = the SUMMARY file.  Sometimes this is the screen.
*                 If so, it may not need to be opened.
*        ispecs = the SPECS file, containing one or more problem specs.
*                 This file is not rewound after use, because it may
*                 contain another SPECS file.
*        iscr   = SCRATCH file -- no longer used.
*
*        Files may be opened directly or by calling m1open, e.g.
*           if (iprint .gt. 0) open( unit=iprint, file='minos.print' )
*        or
*           call m1open( iprint, 'OUT' )
*
*        iread and ispecs remain the same throughout the run.
*        iprint and isumm may be altered by the SPECS file
*        and therefore need to be opened twice.
*        ---------------------------------------------------------------
      
         iread  = 5
         iprinx = iprint
         isummx = isumm
         if (ispecs .gt. 0) call m1open( ispecs, 'IN ' )
         if (iprint .gt. 0) call m1open( iprint, 'OUT' )
         if (isumm  .gt. 0) call m1open( isumm , 'OUT' )
      else      
*        ---------------------------------------------------------------
*        Mode 2: Define files mentioned in the SPECS file just read.
*        Input files are opened first.  Only one basis file is needed.
*        ---------------------------------------------------------------
         if (imps  .le. 0     ) imps = ispecs
         if (imps  .ne. ispecs) call m1open( imps , 'IN ' )
      
         if      (ioldb .gt. 0) then
                           call m1open( ioldb, 'IN ' )
         else if (insrt .gt. 0) then
                           call m1open( insrt, 'IN ' )
         else if (iload .gt. 0) then
                           call m1open( iload, 'IN ' )
         end if
         
         if (iback .gt. 0) call m1open( iback, 'OUT' )
         if (idump .gt. 0) call m1open( idump, 'OUT' )
         if (inewb .gt. 0) call m1open( inewb, 'OUT' )
         if (ipnch .gt. 0) call m1open( ipnch, 'OUT' )
         if (isoln .gt. 0) call m1open( isoln, 'OUT' )

*        Open new Print or Summary files if they were altered
*        by the Specs file.

         if (iprint .ne. iprinx) then
            if (iprint .gt. 0) call m1open( iprint, 'OUT' )
         end if
         if (isumm  .ne. isummx) then
            if (isumm  .gt. 0) call m1open( isumm , 'OUT' )
         end if
      end if
   
*     end of mifile
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine mispec( ispecx, iprinx, isummx, nwcore, inform )

*     ------------------------------------------------------------------
*     mispec  is called by minoss (not by MINOS) to do the following:
*     1. Open default files (Specs, Print, Summary).
*     2. Initialize title.
*     3. Set options to default values.
*     4. Read the Specs file if any.
*
*     01 Oct 1991: First version.
*     27 Jun 1992: Don't read a Specs file if ispecs <= 0.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m2mapz/ maxw  ,maxz

      external           m3key
      character*30       title

*     Open the specified Specs, Print and Summary files.

      ispecs = ispecx
      iprint = iprinx
      isumm  = isummx
      call mifile( 1 )

      ncalls = 1
      maxw   = 0
      maxz   = nwcore
      inform = 0

*     ------------------------------------------------------------------
*     Define the MINOS title and read the Specs file (if any).
*     minoss will check the options later and maybe print them.
*     ------------------------------------------------------------------
      call m1init( title )
      if (ispecs .gt. 0) then
         call opfile( ncalls, ispecs, m3key,
     $                title , iprint, isumm, inform )
      else
*        Just set options to default values.
         call m3dflt( 1 )
      end if

*     end of mispec
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine misolv( mimode, start,
     $                   mxx, nxx, nbxx, nexx, nkax, nnamex,
     $                   iobjxx, objadd,
     $                   a, ha, ka, bl, bu, name1, name2,
     $                   hs, xn, pi, rc, 
     $                   inform, ns, z, nwcore )

      implicit           double precision (a-h,o-z)
      character*(*)      start
      integer*4          ha(nexx), hs(nbxx)
      integer            ka(nkax), name1(nnamex), name2(nnamex)
      double precision   a(nexx) , bl(nbxx), bu(nbxx)
      double precision   xn(nbxx), pi(mxx) , rc(nxx) , z(nwcore)

*     ------------------------------------------------------------------
*     misolv solves the current problem.
*
*     On entry,
*     the SPECS file has been read,
*     all data items have been loaded (including a, ha, ka, ...),
*     and workspace has been allocated within z.
*
*     mimode  =  1 if the call is from minos3 (stand-alone MINOS).
*            ge  2 if the call is from minoss.
*
*     On exit,
*     inform  =  0 if an optimal solution was found,
*             =  1 if the problem was infeasible,
*             =  2 if the problem was unbounded,
*             =  3 if the Iteration limit was exceeded,
*            ge  4 if iterations were terminated by some other
*                  error condition (see the MINOS user's guide).
*
*     01 Oct 1991: minoss, mispec and misolv implemented.
*                  minos1, minos2, minos3 reorganized
*                  to facilitate calling MINOS as a subroutine.
*     25 Nov 1991: nname and rc added as parameters of matmod.
*     10 Apr 1992: objadd added as input parameter.
*     20 Apr 1992: Parameter list revised.  nname, name1, name2 added.
*     27 Jun 1992: Cold, Warm, Hot start implemented.
*     09 Jul 1992: ns initialized here only for Cold starts, just to
*                  help debugging.  m4chek always sets it later.
*     ------------------------------------------------------------------
*
*     All common blocks are listed here for reference.
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
*     File mcommon fortran.


*  Machine-dependent items.

      logical            alone, AMPL, GAMS, MINT, page1, page2
      common    /m1env / alone, AMPL, GAMS, MINT, page1, page2
      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m1savz/ nbytes,newz
      parameter        ( ntime = 5 )
      common    /m1tim / tlast(ntime), tsum(ntime), numt(ntime), ltime
      common    /m1word/ nwordr,nwordi,nwordh


*  Files, maps, parameters.

      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m2len / mrows,mcols,melms
      common    /m2lu1 / minlu,maxlu,lena,nbelem,ip,iq,lenc,lenr,
     $                   locc,locr,iploc,iqloc,lua,indc,indr
      common    /m2lu2 / factol(5),lamin,nsing1,nsing2
      common    /m2lu3 / lenl,lenu,ncp,lrow,lcol
      common    /m2lu4 / parmlu(30),luparm(30)
      common    /m2mapa/ ne    ,nka   ,la    ,lha   ,lka
      common    /m2mapz/ maxw  ,maxz
      common    /m2parm/ dparm(30),iparm(30)


*  Problem size, MPS names, Scale options.

      common    /m3len / m     ,n     ,nb    ,nscl
      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m3mps1/ lname1,lname2,lkeynm,nname
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3mps5/ aelem(2), id(6), iblank
      common    /m3scal/ sclobj,scltol,lscale


*  LP items.

      common    /m5len / maxr  ,maxs  ,mbs   ,nn    ,nn0   ,nr    ,nx
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m5freq/ kchk,kinv,ksav,klog,ksumm,i1freq,i2freq,msoln
      common    /m5inf / prinf, duinf, jprinf, jduinf
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m5log2/ jq1,jq2,jr1,jr2,lines1,lines2
      common    /m5log3/ djq,theta,pivot,cond,nonopt,jp,jq,modr1,modr2
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5lp2 / invrq,invitn,invmod
      common    /m5prc / nparpr,nmulpr,kprc,newsb
      common    /m5step/ featol, tolx0,tolinc,kdegen,ndegen,
     $                   itnfix, nfix(2)
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm


*  Nonlinear objective.

      logical            conv,restrt
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m7loc / lgobj ,lgobj2
      common    /m7cg1 / cgbeta,itncg,msgcg,modcg,restrt
      common    /m7cg2 / lcg1,lcg2,lcg3,lcg4,modtcg,nitncg,nsubsp
      common    /m7conv/ etash,etarg,lvltol,nfail,conv(4)
      common    /m7phes/ rgmin1,rgnrm1,rgnrm2,jz1,jz2,labz,nfullz,mfullz
      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg


*  Nonlinear constraints.

      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /m8al2 / radius,rhsmod,modpen,modrhs
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2
      common    /m8save/ vimax ,virel ,maxvi ,majits,minits,nssave
      common    /m8veri/ jverif(4),lverif(2)


*  Miscellaneous.

      logical            gotbas,gotfac,gothes,gotscl
      common    /cycle1/ gotbas,gotfac,gothes,gotscl
      common    /cycle2/ objtru,suminf,numinf
      common    /cyclcm/ cnvtol,jnew,materr,maxcy,nephnt,nphant,nprint

***** end of file mcommon fortran.
*     ------------------------------------------------------------------

      character*1        ch1
      logical            finish, lincon, nlncon, nlnobj

*     For minoss we have to copy m, n, etc into common.

      m      = mxx
      n      = nxx
      nb     = nbxx
      ne     = nexx
      nka    = nkax
      nname  = nnamex
      iobj   = iobjxx

*     Initialize a few things.

      lincon = nncon  .eq. 0
      nlncon = nncon  .gt. 0
      nlnobj = nnobj  .gt. 0
      ierr   = 0
      lenl   = 0
      lenu   = 0
      ncycle = 0
      ninf   = 0
      nstat1 = 1
      nstat2 = 1
      sclobj = 1.0d+0
      jobj   = 0
      if (iobj .gt. 0) jobj = n + iobj

*     ------------------------------------------------------------------
*     Decode 'start'.
*     ------------------------------------------------------------------
      gotbas = .false.
      gotfac = .false.
      gothes = .false.
      gotscl = .false.
      ch1    = start(1:1)

      if      (ch1 .eq. 'C'  .or.  ch1 .eq. 'c'  .or.
     $         ch1 .eq. 'B'  .or.  ch1 .eq. 'b') then

*        Cold start  or  Basis file.

         istart = 0
         gotbas = (ioldb + insrt + iload) .gt. 0
         ns     = 0

      else if (ch1 .eq. 'W'  .or.  ch1 .eq. 'w') then

*        Warm start.

         istart = 1
         gotbas = .true.

      else if (ch1 .eq. 'H'  .or.  ch1 .eq. 'h') then

*        Hot start.
*        'Hot' is the same as 'Hot FHS'.
*        Look for 'Hot F', 'Hot FH', etc.

         istart = 2
         gotbas = .true.
         nchar  = len( start )
         if (nchar .lt. 5) then
            gotfac = .true.
            gothes = .true.
            gotscl = .true.
         else
            do 100 j = 5, nchar
               ch1 = start(j:j)
               if (ch1 .eq. 'F'  .or.  ch1 .eq. 'f') gotfac = .true.
               if (ch1 .eq. 'H'  .or.  ch1 .eq. 'h') gothes = .true.
               if (ch1 .eq. 'S'  .or.  ch1 .eq. 's') gotscl = .true.
  100       continue
         end if

         if (iprint .gt. 0) then
            write(iprint, 1020) gotbas, gotfac, gothes, gotscl
         end if
         
      else
         istart = 0
         if (iprint .gt. 0) write(iprint, 1030) start
         if (isumm  .gt. 0) write(isumm , 1030) start
      end if

      nssave = ns

*     ------------------------------------------------------------------
*     1. Fiddle with partial price parameter to avoid foolish values.
*        We reduce nparpr if both the row and column section sizes
*        would be smaller than minprc (= 10 say).
*     2. Change Scale option 1 to 0 if all variables are nonlinear.
*     ------------------------------------------------------------------
      minprc = 10
      npr1   = n / nparpr
      npr2   = m / nparpr
      if (max( npr1, npr2 ) .lt. minprc) then
         maxmn  = max( m, n )
         nparpr = maxmn / min( maxmn, minprc )
         npr1   = n / nparpr
         npr2   = m / nparpr
      end if

      if (lscale .eq. 1  .and.  nn .eq. n) lscale = 0

      if (iprint .gt. 0) write(iprint, 1100) lscale, nparpr, npr1, npr2
      if (isumm  .gt. 0) write(isumm , 1110) lscale, nparpr

*     ------------------------------------------------------------------
*     Set the vector of row types and print the matrix statistics.
*     ------------------------------------------------------------------
      call m2amat( 1, m, n, nb,
     $             ne, nka, a, ha, ka,
     $             bl, bu, z(lhrtyp) )

*     ------------------------------------------------------------------
*     Load Jacobian elements in the MPS file from A into gcon and gcon2.
*     ------------------------------------------------------------------
      if (nlncon) then
         call m8augl( 1, m, n, nb, ns, inform,
     $                ne, nka, a, ha, ka,
     $                hs, bl, bu, xn, z, nwcore )
      end if

*     ------------------------------------------------------------------
*     Input a basis file if one exists, thereby defining hs and xn.
*     (Otherwise, m2crsh will be called later to define hs.)
*     At this stage, ncycle = 0.
*     ------------------------------------------------------------------
      call m1page( 1 )
      if (iprint .gt. 0) then
         write(iprint, 1200)
         if (.not. gotbas) write(iprint, 1210)
      end if

      call m4getb( ncycle, istart, m, mbs, n, nb, nn, nname, nscl,
     $             lcrash, ns,
     $             ne, nka, a, ha, ka,
     $             z(lhrtyp), hs, z(lkb), z(lascal), bl, bu,
     $             pi, xn, z(ly), z(ly2), name1, name2,
     $             z, nwcore )
      if (ierr .ne. 0) go to 900


*     ------------------------------------------------------------------
*                             CYCLE  PROCEDURE
*
*     The following notes are relevant if Cycle limit = 2 or more.
*
*  1. Scaling and/or Crash are controlled on each cycle by the following
*     logical variables:
*
*     If gotscl is true, scales are retained from the previous cycle.
*                        Otherwise, scales are recomputed (if lscale>0).
*
*     If gotbas is true, the basis is retained.  Otherwise, Crash is
*                        called.
*
*
*  2. When m5solv is called, Flying Starts are controlled by the
*     following logical variables:
*
*     If gotfac is true, an LU factorization of the basis is assumed
*                        to be present.  (Ignored if there are any
*                        nonlinear constraints.)
*
*     If gothes is true, z(lr) is assumed to contain a useful
*                        reduced-Hessian approximation.
*
*
*  3. For the next cycle,
*        m4getb sets gotscl and gotbas to be true, and
*        m5solv sets gotfac and gothes to current values (usually true).
*     These values will often be appropriate.  However, the expert user
*     of matmod must set some or all of the logical variables to .false.
*     if the problem data or state have been significantly altered.
*
*     For example, if the Jacobian was used by the scaling routine
*     (Scale option 2) and if the Jacobian could be rather different
*     from its value at the start of the previous cycle, it may be
*     advisable to request new scales by setting gotscl = .false.
*
*     Similarly, if matmod alters some matrix elements in columns that
*     are currently basic, one should set gotfac = .false. to force
*     refactorization.  In particular, if the linear objective row c is
*     altered, gotfac should be set to .false., since c is part of the
*     LU factors.
*     ------------------------------------------------------------------

      finish = .false.
      jnew   = n - nphant
      materr = 0
      nprntd = 0
      nsolvd = ncycle

*     If Cycle limit is more than 1, call matmod with ncycle = 0 in case
*     the user wants a chance to set things up before any solves.

      if (maxcy .gt. 1) then
         if (iprint .gt. 0) write(iprint, 3000) ncycle
         if (isumm  .gt. 0) write(isumm , 3000) ncycle
      
         call matmod( ncycle, nprob, finish,
     $                m, n, nb, ne, nka, ns, nscl, nname,
     $                a, ha, ka, bl, bu,
     $                z(lascal), hs, name1, name2,
     $                xn, pi, rc, z, nwcore )
         if (finish) go to 800
      end if

*     ------------------------------------------------------------------
*     Check gradients if requested, before scaling interferes.
*     m4getb has made sure nonlinear xn(j)s are within their bounds.
*     ------------------------------------------------------------------
      if (nlncon) then
         call m8chkj( nncon, nnjac, njac, nx,
     $                ne, nka, ha, ka,
     $                bl, bu, z(lfcon), z(lfcon2),
     $                z(lgcon), z(lgcon2),
     $                xn, z(ly), z(ly2), z, nwcore )
         if (ierr .gt. 0) go to 900
      end if

      if (nlnobj) then
         call m7chkg( nnobj,
     $                bl, bu, z(lgobj), z(lgobj2),
     $                xn, z(ly), z(ly2), z, nwcore )
         if (ierr .gt. 0) go to 900
      end if

*     ==================================================================
*     Start of the Cycle loop.
*     ==================================================================
  500    ncycle = ncycle + 1
         nsolvd = ncycle
         ierr   = 0
         if (.not. gotbas) then
            gotfac = .false.
            gothes = .false.
         end if
         if (ncycle .gt. 1) then
            call m1page( 1 )
            if (iprint .gt. 0) then
               write(iprint, 2000) ncycle
               write(iprint, 1020) gotbas, gotfac, gothes, gotscl
            end if
            if (isumm  .gt. 0) then
               write(isumm , 2000) ncycle
            end if
         end if
      
*        Make sure the Jacobian variables are inside their bounds.
      
         if (nlncon) then
            call m8augl( 2, m, n, nb, ns, inform,
     $                   ne, nka, a, ha, ka,
     $                   hs, bl, bu, xn, z, nwcore )
         end if
      
*        For the first cycle, the row types have been set by m2amat.
*        Reset them for later cycles in case m2scal or m2crsh are
*        called.
      
         if (ncycle .ge. 2) then
            call m2amat( 2, m, n, nb,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, z(lhrtyp) )
         end if
      
*        ---------------------------------------------------------------
*        Evaluate the Jacobian and store it in A (unscaled) for the
*        first major iteration.
*        Compute scales from a, bl, bu (except if gotscl is true).
*        Scale a, bl, bu, xn, pi and fcon.
*        Initialize xlam from pi.
*        Call CRASH if a basis file was not supplied
*        (or if gotbas is false).
*        ---------------------------------------------------------------
         call m4getb( ncycle, istart, m, mbs, n, nb, nn, nname, nscl,
     $                lcrash, ns,
     $                ne, nka, a, ha, ka,
     $                z(lhrtyp), hs, z(lkb), z(lascal), bl, bu,
     $                pi, xn, z(ly), z(ly2), name1, name2,
     $                z, nwcore )
         if (ierr .ne. 0) go to 900
      
*        1. Set ns to match hs(*).
*        2. Set kb(m+1) thru kb(m+ns) to define the initial set of
*           superbasics, except if a Hot start
*           (gotbas and gothes are both true).
*        3. Check that nonbasic xn are within bounds.
      
         call m4chek( m, maxs, mbs, n, nb, ns,
     $                hs, z(lkb), bl, bu, xn )
      
*        ---------------------------------------------------------------
*        Solve the current problem.
*        Bail out if there is a fatal error.
*        ---------------------------------------------------------------
         call m1page( 1 )
         if (iprint .gt. 0) write(iprint, 2100)
      
         call m1time( 2,0 )
         call m5solv( m, maxr, maxs, mbs, n, nb, nn, nn0, nr,
     $                lcrash, ns, nscl, nx, objadd,
     $                ne, nka, a, ha, ka,
     $                z(lhrtyp), hs, z(lkb), z(lascal), bl, bu,
     $                z(lbbl), z(lbbu), fsub, z(lgsub),
     $                z(lgrd), z(lgrd2),
     $                pi, z(lr), rc, z(lrg), z(lrg2),
     $                z(lx), xn, z(ly), z(ly2), z, nwcore )
         call m1time(-2,0 )
      
         if (ierr .ge. 30                   ) go to 900
         if (ierr .ge. 20  .and.  itn .eq. 0) go to 900
      
*        ---------------------------------------------------------------
*        Unscale, compute nonlinear constraint violations,
*        save basis files and prepare to print the solution.
*        Clock 3 is "Output time".
*        ---------------------------------------------------------------
         call m1time( 3,0 )
         call m4savb( 1, m, mbs, n, nb, nn, nname, nscl, msoln, ns,
     $                ne, nka, a, ha, ka,
     $                hs, z(lkb), z(lascal), bl, bu,
     $                name1, name2,
     $                pi, rc, xn, z(ly), z, nwcore )
      
*        In some Cycling applications, it may be desirable to suppress
*        the printing of intermediate solutions.  Otherwise if mode = 2,
*        m4savb prints the solution under the control of msoln
*        (which is set by the  Solution  keyword in the SPECS file).
*        The printed solution may or may not be wanted, as follows:
*     
*        msoln  = 0   means      No
*               = 1   means      If optimal, infeasible or unbounded
*               = 2   means      Yes
*               = 3   means      If error condition
*     
*        This call normally prints the solution when there is no
*        Cycling, because the default values are  maxcy = nprint = 1.
      
         if (ncycle .gt. maxcy - nprint) then
            nprntd = nsolvd
            call m4savb( 2, m, mbs, n, nb, nn, nname, nscl, msoln, ns,
     $                ne, nka, a, ha, ka,
     $                hs, z(lkb), z(lascal), bl, bu,
     $                name1, name2,
     $                pi, rc, xn, z(ly), z, nwcore )
         end if
         call m1time(-3,0 )
      
*        ---------------------------------------------------------------
*        Call the functions one last time with  nstate .ge. 2.
*        We have to disable scaling.
*        mode = 0  tells the functions that gradients are not required.
*        ---------------------------------------------------------------
         if (ierr .eq. 6) go to 800
         lssave = lscale
         lscale = 0
         nstat1 = 2 + ierr
         nstat2 = nstat1
         mode   = 0
         if (nlncon) then
            call m6fcon( mode, nncon, nnjac, njac, z(lfcon), z(lgcon2),
     $                   ne, nka, ha, ka,
     $                   xn, z, nwcore )
         end if
         if (nlnobj) then
            call m6fobj( mode, nnobj, fobj, z(lgobj2), xn, z, nwcore )
         end if
         lscale = lssave
         if (mode .lt. 0) go to 800
         nstat1 = 0
         nstat2 = 0
         
*        ---------------------------------------------------------------
*        Terminate Cycles if m5solv gave a serious error.
*        Otherwise, let the user modify the problem for the next Cycle.
*        ---------------------------------------------------------------
         if (ierr   .ge.    20) go to 800
         if (ncycle .ge. maxcy) go to 800
         if (iprint .gt.     0) write(iprint, 3000) ncycle
         if (isumm  .gt.     0) write(isumm , 3000) ncycle
      
         call matmod( ncycle, nprob, finish,
     $                m, n, nb, ne, nka, ns, nscl, nname,
     $                a, ha, ka, bl, bu,
     $                z(lascal), hs, name1, name2,
     $                xn, pi, rc, z, nwcore )
      
         if (.not. finish) go to 500
*     ==================================================================
*     End of the Cycle loop.
*     ==================================================================


*     Print the final solution if it has not already been printed.

  800 if (nprntd .ne. nsolvd) then
         call m1time( 3,0 )
         call m4savb( 2, m, mbs, n, nb, nn, nname, nscl, msoln, ns,
     $                ne, nka, a, ha, ka,
     $                hs, z(lkb), z(lascal), bl, bu,
     $                name1, name2,
     $                pi, rc, xn, z(ly), z, nwcore )
         call m1time(-3,0 )
      end if
*     ------------------------------------------------------------------
*     Exit.
*     ------------------------------------------------------------------

  900 inform = ierr
      return

 1020 format(/ ' gotbas =', l2, 4x, ' gotfac =', l2, 4x,
     $         ' gothes =', l2, 4x, ' gotscl =', l2)
 1030 format(/ ' XXX Start parameter not recognized:  ', a)
 1100 format(  ' Scale option', i3, ',      Partial price', i8
     $       / ' Partial price section size (A) ', i12
     $       / ' Partial price section size (I) ', i12)
 1110 format(/ ' Scale option', i3, ',    Partial price', i4)
 1200 format(  ' Initial basis' / ' -------------')
 1210 format(/ ' No basis file supplied')
 2000 format(/ ' Start of Cycle', i5,
     $       / ' -------------------')
 2100 format(  ' Iterations' / ' ----------')
 3000 format(/ ' matmod called with ncycle =', i5)

*     end of misolv
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m1clos( lun )

*     ------------------------------------------------------------------
*     m1clos  closes the file with logical unit number lun.
*     This version is trivial and so far is not even used by MINOS.
*     Perhaps some implementations will need something fancier.
*     ------------------------------------------------------------------

      close( lun )

*     end of m1clos
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m1envt( mode )

*     ------------------------------------------------------------------
*     m1envt specifies the environment within which MINOS is being used.
*
*     When mode = 0, the various logicals should be initialized.
*     page1 says whether new pages are ever wanted on file iprint.
*     page2 says whether new pages are ever wanted on file isumm.
*
*     When mode is in the range 1 to 99, each environment does its
*     own thing.
*
*     When mode = 999, MINOS is asking if resource limits have been
*     reached.   To indicate YES, set ierr = 19.
*
*     The various environments are as follows:
*
*     ALONE:
*     This means MINOS is in stand-alone mode---the normal case.
*     Nothing special is done.
*
*     GAMS:
*     When mode = 1, 2, ..., 9 the characters =1, =2, ..., =9
*     are output to the print file.
*     When mode = 999, the resource limits are tested.
*
*     MINT:
*     Since branch-and-bound means a lot of CYCLES, we suppress
*     page ejects.  Otherwise, nothing special as yet.
*
*     AMPL:
*     Nothing special yet, but might want to test resource limits.
*
*     16 Sep 1987  Initial version.
*     24 Apr 1992  AMPL added.
*     ------------------------------------------------------------------

      logical            alone, AMPL, GAMS, MINT, page1, page2
      common    /m1env / alone, AMPL, GAMS, MINT, page1, page2
      common    /m1file/ iread,iprint,isumm
      common    /m5log1/ idebug,ierr,lprint

*     GAMS resource info:
*     ARESLM is limit on cpu time.
*     ATIME0 is starting time.
*     ATMGET is a function returning time so far.
*
*X    REAL               ARESLM, ATIME0, ADELT
*X    COMMON    /GAMS00/ ARESLM, ATIME0, ADELT
*X    REAL               ATMGET
*X    EXTERNAL           ATMGET

      if (mode .le. 0) then
*        ---------------------------------------------------------------
*        mode = 0.    Initialize.
*        page1 and page2 should be false for applications involving
*        many Cycles  (e.g. MINT).
*        ---------------------------------------------------------------
         alone  = .true.
         AMPL   = .false.
         GAMS   = .false.
         MINT   = .false.
         page1  = .true.
         page2  = .false.

      else if (mode .lt. 999) then
*        ---------------------------------------------------------------
*        mode = 1 or more.  Do what has to be done in each environment.
*        ---------------------------------------------------------------
         if (GAMS  .and.  iprint .gt. 0) then
            write(iprint, '(a1, i1)') '=', mode
         end if

      else if (mode .eq. 999) then
*        ---------------------------------------------------------------
*        mode = 999.  Test for excess time, etc.
*        ---------------------------------------------------------------
         if (GAMS) then
*X          IF (ATMGET() - ATIME0 .GE. ARESLM) IERR = 19
         else if (AMPL) then
*X
         end if
      end if

*     end of m1envt
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m1init( title )

      implicit           double precision (a-h,o-z)
      character*30       title

*     ------------------------------------------------------------------
*     m1init defines certain machine-dependent constants.
*
*     eps    = floating-point precision (e.g., 2.0**(-47) for CDC)
*     nwordr = no. of  reals          per word of  z(*)
*     nwordi = no. of  integers       per word of  z(*)
*     nwordh = no. of  half integers  per word of  z(*)
*        where  z(*)  is the main array of storage.
*
*     Original version:  integer*2  and  nwordh = 4  used throughout for
*                        certain arrays:
*                        ha, hs  in MINOS (and maybe a few others),
*                        indc, indr, ip, iq  in LUSOL.
*                        A quirk in LUSOL limits MINOS to 16383 rows
*                        when the limit should have been  32767.
*
*                        At present, nwordr is not used because there
*                        are no real*4 arrays.
*
*     22 May 1992:       integer*4  and  nwordh = 2  now used to allow
*                        essentially any number of rows.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1word/ nwordr,nwordi,nwordh

      title  = 'M I N O S    5.4    (Dec 1992)'
*---------------123456789|123456789|123456789|--------------------------

*---+ IEEE standard: eps = 2**(-52) = 2.22e-16
      eps    = 2.0d+0**(-52)
      nwordr = 2
      nwordi = 2
      nwordh = 2

*     Set other machine-precision constants.

      eps0   = eps**(0.8 d+0)
      eps1   = eps**(0.67d+0)
      eps2   = eps**(0.5 d+0)
      eps3   = eps**(0.33d+0)
      eps4   = eps**(0.25d+0)
      eps5   = eps**(0.2 d+0)
      plinfy = 1.0d+20

*     Set the environment (for later use).

      call m1envt( 0 )

*     end of m1init
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m1open( lun, state )

      integer            lun
      character*3        state

*     ------------------------------------------------------------------
*     m1open  is a machine-dependent routine.
*     It opens the file with logical unit number lun.
*
*     state   is intended to be input as 'IN ' or 'OUT'.  It may
*     be helpful on some systems.  MINOS uses sequential files only
*     and does not need to read and write to the same file.
*
*     'IN ' refers to an existing input file that will not be altered.
*     'OUT' means that a new file will be output.  If the file already
*     exists, it might be OK to overwrite it, but on some systems it
*     is better to create a new version of the file.  The choice is
*     open (to coin a phrase).
*
*
*     15 Jul 1989: First version, follows some of the advice offered
*                  by David Gay, AT&T.
*     ------------------------------------------------------------------

      if ( state .eq. 'IN ' ) then

*        Open an input file (e.g. MPS, OLD BASIS).
*        'OLD' means there will be an error if the file does not exist.
*        Since some systems position existing files at the end
*        (rather than the beginning), a rewind is performed.
      
         open  ( lun, status='OLD' )
         rewind( lun, err=900 )

      else if ( state .eq. 'OUT' ) then

*        Open an output file (e.g. DUMP, SOLUTION).
*        If it is OK to overwrite an existing file, we could do the
*        same as for input:

*---     open  ( lun )
*---     rewind( lun, err=900 )

*        On DEC VAX/VMS systems it is better to let a new generation
*        be created when the first write occurs, so we do nothing:

      end if

  900 return
      
*     end of m1open
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m1page( mode )

*     ------------------------------------------------------------------
*     m1page is an installation-dependent routine.  It is called at
*     points where some users might want output to files iprint or isumm
*     to begin on a new page.
*
*     page1 and page2 have already been set by m1envt.
*     If they are true, a page eject and a blank line are output.
*     Otherwise, just a blank line is output.
*
*     If mode = 1, just the page control is relevant.
*     If mode = 2, GAMS wants m1envt to print an =.
*     If mode = 0  and Summary level = 0, we don't want anything output
*                  to the Summary file.  At present, this is so m8setj
*                  will print just one line per major iteration, with
*                  no blank line in between.
*
*     16-Sep-1987:  First version.
*     20-Mar-1988:  mode 2 added.
*     12-Dec-1991:  mode 0 added.
*     ------------------------------------------------------------------

      logical            alone, AMPL, GAMS, MINT, page1, page2
      common    /m1env / alone, AMPL, GAMS, MINT, page1, page2
      common    /m1file/ iread,iprint,isumm
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed

      if (iprint .gt. 0) then
         if ( page1 ) write(iprint, 1001)
                      write(iprint, 1002)
      end if

      if (mode   .eq. 2) call m1envt( 1 )

      if (isumm  .gt. 0) then
         if ( page2 ) write(isumm , 1001)
         if ( summ1  .or.  mode .ne. 0 )
     $                write(isumm , 1002)
      end if
      return

 1001 format('1')
 1002 format(' ')

*     end of m1page
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m1time( clock, prtopt )

      implicit           double precision (a-h,o-z)
      integer            clock, prtopt

*     ------------------------------------------------------------------
*     m1time, m1timp and m1cpu are derived from timer, timout and nowcpu
*     written for DEC VAX/VMS systems by Irvin Lustig,
*     Department of Operations Research, Stanford University, 1987.
*
*     MINOS  calls m1time only.  m1time calls m1cpu  and  m1timp.
*     Only m1cpu is intrinsically machine dependent.
*
*     If a timer is available, call it in m1cpu  and arrange that
*     m1cpu  returns the current CPU time in seconds.
*
*     If a timer is not available or not wanted, set time = 0.0 in m1cpu.
*     Timing will be turned off and m1timp will not be called.
*     ------------------------------------------------------------------
*
*     m1time turns on or off a selected clock and optionally prints
*     statistics regarding all clocks or just the clock chosen.
*
*     The value of abs(clock) is which clock to use.
*     If clock = 0 and prtopt = 0, all clocks and statistics are reset.
*     If clock > 0, the clock is reset to start timing at the
*                   current time (determined by calling the
*                   machine-dependent subroutine m1cpu).
*     If clock < 0, the clock is turned off and the statistic is
*                   recorded for the amount of time since the clock
*                   was turned on.
*
*     prtopt is the print option.
*     If ltime < 0, nothing is printed.  Otherwise,
*     prtopt = 0 indicates print nothing,
*            = 1 indicates print last time for this clock,
*                only if clock < 0 (it has just been turned off),
*            = 2 indicates print total time for all clocks,
*            = 3 indicates print mean  time for all clocks.
*
*     The procedure for adding a new timer n is as follows:
*     1)  Change ntime to n in the parameter statement below (and in
*         all other routines referencing common block /m1tim /).
*     2)  Expand the array "label" to length n in subroutine m1timp.
*
*     04 Jun 1989: Irv's VMS/VAXC version of m1cpu installed,
*                  with changes to return time in seconds.
*     10 Jul 1992: More clocks added for use in AMPL (and elsewhere).
*     ------------------------------------------------------------------
*
*        Clock 1 is for input time.
*        Clock 2 is for solve time.
*        Clock 3 is for output time.
*        Clock 4 is for the nonlinear constraint functions.
*        Clock 5 is for the nonlinear objective.
*
*        numt(i)  is the number of times clock i has been turned on.
*        tlast(i) is the time at which clock i was last turned on.
*        tsum(i)  is the total time elapsed while clock i was on.
*        ltime    is the Timing level set in the Specs file.

      parameter        ( ntime = 5 )
      common    /m1tim / tlast(ntime), tsum(ntime), numt(ntime), ltime

      external           m1cpu
      double precision   stat, time
      integer            iclock, ilo, ihi

      if (ltime .eq. 0) return
      iclock = iabs(clock)

      if (clock .eq. 0) then
         if (prtopt .eq. 0) then

*           clock = 0, prtopt = 0.  Reset everything.

            call m1cpu ( 1, time )
            call m1cpu ( 0, time )
            do 100 i = 1, ntime
               tlast(i) = time
               tsum(i)  = 0.0
               numt(i)  = 0
  100       continue

*           If the m1cpu( 0, time ) gave time = 0.0, we assume that
*           the clock is a dummy.  Turn off future timing.

            if (time .le. 0.0) ltime = 0
         end if

      else
         call m1cpu ( 0, time )
         if (clock .gt. 0) then
            tlast(iclock) = time
         else
            stat         = time - tlast(iclock)
            tsum(iclock) = tsum(iclock) + stat
            numt(iclock) = numt(iclock) + 1
         end if
      end if

*     Now deal with print options.

      if (prtopt .eq. 0  .or.  ltime .lt. 0) then

*        Do nothing.

      else if (prtopt .eq. 1) then

*        Print statistic for last clock if just turned off.
      
         if (clock .lt. 0) then
            call m1timp( iclock, 'Last time', stat )
         end if

      else

*        prtopt >= 2.  Print all statistics if clock = 0,
*        or print statistic for individual clock.
      
         if (clock .eq. 0) then
            call m1cpu ( -1, time )
            ilo   = 1
            ihi   = ntime
         else
            ilo   = iclock
            ihi   = iclock
         end if
      
         do 400 i = ilo, ihi
            stat  = tsum(i)
            if (prtopt .eq. 2) then
               call m1timp( i, 'Time', stat )
            else if (prtopt .eq. 3) then
               istat = numt(i)
               if (istat .gt. 0) stat = stat / istat
               call m1timp( i, 'Mean time', stat )
            end if
  400    continue
      end if

*     end of m1time
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m1timp( iclock, lstat, stat )

      integer            iclock
      character*(*)      lstat
      double precision   stat

*     ------------------------------------------------------------------
*     m1timp  prints CPU time for m1time on file iprint and/or isumm.
*     It is not intrinsically machine dependent.
*
*     iclock  selects the correct label.
*     lstat   is a string to print to tell which type of statistic.
*     stat    is the statistic to print out.
*             If it is zero, we figure it was never timed, so no print.
*
*     12 Jul 1992: Array of labels avoids multiple formats.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm

      character*24       label(5)
      data               label
     $                 / 'for MPS input',          
     $                   'for solving problem',    
     $                   'for solution output',    
     $                   'for constraint functions',
     $                   'for objective function' /

      if (iclock .eq. 1) then
         if (iprint .gt. 0) write(iprint, 1000)
         if (isumm  .gt. 0) write(isumm , 1000)
      end if

      if (stat .eq. 0.0) return

      if (iprint .gt. 0) write(iprint, 1000) lstat, label(iclock), stat
      if (isumm  .gt. 0) write(isumm , 1000) lstat, label(iclock), stat
      return

 1000 format( 1x, a, 1x, a, t38, f13.2,' seconds')

*     end of m1timp
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m1cpu ( mode, time )

      integer            mode
      double precision   time

*     ------------------------------------------------------------------
*     m1cpu is a machine-dependent routine to return time = cpu time
*     in seconds, so that 2 consecutive calls will indicate the
*     time difference of operations between the 2 calls.
*     The parameter 'mode' indicates what function should be done
*     to the timer.  This allows necessary initialization for certain
*     machines.
*     mode =  1  indicates initialization,
*     mode =  0  indicates normal use,
*     mode = -1  indicates stop the timer.
*
*     For the MicroVax II under VMS, we need to call the correct library
*     routine to get the timer statistics.  These statistics are
*     found by using the times() function in the VaX C Runtime library.
*     To use this version of m1cpu, one must create an options file
*     called  vmsc.opt  with the line
*        SYS$LIBRARY:VAXCRTL/SHARE
*     in it.   Then link using the usual command and append   ,vmsc/opt
*     to the end of the line.  The name vmsc can be anything you desire.
*     ------------------------------------------------------------------

*-->  DEC VAX/VMS
*-->  integer            itimad(4)

*-->  Unix (DECstation)
      real               tarray(2)

      if (mode .eq. 1) then
*        ---------------------------------------------------------------
*        Initialize.
*        ---------------------------------------------------------------
         time   = 0.0

      else if (mode .eq. 0) then
*        ---------------------------------------------------------------
*        Normal call.
*        Return current timer value here.
*        ---------------------------------------------------------------
*-->     On VAX/VMS, itimad(1) returns the number of  centiseconds.
*        call times ( itimad )
*        time   = itimad(1)
*        time   = time * 0.01d+0

*-->     On Unix (DECstation MIPS RISC F77), etime returns seconds.
         time   = etime ( tarray )

*-->     On other machines, to forget about timing, just say
*-->     time   = 0.0

      else if (mode .eq. -1) then
*        ---------------------------------------------------------------
*        Stop the clock.
*        ---------------------------------------------------------------
         time   = 0.0
      end if

*     end of m1cpu
      end
*********************************************************************
*
*     File  mi15blas fortran.
*
*     dasum    daxpy    dcopy    ddot     dnrm2    dscal    idamax
*
*     These correspond to members of the BLAS package (Basic Linear
*     Algebra Subprograms, Lawson et al. (1979), ACM TOMS 5, 3).
*     If possible, they should be replaced by authentic BLAS routines
*     tuned to the machine being used.
*
*     The following utilities are also used:
*
*     dddiv    ddscl    dload    dnorm1
*     hcopy    hload    icopy    iload    iload1
*
*     These could be tuned to the machine being used.
*     dload  is used the most.
*
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine dddiv ( n, d, incd, x, incx )

      implicit           double precision (a-h,o-z)
      double precision   d(*), x(*)

*     dddiv  performs the diagonal scaling  x  =  x / d.

      integer            i, id, ix
      external           dscal
      intrinsic          abs
      parameter        ( one = 1.0d+0 )

      if (n .gt. 0) then
         if (incd .eq. 0  .and.  incx .ne. 0) then
            call dscal ( n, one/d(1), x, abs(incx) )
         else if (incd .eq. incx  .and.  incd .gt. 0) then
            do 10 id = 1, 1 + (n - 1)*incd, incd
               x(id) = x(id) / d(id)
   10       continue
         else
            if (incx .ge. 0) then
               ix = 1
            else
               ix = 1 - (n - 1)*incx
            end if
            if (incd .gt. 0) then
               do 20 id = 1, 1 + (n - 1)*incd, incd
                  x(ix) = x(ix) / d(id)
                  ix    = ix   + incx
   20          continue
            else
               id = 1 - (n - 1)*incd
               do 30  i = 1, n
                  x(ix) = x(ix) / d(id)
                  id    = id + incd
                  ix    = ix + incx
   30          continue
            end if
         end if
      end if

*     end of dddiv
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine ddscl ( n, d, incd, x, incx )

      integer            incd, incx, n
      double precision   d(*), x(*)

*     ddscl  performs the diagonal scaling  x  =  d * x.

      integer            i, id, ix
      external           dscal
      intrinsic          abs

      if (n .gt. 0) then
         if (incd .eq. 0  .and.  incx .ne. 0) then
            call dscal ( n, d(1), x, abs(incx) )
         else if (incd .eq. incx  .and.  incd .gt. 0) then
            do 10 id = 1, 1 + (n - 1)*incd, incd
               x(id) = d(id)*x(id)
   10       continue
         else
            if (incx .ge. 0) then
               ix = 1
            else
               ix = 1 - (n - 1)*incx
            end if
            if (incd .gt. 0) then
               do 20 id = 1, 1 + (n - 1)*incd, incd
                  x(ix) = d(id)*x(ix)
                  ix    = ix + incx
   20          continue
            else
               id = 1 - (n - 1)*incd
               do 30  i = 1, n
                  x(ix) = d(id)*x(ix)
                  id    = id + incd
                  ix    = ix + incx
   30          continue
            end if
         end if
      end if

*     end of ddscl
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine dload ( n, const, x, incx )

      double precision   const
      integer            incx, n
      double precision   x(*)

*     dload loads elements of x with const.

      double precision   zero
      parameter        ( zero = 0.0d+0 )
      integer            ix

      if (n .gt. 0) then
         if (incx .eq. 1  .and.  const .eq. zero) then
            do 10 ix = 1, n
               x(ix) = zero
   10       continue
         else
            do 20 ix = 1, 1 + (n - 1)*incx, incx
               x(ix) = const
   20       continue
         end if
      end if

*     end of dload
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      function   dnorm1( n, x, incx )

      implicit           double precision (a-h,o-z)
      double precision   x(*)

*     dnorm1  returns the 1-norm of the vector  x,  scaled by root(n).
*     This approximates an "average" element of x with some allowance
*     for x being sparse.

      intrinsic          sqrt
      external           dasum

      d      = n
      d      = dasum ( n, x, incx ) / sqrt(d)
      dnorm1 = d

*     end of dnorm1
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine hcopy ( n, hx, incx, hy, incy )

      integer*4          hx(*), hy(*)
      integer            incx, incy

*     hcopy  is the half-integer version of dcopy.
*     In this version of MINOS we no longer use half integers.

      integer            ix, iy

      if (n .gt. 0) then
         if (incx .eq. incy  .and.  incy .gt. 0) then
            do 10 iy  = 1, 1 + (n - 1)*incy, incy
               hy(iy) = hx(iy)
   10       continue
         else
            if (incx .ge. 0) then
               ix = 1
            else
               ix = 1 - (n - 1)*incx
            end if
            if (incy .gt. 0) then
               do 20 iy  = 1, 1 + ( n - 1 )*incy, incy
                  hy(iy) = hx(ix)
                  ix     = ix + incx
   20          continue
            else
               iy = 1 - (n - 1)*incy
               do 30  i  = 1, n
                  hy(iy) = hx(ix)
                  iy    = iy + incy
                  ix    = ix + incx
   30          continue
            end if
         end if
      end if

*     end of hcopy
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine hload ( n, const, hx, incx )

      integer            incx, n
      integer            const
      integer*4          hx(*)

*     hload loads elements of hx with const.
*     Beware that const is INTEGER, not half integer.

      integer            ix

      if (n .gt. 0) then
         if (incx .eq. 1  .and.  const .eq. 0) then
            do 10 ix  = 1, n
               hx(ix) = 0
   10       continue
         else
            do 20 ix  = 1, 1 + (n - 1)*incx, incx
               hx(ix) = const
   20       continue
         end if
      end if

*     end of hload
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine icopy ( n, x, incx, y, incy )

      integer            x(*), y(*)
      integer            incx, incy

*     icopy  is the integer version of dcopy.

      integer            ix, iy

      if (n .gt. 0) then
         if (incx .eq. incy  .and.  incy .gt. 0) then
            do 10 iy = 1, 1 + (n - 1)*incy, incy
               y(iy) = x(iy)
   10       continue
         else
            if (incx .ge. 0) then
               ix = 1
            else
               ix = 1 - (n - 1)*incx
            end if
            if (incy .gt. 0) then
               do 20 iy = 1, 1 + ( n - 1 )*incy, incy
                  y(iy) = x(ix)
                  ix    = ix + incx
   20          continue
            else
               iy = 1 - (n - 1)*incy
               do 30 i  = 1, n
                  y(iy) = x(ix)
                  iy    = iy + incy
                  ix    = ix + incx
   30          continue
            end if
         end if
      end if

*     end of icopy
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine iload ( n, const, x, incx )

      integer            incx, n
      integer            const
      integer            x(*)

*     iload  loads elements of x with const.

      integer            ix

      if (n .gt. 0) then
         if (incx .eq. 1  .and.  const .eq. 0) then
            do 10 ix = 1, n
               x(ix) = 0
   10       continue
         else
            do 20 ix = 1, 1 + (n - 1)*incx, incx
               x(ix) = const
   20       continue
         end if
      end if

*     end of iload
      end

*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine iload1( n, const, x, incx )

      integer            incx, n
      integer            const
      integer            x(*)

*     iload1 loads elements of x with const, by calling iload.
*     iload1 is needed in MINOS because iload  is a file number.

      call iload ( n, const, x, incx )

*     end of iload1
      end
************************************************************************
*
*     File  mi20amat fortran.
*
*     m2core   m2amat   m2aprd   m2apr1   m2apr5
*     m2crsh   m2scal   m2scla   m2unpk   matcol
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2core( mode, mincor )

      implicit           double precision (a-h,o-z)

*     ------------------------------------------------------------------
*     m2core  allocates all array storage for MINOS,
*     using various dimensions stored in common blocks as follows:
*        (m2len )   mrows, mcols, melms
*        (m3len )   nscl
*        (m5len )   maxr, maxs, nn
*        (m7len )   nnobj
*        (m8len )   njac, nncon, nnjac
*
*     if mode = 1, the call is from minos1 to estimate the storage
*                  needed for all stages of the problem.
*     if mode = 2, the call is from m3inpt to find the minimum storage
*                  needed for mps input.
*     if mode = 3, all dimensions should be known exactly.
*                  The call is from m3inpt to find the minimum
*                  storage to solve the problem and output the solution.
*                  The basis factorization routines will say later
*                  if they have sufficient storage.
*     if mode = 4, the call is from minoss and all dimensions are known
*                  exactly.  a, ha, ka etc need not be allocated.
*     ------------------------------------------------------------------

      common    /m1word/ nwordr,nwordi,nwordh
      common    /m2len / mrows,mcols,melms
      common    /m2mapa/ ne    ,nka   ,la    ,lha   ,lka
      common    /m2mapz/ maxw  ,maxz
      common    /m3len / m     ,n     ,nb    ,nscl
      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m3mps1/ lname1,lname2,lkeynm,nname
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5len / maxr  ,maxs  ,mbs   ,nn    ,nn0   ,nr    ,nx
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m7loc / lgobj ,lgobj2
      common    /m7cg2 / lcg1,lcg2,lcg3,lcg4,modtcg,nitncg,nsubsp
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
*     ------------------------------------------------------------------

      m      = mrows
      n      = mcols
      ne     = melms
      mbs    = m + maxs
      nka    = n + 1
      nb     = n + m
      nscl   = nb
      if (lscale .eq. 0) nscl  = 1
      nname  = nb
      if (mode   .eq. 4) nname = 1

      ncoli  = n/nwordi + 1
      nrowi  = m/nwordi + 1
      lenh   = max( 3*nrowi, 100 )

      nn0    = max( nn   , 1 )
      nncon0 = max( nncon, 1 )
      nnobj0 = max( nnobj, 1 )
      maxr   = max( maxr , 1 )
      nr     = maxr*(maxr + 1)/2  +  (maxs - maxr)
      nx     = max( mbs, nn )
      nx2    = 0
      if (lscale .ge. 2) nx2 = nn

      if (mode .le. 3) then

*        MINOS is getting its data from an MPS file.
*        Allocate arrays that are arguments of minoss and misolv.
*        These are for the data,
*                 A, ha, ka, bl, bu, name1, name2,
*        and for the solution
*                 hs, xn, pi, rc.

         la     = maxw   + 1
         lha    = la     + ne
         lka    = lha    + 1 + (ne /nwordh)
         lbl    = lka    + 1 + (nka/nwordi)
         lbu    = lbl    + nb
         lname1 = lbu    + nb
         lname2 = lname1 + 1 + (nname/nwordi)
         lhs    = lname2 + 1 + (nname/nwordi)
         lxn    = lhs    + 1 + (nb /nwordh)
         lpi    = lxn    + nb
         lrc    = lpi    + m
         minsub = lrc    + nb
      else

*        The subroutine version can use all of z except up to maxw.

         minsub = maxw   + 1
      end if

*     Allocate arrays needed during and after MPS input.

      lhrtyp = minsub
      lkb    = lhrtyp + 1 + (mbs/nwordh)
      minz   = lkb    + 1 + (mbs/nwordi)

      if (mode .le. 2) then

*        Allocate additional arrays needed for MPS input.
*        Currently just keynam -- the hash table.

         lkeynm = minz
         minmps = lkeynm + lenh
      end if

*     LP and reduced-gradient algorithm.
***** Beware -- length  0000  is used below for arrays that are
***** not needed in this version of MINOS.

      lascal = minz
      lpi2   = lascal + nscl
      lw     = lpi2   + 0000
      lw2    = lw     + nx
      lx     = lw2    + 0000
      lx2    = lx     + nx
      ly     = lx2    + nx2
      ly2    = ly     + nx
      lgsub  = ly2    + nx
      lgsub2 = lgsub  + nn
      lgrd   = lgsub2 + 0000
      lr     = lgrd   + mbs
      lrg    = lr     + nr
      lrg2   = lrg    + maxs
      minz   = lrg2   + maxs

*     Nonlinear objective.

      lgobj  = minz
      lgobj2 = lgobj  + nnobj
      minz   = lgobj2 + nnobj

*     Nonlinear constraints.
*     If Jacobian = Dense we can set njac correctly.
*     Otherwise we have to guess if the MPS file hasn't been
*     input yet (mode = 1 or 2).

      if (nden .eq. 1) then
         njac = nncon * nnjac
      else if (mode .le. 2) then
         njac =     5 * nnjac
      end if
      njac   = max( njac, 1 )
      lfcon  = minz
      lfcon2 = lfcon  + nncon
      lfdif  = lfcon2 + nncon
      lfdif2 = lfdif  + nncon
      lfold  = lfdif2 + 0000
      lblslk = lfold  + nncon
      lbuslk = lblslk + nncon
      lxlam  = lbuslk + nncon
      lrhs   = lxlam  + nncon
      lgcon  = lrhs   + nncon
      lgcon2 = lgcon  + njac
      lxdif  = lgcon2 + njac
      lxold  = lxdif  + nn
      minz   = lxold  + nn

*     Work arrays that can be overwritten by the names (see below)
*     in between cycles.

      lbbl   = minz
      lbbu   = lbbl   + mbs
      lgrd2  = lbbu   + mbs
      minz   = lgrd2  + mbs

*     Allocate arrays for the basis factorization routines.
*     minz points to the beginning of the LU factorization.
*     This is the beginning of free space between cycles
*     if the LU factors are allowed to be overwritten.

      call m2bmap( mode, m, n, ne, minz, maxz, nguess )

      if (mode .eq. 1) mincor = max( minmps, nguess, minz )
      if (mode .eq. 2) mincor = minmps
      if (mode .ge. 3) mincor = minz

*     end of m2core
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2amat( mode, m, n, nb,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, hrtype )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hrtype(m)
      integer            ka(nka)
      double precision   a(ne), bl(nb), bu(nb)

*     ------------------------------------------------------------------
*     If mode = 1 or 2,  m2amat defines hrtype, the set of row types.
*     If mode = 1,       m2amat also prints the matrix statistics.
*
*     The vector of row-types is as follows:
*        hrtype(i) = 0  for e rows (equalities),
*        hrtype(i) = 2  for n rows (objective or free rows),
*        hrtype(i) = 1  otherwise.
*     They are used in m2scal and m2crsh.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m8len / njac  ,nncon ,nncon0,nnjac

      intrinsic          abs, max, min
      parameter        ( zero = 0.0d+0 )

      bplus  = 0.9*plinfy
      bminus = - bplus

*     Construct the vector of row-types.

      nfixed = 0
      nfree  = 0
      nnorml = 0
      do 100 i  = 1, m
         j      = n + i
         b1     = bl(j)
         b2     = bu(j)
         itype  = 1
         if (b1 .eq. b2                          ) itype = 0
         if (b1 .le. bminus  .and.  b2 .ge. bplus) itype = 2
         hrtype(i) = itype
         if (itype .eq. 0) nfixed = nfixed + 1
         if (itype .eq. 2) nfree  = nfree  + 1
         if (itype .ne. 1) go to 100
         if (b1 .le. bminus  .or.   b2 .ge. bplus) nnorml = nnorml + 1
  100 continue

      if (mode   .eq. 2) return
      if (iprint .le. 0) return

      nbnded = m - nfixed - nfree - nnorml
      write(iprint, 2200)
      write(iprint, 2300) ' Rows   ', m, nnorml, nfree, nfixed, nbnded

      nfixed = 0
      nfree  = 0
      nnorml = 0

      do 200 j  = 1, n
         b1     = bl(j)
         b2     = bu(j)
         if (b1 .eq. b2) then
             nfixed = nfixed + 1
         else
             if      (b1 .eq. zero  ) then
                if      (b2 .ge. bplus) then
                    nnorml = nnorml + 1
                end if
             else if (b1 .le. bminus) then
                if      (b2 .eq. zero ) then
                    nnorml = nnorml + 1
                else if (b2 .ge. bplus) then
                    nfree  = nfree  + 1
                end if
             end if
         end if
  200 continue

      nbnded = n - nfixed - nfree - nnorml
      write(iprint, 2300) ' Columns', n, nnorml, nfree, nfixed, nbnded

*     Find the biggest and smallest elements in a, excluding free rows
*     and fixed columns.
*     Also find the largest objective coefficients in non-fixed columns.

      aijmax = zero
      aijmin = bplus
      cmax   = zero
      cmin   = bplus
      ncost  = 0

      do 300 j = 1, n
         if (bl(j) .lt. bu(j)) then
            do 250 k = ka(j), ka(j+1) - 1
               i     = ha(k)
               if (hrtype(i) .eq. 2) then
                   if (i .eq. iobj) then
                       aij   = abs( a(k) )
                       if (aij .gt. zero) then
                           ncost  = ncost + 1
                           cmax   = max( cmax, aij )
                           cmin   = min( cmin, aij )
                       end if
                   end if
               else
                   aij    = abs( a(k) )
                   aijmax = max( aijmax, aij )
                   aijmin = min( aijmin, aij )
               end if
  250       continue
         end if
  300 continue

      if (aijmin .eq. bplus) aijmin = zero
      adnsty = 100.0*ne / (m*n)
      write(iprint, 2400) ne, adnsty, aijmax, aijmin, ncost
      if (ncost .gt. 0) write(iprint, 2410) cmax, cmin

*     Print a few things to be gathered as statistics
*     from a bunch of test runs.

      write(iprint, 2500) nncon, m-nncon, n
      return

 2200 format(///
     $   ' Matrix Statistics' /
     $   ' -----------------' /
     $   15x, 'Total', 6x, 'Normal', 8x, 'Free',  7x, 'Fixed',
     $    5x, 'Bounded')
 2300 format(a, 5i12)
 2400 format(/ ' No. of matrix elements', i21, 5x, 'Density', f12.3
     $       / ' Biggest ', 1p, e35.4, '  (excluding fixed columns,'
     $       / ' Smallest',     e35.4, '   free rows, and RHS)'
     $      // ' No. of objective coefficients', i14)
 2410 format(  ' Biggest ', 1p, e35.4, '  (excluding fixed columns)'
     $       / ' Smallest',     e35.4)
 2500 format(//' Number of Nonlinear constraints', i12
     $       / ' Number of Linear    constraints', i12
     $       / ' Number of Variables            ', i12)

*     end of m2amat
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2aprd( mode, x, lenx, y, leny,
     $                   ne, nka, a, ha, ka,
     $                   z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne)
      double precision   x(lenx), y(leny), z(nwcore)

*     ------------------------------------------------------------------
*     m2aprd computes various products involving the matrix  A.
*     For mode 1 to 4,  x is the basic and/or superbasic variables.
*     For mode 5 to 8,  x is xn, the variables in natural order.
*
*     mode   function                       lenx  leny  called by
*     ----   --------                       ----  ----  ---------
*       1    y  =  y - B*x                  m     m     not used
*       1    y  =  y - (B S)*x              ms    m     not used
*       2    y  =  y - S*x                  ns    m     m7rgit
*       3    y  =  y - B(t)*x               m     m     not used
*       3    y  =  y - (B S)(t)*x           m     ms    not used
*       4    y  =  y - S(t)*x               m     ms    m7chzq, m7rg
*
*       5    y  =  y - A*xn                 n     m     m5setx, m5solv
*       6                                               not used
*       7    y  =  y - (nonlinear  A)*xn    nnjac nncon m6fun1, m8setj
*       8    y  =  y - (linear     A)*xn    n     nncon m8viol
*     mode 7 and 8 deal with nonlinear rows only and ignore slacks.
*
*     14 Oct 1991: a, ha, ka added as parameters to help with minoss.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m3len / m     ,n     ,nb    ,nscl
      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m5len / maxr  ,maxs  ,mbs   ,nn    ,nn0   ,nr    ,nx
      common    /m8len / njac  ,nncon ,nncon0,nnjac

      tolz   = eps0

      if (mode .lt. 5) then
         call m2apr1( mode, m, mbs, n, tolz,
     $                ne, nka, a, ha, ka, z(lkb),
     $                x, lenx, y, leny )
      else
         call m2apr5( mode, m, n, nb, nncon, nnjac, tolz,
     $                ne, nka, a, ha, ka,
     $                x, lenx, y, leny )
      end if

*     end of m2aprd
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2apr1( mode, m, mbs, n, tolz,
     $                   ne, nka, a, ha, ka, kb, x, lenx, y, leny )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka), kb(mbs)
      double precision   a(ne)
      double precision   x(lenx), y(leny)

*     ------------------------------------------------------------------
*     m2apr1  computes various matrix-vector products involving
*     B  and  S,  the basic and superbasic columns of  A.
*
*     mode=1    y  =  y - B*x   or   y - (B S)*x  (depending on  lenx)
*     mode=2    y  =  y - S*x
*     mode=3    y  =  y - B(transpose)*x   or   y - (B S)(transpose)*x
*    (mode 3  is not used as yet)
*     mode=4    y  =  y - S(transpose)*x
*     ------------------------------------------------------------------

      kbase  = 0
      if (mode .le. 2) then
*        -------------
*        mode 1 and 2.
*        -------------
         if (mode .eq. 2) kbase = m

         do 200 k = 1, lenx
            xj    = x(k)
            if (abs(xj) .le. tolz) go to 200
            j     = kb(kbase + k)
            if (j .le. n) then
               do 150 i = ka(j), ka(j+1) - 1
                  ir    = ha(i)
                  y(ir) = y(ir) - a(i)*xj
  150          continue
            else
               ir    = j - n
               y(ir) = y(ir) - xj
            end if
  200    continue
      else
*        -------------
*        mode 3 and 4.
*        -------------
         if (mode .eq. 4) kbase = m

         do 400 k = 1, leny
            t     = y(k)
            j     = kb(kbase + k)
            if (j .le. n) then
               do 350 i = ka(j), ka(j+1) - 1
                  ir    = ha(i)
                  t     = t - x(ir)*a(i)
  350          continue
               y(k) = t
            else
               ir   = j - n
               y(k) = t - x(ir)
            end if
  400    continue
      end if

*     end of m2apr1
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2apr5( mode, m, n, nb, nncon, nnjac, tolz,
     $                   ne, nka, a, ha, ka, xn, lenx, y, leny )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne)
      double precision   xn(lenx), y(leny)

*     ------------------------------------------------------------------
*     m2apr5  computes products involving parts of  A  and  xn.
*     ------------------------------------------------------------------

      intrinsic          abs

      if (mode .eq. 5) then
*        ----------------------------------
*        mode = 5.   Set  y = y - A*xn.
*        ----------------------------------
         do 500 j = 1, n
            xj    = xn(j)
            if (abs( xj ) .gt. tolz) then
               do 450 k = ka(j), ka(j+1) - 1
                  ir    = ha(k)
                  y(ir) = y(ir) - a(k)*xj
  450          continue
            end if
  500    continue

      else
*        ------------------------------------------------------------
*        If mode = 7, set  y  =  y - (Jacobian)*xn.
*        If mode = 8, set  y  =  y - (linear A)*xn.
*        Only the first nncon rows are involved, and no slacks.
*        ------------------------------------------------------------
         if (mode .eq. 7) then
            j1 = 1
            j2 = nnjac
         else
            j1 = nnjac + 1
            j2 = n
         end if

         do 800 j = j1, j2
            xj    = xn(j)
            if (abs( xj ) .gt. tolz) then
               do 750 k = ka(j), ka(j+1) - 1
                  ir    = ha(k)
                  if (ir .le. nncon) y(ir) = y(ir) - a(k)*xj
  750          continue
            end if
  800    continue
      end if

*     end of m2apr5
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2crsh( lcrash, m     , n     , nb    , nn    ,
     $                   ne    , nka   , a     , ha    , ka    ,
     $                   hpiv  , hs    , hrtype, bl    , bu    ,
     $                   xn    , z     , nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hpiv(m), hs(nb), hrtype(m)
      integer            ka(nka)
      double precision   a(ne), bl(nb), bu(nb), xn(nb), z(nwcore )

*     ------------------------------------------------------------------
*     m2crsh  looks for a basis in the columns of ( A  I ).
*
*     ON ENTRY
*
*     iparm(1)  = Crash option    (icrash)   has been used by m4getb
*                 to set lcrash.
*     dparm(5)  = Crash tolerance (tcrash).  Default = 0.1
*
*     lcrash      specifies the action to be taken by Crash.
*        0,1,2,3  The call is from m4getb.
*        4        The call is from m5solv.
*        5        The call is from m8setj.
*
*        0        The all-slack basis is set up.
*
*        1        A triangular crash is applied to the columns of A.
*                 hs(1:n) is used to help select columns.
*                 tcrash is used to ignore small entries in each column.
*                 Depending on the size of tcrash, the resulting basis
*                 will be nearly (but not strictly) lower triangular.
*
*        2        As for 1, but nonlinear rows are ignored.
*
*        3        As for 2, but linear LG rows are also ignored.
*
*        4        Linear LG rows are now included.
*                 All hs(1:nb) and xn(n+i) are defined.
*                 Slack values of xn(n+i) are used to select LG rows.
*
*        5        Nonlinear rows are now included.
*
*     hrtype(*)   should be defined as described in m2amat:
*     hrtype(i) = 0  for E rows      (equalities)
*     hrtype(i) = 1  for L or G rows (inequalities)
*     hrtype(i) = 2  for N rows      (objective or free rows)
*
*     xn          If lcrash <= 4, xn(1:n) is used to initialize
*                 slacks as xn(n+1:nb) = - A*xn.
*                 Used to select slacks from LG rows to be in B (basis).
*                 If lcrash  = 5, xn(n+1:n+nncon) contains slack values
*                 evaluated from xn(1:n) and fcon(*).
*                 Used to select slacks from nonlinear rows to be in B.
*
*     hs          If lcrash = 1, 2 or 3, hs(1:n)  is used.
*                 If lcrash =    4 or 5, hs(1:nb) is used.
*                 If hs(j) =  0, 1 or 3, column j is eligible for B,
*                                        with 3 being "preferred".
*                 If hs(j) =  2, 4 or 5, column j is ignored.
*
*
*     Crash has several stages.
*
*     Stage 1: Insert any slacks (N, L or G rows, hrtype = 1 or 2).
*
*     Stage 2: Do triangular crash on any free columns (wide bounds)
*
*     Stage 3: Do triangular crash on "preferred" columns (hs(j) < 0).
*              For the linear crash, this includes variables set
*              between their bounds in the MPS file via FR INITIAL.
*              For the nonlinear crash, it includes nonbasics
*              between their bounds.
*              (That is, "pegged" variables in both cases.)
*
*     Stage 4: Grab unit columns.
*
*     Stage 5: Grab double columns.
*
*     Stage 6: Do triangular crash on all columns.
*
*     Slacks are then used to pad the basis.
*
*
*     ON EXIT
*
*     hs          is set to denote an initial (B S N) partition.
*                 hs(j) = 3 denotes variables for the initial basis.
*                 If hs(j) = 2 still, variable j will be superbasic.
*                 If hs(j) = 4 or 5 still, it will be changed to 0 or 1
*                 by m4chek and variable j will be nonbasic.
*
*     xn          If lcrash <= 4, slacks xn(n+1:nb) are initialized.
*
*     ------------------------------------------------------------------
*        Nov 1986: Essentially the same as in 1976.
*                  Crash tolerance added.
*                  Attention paid to various hs values.
*
*     12 Nov 1988: After free rows and columns have been processed
*                  (stage 1 and 2), slacks on L or G rows are inserted
*                  if their rows have not yet been assigned.
*
*     28 Dec 1988: Refined as follows.
*                  Stage 1 inserts free and preferred rows (slacks).
*                  Stage 2 performs a triangular crash on free or
*                          preferred columns, ignoring free rows.
*                          Unpivoted L or G slacks are then inserted.
*                  Stage 3 performs a triangular crash on the other
*                          columns, ignoring rows whose slack is basic.
*                          (Such rows form the top part of U.  The
*                          remaining rows form the beginning of L.)
*
*     30 Apr 1989: Stage 1 now also looks for singleton columns
*                  (ignoring free and preferred rows).
*     05 May 1989: Stage 2 doesn't insert slacks if crash option < 0.
*
*     06 Dec 1989: Stage 2, 3, 4 modified.  Columns of length 2 are
*                  now treated specially.
*
*     20 Dec 1989: Stage 2 thru 5 modified.  Free columns done before
*                  unit and double columns.
*
*     19 May 1992: xn now used to help initialize slacks.
*                  Stage 1 thru 7 redefined as above.
*
*     01 Jun 1992: abs used to define closeness of slacks to bounds.
*                  Unfortunately, xn(1:n) seldom has meaningful values.
*
*     02 Jun 1992: Poor performance on the larger problems.
*                  Reverted to simple approach: all slacks grabbed.
*
*     04 Jun 1992: Compromise -- Crash 3 now has 3 phases:
*                  (a) E rows.
*                  (b) LG rows.
*                  (c) Nonlinear rows.
*                  xn(1:n) should then define the slack values better
*                  for (b) and (c).
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2parm/ dparm(30),iparm(30)
      common    /m5log1/ idebug,ierr,lprint
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m8len / njac  ,nncon ,nncon0,nnjac

      equivalence      ( iparm(1), icrash )
      equivalence      ( dparm(5), tcrash )

      intrinsic          abs, max
      double precision   apiv(2)
      integer            ipiv(2)
      parameter        ( nstage = 6 )
      integer            num(nstage), stage
      logical            free, prefer, gotslk,
     $                   stage2, stage3, stage4, stage5
      parameter        ( zero  = 0.0d+0,
     $                   small = 1.0d-3,  big = 1.0d+4 )

      if (iprint .gt. 0) then
         if (lcrash .le. 3) write(iprint, 1000) icrash
         if (lcrash .eq. 3  .and.  nncon .lt. m) write(iprint, 1030)
         if (lcrash .eq. 4) write(iprint, 1040)
         if (lcrash .eq. 5) write(iprint, 1050)
      end if

      if (lcrash .le. 4) then

*        Sets slacks xn(n+1:nb) = - A*xn.
*        This is where the slacks are initialized.
*        They may be altered later (see the end of Crash).

         call dload ( m,  zero, xn(n+1), 1 )
         call m2aprd( 5, xn, n, xn(n+1), m,
     $                ne, nka, a, ha, ka,
     $                z, nwcore )
      end if

*     ------------------------------------------------------------------
*     For Crash option 0, set hs(j) = 3 for all slacks and quit.
*     ------------------------------------------------------------------
      if (lcrash .eq. 0) then
         call hload ( n, 0, hs     , 1 )
         call hload ( m, 3, hs(n+1), 1 )
         go to 900
      end if

*     ------------------------------------------------------------------
*     Crash option 1, 2 or 3.   lcrash = 1, 2, 3, 4, or 5.
*     tolslk measures closeness of slacks to bounds.
*     i1,i2  are the first and last rows of A involved in Stage 1.
*     ------------------------------------------------------------------
      tolslk = 0.25
      call iload ( nstage, 0, num, 1 )

      if (lcrash .le. 3) then
*        ---------------------------------------------------------------
*        First call.   lcrash = 1, 2 or 3.
*        Initialize hpiv(*) for all rows and hs(*) for all slacks.
*        ---------------------------------------------------------------
         i1     = 1
         if (lcrash .ge. 2) i1 = nncon + 1
         i2     = m
         nrows  = i2 - i1 + 1

*        Make sure there are no basic columns already (hs(j) = 3).
*        If there are, make them "preferred".

         do 20 j = 1, n
            if (hs(j) .eq. 3) hs(j) = -1
   20    continue

*        Make relevant rows available:  hpiv(i) = 1, hs(n+i) = 0.

         if (nrows .gt. 0) then
            call hload ( nrows, 1, hpiv(i1), 1 )
            call hload ( nrows, 0, hs(n+i1), 1 )
         end if

         if (lcrash .eq. 1) then
            nbasic = 0
         else

*           lcrash = 2 or 3:  Insert nonlinear slacks.

            nbasic = nncon
            if (nncon .gt. 0) then
               call hload ( nncon, 3, hpiv   , 1 )
               call hload ( nncon, 3, hs(n+1), 1 )
            end if
         end if

         if (lcrash .eq. 3) then

*           Insert linear inequality slacks (including free rows).

            do 25 i = i1, m
               if (hrtype(i) .ge. 1) then
                  nbasic  = nbasic + 1
                  nrows   = nrows  - 1
                  hpiv(i) = 3
                  hs(n+i) = 3
               end if
   25       continue
         end if

*        We're done if there are no relevant rows.

         if (nrows .eq. 0) go to 800

      else
*        ---------------------------------------------------------------
*        Second or third call.  lcrash = 4 or 5.
*        Initialize hpiv(*) for all rows.
*        hs(*) already defines a basis for the full problem,
*        but we want to do better by including only some of the slacks.
*        ---------------------------------------------------------------
         if (lcrash .eq. 4) then
*           ------------------------------------------------------------
*           Crash on linear LG rows.
*           ------------------------------------------------------------
            if (nncon .eq. m) go to 900
            i1     = nncon + 1
            i2     = m

*           Mark nonlinear rows as pivoted: hpiv(i) = 3.

            nbasic = nncon
            if (nbasic .gt. 0) then
               call hload ( nbasic, 3, hpiv, 1 )
            end if

*           Mark linear E  rows as pivoted: hpiv(i) = 3
*           Make linear LG rows available:  hpiv(i) = 1, hs(n+i) = 0.

            do 30 i = i1, m
               if (hrtype(i) .eq. 0) then
                  nbasic  = nbasic + 1
                  hpiv(i) = 3
               else
                  hpiv(i) = 1
                  hs(n+i) = 0
               end if
   30       continue

*           Mark linear LG rows with hpiv(i) = 2
*           if any basic columns contain a nonzero in row i.

            do 50 j = 1, n
               if (hs(j) .eq. 3) then
                  do 40 k = ka(j), ka(j+1) - 1
                     i    = ha(k)
                     if (hrtype(i) .eq. 1) then
                        if (i .gt. nncon) then
                           if (a(k) .ne. zero) hpiv(i) = 2
                        end if
                     end if
   40             continue
               end if
   50       continue

         else
*           ------------------------------------------------------------
*           lcrash = 5.  Crash on nonlinear rows.
*           ------------------------------------------------------------
            i1     = 1
            i2     = nncon

*           Mark all linear rows as pivoted: hpiv(i) = 3

            nbasic = m - nncon
            if (nbasic .gt. 0) then
               call hload ( nbasic, 3, hpiv(nncon+1), 1 )
            end if

*           Make nonlinear rows available:  hpiv(i) = 1, hs(n+i) = 0.

            call hload ( nncon, 1, hpiv   , 1 )
            call hload ( nncon, 0, hs(n+1), 1 )

*           Mark nonlinear rows with hpiv(i) = 2
*           if any basic columns contain a nonzero in row i.

            do 70 j = 1, n
               if (hs(j) .eq. 3) then
                  do 60 k = ka(j), ka(j+1) - 1
                     i    = ha(k)
                     if (i .le. nncon) then
                        if (a(k) .ne. zero) hpiv(i) = 2
                     end if
   60             continue
               end if
   70       continue
         end if
      end if

*     ------------------------------------------------------------------
*     Stage 1: Insert relevant slacks (N, L or G rows, hrtype = 1 or 2).
*              If lcrash = 4 or 5, grab them only if they are more than
*              tolslk from their bound.
*     ------------------------------------------------------------------
      stage  = 1
      gotslk = lcrash .eq. 4  .or.  lcrash .eq. 5

      do 100 i = i1, i2
         j     = n + i
         if (hs(j) .le. 1  .and.  hrtype(i) .gt. 0) then
            if (gotslk) then
               d1 = xn(j) - bl(j)
               d2 = bu(j) - xn(j)
               if (min( d1, d2 ) .le. tolslk) then
              
*                 The slack is close to a bound or infeasible.
*                 Move it exactly onto the bound.
              
                  if (d1 .le. d2) then
                     xn(j) = bl(j)
                     hs(j) = 0
                  else
                     xn(j) = bu(j)
                     hs(j) = 1
                  end if
                  go to 100
               end if
            end if

            nbasic     = nbasic     + 1
            num(stage) = num(stage) + 1
            hpiv(i)    = 3
            hs(j)      = 3
         end if
  100 continue

      if (nbasic .eq. m) go to 700

*     ------------------------------------------------------------------
*     Apply a triangular crash to various subsets of the columns of A.
*
*        hpiv(i) = 1  if row i is unmarked (initial state).
*        hpiv(i) = 3  if row i has been given a pivot
*                     in one of a set of triangular columns.
*        hpiv(i) = 2  if one of the triangular columns contains
*                     a nonzero in row i below the triangle.
*     ------------------------------------------------------------------

      do 600 stage = 2, nstage
         stage2    = stage .eq. 2
         stage3    = stage .eq. 3
         stage4    = stage .eq. 4
         stage5    = stage .eq. 5

*        ---------------------------------------------------------------
*        Main loop for triangular crash.
*        ---------------------------------------------------------------
         do 200  j = 1, n
            js     = hs(j)
            if (js    .gt.    1 ) go to 200
            if (bl(j) .eq. bu(j)) go to 200

            if ( stage2 ) then
               free   = bl(j) .le. - big  .and.  bu(j) .ge. big
               if ( .not. free  ) go to 200

            else if ( stage3 ) then
               prefer = js .lt. 0
               if ( .not. prefer) go to 200
            end if

*           Find the biggest aij, ignoring free rows.

            k1     = ka(j)
            k2     = ka(j+1) - 1
            aimax  = zero

            do 130  k = k1, k2
               i      = ha(k)
               if (hrtype(i) .ne. 2) then
                  ai    = a(k)
                  aimax = max( aimax, abs( ai ) )
               end if
  130       continue

*           Prevent small pivots if crash tol is too small.

            if (aimax .le. small) go to 200

*           Find the biggest pivots in rows that are still
*           unpivoted and unmarked.  Ignore smallish elements.
*           nz counts the number of relevant nonzeros.

            aitol   = aimax * tcrash
            nz      = 0
            npiv    = 0
            ipiv(1) = 0
            ipiv(2) = 0
            apiv(1) = zero
            apiv(2) = zero

            do 150 k = k1, k2
               i     = ha(k)
               if (hs(n+i) .ne. 3) then
                  ai = abs( a(k) )
                  if (ai .gt. aitol) then
                     nz = nz + 1
                     ip = hpiv(i)
                     if (ip .le. 2) then
                        if (apiv(ip) .lt. ai) then
                            apiv(ip) = ai
                            ipiv(ip) = i
                        end if
                     else
                        npiv = npiv + 1
                     end if
                  end if
               end if
  150       continue

*           Grab unit or double columns.

            if      ( stage4 ) then
               if (nz .ne. 1) go to 200
            else if ( stage5 ) then
               if (nz .ne. 2) go to 200
            end if

*           See if the column contained a potential pivot.
*           An unmarked row is favored over a marked row.

            ip     = 1
            if (ipiv(1)  .eq.  0  .and.  npiv .eq. 0) ip = 2
            i      = ipiv(ip)

            if (i .gt. 0) then
               nbasic     = nbasic     + 1
               num(stage) = num(stage) + 1
               hpiv(i)    = 3
               hs(j)      = 3
               if (nbasic .ge. m) go to 700

*              Mark off all relevant unmarked rows.

               do 180 k = k1, k2
                  i     = ha(k)
                  if (hs(n+i) .ne. 3) then
                     ai = abs( a(k) )
                     if (ai .gt. aitol) then
                        if (hpiv(i) .eq. 1) hpiv(i) = 2
                     end if
                  end if
  180          continue
            end if
  200    continue
  600 continue

*     ------------------------------------------------------------------
*     All stages finished.
*     Fill remaining gaps with slacks.
*     ------------------------------------------------------------------
  700 npad   = m - nbasic
      if (iprint .gt. 0) write(iprint, 1200) num, npad

      if (npad .gt. 0) then
         do 720 i = 1, m
            if (hpiv(i) .lt. 3) then
               nbasic  = nbasic + 1
               hs(n+i) = 3
               if (nbasic .ge. m) go to 800
            end if
  720    continue
      end if

*     ------------------------------------------------------------------
*     Make sure there aren't lots of nonbasic slacks floating off
*     their bounds.  They could take lots of iterations to move.
*     ------------------------------------------------------------------
  800 do 850 i = i1, i2
         j     = n + i
         if (hs(j) .le. 1  .and.  hrtype(i) .gt. 0) then
            d1 = xn(j) - bl(j)
            d2 = bu(j) - xn(j)
            if (min( d1, d2 ) .le. tolslk) then
              
*              The slack is close to a bound or infeasible.
*              Move it exactly onto the bound.
              
               if (d1 .le. d2) then
                  xn(j) = bl(j)
                  hs(j) = 0
               else
                  xn(j) = bu(j)
                  hs(j) = 1
               end if
            end if
         end if
  850 continue

  900 return

 1000 format(// ' Crash option', i3)
 1030 format(   ' Crash on linear E  rows:')
 1040 format(   ' Crash on linear LG rows:')
 1050 format(   ' Crash on nonlinear rows:')
 1200 format(
     $   ' Slacks', i6, '  Free cols', i6, '  Preferred', i6
     $ / ' Unit  ', i6, '  Double   ', i6, '  Triangle ', i6,
     $     '  Pad', i6)

*     end of m2crsh
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2scal( m, n, nb, ne, nka, nn, nncon, nnjac,
     $                   hrtype, ha, ka, a, ascale, bl, bu, rmin, rmax )

      implicit           double precision (a-h,o-z)
      integer*4          hrtype(m), ha(ne)
      integer            ka(nka)
      double precision   a(ne), ascale(nb), bl(nb), bu(nb)
      double precision   rmin(m), rmax(m)

*     ------------------------------------------------------------------
*     m2scal computes scale factors ascale from A, bl, bu.
*
*     In phase 1, an iterative procedure based on geometric means is
*     used to compute scales from a alone.  This procedure is derived
*     from a routine written by Robert Fourer, 1979.  The main steps
*     are:
*
*        (1) Compute aratio = max(i1,i2,j)  a(i1,j) / a(i2,j).
*        (2) Divide each row i by
*               ( min(j) a(i,j) * max(j) a(i,j) ) ** 1/2.
*        (3) Divide each column j by
*               ( min(i) a(i,j) * max(i) a(i,j) ) ** 1/2.
*        (4) Compute sratio as in (1).
*        (5) If sratio .lt. scltol * aratio, repeat from step (1).
*
*        Free rows (hrtype=2) and fixed columns (bl=bu) are not used
*        at this stage.
*
*     In phase 2, the scales for free rows are set to be their largest
*     element.
*
*     In phase 3, fixed columns are summed in order to compute
*     a scale factor sigma that allows for the effective rhs of the
*     constraints.  All scales are then multiplied by sigma.
*
*
*     If lscale = 1, the first nncon rows and the first nn columns will
*     retain scales of 1.0 during phases 1-2, and phase 3 will not be
*     performed.  (This takes effect if the problem is nonlinear but
*     the user has specified scale linear variables only.)
*     However, all rows    contribute to the linear column scales,
*     and      all columns contribute to the linear row    scales.
*
*     If lscale = 2, all rows and columns are scaled.  To guard against
*     misleadingly small Jacobians, if the maximum element in any of
*     the first nncon rows or the first nnjac columns is less than
*     smallj, the corresponding row or column retains a scale of 1.0.
*
*     Initial version  March 1981.
*     Feb 1983.  For convenience, everything is now double-precision.
*     Aug 1983.  Revised to damp the effect of very small elements.
*                On each pass, a new row or column scale will not be
*                smaller than  sqrt(damp)  times the largest (scaled)
*                element in that row or column.
*     Oct 1986.  Scale option 2 and phase 3 implemented.
*     Nov 1989.  Large scales are now reduced to at most 0.1/tolx (say)
*                to help prevent obvious infeasibilities when the
*                problem is unscaled.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2parm/ dparm(30),iparm(30)
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5log1/ idebug,ierr,lprint
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm

      intrinsic          abs, min, max, sqrt

      double precision   ac, ar, amin, amax, aratio, bnd, b1, b2,
     $                   cmin, cmax, cratio, damp, sigma,
     $                   small, smallj, sratio

      parameter        ( zero = 0.0d+0,  one    = 1.0d+0,
     $                   damp = 1.0d-4,  smallj = 1.0d-2 )

      if (iprint .gt.  0 ) write(iprint, 1000)
      if (scltol .ge. one) scltol = 0.99
      bplus  = 0.1*plinfy
      aratio = bplus
      maxk   = 11
      if (lscale .eq. 1) then
*        Do linear rows and columns only.
         istart = nncon + 1
         jstart = nn    + 1
      else
*        Do all rows and cols if lscale .ge. 2
         istart = 1
         jstart = 1
      end if

      do 50 j = 1, nb
         ascale(j) = one
   50 continue

      if (jstart .gt. n) go to 900

*     ------------------------------------------------------------------
*     Main loop for phase 1.
*     Only the following row-types are used:
*        hrtype(i) = 2       for n rows (objective or free rows),
*        hrtype(i) = 0 or 1  otherwise.
*     ------------------------------------------------------------------

      do 400 kk = 1, maxk

*        Find the largest column ratio.
*        Also set new column scales (except on pass 0).

         npass  = kk - 1
         amin   = bplus
         amax   = zero
         small  = smallj
         sratio = one

         do 250 j = jstart, n
            if (bl(j) .lt. bu(j)) then
               cmin  = bplus
               cmax  = zero

               do 230  k = ka(j), ka(j+1) - 1
                  i      = ha(k)
                  if (hrtype(i) .eq. 2   ) go to 230
                  ar     = abs( a(k) )
                  if (ar        .eq. zero) go to 230
                  ar     = ar / ascale(n+i)
                  cmin   = min( cmin, ar )
                  cmax   = max( cmax, ar )
  230          continue

               ac     = max( cmin, damp*cmax )
               ac     = sqrt( ac ) * sqrt( cmax )
               if (j     .gt. nnjac) small     = zero
               if (cmax  .le. small) ac        = one
               if (npass .gt. 0    ) ascale(j) = ac
               amin   = min( amin, cmin / ascale(j) )
               amax   = max( amax, cmax / ascale(j) )
               cratio = cmax / cmin
               sratio = max( sratio, cratio )
            end if
  250    continue

         if (iprint .gt. 0) then
            write(iprint, 1200) npass, amin, amax, sratio
         end if
         if (npass  .ge. 3       .and.
     $       sratio .ge. aratio*scltol) go to 420
         if (kk     .eq. maxk         ) go to 420
         aratio = sratio


*        Set new row scales for the next pass.

         if (istart .le. m) then
            do 300  i  = istart, m
               rmin(i) = bplus
               rmax(i) = zero
  300       continue

            do 350 j = 1, n
               if (bl(j) .lt. bu(j)) then
                  ac = ascale(j)

                  do 330  k  = ka(j), ka(j+1) - 1
                     i       = ha(k)
                     if (i  .lt. istart) go to 330
                     ar      = abs( a(k) )
                     if (ar .eq. zero  ) go to 330
                     ar      = ar / ac
                     rmin(i) = min( rmin(i), ar )
                     rmax(i) = max( rmax(i), ar )
  330             continue
               end if
  350       continue

            do 360 i = istart, m
               j     = n + i
               ar    = rmax(i)
               if (i .le. nncon  .and.  ar .le. smallj) then
                  ascale(j) = one
               else
                  ac        = max( rmin(i), damp*ar )
                  ascale(j) = sqrt( ac ) * sqrt( ar )
               end if
  360       continue
         end if
  400 continue

*     ------------------------------------------------------------------
*     End of main loop.
*     ------------------------------------------------------------------

*     Invert the column scales, so that structurals and logicals
*     can be treated the same way during subsequent unscaling.
*     Find the min and max column scales while we're at it.
*     Nov 1989: nclose counts how many are "close" to 1.
*     For problems that are already well-scaled, it seemed sensible to
*     set the "close" ones exactly equal to 1.
*     Tried "close" = (0.5,2.0) and (0.9,1.1), but they helped only
*     occasionally.  Have decided not to interfree.

  420 amin   = bplus
      amax   = zero
      close1 = 0.5
      close2 = 2.0
      nclose = 0


      do 430 j = 1, n
         ac    = one / ascale(j)

         if (amin .gt. ac) then
             amin   =  ac
             jmin   =  j
         end if
         if (amax .lt. ac) then
             amax   =  ac
             jmax   =  j
         end if

         if (ac .gt. close1  .and.  ac .lt. close2) then
             nclose =  nclose + 1
*----        ac     =  one
         end if

         ascale(j)  = ac
  430 continue

*     Remember, column scales are upside down.

      amax   = one / amax
      amin   = one / amin
      k      = max( 1, n - jstart + 1 )
      percnt = nclose * 100.0 / k
      if (iprint .gt. 0) then
         write(iprint, 1300)
         write(iprint, 1310) 'Col', jmax, amax, 'Col', jmin, amin,
     $                       nclose, percnt
      end if

*     ------------------------------------------------------------------
*     Phase 2.  Deal with empty rows and free rows.
*     Find the min and max row scales while we're at it.
*     ------------------------------------------------------------------
      amin   = bplus
      amax   = zero
      imin   = 0
      imax   = 0
      nclose = 0

      do 440 i = istart, m
         j     = n + i

         if (hrtype(i) .eq. 2) then
            ar = rmax(i)
            if (ar .eq. zero) ar = one
         else
            ar = ascale(j)
            if (ar .eq. zero) ar = one

            if (amin .gt. ar) then
                amin  =   ar
                imin  =   i
            end if
            if (amax .lt. ar) then
                amax  =   ar
                imax  =   i
            end if

            if (ar .gt. close1  .and.  ar .lt. close2) then
                nclose =  nclose + 1
*----           ar     =  one
            end if
         end if

         ascale(j) = ar
  440 continue

      if (imin .eq. 0) then
          amin   = zero
          amax   = zero
      end if
      k      = max( 1, m - istart + 1 )
      percnt = nclose * 100.0 / k
      if (iprint .gt. 0) then
         write(iprint, 1310) 'Row', imin, amin, 'Row', imax, amax,
     $                       nclose, percnt
      end if

*     ------------------------------------------------------------------
*     Phase 3.
*     Compute what is effectively the rhs for the constraints.
*     We set  rmax  =  ( A  I ) * x  for fixed columns and slacks,
*     including positive lower bounds and negative upper bounds.
*     ------------------------------------------------------------------
      if (lscale .ge. 2) then
         call dload ( m, zero, rmax, 1 )

         do 500 j = 1, nb
            bnd   = zero
            b1    = bl(j)
            b2    = bu(j)
            if (b1  .eq. b2  ) bnd = b1
            if (b1  .gt. zero) bnd = b1
            if (b2  .lt. zero) bnd = b2
            if (bnd .eq. zero) go to 500

            if (j   .le. n   ) then
               do 480 k = ka(j), ka(j+1) - 1
                  i       = ha(k)
                  rmax(i) = rmax(i)  +  a(k) * bnd
  480          continue
            else
*              Free slacks never get here,
*              so we don't have to skip them.
               i       = j - n
               rmax(i) = rmax(i)  +  bnd
            end if
  500    continue

*        We don't want nonzeros in free rows to interfere.

         do 520 i = 1, m
            if (hrtype(i) .eq. 2) rmax(i) = zero
  520    continue

*        Scale rmax = rmax / (row scales),  and use its norm sigma
*        to adjust both row and column scales.

         ac     = dnorm1( m, rmax, 1 )
         call dddiv ( m, ascale(n+1), 1, rmax, 1 )
         sigma  = dnorm1( m, rmax, 1 )
         if (iprint .gt. 0) write(iprint, 1400) ac, sigma
         sigma  = max( sigma, one )
         call dscal ( nb, sigma, ascale, 1 )

*        Big scales might lead to excessive infeasibility when the
*        problem is unscaled.  If any are too big, scale them down.

         amax  = zero
         do 540 j = 1, n
            amax  = max( amax, ascale(j) )
  540    continue

         do 550 i = 1, m
            if (hrtype(i) .ne. 2) then
               amax  = max( amax, ascale(n + i) )
            end if
  550    continue

         big    = 0.1 / tolx
         sigma  = big / amax
         if (sigma .lt. one) then
            call dscal ( nb, sigma, ascale, 1 )
            if (iprint .gt. 0) write(iprint, 1410) sigma
         end if
      end if

      if (iparm(4) .gt. 0  .and.  iprint .gt. 0) then
        if (istart .le. m) write(iprint,1500) (i,ascale(n+i),i=istart,m)
        if (jstart .le. n) write(iprint,1600) (j,ascale(j  ),j=jstart,n)
      end if

  900 return

 1000 format(// ' Scaling' / ' -------'
     $        / '             Min elem    Max elem       Max col ratio')
 1200 format(   ' After', i3, 1p, e12.2, e12.2, 0p, f20.2)
 1300 format(/  12x, 'Min scale', 23x, 'Max scale', 6x,
     $          'Between 0.5 and 2.0')
 1310 format(1x, a, i7, 1p, e10.1, 12x, a, i7, e10.1, i17, 0p, f8.1)
 1400 format(/  ' Norm of fixed columns and slacks', 1p, e20.1
     $       /  ' (before and after row scaling)  ', 1p, e20.1)
 1410 format(   ' Scales are large --- reduced by ', 1p, e20.1)
 1500 format(// ' Row scales  r(i)',
     $      8x, ' a(i,j)  =   r(i)  *  scaled a(i,j)  /  c(j)'
     $        / ' ----------------'    // 5(i6, g16.5))
 1600 format(// ' Column scales  c(j)',
     $      5x, ' x(j)    =   c(j)  *  scaled x(j)'
     $        / ' -------------------' // 5(i6, g16.5))

*     end of m2scal
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2scla( mode, m, n, nb, ne, nka,
     $                   ha, ka, a, ascale, bl, bu, pi, xn )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne), ascale(nb), bl(nb), bu(nb)
      double precision   pi(m), xn(nb)

*     ------------------------------------------------------------------
*     m2scla scales or unscales a, bl, bu, pi, xn using ascale.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj

      parameter        ( one = 1.0d+0 )

      bplus  = 0.1*plinfy
      if (mode .eq. 1) then
*        ---------------------------------------------------------------
*        mode = 1  ---  scale a, bl, bu, xn and pi.
*        ---------------------------------------------------------------
         do 150 j = 1, nb
            scale = ascale(j)
            if (j .le. n) then
               do 110 k = ka(j), ka(j+1) - 1
                  i     = ha(k)
                  a(k)  = a(k) * ( scale / ascale(n+i) )
  110          continue
            end if
            xn(j) = xn(j) / scale
            if (bl(j) .gt. -bplus) bl(j) = bl(j) / scale
            if (bu(j) .lt.  bplus) bu(j) = bu(j) / scale
  150    continue

         call ddscl ( m, ascale(n+1), 1, pi, 1 )
         if (jobj .gt. 0) sclobj = ascale(jobj)
      else
*        ---------------------------------------------------------------
*        mode = 2  ---  unscale everything.
*        ---------------------------------------------------------------
         do 250 j = 1, nb
            scale = ascale(j)
            if (j .le. n) then
               do 210 k = ka(j), ka(j+1) - 1
                  i     = ha(k)
                  a(k)  = a(k) * ( ascale(n+i) / scale )
  210          continue
            end if
            xn(j) = xn(j) * scale
            if (bl(j) .gt. -bplus) bl(j) = bl(j) * scale
            if (bu(j) .lt.  bplus) bu(j) = bu(j) * scale
  250    continue

         call dddiv ( m, ascale(n+1), 1, pi, 1 )
         sclobj = one
      end if

*     end of m2scla
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2unpk( jq, m, n, ne, nka, a, ha, ka, y )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne)
      double precision   y(m)

*     ------------------------------------------------------------------
*     m2unpk  expands the jq-th column of  ( A  I )  into  y.
*     ------------------------------------------------------------------

      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

      call dload ( m, zero, y, 1 )

      if (jq .le. n) then
         do 10 k = ka(jq), ka(jq+1) - 1
            i    = ha(k)
            y(i) = a(k)
   10    continue
      else
         islack    = jq - n
         y(islack) = one
      end if

*     end of m2unpk
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine matcol( m, n, nb, ne, nka,
     $                   a, ha, ka, bl, bu, col, ztol )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne), bl(nb), bu(nb), col(m)

*     ------------------------------------------------------------------
*     matcol  creates a new matrix column from the dense vector  col.
*     It becomes column number  jnew,  which is updated accordingly.
*     Elements as small as the zero tolerance  ztol  are not retained.
*     Default bounds are given to the new variable.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /cyclcm/ cnvtol,jnew,materr,maxcy,nephnt,nphant,nprint

      intrinsic          abs

      if (jnew .ge. n) go to 900
      jnew   = jnew + 1
      ia     = ka(jnew)

      do 100 i  = 1, m
         if (abs( col(i) )  .le.  ztol) go to 100
         if (ia .gt. ne) go to 910
         a (ia) = col(i)
         ha(ia) = i
         ia     = ia + 1
  100 continue

      if (ia .eq. ka(jnew)) go to 920
      bl(jnew) = bstruc(1)
      bu(jnew) = bstruc(2)
      ka(jnew + 1) = ia
      return

*     Too many columns.

  900 if (iprint .gt. 0) write(iprint, 1000)
      if (isumm  .gt. 0) write(isumm , 1000)
      go to 990

*     Too many elements.

  910 if (iprint .gt. 0) write(iprint, 1100)
      if (isumm  .gt. 0) write(isumm , 1100)
      go to 990

*     Zero column.

  920 if (iprint .gt. 0) write(iprint, 1200)
      if (isumm  .gt. 0) write(isumm , 1200)

*     Error exit.

  990 materr = materr + 1
      return

 1000 format(/ ' XXX  MATCOL  error.  Not enough Phantom columns.')
 1100 format(/ ' XXX  MATCOL  error.  Not enough Phantom elements.')
 1200 format(/ ' XXX  MATCOL  error.  New column of  A  was zero.')

*     end of matcol
      end
************************************************************************
*
*     File  mi25bfac fortran.
*
*     m2bfac   m2bmap   m2belm   m2bsol   m2sing
*     lu1fac   lu1fad   lu1gau   lu1mar   lu1pen
*     lu1max   lu1or1   lu1or2   lu1or3   lu1or4
*     lu1pq1   lu1pq2   lu1pq3   lu1rec
*     lu6chk   lu6sol   lu7add   lu7elm   lu7for   lu7zap   lu8rpc
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2bfac( gotfac, nfac, m, mbs, n, nb, nr, nn, ns,
     $                   lcrash, fsub, objadd,
     $                   ne, nka, a, ha, ka,
     $                   kb, hs, bl, bu, bbl, bbu,
     $                   r, w, x, xn, y, y2, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            gotfac
      integer*4          ha(ne), hs(nb)
      integer            ka(nka),kb(mbs)
      double precision   a(ne),  bl(nb), bu(nb), bbl(mbs), bbu(mbs),
     $                   r(nr),  w(m),   x(mbs), xn(nb),
     $                   y(mbs), y2(m),  z(nwcore)

*     ------------------------------------------------------------------
*     m2bfac  computes a factorization of the current basis.
*     gotfac  must be false the first time m2bfac is called.
*     It might be true after the first cycle.
*
*     If lcrash = 3, linear inequality rows (LG rows) are to be
*     treated as free rows.
*
*     04 May 1992: invmod and invitn not reset if gotfac is true.
*                  This will help m5solv keep track of LU properly
*                  during later cycles.
*     04 Jun 1992: lcrash added.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2lu1 / minlu,maxlu,lena,nbelem,ip,iq,lenc,lenr,
     $                   locc,locr,iploc,iqloc,lua,indc,indr
      common    /m2lu4 / parmlu(30),luparm(30)
      common    /m2mapz/ maxw,maxz
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5lp2 / invrq,invitn,invmod
      common    /m5prc / nparpr,nmulpr,kprc,newsb

      logical            modtol, prnt

      nfac   = nfac + 1
      obj    = sinf
      if (ninf .eq. 0) obj = minimz * fsub  +  objadd
      prnt   = iprint .gt. 0   .and.  mod(lprint,10) .gt. 0
      if (prnt) write(iprint, 1000) nfac, invrq, itn, ninf, obj

      m1     = m + 1
      ms     = m + ns
      l1     = n + 1
      ntry   = 0
      maxtry = 10
      if (gotfac) then
         if (invrq .eq. 0) go to 500
      end if

*     Load the basic variables into  kb,  slacks first.
*     Set kobj to tell us where the linear objective is.

   20 ntry   = ntry + 1
      invrq  = 0
      invitn = 0
      invmod = 0
      ierr   = 0
      k      = 0

      do 50 j  = l1, nb
         if (hs(j) .eq. 3) then
            k     = k + 1
            kb(k) = j
            if (j .eq. jobj) kobj = k
         end if
   50 continue

      nbelem = k
      nslack = k
      nonlin = 0
      do 100 j = 1, n
         if (hs(j) .eq. 3) then
            k     = k + 1
            if (k  .le. m) then
               kb(k) = j
               if (j .le. nn) nonlin = nonlin + 1
               nbelem = nbelem + ka(j+1) - ka(j)
            end if
         end if
  100 continue

      nbasic = k
      if (nbasic .gt. m) go to 920

      if (nbasic .lt. m) then

*        Not enough basics.
*        Set the remaining kb(k) = 0 for m2belm and m2sing.

         j      = nbasic + 1
         do 150 k = j, m
            kb(k) = 0
  150    continue
      end if

*     Make the objective the  iobj-th  basic, so we know where it is.

      if (iobj .gt. 0) then
         j        = kb(iobj)
         kb(iobj) = jobj
         kb(kobj) = j
      end if

      lin    = nbasic - nslack - nonlin
      if (lin .lt. 0) lin = 0
      bdnsty = 100.0 * nbelem / (m*m)
      if (prnt) write(iprint, 1010) nonlin, lin, nslack, nbelem, bdnsty

*     -----------------------------------------------------------------
*     Load the basis matrix into the LU arrays.
*     Then factorize it.
*
*     modtol says if this is the first iteration after Crash.
*     If so, we use big singularity tols to prevent getting an
*     unnecessarily ill-conditioned starting basis.
*     -----------------------------------------------------------------
      minlen = nbelem*5/4
      if (minlen .gt. lena) go to 940

      call m2belm( m, n, ne, nka, a, ha, ka, kb,
     $             z(lua), z(indc), z(indr), lena )

      modtol = itn .eq. 0  .and.  lcrash .gt. 0

      if (modtol) then
         utol1     = parmlu(4)
         utol2     = parmlu(5)
         parmlu(4) = max( utol1, eps3 )
         parmlu(5) = max( utol2, eps3 )
      end if

      call m2bsol( 0, m, w, y, z, nwcore )

      if (modtol) then
         parmlu(4) = utol1
         parmlu(5) = utol2
      end if

      nsing  = luparm(11)
      minlen = max( minlen, luparm(13) )
      if (ierr  .ge. 7) go to 940
      if (ierr  .ge. 2) go to 950
      ierr   = 0
      if (nsing .eq. 0) go to 500

*     -----------------------------------------------------------------
*     The basis appears to be singular.
*     Suspect columns are indicated by non-positive components of w(*).
*     Replace them by the relevant slacks and try again.
*     -----------------------------------------------------------------
      if (ntry .gt. maxtry) go to 960
      call m2sing( m, n, nb, w, z(ip), z(iq), bl, bu, hs, kb, xn )

*     See if any superbasics slacks were made basic.

      if (ns .gt. 0) then
         ns0    = ns
         do 410 k = 1, ns0
            jq    = 1 + ns0 - k
            j     = kb(m+jq)
            if (hs(j) .eq. 3) then
               call m6rdel( m, 1, nr, ns, ms,
     $                      kb, bbl, bbu, x, r, x, x, jq, .false. )
               ns    = ns - 1
               ms    = m  + ns
            end if
  410    continue
         if (ns .lt. ns0) r(1) = 0.0
      end if

      go to 20

*     ------------------------------------------------------------------
*     Compute the basic variables and check that  A * xn = 0.
*     If gotfac was true, ntry = 0.
*     ------------------------------------------------------------------
  500 gotfac = .false.
      call m5setx( 1, m, n, nb, ms, kb,
     $             ne, nka, a, ha, ka,
     $             bl, bu, x, xn, y, y2, z, nwcore )
      if (ierr .gt. 0  .and.  ntry .eq. 0) go to 20
      if (ierr .gt. 0) go to 980

*     Load basic and superbasic bounds into bbl, bbu.

      do 550 k  = 1, ms
         j      = kb(k)
         bbl(k) = bl(j)
         bbu(k) = bu(j)
  550 continue

*     For Crash option 3, linear LG rows should appear to be free.

      if (lcrash .eq. 3) then
         do 600 k = 1, ms
            j     = kb(k)
            if (j .gt. n) then
               if (bl(j) .lt. bu(j)) then
                  bbl(k) = - plinfy
                  bbu(k) = + plinfy
               end if
            end if
  600    continue
      end if

*     Normal exit.

      if (idebug .eq. 100) then
         if (iprint .gt. 0) write(iprint, 2000) (kb(k), x(k), k = 1, ms)
      end if
  900 return

*     -------------------------------------------------
*     Error exits.
*     m1page( ) decides whether to write on a new page.
*     m1page(2) also writes '=1' if GAMS.
*     -------------------------------------------------

*     Wrong number of basics.

  920 ierr   = 32
      call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1300) nbasic
      if (isumm  .gt. 0) write(isumm , 1300) nbasic
      return

*     Not enough memory.

  940 ierr   = 20
      more   = maxz + 3*(minlen - lena)
      call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1400) maxz, more
      if (isumm  .gt. 0) write(isumm , 1400) maxz, more
      return

*     Error in the LU package.

  950 ierr   = 21
      call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1500)
      if (isumm  .gt. 0) write(isumm , 1500)
      return

*     The basis is structurally singular even after the third try.
*     Time to give up.

  960 ierr   = 22
      call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1600) ntry
      if (isumm  .gt. 0) write(isumm , 1600) ntry
      return

*     Fatal row error.

  980 ierr   = 10
      return


 1000 format(/ ' Factorize', i6,
     $   '    Demand', i9, '    Iteration', i6,
     $   '    Infeas', i9, '    Objective', 1p, e17.9)
 1010 format(' Nonlinear', i6, '    Linear', i9, '    Slacks', i9,
     $       '    Elems', i10, '    Density', f8.2)
 1300 format(' EXIT -- system error.  Wrong no. of basic variables:',
     $         i8)
 1400 format(' EXIT -- not enough storage for the basis factors'
     $   // ' Words available =', i8
     $   // ' Increase Workspace (total) to at least', i8)
 1500 format(' EXIT -- error in basis package')
 1600 format(' EXIT -- the basis is singular after', i4,
     $   '  factorization attempts')
 2000 format(/ ' BS and SB values:' // (5(i7, g17.8)))

*     end of m2bfac
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2bmap( mode, m, n, ne, minz, maxz, nguess )

*     ------------------------------------------------------------------
*     m2bmap sets up the core allocation for the basis factors.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m1word/ nwordr,nwordi,nwordh
      common    /m2lu1 / minlu,maxlu,lena,nbelem,ip,iq,lenc,lenr,
     $                   locc,locr,iploc,iqloc,lua,indc,indr

      intrinsic          max, min

      minlu  = minz
      maxlu  = maxz
      mh     = (m - 1)/nwordh + 1
      mi     = (m - 1)/nwordi + 1
      ip     = minlu
      iq     = ip     + mh
      lenc   = iq     + mh
      lenr   = lenc   + mh
      locc   = lenr   + mh
      locr   = locc   + mi
      iploc  = locr   + mi
      iqloc  = iploc  + mh
      lua    = iqloc  + mh
      lena   = (maxlu - lua - 1)*nwordh/(nwordh + 2)
      indc   = lua    + lena
      indr   = indc   + (lena - 1)/nwordh + 1

*     Estimate the number of nonzeros in the basis factorization.
*     necola = estimate of nonzeros per column of  a.
*     We guess that the density of the basis factorization is
*     2 times as great, and then allow 1 more such lot for elbow room.
*     18 sep 1989: Tony and Alex change m to min( m, n ) below.

      necola = ne / n
      necola = max( necola, 5 )
      mina   = 3 * min( m, n ) * necola
      nguess = lua + mina + 2*mina/nwordh
      if (mode .ge. 3) then
         if (iprint .gt. 0) write(iprint, 1000) lena
      end if
      return

 1000 format(/ ' Nonzeros allowed for in LU factors', i9)

*     end of m2bmap
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2belm( m, n, ne, nka, a, ha, ka, kb,
     $                   alu, indc, indr, lena )

      implicit           double precision (a-h,o-z)
      double precision   a(ne), alu(lena)
      integer*4          ha(ne), indc(lena), indr(lena)
      integer            ka(nka), kb(m)

*     ------------------------------------------------------------------
*     m2belm  extracts the basis elements from the constraint matrix,
*     ready for use by the LU factorization routines.
*     ------------------------------------------------------------------

      parameter        ( one = 1.0d+0 )

      nz     = 0
      do 200 k = 1, m
         j     = kb(k)
         if (j .eq. 0) go to 200
         if (j .le. n) then
            i1    = ka(j)
            i2    = ka(j+1) - 1
            do 100  i   = i1, i2
               ir       = ha(i)
               nz       = nz + 1
               alu(nz)  = a(i)
               indc(nz) = ir
               indr(nz) = k
  100       continue
         else

*           Treat slacks specially.

            nz       = nz + 1
            alu(nz)  = one
            indc(nz) = j - n
            indr(nz) = k
         end if
  200 continue

*     end of m2belm
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2bsol( mode, m, w, y, z, nwcore )

      implicit           double precision (a-h,o-z)
      double precision   w(m), y(m), z(nwcore)

*     ------------------------------------------------------------------
*     m2bsol  calls up the relevant basis-factorization routines.
*
*     mode
*     ----
*      0    Factorize current basis from scratch, so that  B = L*U.
*      1    Solve  L*w = w(input).  y is not touched.
*      2    Solve  L*w = w(input)  and solve  B*y = w(input).
*      3    Solve  B(transpose)*y = w.  Note that w is destroyed.
*      4    Update the LU factors when the jp-th column is replaced
*           by a vector v.  jp is in common block /m5log3/.
*           On input, w must satisfy L*w = v.  w will be destroyed.
*      5    Solve  L(transpose)*w = w(input).  y is not touched.
*
*     The following tolerances are used...
*
*     luparm(3) = maxcol   lu1fac: maximum number of columns
*                          searched allowed in a Markowitz-type
*                          search for the next pivot element.
*     parmlu(1) = lmax1  = maximum multiplier allowed in  L  during
*                          refactorization.
*     parmlu(2) = lmax2  = maximum multiplier allowed during updates.
*     parmlu(3) = small  = minimum element kept in  B  or in
*                          transformed matrix during elimination.
*     parmlu(4) = utol1  = abs tol for flagging small diagonals of  U.
*     parmlu(5) = utol2  = rel tol for flagging small diagonals of  U.
*     parmlu(6) = uspace = factor allowing waste space in row/col lists.
*     parmlu(7) = dens1  = the density at which the Markowitz strategy
*                          should search maxcol columns and no rows.
*     parmlu(8) = dens2  = the density at which the Markowitz strategy
*                          should search only 1 column.
*                          (In one version of lu1fac, the remaining
*                          matrix is treated as dense if there is
*                          sufficient storage.)
*
*     25 Nov 1991: parmlu(1,2,4,5,8) are now defined by the SPECS file.
*     12 Jun 1992: Decided lu1fac's switch to dense LU was giving
*                  trouble.  Went back to treating all of LU as sparse.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2lu1 / minlu,maxlu,lena,nbelem,ip,iq,lenc,lenr,
     $                   locc,locr,iploc,iqloc,lua,indc,indr
      common    /m2lu2 / factol(5),lamin,nsing1,nsing2
      common    /m2lu3 / lenl,lenu,ncp,lrow,lcol
      common    /m2lu4 / parmlu(30),luparm(30)
      common    /m5freq/ kchk,kinv,ksav,klog,ksumm,i1freq,i2freq,msoln
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m5log1/ idebug,ierr,lprint
      common    /m5log3/ djq,theta,pivot,cond,nonopt,jp,jq,modr1,modr2
      common    /m5lp2 / invrq,invitn,invmod


      if (mode .eq. 0) then
*        ---------------------------------------------------------------
*        mode = 0.    Factorize the basis.
*        Note that parmlu(1,2,4,5,8) are defined by the SPECS file
*        in m3dflt and m3key.
*        ---------------------------------------------------------------
         luparm(1) = iprint
         luparm(2) = 0
         if (idebug .eq. 51) luparm(2) =  1
         if (idebug .eq. 52) luparm(2) =  2
         if (iprint .le.  0) luparm(2) = -1
         luparm(3) = 5
         if (i1freq .gt.  0) luparm(3) = i1freq
*       (For test purposes,  i1freq in the Specs file can alter maxcol.)

         parmlu(3) = eps0
         parmlu(6) = 3.0
         parmlu(7) = 0.3

         call lu1fac( m,  m   , nbelem  , lena   , luparm , parmlu,
     $                z(lua  ), z(indc ), z(indr), z(ip)  , z(iq) ,
     $                z(lenc ), z(lenr ), z(locc), z(locr),
     $                z(iploc), z(iqloc), z(ly  ), z(ly2 ), w, inform )

         lamin  = luparm(13)
         ndens1 = luparm(17)
         ndens2 = luparm(18)
         lenl   = luparm(23)
         lenu   = luparm(24)
         lrow   = luparm(25)
         ncp    = luparm(26)
         mersum = luparm(27)
         nutri  = luparm(28)
         nltri  = luparm(29)
         amax   = parmlu(10)
         elmax  = parmlu(11)
         umax   = parmlu(12)
         dumin  = parmlu(14)

         ierr   = inform
         bdincr = (lenl + lenu - nbelem) * 100.0 / max( nbelem, 1 )
         avgmer = mersum
         floatm = m
         avgmer = avgmer / floatm
         growth = umax   / (amax + eps)
         nbump  = m - nutri - nltri
         if (iprint .gt. 0  .and.  mod(lprint,10) .gt. 0) then
            write(iprint, 1000) ncp   , avgmer, lenl , lenu  ,
     $                          bdincr, m     , nutri, ndens1,
     $                          elmax , amax  , umax , dumin ,
     $                          growth, nltri , nbump, ndens2
         end if

      else if (mode .le. 2) then
*        ---------------------------------------------------------------
*        mode = 1 or 2.    Solve   L*w = w(input).
*        When LU*y = w is being solved in MINOS, norm(w) will sometimes
*        be small (e.g. after periodic refactorization).  Hence for
*        mode 2 we scale parmlu(3) to alter what lu6sol thinks is small.
*        ---------------------------------------------------------------
         small  = parmlu(3)
         if (mode .eq. 2) parmlu(3) = small * dnorm1( m, w, 1 )

         call lu6sol( 1, m, m, w, y, lena, luparm, parmlu,
     $                z(lua ), z(indc), z(indr), z(ip), z(iq),
     $                z(lenc), z(lenr), z(locc), z(locr), inform )

         parmlu(3) = small

         if (mode .eq. 2) then
*           ------------------------------------------------------------
*           mode = 2.    Solve  U*y = w.
*           ------------------------------------------------------------
            call lu6sol( 3, m, m, w, y, lena, luparm, parmlu,
     $                   z(lua ), z(indc), z(indr), z(ip), z(iq),
     $                   z(lenc), z(lenr), z(locc), z(locr), inform )
         end if

      else if (mode .eq. 3) then
*        ---------------------------------------------------------------
*        mode = 3.    Solve  B(transpose)*y = w.
*        ---------------------------------------------------------------
         call lu6sol( 6, m, m, y, w, lena, luparm, parmlu,
     $                z(lua ), z(indc), z(indr), z(ip), z(iq),
     $                z(lenc), z(lenr), z(locc), z(locr), inform )

      else if (mode .eq. 4) then
*        ---------------------------------------------------------------
*        mode = 4.    Update the LU factors of  B  after basis change.
*        ---------------------------------------------------------------
         invmod = invmod + 1
         call lu8rpc( 1, 2, m, m, jp, w, w,
     $                lena, luparm, parmlu,
     $                z(lua ), z(indc), z(indr), z(ip), z(iq),
     $                z(lenc), z(lenr), z(locc), z(locr),
     $                inform, diag, wnorm )
         if (inform .ne. 0) invrq = 7
         lenl   = luparm(23)
         lenu   = luparm(24)
         lrow   = luparm(25)
         ncp    = luparm(26)
      end if

      return

 1000 format(' Compressns', i5, '    Merit', f10.2,
     $       '    lenL', i11, '    lenU', i11, '    Increase', f7.2,
     $       '    m ', i6, '  Ut', i6, '  d1', i6, 1p
     $       /  ' Lmax', e11.1, '    Bmax', e11.1,
     $       '    Umax', e11.1, '    Umin', e11.1,
     $       '    Growth',e9.1, '    Lt', i6, '  bp', i6, '  d2', i6)

*     end of m2bsol
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m2sing( m, n, nb, w, ip, iq, bl, bu, hs, kb, xn )

      implicit           double precision (a-h,o-z)
      integer*4          ip(m), iq(m), hs(nb)
      integer            kb(m)
      double precision   bl(nb), bu(nb), w(m), xn(nb)

*     -----------------------------------------------------------------
*     m2sing  is called if the LU factorization of the basis appears
*     to be singular.   If  w(j)  is not positive, the  j-th  basic
*     variable  kb(j)  is replaced by the appropriate slack.
*     If any kb(j) = 0, only a partial basis was supplied.
*
*     08 Apr 1992: Now generate internal values for hs(j) if necessary
*                  (-1 or 4) to be compatible with m5hs.
*     -----------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm

      intrinsic          max, min
      parameter        ( zero = 0.0d+0,  nprint = 5 )

      nsing  = 0
      do 100 k  = 1, m
         j      = iq(k)
         if (w(j) .gt. zero) go to 100
         j      = kb(j)
         if (j    .gt.  0  ) then

*           Make variable  j  nonbasic (and feasible).
*           hs(j) = -1 means xn(j) is strictly between its bounds.

            if      (xn(j) .le. bl(j)) then
               xn(j) =  bl(j)
               hs(j) =  0
            else if (xn(j) .ge. bu(j)) then
               xn(j) =  bu(j)
               hs(j) =  1
            else
               hs(j) = -1
            end if

            if (bl(j) .eq. bu(j)) hs(j) = 4
         end if

*        Make the appropriate slack basic.

         i       = ip(k)
         hs(n+i) = 3
         nsing   = nsing + 1
         if (nsing .le. nprint) then
            if (iprint .gt. 0) write(iprint, 1000) j, i
            if (isumm  .gt. 0) write(isumm , 1000) j, i
         end if
  100 continue

      if (nsing .gt. nprint) then
         if (iprint .gt. 0) write(iprint, 1100) nsing
         if (isumm  .gt. 0) write(isumm , 1100) nsing
      end if
      return

 1000 format(' Column', i7, '  replaced by slack', i7)
 1100 format(' and so on.  Total slacks inserted =', i6)

*     end of m2sing
      end

************************************************************************
*
*     File  LU1.sparse  FORTRAN
*
*     lu1fac   lu1fad   lu1gau   lu1mar   lu1pen
*     lu1max   lu1or1   lu1or2   lu1or3   lu1or4     < File LU1UTIL
*     lu1pq1   lu1pq2   lu1pq3   lu1rec
*
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1fac( m    , n    , nelem, lena , luparm, parmlu,
     $                   a    , indc , indr , ip   , iq    ,
     $                   lenc , lenr , locc , locr ,
     $                   iploc, iqloc, ipinv, iqinv, w     , inform )

      implicit           double precision (a-h,o-z)
      integer            luparm(30)
      double precision   parmlu(30), a(lena)   , w(n)
      integer*4          indc(lena), indr(lena), ip(m)   , iq(n),
     $                   lenc(n)   , lenr(m)   ,
     $                   iploc(n)  , iqloc(m)  , ipinv(m), iqinv(n)
      integer            locc(n)   , locr(m)

*     ------------------------------------------------------------------
*     lu1fac computes a factorization A = L*U, where A is a sparse
*     matrix with m rows and n columns, P*L*P' is lower triangular
*     and P*U*Q is upper triangular for certain permutations P, Q
*     (which are returned in the arrays ip, iq).
*     Stability is ensured by limiting the size of the elements of L.
*
*     The nonzeros of A are input via the parallel arrays  a, indc, indr
*     which should contain nelem entries of the form     aij,    i,    j
*     in any order.
*
*     ******************************************************************
*     *        Beware !!!   The row indices i must be in indc,         *
*     *              and the column indices j must be in indr.         *
*     ******************************************************************
*
*     It does not matter if some of the entries in a(*) are zero.
*     Entries satisfying  abs( a(i) ) .le. parmlu(3)  are ignored.
*     Other parameters in luparm and parmlu are described below.
*
*     The matrix A may be singular.  On exit, nsing = luparm(11) gives
*     the number of apparent singularities.  This is the number of
*     "small" diagonals of the permuted factor U, as judged by
*     the input tolerances utol1 = parmlu(4) and  utol2 = parmlu(5).
*     The diagonal element diagj associated with column j of A is
*     "small" if
*                 abs( diagj ) .le. utol1
*     or
*                 abs( diagj ) .le. utol2 * max( uj ),
*
*     where max( uj ) is the maximum element in the j-th column of U.
*     The position of such elements is returned in w(*).  In general,
*     w(j) = + max( uj ),  but if column j is a singularity,
*     w(j) = - max( uj ).  Thus, w(j) .le. 0 if column j appears to be
*     dependent on the other columns of A.
*     ==================================================================
*
*
*     Notes on the array names
*     ------------------------
*
*     During the LU factorization, the sparsity pattern of the matrix
*     being factored is stored twice: in a column list and a row list.
*
*     The column list is ( a, indc, locc, lenc )
*     where
*           a(*)    holds the nonzeros,
*           indc(*) holds the indices for the column list,
*           locc(j) points to the start of column j in a(*) and indc(*),
*           lenc(j) is the number of nonzeros in column j.
*
*     The row list is    (    indr, locr, lenr )
*     where
*           indr(*) holds the indices for the row list,
*           locr(i) points to the start of row i in indr(*),
*           lenr(i) is the number of nonzeros in row i.
*
*
*     At all stages of the LU factorization, ip contains a complete
*     row permutation.  At the start of stage k,  ip(1), ..., ip(k-1)
*     are the first k-1 rows of the final row permutation P.
*     The remaining rows are stored in an ordered list
*                          ( ip, iploc, ipinv )
*     where
*           iploc(nz) points to the start in ip(*) of the set of rows
*                     that currently contain nz nonzeros,
*           ipinv(i)  points to the position of row i in ip(*).
*
*     For example,
*           iploc(1) = k   (and this is where rows of length 1 begin),
*           iploc(2) = k+p  if there are p rows of length 1
*                          (and this is where rows of length 2 begin).
*
*     Similarly for iq, iqloc, iqinv.
*     ==================================================================
*
*
*     00 Jun 1983  Original version.
*     00 Jul 1987  nrank  saved in luparm(16).
*     12 Apr 1989  ipinv, iqinv added as workspace.
*     26 Apr 1989  maxtie replaced by maxcol in Markowitz search.
*     16 Mar 1992  jumin  saved in luparm(19).
*     10 Jun 1992  lu1fad has to move empty rows and cols to the bottom
*                  (via lu1pq3) before doing the dense LU.
*     12 Jun 1992  Deleted dense LU (lu1ful and lu1vlu).
*
*     Systems Optimization Laboratory, Stanford University.
*  ---------------------------------------------------------------------
*
*  Input parameters                                        Typical value
*
*  luparm( 1) = nout     File number for printed messages.         6
*  luparm( 2) = lprint   Print level.                              0
*                    < 0 suppresses output.
*                    = 0 gives error messages.
*                    = 1 gives debug output from some of the
*                        other routines in LUSOL.
*                   >= 2 gives the pivot row and column and the
*                        no. of rows and columns involved at
*                        each elimination step in lu1fac.
*  luparm( 3) = maxcol   lu1fac: maximum number of columns         5
*                        searched allowed in a Markowitz-type
*                        search for the next pivot element.
*                        For some of the factorization, the
*                        number of rows searched is
*                        maxrow = maxcol - 1.
*
*
*  Output parameters
*
*  luparm(10) = inform   Return code from last call to any LU routine.
*  luparm(11) = nsing    No. of singularities marked in the
*                        output array w(*).
*  luparm(12) = jsing    Column index of last singularity.
*  luparm(13) = minlen   Minimum recommended value for  lena.
*  luparm(14) = maxlen   ?
*  luparm(15) = nupdat   No. of updates performed by the lu8 routines.
*  luparm(16) = nrank    No. of nonempty rows of U.
*  luparm(17) = ndens1   No. of columns remaining when the density of
*                        the matrix being factorized reached dens1.
*  luparm(18) = ndens2   No. of columns remaining when the density of
*                        the matrix being factorized reached dens2.
*  luparm(19) = jumin    The column index associated with dumin.
*  luparm(20) = numl0    No. of columns in initial  L.
*  luparm(21) = lenl0    Size of initial  L  (no. of nonzeros).
*  luparm(22) = lenu0    Size of initial  U.
*  luparm(23) = lenl     Size of current  L.
*  luparm(24) = lenu     Size of current  U.
*  luparm(25) = lrow     Length of row file.
*  luparm(26) = ncp      No. of compressions of LU data structures.
*  luparm(27) = mersum   lu1fac: sum of Markowitz merit counts.
*  luparm(28) = nutri    lu1fac: triangular rows in U.
*  luparm(29) = nltri    lu1fac: triangular rows in L.
*  luparm(30) =
*
*
*  Input parameters                                        Typical value
*
*  parmlu( 1) = elmax1   Max multiplier allowed in  L           10.0
*                        during factor.
*  parmlu( 2) = elmax2   Max multiplier allowed in  L           10.0
*                        during updates.
*  parmlu( 3) = small    Absolute tolerance for             eps**0.8
*                        treating reals as zero.     IBM double: 3.0d-13
*  parmlu( 4) = utol1    Absolute tol for flagging          eps**0.66667
*                        small diagonals of U.       IBM double: 3.7d-11
*  parmlu( 5) = utol2    Relative tol for flagging          eps**0.66667
*                        small diagonals of U.       IBM double: 3.7d-11
*  parmlu( 6) = uspace   Factor limiting waste space in  U.      3.0
*                        In lu1fac, the row or column lists
*                        are compressed if their length
*                        exceeds uspace times the length of
*                        either file after the last compression.
*  parmlu( 7) = dens1    The density at which the Markowitz      0.3
*                        strategy should search maxcol columns
*                        and no rows.
*  parmlu( 8) = dens2    the density at which the Markowitz      0.6
*                        strategy should search only 1 column
*                        or (preferably) use a dense LU for
*                        all the remaining rows and columns.
*
*
*  Output parameters
*
*  parmlu(10) = amax     Maximum element in  A.
*  parmlu(11) = elmax    Maximum multiplier in current  L.
*  parmlu(12) = umax     Maximum element in current  U.
*  parmlu(13) = dumax    Maximum diagonal in  U.
*  parmlu(14) = dumin    Minimum diagonal in  U.
*  parmlu(15) =
*  parmlu(16) =
*  parmlu(17) =
*  parmlu(18) =
*  parmlu(19) =
*  parmlu(20) = resid    lu6sol: residual after solve with U or U'.
*  ...
*  parmlu(30) =
*  ---------------------------------------------------------------------

      parameter        ( zero = 0.0d+0 )

*     Grab relevant input parameters.

      nout   = luparm(1)
      lprint = luparm(2)
      small  = parmlu(3)

*     Initialize output parameters.

      inform = 0
      minlen = nelem + 2*(m + n)
      nrank  = 0
      numl0  = 0
      lenl   = 0
      lenu   = 0
      lrow   = 0
      lcol   = nelem
      mersum = 0
      nutri  = m
      nltri  = 0
      amax   = 1.0

*     Initialize workspace parameters.

      luparm(26) = 0
      if (lena .lt. minlen) go to 970


*     ------------------------------------------------------------------
*     Organize the  aij's  in  a, indc, indr.
*     lu1or1  deletes small entries, tests for illegal  i,j's,
*             and counts the nonzeros in each row and column.
*     lu1or2  reorders the elements of  A  by columns.
*     lu1or3  uses the column list to test for duplicate entries
*             (same indices  i,j).
*     lu1or4  constructs a row list from the column list.
*     ------------------------------------------------------------------
      call lu1or1( m   , n    , nelem, small,
     $             a   , indc , indr , lenc , lenr,
     $             amax, numnz, lerr , inform )
      if (inform .ne. 0) go to 930

      nelem  = numnz
      if (nelem .gt. 0) then
         call lu1or2( n, nelem, a, indc, indr, lenc, locc )
         call lu1or3( m, n, nelem, indc, lenc, locc, w,
     $                lerr, inform )
         if (inform .ne. 0) go to 940

         call lu1or4( m, n, nelem,
     $                indc, indr, lenc, lenr, locc, locr )
      end if

*     ------------------------------------------------------------------
*     Set up lists of rows and columns with equal numbers of nonzeros,
*     using  indc(*)  as workspace.
*     Then compute the factorization  A = L*U.
*     ------------------------------------------------------------------
      call lu1pq1( m, n, lenr, ip, iploc, ipinv, indc(nelem + 1) )
      call lu1pq1( n, m, lenc, iq, iqloc, iqinv, indc(nelem + 1) )

      call lu1fad( m     , n    , nelem, lena  , luparm, parmlu,
     $             a     , indc , indr , ip    , iq    ,
     $             lenc  , lenr , locc , locr  ,
     $             iploc , iqloc, ipinv, iqinv ,
     $             inform, lenl , lenu , minlen, mersum,
     $             nutri , nltri )

      if (inform .eq. 7) go to 970
      if (inform .gt. 0) go to 980

*     Move empty rows and cols to the end of the permutations  ip, iq.

      call lu1pq3( m, lenr, ip, mrank )
      call lu1pq3( n, lenc, iq, nrank )
      nrank  = min( mrank, nrank )

*     ------------------------------------------------------------------
*     The LU factors are at the top of  a, indc, indr,
*     with the columns of  L  and the rows of  U  in the order
*
*        ( free )   ... ( u3 ) ( l3 ) ( u2 ) ( l2 ) ( u1 ) ( l1 ).
*
*     Starting with ( l1 ) and ( u1 ), move the rows of  U  to the
*     left and the columns of  L  to the right, giving
*
*        ( u1 ) ( u2 ) ( u3 ) ...   ( free )   ... ( l3 ) ( l2 ) ( l1 ).
*
*     Also, set  numl0 = the number of nonempty columns of  U.
*     ------------------------------------------------------------------
      lu     = 0
      ll     = lena + 1
      lm     = ll
      ltopl  = ll - lenl - lenu
      lrow   = lenu

      do 360  k  = 1, nrank
         i       =   ip(k)
         lenuk   = - lenr(i)
         lenr(i) =   lenuk
         j       =   iq(k)
         lenlk   = - lenc(j) - 1
         if (lenlk .gt. 0) then
             numl0        = numl0 + 1
             iqloc(numl0) = lenlk
         end if

         if (lu + lenuk .lt. ltopl) then
*           ============================================================
*           There is room to move ( uk ).  Just right-shift ( lk ).
*           ============================================================
            do 310 idummy = 1, lenlk
               ll       = ll - 1
               lm       = lm - 1
               a(ll)    = a(lm)
               indc(ll) = indc(lm)
               indr(ll) = indr(lm)
  310       continue
         else
*           ============================================================
*           There is no room for ( uk ) yet.  We have to
*           right-shift the whole of the remaining LU file.
*           Note that ( lk ) ends up in the correct place.
*           ============================================================
            llsave = ll - lenlk
            nmove  = lm - ltopl

            do 330 idummy = 1, nmove
               ll       = ll - 1
               lm       = lm - 1
               a(ll)    = a(lm)
               indc(ll) = indc(lm)
               indr(ll) = indr(lm)
  330       continue

            ltopl  = ll
            ll     = llsave
            lm     = ll
         end if

*        =========================================================
*        Left-shift ( uk ).
*        =========================================================
         locr(i) = lu + 1
         l2      = lm - 1
         lm      = lm - lenuk

         do 350 l = lm, l2
            lu       = lu + 1
            a(lu)    = a(l)
            indr(lu) = indr(l)
  350    continue
  360 continue

*     ------------------------------------------------------------------
*     Save the lengths of the nonempty columns of  L,
*     and initialize  locc(j)  for the LU update routines.
*     ------------------------------------------------------------------
      do 370  k  = 1, numl0
         lenc(k) = iqloc(k)
  370 continue

      do 390  j  = 1, n
         locc(j) = 0
  390 continue

*     ------------------------------------------------------------------
*     Test for singularity.
*     lu6chk  sets  nsing, jsing, jumin, elmax, umax, dumax, dumin.
*     inform = 1  if there are singularities (nsing gt 0).
*     ------------------------------------------------------------------
      luparm(16) = nrank
      luparm(23) = lenl
      call lu6chk( 1, m, n, w, lena, luparm, parmlu,
     $             a, indc, indr, ip, iq,
     $             lenc, lenr, locc, locr, inform )
      go to 990

*     ------------
*     Error exits.
*     ------------
  930 inform = 3
      if (lprint .ge. 0) write(nout, 1300) lerr, indc(lerr), indr(lerr)
      go to 990

  940 inform = 4
      if (lprint .ge. 0) write(nout, 1400) lerr, indc(lerr), indr(lerr)
      go to 990

  970 inform = 7
      if (lprint .ge. 0) write(nout, 1700) lena, minlen
      go to 990

  980 inform = 8
      if (lprint .ge. 0) write(nout, 1800)

*     Store output parameters.

  990 luparm(10) = inform
      luparm(13) = minlen
      luparm(15) = 0
      luparm(16) = nrank
      luparm(20) = numl0
      luparm(21) = lenl
      luparm(22) = lenu
      luparm(23) = lenl
      luparm(24) = lenu
      luparm(25) = lrow
      luparm(27) = mersum
      luparm(28) = nutri
      luparm(29) = nltri
      parmlu(10) = amax
      return

 1300 format(/ ' lu1fac  error...  entry  a(', i8, ')  has an illegal',
     $         ' row or column index'
     $       //' indc, indr =', 2i8)
 1400 format(/ ' lu1fac  error...  entry  a(', i8, ')  has the same',
     $         ' indices as an earlier entry'
     $       //' indc, indr =', 2i8)
 1700 format(/ ' lu1fac  error...  insufficient storage'
     $       //' Increase  lena  from', i8, '  to at least', i8)
 1800 format(/ ' lu1fac  error...  fatal bug',
     $         '   (sorry --- this should never happen)')
*     end of lu1fac
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1fad( m     , n    , nelem, lena  , luparm, parmlu,
     $                   a     , indc , indr , ip    , iq    ,
     $                   lenc  , lenr , locc , locr  ,
     $                   iploc , iqloc, ipinv, iqinv ,
     $                   inform, lenl , lenu , minlen, mersum,
     $                   nutri , nltri )

      implicit           double precision (a-h,o-z)
      integer            luparm(30)
      double precision   parmlu(30), a(lena)
      integer*4          indc(lena), indr(lena), ip(m), iq(n)
      integer*4          lenc(n)   , lenr(m)
      integer            locc(n)   , locr(m)
      integer*4          iploc(n)  , iqloc(m), ipinv(m), iqinv(n)

*     ------------------------------------------------------------------
*     lu1fad  is a driver for the numerical phase of lu1fac.
*     At each stage it computes a column of  L  and a row of  U,
*     using a Markowitz criterion to select the pivot element,
*     subject to a stability criterion that bounds the elements of  L.
*
*     00 Jan 1986  Version documented in LUSOL paper:
*                  Gill, Murray, Saunders and Wright (1987),
*                  Maintaining LU factors of a general sparse matrix,
*                  Linear algebra and its applications 88/89, 239-270.
*
*     02 Feb 1989  Following Suhl and Aittoniemi (1987), the largest
*                  element in each column is now kept at the start of
*                  the column, i.e. in position locc(j) of a and indc.
*                  This should speed up the Markowitz searches.
*                  To save time on highly triangular matrices, we wait
*                  until there are no further columns of length 1
*                  before setting and maintaining that property.
*
*     12 Apr 1989  ipinv and iqinv added (inverses of ip and iq)
*                  to save searching ip and iq for rows and columns
*                  altered in each elimination step.  (Used in lu1pq2)
*
*     19 Apr 1989  Code segmented to reduce its size.
*                  lu1gau does most of the Gaussian elimination work.
*                  lu1mar does just the Markowitz search.
*                  lu1max moves biggest elements to top of columns.
*                  lu1pen deals with pending fill-in in the row list.
*                  lu1pq2 updates the row and column permutations.
*
*     26 Apr 1989  maxtie replaced by maxcol, maxrow in the Markowitz
*                  search.  maxcol, maxrow change as density increases.
*
*     Systems Optimization Laboratory, Stanford University.
*     ------------------------------------------------------------------

      intrinsic          abs  , min , max
      logical            utri , ltri, spars1, spars2, dense
      double precision   lmax, small

      double precision   zero,             one,
     $                   lmin
      parameter        ( zero = 0.0d+0,    one = 1.0d+0,
     $                   lmin = 1.0001d+0 )

*     ------------------------------------------------------------------
*     Local variables
*     ---------------
*
*     lcol   is the length of the column file.  It points to the last
*            nonzero in the column list.
*     lrow   is the analogous quantity for the row file.
*     lfile  is the file length (lcol or lrow) after the most recent
*            compression of the column list or row list.
*     nrowd  and  ncold  are the number of rows and columns in the
*            matrix defined by the pivot column and row.  They are the
*            dimensions of the submatrix D being altered at this stage.
*     melim  and  nelim  are the number of rows and columns in the
*            same matrix D, excluding the pivot column and row.
*     mleft  and  nleft  are the number of rows and columns
*            still left to be factored.
*     nzchng is the increase in nonzeros in the matrix that remains
*            to be factored after the current elimination
*            (usually negative).
*     nzleft is the number of nonzeros still left to be factored.
*     nspare is the space we leave at the end of the last row or
*            column whenever a row or column is being moved to the end
*            of its file.  nspare = 1 or 2 might help reduce the
*            number of file compressions when storage is tight.
*
*     The row and column ordering permutes A into the form
*
*                        ------------------------
*                         \                     |
*                          \         U1         |
*                           \                   |
*                            --------------------
*                            |\
*                            | \
*                            |  \
*            P A Q   =       |   \
*                            |    \
*                            |     --------------
*                            |     |            |
*                            |     |            |
*                            | L1  |     A2     |
*                            |     |            |
*                            |     |            |
*                            --------------------
*
*     where the block A2 is factored as  A2 = L2 U2.
*     The phases of the factorization are as follows.
*
*     utri   is true when U1 is being determined.
*            Any column of length 1 is accepted immediately.
*
*     ltri   is true when L1 is being determined.
*            lu1mar exits as soon as an acceptable pivot is found
*            in a row of length 1.
*
*     spars1 is true while the density of the (modified) A2 is less
*            than the parameter dens1 = parmlu(7) = 0.3 say.
*            lu1mar searches maxcol columns and maxrow rows,
*            where  maxcol = luparm(3),  maxrow = maxcol - 1.
*            lu1max is used to keep the biggest element at the top
*            of all remaining columns.
*
*     spars2 is true while the density of the modified A2 is less
*            than the parameter dens2 = parmlu(8) = 0.6 say.
*            lu1mar searches maxcol columns and no rows.
*            lu1max fixes up only the first maxcol columns.
*
*     dense  is true once the density of A2 reaches dens2.
*            lu1mar searches only 1 column (the shortest).
*            lu1max fixes up only the first column.
*
*     ------------------------------------------------------------------
*     To eliminate timings, comment out all lines containing "time".
*     ------------------------------------------------------------------

*     integer            eltime, mktime

*     call timer ( 'start ', 3 )
*     ntime  = (n / 4.0) + lmin

      nout   = luparm(1)
      lprint = luparm(2)
      maxcol = luparm(3)
      maxrow = maxcol - 1
      lenl   = 0
      lenu   = 0
      lfile  = nelem
      lrow   = nelem
      lcol   = nelem
      lu1    = lena + 1
      minmn  = min( m, n )
      maxmn  = max( m, n )
      nzleft = nelem
      nspare = 1

      lmax   = parmlu(1)
      small  = parmlu(3)
      uspace = parmlu(6)
      dens1  = parmlu(7)
      dens2  = parmlu(8)
      utri   = .true.
      ltri   = .false.
      spars1 = .false.
      spars2 = .false.
      dense  = .false.

*     Check parameters.

      lmax   = max( lmax, lmin )

*     Initialize output parameters.

      ndens1 = 0
      ndens2 = 0

*     ------------------------------------------------------------------
*     Start of main loop.
*     ------------------------------------------------------------------

      do 800 nrowu = 1, minmn

*        mktime = (nrowu / ntime) + 4
*        eltime = (nrowu / ntime) + 9
         mleft  = m + 1 - nrowu
         nleft  = n + 1 - nrowu

*        Bail out if there are no nonzero rows left.

         if (iploc(1) .gt. m) go to 900

*        ===============================================================
*        Find a suitable pivot element.
*        ===============================================================

         if ( utri ) then
*           ------------------------------------------------------------
*           So far all columns have had length 1.
*           We are still looking for the (backward) triangular part of A
*           that forms the first rows and columns of U.
*           ------------------------------------------------------------
            lq1    = iqloc(1)
            lq2    = n
            if (m   .gt.   1) lq2 = iqloc(2) - 1

            if (lq1 .le. lq2) then

*              We still have a column of length 1.  Grab it!

               jbest  = iq(lq1)
               lc     = locc(jbest)
               ibest  = indc(lc)
               mbest  = 0
            else

*              This is the end of the U triangle.
*              Move the largest elements to the top of each column
*              to make the remaining Markowitz searches more efficient.
*              We will not return to this part of the code.

               utri   = .false.
               ltri   = .true.
               spars1 = .true.
               nutri  =  nrowu - 1
               call lu1max( lq1, n, iq, a, indc, lenc, locc )
            end if
         end if

         if ( spars1 ) then
*           ------------------------------------------------------------
*           Perform a normal Markowitz search.
*           Search cols of length 1, then rows of length 1,
*           then   cols of length 2, then rows of length 2, etc.
*           ------------------------------------------------------------
*           call timer ( 'start ', mktime )
            call lu1mar( m    , n     , lena  , maxmn,
     $                   lmax , maxcol, maxrow,
     $                   ibest, jbest , mbest ,
     $                   a    , indc  , indr  , ip   , iq,
     $                   lenc , lenr  , locc  , locr ,
     $                   iploc, iqloc )
*           call timer ( 'finish', mktime )

            if ( ltri ) then

*              So far all rows have had length 1.
*              We are still looking for the (forward) triangle of A
*              that forms the first rows and columns of L.

               if (mbest .gt. 0) then
                   ltri   = .false.
                   nltri  =  nrowu - 1 - nutri
               end if
            else

*              See if what's left is as dense as dens1.

               if (nzleft  .ge.  mleft * nleft * dens1) then
                   spars1 = .false.
                   spars2 = .true.
                   ndens1 =  nleft
                   maxrow =  0
               end if
            end if

         else if ( spars2 .or. dense ) then
*           ------------------------------------------------------------
*           Perform a restricted Markowitz search,
*           looking at only the first maxcol columns.  (maxrow = 0.)
*           ------------------------------------------------------------
*           call timer ( 'start ', mktime )
            call lu1mar( m    , n     , lena  , maxmn,
     $                   lmax , maxcol, maxrow,
     $                   ibest, jbest , mbest ,
     $                   a    , indc  , indr  , ip   , iq,
     $                   lenc , lenr  , locc  , locr ,
     $                   iploc, iqloc )
*           call timer ( 'finish', mktime )

*           See if what's left is as dense as dens2.

            if ( spars2 ) then
               if (nzleft  .ge.  mleft * nleft * dens2) then
                   spars2 = .false.
                   dense  = .true.
                   ndens2 =  nleft
                   maxcol =  1
               end if
            end if
         end if

*        ===============================================================
*        The best  aij  has been found.
*        The pivot row  ibest  and the pivot column  jbest
*        Define a dense matrix  D  of size  nrowd  by  ncold.
*        ===============================================================
         ncold  = lenr(ibest)
         nrowd  = lenc(jbest)
         melim  = nrowd  - 1
         nelim  = ncold  - 1
         mersum = mersum + mbest
         lenl   = lenl   + melim
         lenu   = lenu   + ncold
         if (lprint .ge. 2)
     $   write(nout, 1100) nrowu, ibest, jbest, nrowd, ncold

*        ===============================================================
*        Allocate storage for the next column of  L  and next row of  U.
*        Initially the top of a, indc, indr are used as follows:
*
*                   ncold       melim       ncold        melim
*
*        a      |...........|...........|ujbest..ujn|li1......lim|
*
*        indc   |...........|  lenr(i)  |  lenc(j)  |  markl(i)  |
*
*        indr   |...........| iqloc(i)  |  jfill(j) |  ifill(i)  |
*
*              ^           ^             ^           ^            ^
*              lfree   lsave             lu1         ll1          oldlu1
*
*        Later the correct indices are inserted:
*
*        indc   |           |           |           |i1........im|
*
*        indr   |           |           |jbest....jn|ibest..ibest|
*
*        ===============================================================
         ll1    = lu1   - melim
         lu1    = ll1   - ncold
         lsave  = lu1   - nrowd
         lfree  = lsave - ncold

*        Make sure the column file has room.
*        Also force a compression if its length exceeds a certain limit.

         limit  = uspace * lfile  +  m  +  n  +  1000
         minfre = ncold  + melim
         nfree  = lfree  - lcol
         if (nfree .lt. minfre  .or.  lcol .gt. limit) then
            call lu1rec( n, .true., luparm,
     $                   lcol, lena, a, indc, lenc, locc )
            lfile  = lcol
            nfree  = lfree - lcol
            if (nfree .lt. minfre) go to 970
         end if

*        Make sure the row file has room.

         minfre = melim + ncold
         nfree  = lfree - lrow
         if (nfree .lt. minfre  .or.  lrow .gt. limit) then
            call lu1rec( m, .false., luparm,
     $                   lrow, lena, a, indr, lenr, locr )
            lfile  = lrow
            nfree  = lfree - lrow
            if (nfree .lt. minfre) go to 970
         end if

*        ===============================================================
*        Move the pivot element to the front of its row
*        and to the top of its column.
*        ===============================================================
         lpivr  = locr(ibest)
         lpivr1 = lpivr + 1
         lpivr2 = lpivr + nelim

         do 330 l = lpivr, lpivr2
            if (indr(l) .eq. jbest) go to 335
  330    continue

  335    indr(l)     = indr(lpivr)
         indr(lpivr) = jbest

         lpivc  = locc(jbest)
         lpivc1 = lpivc + 1
         lpivc2 = lpivc + melim

         do 340 l = lpivc, lpivc2
            if (indc(l) .eq. ibest) go to 345
  340    continue

  345    indc(l)     = indc(lpivc)
         indc(lpivc) = ibest
         abest       = a(l)
         a(l)        = a(lpivc)
         a(lpivc)    = abest

*        ===============================================================
*        Delete the pivot row from the column file
*        and store it as the next row of  U.
*        set  indr(lu) = 0     to initialize jfill ptrs on columns of D,
*             indc(lu) = lenj  to save the original column lengths.
*        ===============================================================
         a(lu1)    = abest
         indr(lu1) = jbest
         indc(lu1) = nrowd
         lu        = lu1

         do 360 lr   = lpivr1, lpivr2
            lu       = lu + 1
            j        = indr(lr)
            lenj     = lenc(j)
            lenc(j)  = lenj - 1
            lc1      = locc(j)
            last     = lc1 + lenc(j)

            do 350 l = lc1, last
               if (indc(l) .eq. ibest) go to 355
  350       continue

  355       a(lu)      = a(l)
            indr(lu)   = 0
            indc(lu)   = lenj
            a(l)       = a(last)
            indc(l)    = indc(last)
            indc(last) = 0
  360    continue

         if (indc(lcol) .eq. 0) lcol = lcol - 1

*        ===============================================================
*        Delete the pivot column from the row file
*        and store the nonzeros of the next column of  L.
*        Set  indc(ll) = 0     to initialize markl(*) markers,
*             indr(ll) = 0     to initialize ifill(*) row fill-in cntrs,
*             indc(ls) = leni  to save the original row lengths,
*             indr(ls) = iqloc(i)    to save parts of  iqloc(*),
*             iqloc(i) = lsave - ls  to point to the nonzeros of  L
*                      = -1, -2, -3, ... in mark(*).
*        ===============================================================
         indc(lsave) = ncold
         if (melim .eq. 0) go to 700

         ll     = ll1 - 1
         ls     = lsave
         abest  = one / abest

         do 390 lc   = lpivc1, lpivc2
            ll       = ll + 1
            ls       = ls + 1
            i        = indc(lc)
            leni     = lenr(i)
            lenr(i)  = leni - 1
            lr1      = locr(i)
            last     = lr1 + lenr(i)

            do 380 l = lr1, last
               if (indr(l) .eq. jbest) go to 385
  380       continue

  385       indr(l)    = indr(last)
            indr(last) = 0

            a(ll)      = - a(lc) * abest
            indc(ll)   = 0
            indr(ll)   = 0
            indc(ls)   = leni
            indr(ls)   = iqloc(i)
            iqloc(i)   = lsave - ls
  390    continue

         if (indr(lrow) .eq. 0) lrow = lrow - 1

*        ===============================================================
*        Do the Gaussian elimination.
*        This involves adding a multiple of the pivot column
*        to all other columns in the pivot row.
*
*        Sometimes more than one call to lu1gau is needed to allow
*        compression of the column file.
*        lfirst  says which column the elimination should start with.
*        minfre  is a bound on the storage needed for any one column.
*        lu      points to off-diagonals of u.
*        nfill   keeps track of pending fill-in in the row file.
*        ===============================================================
         if (nelim .eq. 0) go to 700
         lfirst = lpivr1
         minfre = mleft + nspare
         lu     = 1
         nfill  = 0

* 400    call timer ( 'start ', eltime )
  400    call lu1gau( m     , melim , ncold , nspare, small,
     $                lpivc1, lpivc2, lfirst, lpivr2, lfree, minfre,
     $                lrow  , lcol  , lu    , nfill ,
     $                a     , indc  , indr  ,
     $                lenc  , lenr  , locc  , locr  ,
     $                iqloc , a(ll1), indc(ll1),
     $                        a(lu1), indr(ll1), indr(lu1) )
*        call timer ( 'finish', eltime )

         if (lfirst .gt. 0) then

*           The elimination was interrupted.
*           Compress the column file and try again.
*           lfirst, lu and nfill have appropriate new values.

            call lu1rec( n, .true., luparm,
     $                   lcol, lena, a, indc, lenc, locc )
            lfile  = lcol
            lpivc  = locc(jbest)
            lpivc1 = lpivc + 1
            lpivc2 = lpivc + melim
            nfree  = lfree - lcol
            if (nfree .lt. minfre) go to 970
            go to 400
         end if

*        ===============================================================
*        The column file has been fully updated.
*        Deal with any pending fill-in in the row file.
*        ===============================================================
         if (nfill .gt. 0) then

*           Compress the row file if necessary.
*           lu1gau has set nfill to be the number of pending fill-ins
*           plus the current length of any rows that need to be moved.

            minfre = nfill
            nfree  = lfree - lrow
            if (nfree .lt. minfre) then
               call lu1rec( m, .false., luparm,
     $                      lrow, lena, a, indr, lenr, locr )
               lfile  = lrow
               lpivr  = locr(ibest)
               lpivr1 = lpivr + 1
               lpivr2 = lpivr + nelim
               nfree  = lfree - lrow
               if (nfree .lt. minfre) go to 970
            end if

*           Move rows that have pending fill-in to end of the row file.
*           Then insert the fill-in.

            call lu1pen( m     , melim , ncold , nspare,
     $                   lpivc1, lpivc2, lpivr1, lpivr2, lrow,
     $                   lenc  , lenr  , locc  , locr  ,
     $                   indc  , indr  , indr(ll1), indr(lu1) )
         end if

*        ===============================================================
*        Restore the saved values of  iqloc.
*        Insert the correct indices for the col of L and the row of U.
*        ===============================================================
  700    lenr(ibest) = 0
         lenc(jbest) = 0

         ll          = ll1 - 1
         ls          = lsave

         do 710  lc  = lpivc1, lpivc2
            ll       = ll + 1
            ls       = ls + 1
            i        = indc(lc)
            iqloc(i) = indr(ls)
            indc(ll) = i
            indr(ll) = ibest
  710    continue

         lu          = lu1 - 1

         do 720  lr  = lpivr, lpivr2
            lu       = lu + 1
            indr(lu) = indr(lr)
  720    continue

*        ===============================================================
*        Free the space occupied by the pivot row
*        and update the column permutation.
*        Then free the space occupied by the pivot column
*        and update the row permutation.
*
*        nzchng is found in both calls to lu1pq2, but we use it only
*        after the second.
*        ===============================================================
         call lu1pq2( ncold, nzchng,
     $                indr(lpivr), indc( lu1 ), lenc, iqloc, iq, iqinv )

         call lu1pq2( nrowd, nzchng,
     $                indc(lpivc), indc(lsave), lenr, iploc, ip, ipinv )

         nzleft = nzleft + nzchng

*        ===============================================================
*        Move the largest element to the top of each relevant column.
*        ===============================================================
         if ( spars1 ) then

*           Do all modified columns.

            if ( nelim .gt. 0 ) then
*              call timer ( 'start ', 16 )
               call lu1max( lu1+1, lu, indr, a, indc, lenc, locc )
*              call timer ( 'finish', 16 )
            end if

         else if ( spars2 .or. dense ) then

*           Just do the maxcol shortest columns.

*           call timer ( 'start ', 16 )
            lq1    = iqloc(1)
            lq2    = min( lq1 + maxcol - 1, n )
            call lu1max( lq1, lq2, iq, a, indc, lenc, locc )
*           call timer ( 'finish', 16 )
         end if

*        ===============================================================
*        Negate lengths of pivot row and column so they will be
*        eliminated during compressions.
*        ===============================================================
         lenr(ibest) = - ncold
         lenc(jbest) = - nrowd

*        Test for fatal bug: row or column lists overwriting L and U.

         if (lrow .gt. lsave) go to 980
         if (lcol .gt. lsave) go to 980

*        Reset the length of the row file if pivot row was at the end.
*        Similarly for the column file.

         if (lrow .eq. lpivr2) then
            lrow   = lpivr
            do 770 l  = 1, lpivr
               if (indr(lrow) .ne. 0) go to 780
               lrow   = lrow - 1
  770       continue
         end if

  780    if (lcol .eq. lpivc2) then
            lcol   = lpivc
            do 790 l  = 1, lpivc
               if (indc(lcol) .ne. 0) go to 800
               lcol   = lcol - 1
  790       continue
         end if
  800 continue

*     ------------------------------------------------------------------
*     End of main loop.
*     ------------------------------------------------------------------

*     Normal exit.

  900 inform = 0
      minlen = lenl  +  lenu  +  2*(m + n)
      go to 990

*     Not enough space free after a compress.
*     Set  minlen  to an estimate of the necessary value of  lena.

  970 inform = 7
      minlen = lena  +  lfile  +  2*(m + n)
      go to 990

*     Fatal error.  This will never happen!
*    (Famous last words.)

  980 inform = 8

*     exit.

  990 luparm(17) = ndens1
      luparm(18) = ndens2
*     call timer ( 'finish', 3 )
      return

 1100 format(' lu1fad.  nrowu =', i7, '   ibest, jbest =', 2i7,
     $       '   nrowd, ncold =', 2i5)
*     end of lu1fad
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1gau( m     , melim , ncold , nspare, small,
     $                   lpivc1, lpivc2, lfirst, lpivr2, lfree, minfre,
     $                   lrow  , lcol  , lu    , nfill ,
     $                   a     , indc  , indr  ,
     $                   lenc  , lenr  , locc  , locr  ,
     $                   mark  , al    , markl ,
     $                           au    , ifill , jfill )

      implicit           double precision (a-h,o-z)
      double precision   a(*)        , al(melim)   , au(ncold)
      integer*4          indc(*)     , indr(*)     , lenc(*)  , lenr(*),
     $                   mark(*)     , markl(melim),
     $                   ifill(melim), jfill(ncold)
      integer            locc(*)     , locr(*)

*     ------------------------------------------------------------------
*     lu1gau does most of the work for each step of Gaussian elimination.
*     A multiple of the pivot column is added to each other column j
*     in the pivot row.  The column list is fully updated.
*     The row list is updated if there is room, but some fill-ins may
*     remain, as indicated by ifill and jfill.
*
*
*  Input:
*     lfirst   is the first column to be processed.
*     lu + 1   is the corresponding element of U in au(*).
*     nfill    keeps track of pending fill-in.
*     a(*)     contains the nonzeros for each column j.
*     indc(*)  contains the row indices for each column j.
*     al(*)    contains the new column of L.  A multiple of it is
*              used to modify each column.
*     mark(*)  has been set to -1, -2, -3, ... in the rows
*              corresponding to nonzero 1, 2, 3, ... of the col of L.
*     au(*)    contains the new row of U.  Each nonzero gives the
*              required multiple of the column of L.
*
*  Workspace:
*     markl(*) marks the nonzeros of L actually used.
*              (A different mark, namely j, is used for each column.)
*
*  Output:
*     lfirst    = 0 if all columns were completed,
*               > 0 otherwise.
*     lu        returns the position of the last nonzero of U
*               actually used, in case we come back in again.
*     nfill     keeps track of the total extra space needed in the
*               row file.
*     ifill(ll) counts pending fill-in for rows involved in the new
*               column of L.
*     jfill(lu) marks the first pending fill-in stored in columns
*               involved in the new row of U.
*
*     16 Apr 1989  First version.
*     23 Apr 1989  lfirst, lu, nfill are now input and output
*                  to allow re-entry if elimination is interrupted.
*     ------------------------------------------------------------------

      intrinsic          abs
      logical            atend

      do 600 lr = lfirst, lpivr2
         j      = indr(lr)
         lenj   = lenc(j)
         nfree  = lfree - lcol
         if (nfree .lt. minfre) go to 900

*        ---------------------------------------------------------------
*        Inner loop to modify existing nonzeros in column  j.
*        Loop 440 performs most of the arithmetic involved in the
*        whole LU factorization.
*        ndone counts how many multipliers were used.
*        ndrop counts how many modified nonzeros are negligibly small.
*        ---------------------------------------------------------------
         lu     = lu + 1
         uj     = au(lu)
         lc1    = locc(j)
         lc2    = lc1 + lenj - 1
         atend  = lcol .eq. lc2
         ndone  = 0
         if (lenj .eq. 0) go to 500

         ndrop  = 0

         do 440 l = lc1, lc2
            i        =   indc(l)
            ll       = - mark(i)
            if (ll .gt. 0) then
               ndone     = ndone + 1
               markl(ll) = j
               a(l)      = a(l)  +  al(ll) * uj
               if (abs( a(l) ) .le. small) then
                  ndrop  = ndrop + 1
               end if
            end if
  440    continue

*        ---------------------------------------------------------------
*        Remove any negligible modified nonzeros from both
*        the column file and the row file.
*        ---------------------------------------------------------------
         if (ndrop .eq. 0) go to 500
         k      = lc1

         do 480 l = lc1, lc2
            i        = indc(l)
            if (abs( a(l) ) .le. small) go to 460
            a(k)     = a(l)
            indc(k)  = i
            k        = k + 1
            go to 480

*           Delete the nonzero from the row file.

  460       lenj     = lenj    - 1
            lenr(i)  = lenr(i) - 1
            lr1      = locr(i)
            last     = lr1 + lenr(i)

            do 470 lrep = lr1, last
               if (indr(lrep) .eq. j) go to 475
  470       continue

  475       indr(lrep) = indr(last)
            indr(last) = 0
            if (lrow .eq. last) lrow = lrow - 1
  480    continue

*        Free the deleted elements.

         do 490  l  = k, lc2
            indc(l) = 0
  490    continue
         if (atend) lcol = k - 1

*        ---------------------------------------------------------------
*        Deal with the fill-in in column  j.
*        ---------------------------------------------------------------
  500    if (ndone .eq. melim) go to 590

*        See if column j already has room for the fill-in.

         if (atend) go to 540
         last   = lc1  + lenj - 1
         l1     = last + 1
         l2     = last + (melim - ndone)

         do 510 l = l1, l2
            if (indc(l) .gt. 0) go to 520
  510    continue
         go to 540

*        We must move column  j  to the end of the column file.
*        Leave some spare room at the end of the current last column.

  520    atend   = .true.

         do 522  l  = lcol + 1, lcol + nspare
            lcol    = l
            indc(l) = 0
  522    continue

         l1      = lc1
         lc1     = lcol + 1
         locc(j) = lc1

         do 525  l     = l1, last
            lcol       = lcol + 1
            a(lcol)    = a(l)
            indc(lcol) = indc(l)
            indc(l)    = 0
  525    continue

*        ---------------------------------------------------------------
*        Inner loop for the fill-in in column  j.
*        This is usually not very expensive.
*        ---------------------------------------------------------------
  540    indr(lrow+1)  = 0
         last          = lc1 + lenj - 1
         ll            = 0

         do 560   lc   = lpivc1, lpivc2
            ll         = ll + 1
            if (markl(ll)  .eq. j    ) go to 560
            aij        = al(ll) * uj
            if (abs( aij ) .le. small) go to 560
            lenj       = lenj + 1
            last       = last + 1
            a(last)    = aij
            i          = indc(lc)
            indc(last) = i
            leni       = lenr(i)

*           Add the fill-in to row  i  if there is room.

            l          = locr(i) + leni
            if (indr(l) .eq. 0) then
               indr(l)    = j
               lenr(i)    = leni + 1
               if (lrow .lt. l) lrow = l
            else

*              Row i does not have room for the fill-in.
*              Increment ifill(ll) to count how often this has
*              happened to row i.  Also, add m to the row index
*              indc(last) in column j to mark it as a fill-in that is
*              still pending.
*
*              If this is the first pending fill-in for row i,
*              nfill includes the current length of row i
*              (since the whole row has to be moved later).
*
*              If this is the first pending fill-in for column j,
*              jfill(lu) records the current length of column j
*              (to shorten the search for pending fill-ins later).

               if (ifill(ll) .eq. 0) nfill     = nfill + leni + nspare
               if (jfill(lu) .eq. 0) jfill(lu) = lenj
               nfill      = nfill     + 1
               ifill(ll)  = ifill(ll) + 1
               indc(last) = m + i
            end if
  560    continue

         if ( atend ) lcol = last

*        End loop for column  j.  Store its final length.

  590    lenc(j) = lenj
  600 continue

*     Successful completion.

      lfirst = 0
      return

*     Interruption.  We have to come back in after the
*     column file is compressed.  Give lfirst a new value.
*     lu and nfill will retain their current values.

  900 lfirst = lr
      return

*     end of lu1gau
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1mar( m    , n     , lena  , maxmn,
     $                   lmax , maxcol, maxrow,
     $                   ibest, jbest , mbest ,
     $                   a    , indc  , indr  , ip   , iq,
     $                   lenc , lenr  , locc  , locr ,
     $                   iploc, iqloc )

      implicit           double precision (a-h,o-z)
      double precision   lmax      , a(lena)
      integer*4          indc(lena), indr(lena), ip(m)   , iq(n)   ,
     $                   lenc(n)   , lenr(m)   , iploc(n), iqloc(m)
      integer            locc(n)   , locr(m)

*     ------------------------------------------------------------------
*     lu1mar  uses a Markowitz criterion to select a pivot element
*     for the next stage of a sparse LU factorization,
*     subject to a stability criterion that bounds the elements of  L.
*
*     00 Jan 1986  Version documented in LUSOL paper:
*                  Gill, Murray, Saunders and Wright (1987),
*                  Maintaining LU factors of a general sparse matrix,
*                  Linear algebra and its applications 88/89, 239-270.
*
*     02 Feb 1989  Following Suhl and Aittoniemi (1987), the largest
*                  element in each column is now kept at the start of
*                  the column, i.e. in position locc(j) of a and indc.
*                  This should speed up the Markowitz searches.
*
*     26 Apr 1989  Both columns and rows searched during spars1 phase.
*                  Only columns searched during spars2 phase.
*                  maxtie replaced by maxcol and maxrow.
*
*     Systems Optimization Laboratory, Stanford University.
*     ------------------------------------------------------------------

      intrinsic          abs, max
      double precision   lbest
      double precision   zero,           gamma
      parameter        ( zero = 0.0d+0,  gamma  = 2.0d+0 )

*     gamma  is "gamma" in the tie-breaking rule TB4 in the LUSOL paper.

*     ------------------------------------------------------------------
*     Search cols of length nz = 1, then rows of length nz = 1,
*     then   cols of length nz = 2, then rows of length nz = 2, etc.
*     ------------------------------------------------------------------
      mbest  = m * n
      ncol   = 0
      nrow   = 0

      do 300 nz = 1, maxmn
         nz1    = nz - 1
         if (ncol  .ge. maxcol) go to 200
         if (mbest .le. nz1**2) go to 900
         if (nz    .gt. m     ) go to 200

*        ---------------------------------------------------------------
*        Search the set of columns of length  nz.
*        ---------------------------------------------------------------
         lq1    = iqloc(nz)
         lq2    = n
         if (nz .lt. m) lq2 = iqloc(nz + 1) - 1

         do 180 lq = lq1, lq2
            ncol   = ncol + 1
            j      = iq(lq)
            lc1    = locc(j)
            lc2    = lc1 + nz1
            amax   = abs( a(lc1) )

*           Test all aijs in this column.
*           amax is the largest element (the first in the column).
*           cmax is the largest multiplier if aij becomes pivot.

            do 160 lc = lc1, lc2
               i      = indc(lc)
               merit  = nz1 * (lenr(i) - 1)
               if (merit .gt. mbest) go to 160

*              aij  has a promising merit.
*              Apply the stability test.
*              We require  aij  to be sufficiently large compared to
*              all other nonzeros in column  j.  This is equivalent
*              to requiring cmax to be bounded by lmax.

               if (lc .eq. lc1) then

*                 This is the maximum element, amax.
*                 Find the biggest element in the rest of the column
*                 and hence get cmax.  We know cmax .le. 1, but
*                 we still want it exactly in order to break ties.

                  aij    = amax
                  cmax   = zero
                  do 140 l = lc1 + 1, lc2
                     cmax  = max( cmax, abs( a(l) ) )
  140             continue
                  cmax   = cmax / amax
               else

*                 aij is not the biggest element, so cmax .ge. 1.
*                 Bail out if cmax will be too big.

                  aij    = abs( a(lc) )
                  if (amax  .gt.  aij * lmax) go to 160
                  cmax   = amax / aij
               end if

*              aij  is big enough.  Its maximum multiplier is cmax.
*              Accept it immediately if its merit is less than mbest.

               if (merit .eq. mbest) then

*                 Break ties  (merit = mbest).
*                 In this version we minimize cmax
*                 but if it is already small we maximize the pivot.

                  if (lbest .le. gamma  .and.  cmax .le. gamma) then
                     if (abest .ge. aij ) go to 160
                  else
                     if (lbest .le. cmax) go to 160
                  end if
               end if

*              aij  is the best pivot so far.

               ibest  = i
               jbest  = j
               mbest  = merit
               abest  = aij
               lbest  = cmax
  160       continue

*           Finished with that column.

            if (ncol .ge. maxcol) go to 200
  180    continue

*        ---------------------------------------------------------------
*        Search the set of rows of length  nz.
*        ---------------------------------------------------------------
  200    if (nrow  .ge. maxrow) go to 290
         if (mbest .le. nz*nz1) go to 900
         if (nz    .gt. n     ) go to 300
         lp1    = iploc(nz)
         lp2    = m
         if (nz .lt. n) lp2 = iploc(nz + 1) - 1

         do 280 lp = lp1, lp2
            nrow   = nrow + 1
            i      = ip(lp)
            leni   = lenr(i)
            lr1    = locr(i)
            lr2    = lr1 + leni - 1

            do 260 lr = lr1, lr2
               j      = indr(lr)
               merit  = nz1 * (lenc(j) - 1)
               if (merit .gt. mbest) go to 260

*              aij  has a promising merit.
*              Find where  aij  is in column  j.

               lenj   = lenc(j)
               lc1    = locc(j)
               lc2    = lc1 + lenj - 1
               amax   = abs( a(lc1) )
               do 220 lc = lc1, lc2
                  if (indc(lc) .eq. i) go to 230
  220          continue

*              Apply the same stability test as above.

  230          if (lc .eq. lc1) then

*                 This is the maximum element, amax.
*                 Find the biggest element in the rest of the column
*                 and hence get cmax.  We know cmax .le. 1, but
*                 we still want it exactly in order to break ties.

                  aij    = amax
                  cmax   = zero
                  do 240 l = lc1 + 1, lc2
                     cmax  = max( cmax, abs( a(l) ) )
  240             continue
                  cmax   = cmax / amax
               else

*                 aij is not the biggest element, so cmax .ge. 1.
*                 Bail out if cmax will be too big.

                  aij    = abs( a(lc) )
                  if (amax  .gt.  aij * lmax) go to 260
                  cmax   = amax / aij
               end if

*              aij  is big enough.  Its maximum multiplier is cmax.
*              Accept it immediately if its merit is less than mbest.

               if (merit .eq. mbest) then

*                 Break ties as before (merit = mbest).

                  if (lbest .le. gamma  .and.  cmax .le. gamma) then
                     if (abest .ge. aij ) go to 260
                  else
                     if (lbest .le. cmax) go to 260
                  end if
               end if

*              aij  is the best pivot so far.

               ibest  = i
               jbest  = j
               mbest  = merit
               abest  = aij
               lbest  = cmax
               if (nz .eq. 1) go to 900
  260       continue

*           Finished with that row.

            if (nrow .ge. maxrow) go to 290
  280    continue

*        See if it's time to quit.

  290    if (nrow .ge. maxrow  .and.  ncol .ge. maxcol) go to 900
  300 continue

  900 return

*     end of lu1mar
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1pen( m     , melim , ncold , nspare,
     $                   lpivc1, lpivc2, lpivr1, lpivr2, lrow,
     $                   lenc  , lenr  , locc  , locr  ,
     $                   indc  , indr  , ifill , jfill )

      integer*4          indc(*)     , indr(*)     , lenc(*), lenr(*),
     $                   ifill(melim), jfill(ncold)
      integer            locc(*)     , locr(*)

*     ------------------------------------------------------------------
*     lu1pen deals with pending fill-in in the row file.
*     ifill(ll) says if a row involved in the new column of L
*               has to be updated.  If positive, it is the total
*               length of the final updated row.
*     jfill(lu) says if a column involved in the new row of U
*               contains any pending fill-ins.  If positive, it points
*               to the first fill-in in the column that has yet to be
*               added to the row file.
*     ------------------------------------------------------------------

*     Move the rows that have pending fill-in
*     to the end of the row file, leaving space for the fill-in.
*     Leave some spare room at the end of the current last row.

         ll     = 0

         do 650 lc  = lpivc1, lpivc2
            ll      = ll + 1
            if (ifill(ll) .eq. 0) go to 650

            do 620  l  = lrow + 1, lrow + nspare
               lrow    = l
               indr(l) = 0
  620       continue

            i       = indc(lc)
            lr1     = locr(i)
            lr2     = lr1 + lenr(i) - 1
            locr(i) = lrow + 1

            do 630 lr = lr1, lr2
               lrow       = lrow + 1
               indr(lrow) = indr(lr)
               indr(lr)   = 0
  630       continue

            lrow    = lrow + ifill(ll)
  650    continue

*        Scan all columns of  D  and insert the pending fill-in
*        into the row file.

         lu     = 1

         do 680 lr = lpivr1, lpivr2
            lu     = lu + 1
            if (jfill(lu) .eq. 0) go to 680
            j      = indr(lr)
            lc1    = locc(j) + jfill(lu) - 1
            lc2    = locc(j) + lenc(j)   - 1

            do 670 lc = lc1, lc2
               i      = indc(lc) - m
               if (i .gt. 0) then
                  indc(lc)   = i
                  last       = locr(i) + lenr(i)
                  indr(last) = j
                  lenr(i)    = lenr(i) + 1
               end if
  670       continue
  680    continue

*     end of lu1pen
      end

************************************************************************
*
*     file  lu1util  fortran
*
*     lu1max   lu1or1   lu1or2   lu1or3   lu1or4
*     lu1pq1   lu1pq2   lu1pq3   lu1rec
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1max( kol1, kol2, kol,
     $                   a   , indc, lenc, locc )

      implicit           double precision (a-h,o-z)
      double precision   a(*)
      integer*4          kol(kol2), indc(*), lenc(*)
      integer            locc(*)

*     ------------------------------------------------------------------
*     lu1max  moves the largest element in each of a set of columns
*     to the top of its column.
*     ------------------------------------------------------------------

      intrinsic          abs

      do 120  k = kol1, kol2
         j      = kol(k)
         lc1    = locc(j)

*        The next 10 lines are equivalent to
*        l      = idamax( lenc(j), a(lc1), 1 )  +  lc1 - 1
*>>>>>>>>
         lc2    = lc1 + lenc(j) - 1
         amax   = abs( a(lc1) )
         l      = lc1

         do 110 lc = lc1 + 1, lc2
            if (amax .lt. abs( a(lc) )) then
                amax   =  abs( a(lc) )
                l      =  lc
            end if
  110    continue
*>>>>>>>>
         if (l .gt. lc1) then
            amax      = a(l)
            a(l)      = a(lc1)
            a(lc1)    = amax
            i         = indc(l)
            indc(l)   = indc(lc1)
            indc(lc1) = i
         end if
  120 continue

*     end of lu1max
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1or1( m, n, nelem, small,
     $                   a, indc, indr, lenc, lenr,
     $                   amax, numnz, lerr, inform )

      implicit           double precision (a-h,o-z)
      double precision   a(nelem)
      integer*4          indc(nelem), indr(nelem)
      integer*4          lenc(n), lenr(m)

*     ------------------------------------------------------------------
*     lu1or1  organizes the elements of an  m by n  matrix  A  as
*     follows.  On entry, the parallel arrays   a, indc, indr,
*     contain  nelem  entries of the form     aij,    i,    j,
*     in any order.  nelem  must be positive.
*
*     Entries not larger than the input parameter  small  are treated as
*     zero and removed from   a, indc, indr.  The remaining entries are
*     defined to be nonzero.  numnz  returns the number of such nonzeros
*     and  amax  returns the magnitude of the largest nonzero.
*     The arrays  lenc, lenr  return the number of nonzeros in each
*     column and row of  A.
*
*     inform = 0  on exit, except  inform = 1  if any of the indices in
*     indc, indr  imply that the element  aij  lies outside the  m by n
*     dimensions of  A.
*
*     Version of February 1985.
*     ------------------------------------------------------------------
      do 10 i = 1, m
         lenr(i) = 0
   10 continue

      do 20 j = 1, n
         lenc(j) = 0
   20 continue

      amax   = 0.0d+0
      numnz  = nelem
      l      = nelem + 1

      do 100 ldummy = 1, nelem
         l      = l - 1
         if (abs( a(l) ) .gt. small) then
            i      = indc(l)
            j      = indr(l)
            amax   = max( amax, abs( a(l) ) )
            if (i .lt. 1  .or.  i .gt. m) go to 910
            if (j .lt. 1  .or.  j .gt. n) go to 910
            lenr(i) = lenr(i) + 1
            lenc(j) = lenc(j) + 1
         else

*           Replace a negligible element by last element.  Since
*           we are going backwards, we know the last element is ok.

            a(l)    = a(numnz)
            indc(l) = indc(numnz)
            indr(l) = indr(numnz)
            numnz   = numnz - 1
         end if
  100 continue

      inform = 0
      return

  910 lerr   = l
      inform = 1
      return

*     end of lu1or1
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1or2( n, numa, a, inum, jnum, len, loc )
      integer*4          inum(numa), jnum(numa), len(n)
      integer            loc(n)
      double precision   a(numa), ace, acep

*     ------------------------------------------------------------------
*     lu1or2  sorts a list of matrix elements  a(i,j)  into column
*     order, given  numa  entries  a(i,j),  i,  j  in the parallel
*     arrays  a, inum, jnum  respectively.  The matrix is assumed
*     to have  n  columns and an arbitrary number of rows.
*
*     On entry,  len(*)  must contain the length of each column.
*
*     On exit,  a(*) and inum(*)  are sorted,  jnum(*) = 0,  and
*     loc(j)  points to the start of column j.
*
*     lu1or2  is derived from  mc20ad,  a routine in the Harwell
*     Subroutine Library, author J. K. Reid.
*     ------------------------------------------------------------------

*     Set  loc(j)  to point to the beginning of column  j.

      l = 1
      do 150 j  = 1, n
         loc(j) = l
         l      = l + len(j)
  150 continue

*     Sort the elements into column order.
*     The algorithm is an in-place sort and is of order  numa.

      do 230 i = 1, numa
*        Establish the current entry.
         jce     = jnum(i)
         if (jce .eq. 0) go to 230
         ace     = a(i)
         ice     = inum(i)
         jnum(i) = 0

*        Chain from current entry.

         do 200 j = 1, numa

*           The current entry is not in the correct position.
*           Determine where to store it.

            l        = loc(jce)
            loc(jce) = loc(jce) + 1

*           Save the contents of that location.

            acep = a(l)
            icep = inum(l)
            jcep = jnum(l)

*           Store current entry.

            a(l)    = ace
            inum(l) = ice
            jnum(l) = 0

*           If next current entry needs to be processed,
*           copy it into current entry.

            if (jcep .eq. 0) go to 230
            ace = acep
            ice = icep
            jce = jcep
  200    continue
  230 continue

*     Reset loc(j) to point to the start of column j.

      ja = 1
      do 250 j  = 1, n
         jb     = loc(j)
         loc(j) = ja
         ja     = jb
  250 continue

*     end of lu1or2
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1or3( m, n, nelem,
     $                   indc, lenc, locc, iw,
     $                   lerr, inform )

      integer*4          indc(nelem), lenc(n), iw(m)
      integer            locc(n)

*     ------------------------------------------------------------------
*     lu1or3  looks for duplicate elements in an  m by n  matrix  A
*     defined by the column list  indc, lenc, locc.
*     iw  is used as a work vector of length  m.
*
*     Version of February 1985.
*     ------------------------------------------------------------------

      do 100 i = 1, m
         iw(i) = 0
  100 continue

      do 200 j = 1, n
         if (lenc(j) .gt. 0) then
            l1    = locc(j)
            l2    = l1 + lenc(j) - 1

            do 150 l = l1, l2
               i     = indc(l)
               if (iw(i) .eq. j) go to 910
               iw(i) = j
  150       continue
         end if
  200 continue

      inform = 0
      return

  910 lerr   = l
      inform = 1
      return

*     end of lu1or3
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1or4( m, n, nelem,
     $                   indc, indr, lenc, lenr, locc, locr )

      integer*4          indc(nelem), indr(nelem), lenc(n), lenr(m)
      integer            locc(n), locr(m)

*     ------------------------------------------------------------------
*     lu1or4     constructs a row list  indr, locr
*     from a corresponding column list  indc, locc,
*     given the lengths of both columns and rows in  lenc, lenr.
*     ------------------------------------------------------------------

*     Initialize  locr(i)  to point just beyond where the
*     last component of row  i  will be stored.

      l      = 1
      do 10 i = 1, m
         l       = l + lenr(i)
         locr(i) = l
   10 continue

*     By processing the columns backwards and decreasing  locr(i)
*     each time it is accessed, it will end up pointing to the
*     beginning of row  i  as required.

      l2     = nelem
      j      = n + 1

      do 40 jdummy = 1, n
         j  = j - 1
         if (lenc(j) .gt. 0) then
            l1 = locc(j)

            do 30 l = l1, l2
               i        = indc(l)
               lr       = locr(i) - 1
               locr(i)  = lr
               indr(lr) = j
   30       continue

            l2     = l1 - 1
         end if
   40 continue

*     end of lu1or4
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1pq1( m, n, len, iperm, loc, inv, num )

      integer*4          len(m), iperm(m), loc(n), inv(m), num(n)

*     ------------------------------------------------------------------
*     lu1pq1  constructs a permutation  iperm  from the array  len.
*
*     On entry:
*     len(i)  holds the number of nonzeros in the i-th row (say)
*             of an m by n matrix.
*     num(*)  can be anything (workspace).
*
*     On exit:
*     iperm   contains a list of row numbers in the order
*             rows of length 0,  rows of length 1,..., rows of length n.
*     loc(nz) points to the first row containing  nz  nonzeros,
*             nz = 1, n.
*     inv(i)  points to the position of row i within iperm(*).
*     ------------------------------------------------------------------

*     Count the number of rows of each length.

      nzero  = 0
      do 10  nz  = 1, n
         num(nz) = 0
         loc(nz) = 0
   10 continue

      do 20  i   = 1, m
         nz      = len(i)
         if (nz .eq. 0) then
            nzero   = nzero   + 1
         else
            num(nz) = num(nz) + 1
         end if
   20 continue

*     Set starting locations for each length.

      l      = nzero + 1
      do 60  nz  = 1, n
         loc(nz) = l
         l       = l + num(nz)
         num(nz) = 0
   60 continue

*     Form the list.

      nzero  = 0
      do 100  i   = 1, m
         nz       = len(i)
         if (nz .eq. 0) then
            nzero    = nzero + 1
            iperm(nzero) = i
         else
            l        = loc(nz) + num(nz)
            iperm(l) = i
            num(nz)  = num(nz) + 1
         end if
  100 continue

*     Define the inverse of iperm.

      do 120 l  = 1, m
         i      = iperm(l)
         inv(i) = l
  120 continue

*     end of lu1pq1
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1pq2( nzpiv, nzchng,
     $                   indr , lenold, lennew, iqloc, iq, iqinv )

      integer*4          indr(nzpiv), lenold(nzpiv), lennew(*),
     $                   iqloc(*)   , iq(*)        , iqinv(*)

*     ===============================================================
*     lu1pq2 frees the space occupied by the pivot row,
*     and updates the column permutation iq.
*
*     Also used to free the pivot column and update the row perm ip.
*
*     nzpiv   (input)    is the length of the pivot row (or column).
*     nzchng  (output)   is the net change in total nonzeros.
*
*     14 Apr 1989  First version.
*     ===============================================================

      nzchng = 0

      do 200  lr  = 1, nzpiv
         j        = indr(lr)
         indr(lr) = 0
         nz       = lenold(lr)
         nznew    = lennew(j)

         if (nz .ne. nznew) then
            l        = iqinv(j)
            nzchng   = nzchng + (nznew - nz)

*           l above is the position of column j in iq  (so j = iq(l)).

            if (nz .lt. nznew) then

*              Column  j  has to move towards the end of  iq.

  110          next        = nz + 1
               lnew        = iqloc(next) - 1
               if (lnew .ne. l) then
                  jnew        = iq(lnew)
                  iq(l)       = jnew
                  iqinv(jnew) = l
               end if
               l           = lnew
               iqloc(next) = lnew
               nz          = next
               if (nz .lt. nznew) go to 110
            else

*              Column  j  has to move towards the front of  iq.

  120          lnew        = iqloc(nz)
               if (lnew .ne. l) then
                  jnew        = iq(lnew)
                  iq(l)       = jnew
                  iqinv(jnew) = l
               end if
               l           = lnew
               iqloc(nz)   = lnew + 1
               nz          = nz   - 1
               if (nz .gt. nznew) go to 120
            end if

            iq(lnew) = j
            iqinv(j) = lnew
         end if
  200 continue

*     end of lu1pq2
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1pq3( n, len, iperm, nrank )

      integer*4          len(n), iperm(n)

*     ------------------------------------------------------------------
*     lu1pq3  looks at the permutation  iperm(*)  and moves any entries
*     to the end whose corresponding length  len(*)  is zero.
*     ------------------------------------------------------------------

      nrank  = n

      do 20 k = n, 1, -1
         i    = iperm(k)

         if (len(i) .eq. 0) then
            nrank  = nrank - 1
            if (k .le. nrank) then
               do 10 l = k, nrank
                  iperm(l) = iperm(l+1)
   10          continue
               iperm(nrank + 1) = i
            end if
         end if
   20 continue

*     end of lu1pq3
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu1rec( n, reals, luparm,
     $                   ltop, lena, a, ind, len, loc )

      logical            reals
      integer            luparm(30)
      double precision   a(lena)
      integer*4          ind(lena), len(n)
      integer            loc(n)

*     ------------------------------------------------------------------
*     lu1rec  recovers space in  ind(*)  and optionally  a(*),
*     by eliminating entries for which  ind(l)  is zero.
*     The elements of  ind(*)  must not be negative.
*
*     If  len(i)  is positive, entry  i  contains that many elements,
*     starting at  loc(i).  Otherwise, entry  i  is eliminated.
*     ------------------------------------------------------------------

      do 10 i = 1, n
         leni = len(i)
         if (leni .gt. 0) then
            l      = loc(i) + leni - 1
            len(i) = ind(l)
            ind(l) = - i
         end if
   10 continue

      k      = 0
      last   = 0

      do 20 l = 1, ltop
         if (ind(l) .ne. 0) then
            k      = k + 1
            i      = ind(l)
            ind(k) = i
            if (reals) a(k) = a(l)
            if (i .lt. 0) then

*              This is the end of entry  i.

               i      = - i
               ind(k) = len(i)
               loc(i) = last + 1
               len(i) = k    - last
               last   = k
            end if
         end if
   20 continue

      nout   = luparm(1)
      lprint = luparm(2)
      if (lprint .ge. 1) write(nout, 1000) ltop, k, reals
      luparm(26) = luparm(26) + 1
      ltop       = k
      return

 1000 format(' lu1rec.   File compressed from', i8, '   to', i8, l4)

*     end of lu1rec
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
*     file  lu6a    fortran
*
*     lu6chk   lu6sol
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu6chk( mode, m, n, w,
     $                   lena, luparm, parmlu,
     $                   a, indc, indr, ip, iq,
     $                   lenc, lenr, locc, locr,
     $                   inform )

      implicit           double precision (a-h,o-z)
      integer            luparm(30)
      double precision   parmlu(30), a(lena), w(n)
      integer*4          indc(lena), indr(lena), ip(m), iq(n)
      integer*4          lenc(n), lenr(m)
      integer            locc(n), locr(m)

*     ------------------------------------------------------------------
*     lu6chk  looks at the LU factorization  A = L*U.
*
*     If  mode = 1,  the important input parameters are
*
*                    utol1  = parmlu(4),
*                    utol2  = parmlu(5),
*
*     and the significant output parameters are
*
*                    inform = luparm(10),
*                    nsing  = luparm(11),
*                    jsing  = luparm(12),
*                    jumin  = luparm(19),
*                    elmax  = parmlu(11),
*                    umax   = parmlu(12).
*                    dumax  = parmlu(13),
*                    dumin  = parmlu(14),
*                    and      w(*).
*
*     elmax  and  umax   return the largest elements in  L  and  U.
*     dumax  and  dumin  return the largest and smallest diagonals of  U
*                        (excluding diagonals that are exactly zero).
*
*     In general,  w(j)  is set to the maximum absolute element in
*     the j-th column of  U.  However, if the corresponding diagonal
*     of  U  is small in absolute terms or relative to  w(j)
*     (as judged by the parameters  utol1, utol2  respectively),
*     then  w(j)  is changed to  - w(j).
*
*     Thus, if  w(j)  is not positive, the j-th column of  A
*     appears to be dependent on the other columns of  A.
*     The number of such columns, and the position of the last one,
*     are returned as  nsing  and  jsing.
*
*     Note that nrank is assumed to be set already, and is not altered.
*     typically, nsing will satisfy     nrank + nsing  = n,  but if
*     utol1 and utol2 are rather large, nsing may exceed n - nrank.
*
*     inform = 0  if  A  appears to have full column rank  (nsing = 0).
*     inform = 1  otherwise  (nsing .gt. 0).
*
*     Version of July 1987.
*     9 may 1988: f77 version.
*     ------------------------------------------------------------------

      intrinsic          abs, max, min
      parameter        ( zero = 0.0d+0 )

      nout   = luparm(1)
      lprint = luparm(2)
      nrank  = luparm(16)
      lenl   = luparm(23)
      utol1  = parmlu(4)
      utol2  = parmlu(5)
      inform = 0

*     ------------------------------------------------------------------
*     Find  elmax.
*     ------------------------------------------------------------------
      elmax  = zero

      do 120 l = lena + 1 - lenl, lena
         elmax = max( elmax, abs(a(l)) )
  120 continue

*     ------------------------------------------------------------------
*     Find  umax,  and set  w(j) = maximum element in j-th column of  U.
*     ------------------------------------------------------------------
      umax   = zero
      do 220 j = 1, n
         w(j)  = zero
  220 continue

      do 260 k = 1, nrank
         i     = ip(k)
         l1    = locr(i)
         l2    = l1 + lenr(i) - 1

         do 250 l = l1, l2
            j     = indr(l)
            aij   = abs( a(l) )
            w(j)  = max( w(j), aij )
            umax  = max( umax, aij )
  250    continue
  260 continue

*     ------------------------------------------------------------------
*     Negate  w(j)  if the corresponding diagonal of  U  is
*     too small in absolute terms or relative to the other elements
*     in the same column of  U.
*     Also find  dumax  and  dumin,  the extreme diagonals of  U.
*     ------------------------------------------------------------------
      nsing  = 0
      jsing  = 0
      jumin  = 0
      dumax  = zero
      dumin  = 1.0d+30

      do 320 k = 1, n
         j     = iq(k)
         if (k .gt. nrank) go to 310
            i      = ip(k)
            l1     = locr(i)
            diag   = abs( a(l1) )
            dumax  = max( dumax, diag )
            if (dumin .gt. diag) then
                dumin  =   diag
                jumin  =   j
            end if
         if (diag .gt. utol1  .and.  diag .gt. utol2 * w(j)) go to 320

  310       nsing  = nsing + 1
            jsing  = j
            w(j)   = - w(j)
  320 continue

      if (jumin .eq. 0) dumin = zero
      luparm(11) = nsing
      luparm(12) = jsing
      luparm(19) = jumin
      parmlu(11) = elmax
      parmlu(12) = umax
      parmlu(13) = dumax
      parmlu(14) = dumin
      if (nsing .gt. 0) then

*        The matrix has been judged singular.

         inform = 1
         ndefic = n - nrank
         if (lprint .ge. 0)
     $   write(nout, 1100) nrank, ndefic, nsing, jsing, dumax, dumin
      end if

*     Exit.

      luparm(10) = inform
      return

 1100 format(/ ' lu6chk  warning.  The matrix appears to be singular.'
     $       / '     nrank =', i7,       8x, 'rank of U'
     $       / ' n - nrank =', i7,       8x, 'rank deficiency'
     $       / '     nsing =', i7,       8x, 'singularities'
     $       / '     jsing =', i7,       8x, 'last singular column'
     $       / '     dumax =', 1p,e11.2, 4x, 'largest  triangular diag'
     $       / '     dumin =', 1p,e11.2, 4x, 'smallest triangular diag')
*     end of lu6chk
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu6sol( mode, m, n, v, w,
     $                   lena, luparm, parmlu,
     $                   a, indc, indr, ip, iq,
     $                   lenc, lenr, locc, locr,
     $                   inform )

      implicit           double precision (a-h,o-z)
      integer            luparm(30)
      double precision   parmlu(30), a(lena), v(m), w(n)
      integer*4          indc(lena), indr(lena), ip(m), iq(n)
      integer*4          lenc(n), lenr(m)
      integer            locc(n), locr(m)

*     ------------------------------------------------------------------
*     lu6sol  uses the factorization   A = L*U   as follows...
*
*     mode
*     ----
*      1    v  solves   L*v    = v(input).     w  is not touched.
*      2    v  solves   L(t)*v = v(input).     w  is not touched.
*      3    w  solves   U*w    = v.            v  is not altered.
*      4    v  solves   U(t)*v = w.            w  is destroyed.
*      5    w  solves   A*w    = v.            v  is altered as in 1.
*      6    v  solves   A(t)*v = w.            w  is destroyed.
*
*     if  mode .ge. 3,  v  and  w  must not be the same arrays.
*
*     ip(*), iq(*)      hold row and column numbers in pivotal order.
*     lenc(k)           is the length of the k-th column of initial  L.
*     lenr(i)           is the length of the i-th row of  U.
*     locc(*)           is not used.
*     locr(i)           is the start  of the i-th row of  U.
*
*     U is assumed to be in upper-trapezoidal form (nrank by n).
*     the first entry for each row is the diagonal element
*     (according to the permutations  ip, iq).  It is stored at
*     location  locr(i)  in  a(*), indr(*).
*
*     On exit,  inform = 0  except as follows.
*     if  mode .ge. 3  and if  U  (and hence  A)  is singular, then
*     inform = 1  if there is a nonzero residual in solving the system
*     involving  U.  parmlu(20)  returns the norm of the residual.
*
*     Version of July 1987.
*     9 May 1988: f77 version.
*     ------------------------------------------------------------------

      intrinsic          abs
      parameter        ( zero = 0.0d+0 )

      nrank  = luparm(16)
      numl0  = luparm(20)
      lenl0  = luparm(21)
      if (numl0 .eq. 0) lenl0 = 0
      lenl   = luparm(23)
      small  = parmlu(3)
      inform = 0
      nrank1 = nrank + 1
      resid  = zero
      go to (100, 200, 300, 400, 100, 400), mode

*     ==================================================================
*     mode = 1 or 5.    Solve  l * v(new) = v(old).
*     ==================================================================
  100 l1     = lena + 1

      do 120 k = 1, numl0
         len   = lenc(k)
         l     = l1
         l1    = l1 - len
         ipiv  = indr(l1)
         vpiv  = v(ipiv)
         if (abs( vpiv ) .le. small) go to 120

*        ***** The following loop could be coded specially.

         do 110 ldummy = 1, len
            l    = l - 1
            j    = indc(l)
            v(j) = v(j)  +  a(l) * vpiv
  110    continue
  120 continue

  150 l      = lena - lenl0 + 1
      numl   = lenl - lenl0

*     ***** The following loop could be coded specially.

      do 160 ldummy = 1, numl
         l     = l - 1
         i     = indr(l)
         if (abs( v(i) ) .le. small) go to 160
         j     = indc(l)
         v(j)  = v(j)  +  a(l) * v(i)
  160 continue

  190 if (mode .eq. 5) go to 300
      go to 900

*     ==================================================================
*     mode = 2.    Solve  L(transpose) * v(new) = v(old).
*     ==================================================================
  200 l1     = lena - lenl + 1
      l2     = lena - lenl0

*     ***** The following loop could be coded specially.

      do 220 l = l1, l2
         j     = indc(l)
         if (abs( v(j) ) .le. small) go to 220
         i     = indr(l)
         v(i)  = v(i)  +  a(l) * v(j)
  220 continue

      do 280 k = numl0, 1, -1
         len   = lenc(k)
         sum   = zero
         l1    = l2 + 1
         l2    = l2 + len

*        ***** The following loop could be coded specially.

         do 270 l = l1, l2
            j     = indc(l)
            sum   = sum  +  a(l) * v(j)
  270    continue

         ipiv    = indr(l1)
         v(ipiv) = v(ipiv) + sum
  280 continue

      go to 900

*     ==================================================================
*     mode = 3.    Solve   U * w  =  v.
*     ==================================================================

*     Find the first nonzero in  v(1) ... v(nrank),  counting backwards.

  300 do 310 klast = nrank, 1, -1
         i      = ip(klast)
         if (abs( v(i) ) .gt. small) go to 320
  310 continue

  320 do 330 k = klast + 1, n
         j     = iq(k)
         w(j)  = zero
  330 continue

*     Do the back-substitution, using rows  1  to  klast  of  U.

  340 do 380 k  = klast, 1, -1
         i      = ip(k)
         t      = v(i)
         l1     = locr(i)
         l2     = l1 + 1
         l3     = l1 + lenr(i) - 1

*        ***** The following loop could be coded specially.

         do 350 l = l2, l3
            j     = indr(l)
            t     = t  -  a(l) * w(j)
  350    continue

         j      = iq(k)
         if (abs( t ) .le. small) then
            w(j)   = zero
         else
            w(j)   = t / a(l1)
         end if
  380 continue

*     Compute residual for overdetermined systems.

      do 390 k = nrank1, m
         i     = ip(k)
         resid = resid  +  abs( v(i) )
  390 continue

      go to 900

*     ==================================================================
*     mode = 4 or 6.    Solve   U(transpose) * v  =  w.
*     ==================================================================
  400 do 410 k = nrank1, m
         i     = ip(k)
         v(i)  = zero
  410 continue

*     Do the forward-substitution, skipping columns of  U(transpose)
*     when the associated element of  w(*)  is negligible.

      do 480 k = 1, nrank
         i      = ip(k)
         j      = iq(k)
         t      = w(j)
         if (abs( t ) .le. small) then
            v(i) = zero
            go to 480
         end if

         l1     = locr(i)
         t      = t / a(l1)
         v(i)   = t
         l2     = l1 + lenr(i) - 1
         l1     = l1 + 1

*        ***** The following loop could be coded specially.

         do 450 l = l1, l2
            j     = indr(l)
            w(j)  = w(j)  -  t * a(l)
  450    continue
  480 continue

*     Compute residual for overdetermined systems.

      do 490 k = nrank1, n
         j     = iq(k)
         resid = resid  +  abs( w(j) )
  490 continue

      if (mode .eq. 6) go to 200

*     Exit.

  900 if (resid .gt. zero) inform = 1
      luparm(10) = inform
      parmlu(20) = resid
      return

*     end of lu6sol
      end

************************************************************************
*
*     file  lu7a    fortran
*
*     lu7add   lu7cyc   lu7elm   lu7for   lu7rnk   lu7zap
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu7add( m, n, jadd, v,
     $                   lena, luparm, parmlu,
     $                   lenl, lenu, lrow, nrank,
     $                   a, indr, ip, lenr, locr,
     $                   inform, klast, vnorm )

      implicit           double precision (a-h,o-z)
      integer            luparm(30)
      double precision   parmlu(30), a(lena), v(m)
      integer*4          indr(lena), ip(m), lenr(m)
      integer            locr(m)

*     ------------------------------------------------------------------
*     lu7add  inserts the first nrank elements of the vector v(*)
*     as column  jadd  of  U.  We assume that  U  does not yet have any
*     entries in this column.
*     Elements no larger than  parmlu(3)  are treated as zero.
*     klast  will be set so that the last row to be affected
*     (in pivotal order) is row  ip(klast).
*
*     09 May 1988: First f77 version.
*     ------------------------------------------------------------------

      intrinsic          abs
      parameter        ( zero = 0.0d+0 )

      small  = parmlu(3)
      vnorm  = zero
      klast  = 0

      do 200 k  = 1, nrank
         i      = ip(k)
         if (abs( v(i) ) .le. small) go to 200
         klast  = k
         vnorm  = vnorm  +  abs( v(i) )
         leni   = lenr(i)

*        Compress row file if necessary.

         minfre = leni + 1
         nfree  = lena - lenl - lrow
         if (nfree .lt. minfre) then
            call lu1rec( m, .true., luparm, lrow, lena,
     $                   a, indr, lenr, locr )
            nfree  = lena - lenl - lrow
            if (nfree .lt. minfre) go to 970
         end if

*        Move row  i  to the end of the row file,
*        unless it is already there.
*        No need to move if there is a gap already.

         if (leni .eq. 0) locr(i) = lrow + 1
         lr1    = locr(i)
         lr2    = lr1 + leni - 1
         if (lr2    .eq.   lrow) go to 150
         if (indr(lr2+1) .eq. 0) go to 180
         locr(i) = lrow + 1

         do 140 l = lr1, lr2
            lrow       = lrow + 1
            a(lrow)    = a(l)
            j          = indr(l)
            indr(l)    = 0
            indr(lrow) = j
  140    continue

  150    lr2     = lrow
         lrow    = lrow + 1

*        Add the element of  v.

  180    lr2       = lr2 + 1
         a(lr2)    = v(i)
         indr(lr2) = jadd
         lenr(i)   = leni + 1
         lenu      = lenu + 1
  200 continue

*     Normal exit.

      inform = 0
      go to 990

*     Not enough storage.

  970 inform = 7

  990 return

*     end of lu7add
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu7cyc( kfirst, klast, ip )

      integer*4          ip(klast)

*     ------------------------------------------------------------------
*     lu7cyc performs a cyclic permutation on the row or column ordering
*     stored in ip, moving entry kfirst down to klast.
*     If kfirst .ge. klast, lu7cyc should not be called.
*     Sometimes klast = 0 and nothing should happen.
*
*     09 May 1988: First f77 version.
*     ------------------------------------------------------------------

      if (kfirst .lt. klast) then
         ifirst = ip(kfirst)

         do 100 k = kfirst, klast - 1
            ip(k) = ip(k + 1)
  100    continue

         ip(klast) = ifirst
      end if

*     end of lu7cyc
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu7elm( m, n, jelm, v,
     $                   lena, luparm, parmlu,
     $                   lenl, lenu, lrow, nrank,
     $                   a, indc, indr, ip, iq, lenr, locc, locr,
     $                   inform, diag )

      implicit           double precision (a-h,o-z)
      integer            luparm(30)
      double precision   parmlu(30), a(lena), v(m)
      integer*4          indc(lena), indr(lena), ip(m), iq(n), lenr(m)
      integer            locc(n), locr(m)

*     ------------------------------------------------------------------
*     lu7elm  eliminates the subdiagonal elements of a vector  v(*),
*     where  L*v = y  for some vector y.
*     If  jelm > 0,  y  has just become column  jelm  of the matrix  A.
*     lu7elm  should not be called unless  m  is greater than  nrank.
*
*     09 May 1988: First f77 version.
*                  No longer calls lu7for at end.  lu8rpc, lu8mod do so.
*     ------------------------------------------------------------------

      intrinsic          abs
      parameter        ( zero = 0.0d+0 )

      small  = parmlu(3)
      nrank1 = nrank + 1
      diag   = zero

*     Compress row file if necessary.

      minfre = m - nrank
      nfree  = lena - lenl - lrow
      if (nfree .ge. minfre) go to 100
      call lu1rec( m, .true., luparm, lrow, lena, a, indr, lenr, locr )
      nfree  = lena - lenl - lrow
      if (nfree .lt. minfre) go to 970

*     Pack the subdiagonals of  v  into  L,  and find the largest.

  100 vmax   = zero
      kmax   = 0
      l      = lena - lenl + 1

      do 200 k = nrank1, m
         i       = ip(k)
         vi      = abs( v(i) )
         if (vi .le. small) go to 200
         l       = l - 1
         a(l)    = v(i)
         indc(l) = i
         if (vmax .ge. vi ) go to 200
         vmax    = vi
         kmax    = k
         lmax    = l
  200 continue

      if (kmax .eq. 0) go to 900

*     ------------------------------------------------------------------
*     Remove  vmax  by overwriting it with the last packed  v(i).
*     Then set the multipliers in  L  for the other elements.
*     ------------------------------------------------------------------

      imax       = ip(kmax)
      vmax       = a(lmax)
      a(lmax)    = a(l)
      indc(lmax) = indc(l)
      l1         = l + 1
      l2         = lena - lenl
      lenl       = lenl + (l2 - l)

      do 300 l = l1, l2
         a(l)    = - a(l) / vmax
         indr(l) =   imax
  300 continue

*     Move the row containing vmax to pivotal position nrank + 1.

      ip(kmax  ) = ip(nrank1)
      ip(nrank1) = imax
      diag       = vmax

*     ------------------------------------------------------------------
*     If jelm is positive, insert  vmax  into a new row of  U.
*     This is now the only subdiagonal element.
*     ------------------------------------------------------------------

      if (jelm .gt. 0) then
         lrow       = lrow + 1
         locr(imax) = lrow
         lenr(imax) = 1
         a(lrow)    = vmax
         indr(lrow) = jelm
      end if

      inform = 1
      go to 990

*     No elements to eliminate.

  900 inform = 0
      go to 990

*     Not enough storage.

  970 inform = 7

  990 return

*     end of lu7elm
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu7for( m, n, kfirst, klast,
     $                   lena, luparm, parmlu,
     $                   lenl, lenu, lrow,
     $                   a, indc, indr, ip, iq, lenr, locc, locr,
     $                   inform, diag )

      implicit           double precision (a-h,o-z)
      integer            luparm(30)
      double precision   parmlu(30), a(lena)
      integer*4          indc(lena), indr(lena), ip(m), iq(n), lenr(m)
      integer            locc(n), locr(m)

*     ------------------------------------------------------------------
*     lu7for  (forward sweep) updates the LU factorization  A = L*U
*     when row  iw = ip(klast)  of  U  is eliminated by a forward
*     sweep of stabilized row operations, leaving  ip * U * iq  upper
*     triangular.
*
*     The row permutation  ip  is updated to preserve stability and/or
*     sparsity.  The column permutation  iq  is not altered.
*
*     kfirst  is such that row  ip(kfirst)  is the first row involved
*     in eliminating row  iw.  (Hence,  kfirst  marks the first nonzero
*     in row  iw  in pivotal order.)  If  kfirst  is unknown it may be
*     input as  1.
*
*     klast   is such that row  ip(klast)  is the row being eliminated.
*     klast   is not altered.
*
*     lu7for  should be called only if  kfirst .le. klast.
*     If  kfirst = klast,  there are no nonzeros to eliminate, but the
*     diagonal element of row  ip(klast)  may need to be moved to the
*     front of the row.
*
*     On entry,  locc(*)  must be zero.
*
*     On exit:
*     inform = 0  if row iw has a nonzero diagonal (could be small).
*     inform = 1  if row iw has no diagonal.
*     inform = 7  if there is not enough storage to finish the update.
*
*     On a successful exit (inform le 1),  locc(*)  will again be zero.
*
*        Jan 1985: Final f66 version.
*     09 May 1988: First f77 version.
*     ------------------------------------------------------------------

      intrinsic          abs, min
      parameter        ( zero = 0.0d+0 )

      double precision   lmax
      logical            swappd

      lmax   = parmlu(2)
      small  = parmlu(3)
      uspace = parmlu(6)
      kbegin = kfirst
      swappd = .false.

*     We come back here from below if a row interchange is performed.

  100 iw     = ip(klast)
      lenw   = lenr(iw)
      if (lenw   .eq.   0  ) go to 910
      lw1    = locr(iw)
      lw2    = lw1 + lenw - 1
      jfirst = iq(kbegin)
      if (kbegin .ge. klast) go to 700

*     Make sure there is room at the end of the row file
*     in case row  iw  is moved there and fills in completely.

      minfre = n + 1
      nfree  = lena - lenl - lrow
      if (nfree .lt. minfre) then
         call lu1rec( m, .true., luparm, lrow, lena,
     $                a, indr, lenr, locr )
         lw1    = locr(iw)
         lw2    = lw1 + lenw - 1
         nfree  = lena - lenl - lrow
         if (nfree .lt. minfre) go to 970
      end if

*     Set markers on row  iw.

      do 120 l = lw1, lw2
         j       = indr(l)
         locc(j) = l
  120 continue


*     ==================================================================
*     Main elimination loop.
*     ==================================================================
      kstart = kbegin
      kstop  = min( klast, n )

      do 500 k  = kstart, kstop
         jfirst = iq(k)
         lfirst = locc(jfirst)
         if (lfirst .eq. 0) go to 490

*        Row  iw  has its first element in column  jfirst.

         wj     = a(lfirst)
         if (k .eq. klast) go to 490

*        ---------------------------------------------------------------
*        We are about to use the first element of row  iv
*               to eliminate the first element of row  iw.
*        However, we may wish to interchange the rows instead,
*        to preserve stability and/or sparsity.
*        ---------------------------------------------------------------
         iv     = ip(k)
         lenv   = lenr(iv)
         lv1    = locr(iv)
         vj     = zero
         if (lenv      .eq.   0   ) go to 150
         if (indr(lv1) .ne. jfirst) go to 150
         vj     = a(lv1)
         if (            swappd               ) go to 200
         if (lmax * abs( wj )  .lt.  abs( vj )) go to 200
         if (lmax * abs( vj )  .lt.  abs( wj )) go to 150
         if (            lenv  .le.  lenw     ) go to 200

*        ---------------------------------------------------------------
*        Interchange rows  iv  and  iw.
*        ---------------------------------------------------------------
  150    ip(klast) = iv
         ip(k)     = iw
         kbegin    = k
         swappd    = .true.
         go to 600

*        ---------------------------------------------------------------
*        Delete the eliminated element from row  iw
*        by overwriting it with the last element.
*        ---------------------------------------------------------------
  200    a(lfirst)    = a(lw2)
         jlast        = indr(lw2)
         indr(lfirst) = jlast
         indr(lw2)    = 0
         locc(jlast)  = lfirst
         locc(jfirst) = 0
         lenw         = lenw - 1
         lenu         = lenu - 1
         if (lrow .eq. lw2) lrow = lrow - 1
         lw2          = lw2  - 1

*        ---------------------------------------------------------------
*        Form the multiplier and store it in the  L  file.
*        ---------------------------------------------------------------
         if (abs( wj ) .le. small) go to 490
         amult   = - wj / vj
         l       = lena - lenl
         a(l)    = amult
         indr(l) = iv
         indc(l) = iw
         lenl    = lenl + 1

*        ---------------------------------------------------------------
*        Add the appropriate multiple of row  iv  to row  iw.
*        We use two different inner loops.  The first one is for the
*        case where row  iw  is not at the end of storage.
*        ---------------------------------------------------------------
         if (lenv .eq. 1) go to 490
         lv2    = lv1 + 1
         lv3    = lv1 + lenv - 1
         if (lw2 .eq. lrow) go to 400

*        ...............................................................
*        This inner loop will be interrupted only if
*        fill-in occurs enough to bump into the next row.
*        ...............................................................
         do 350 lv = lv2, lv3
            jv     = indr(lv)
            lw     = locc(jv)
            if (lw .gt. 0) then

*              No fill-in.

               a(lw)  = a(lw)  +  amult * a(lv)
               if (abs( a(lw) ) .le. small) then

*                 Delete small computed element.

                  a(lw)     = a(lw2)
                  j         = indr(lw2)
                  indr(lw)  = j
                  indr(lw2) = 0
                  locc(j)   = lw
                  locc(jv)  = 0
                  lenu      = lenu - 1
                  lenw      = lenw - 1
                  lw2       = lw2  - 1
               end if
            else

*              Row  iw  doesn't have an element in column  jv  yet
*              so there is a fill-in.

               if (indr(lw2+1) .ne. 0) go to 360
               lenu      = lenu + 1
               lenw      = lenw + 1
               lw2       = lw2  + 1
               a(lw2)    = amult * a(lv)
               indr(lw2) = jv
               locc(jv)  = lw2
            end if
  350    continue

         go to 490

*        Fill-in interrupted the previous loop.
*        Move row  iw  to the end of the row file.

  360    lv2      = lv
         locr(iw) = lrow + 1

         do 370 l = lw1, lw2
            lrow       = lrow + 1
            a(lrow)    = a(l)
            j          = indr(l)
            indr(l)    = 0
            indr(lrow) = j
            locc(j)    = lrow
  370    continue

         lw1    = locr(iw)
         lw2    = lrow

*        ...............................................................
*        Inner loop with row  iw  at the end of storage.
*        ...............................................................
  400    do 450 lv = lv2, lv3
            jv     = indr(lv)
            lw     = locc(jv)
            if (lw .gt. 0) then

*              No fill-in.

               a(lw)  = a(lw)  +  amult * a(lv)
               if (abs( a(lw) ) .le. small) then

*                 Delete small computed element.

                  a(lw)     = a(lw2)
                  j         = indr(lw2)
                  indr(lw)  = j
                  indr(lw2) = 0
                  locc(j)   = lw
                  locc(jv)  = 0
                  lenu      = lenu - 1
                  lenw      = lenw - 1
                  lw2       = lw2  - 1
               end if
            else

*              Row  iw  doesn't have an element in column  jv  yet
*              so there is a fill-in.

               lenu      = lenu + 1
               lenw      = lenw + 1
               lw2       = lw2  + 1
               a(lw2)    = amult * a(lv)
               indr(lw2) = jv
               locc(jv)  = lw2
            end if
  450    continue

         lrow   = lw2

*        The  k-th  element of row  iw  has been processed.
*        Reset  swappd  before looking at the next element.

  490    swappd = .false.
  500 continue

*     ==================================================================
*     End of main elimination loop.
*     ==================================================================

*     Cancel markers on row  iw.

  600 lenr(iw) = lenw
      if (lenw .eq. 0) go to 910
      do 620 l = lw1, lw2
         j       = indr(l)
         locc(j) = 0
  620 continue

*     Move the diagonal element to the front of row  iw.
*     At this stage,  lenw gt 0  and  klast le n.

  700 do 720 l = lw1, lw2
         ldiag = l
         if (indr(l) .eq. jfirst) go to 730
  720 continue
      go to 910

  730 diag        = a(ldiag)
      a(ldiag)    = a(lw1)
      a(lw1)      = diag
      indr(ldiag) = indr(lw1)
      indr(lw1)   = jfirst

*     If an interchange is needed, repeat from the beginning with the
*     new row  iw,  knowing that the opposite interchange cannot occur.

      if (swappd) go to 100
      inform = 0
      go to 950

*     Singular.

  910 diag   = zero
      inform = 1

*     Force a compression if the file for  U  is much longer than the
*     no. of nonzeros in  U  (i.e. if  lrow  is much bigger than  lenu).
*     This should prevent memory fragmentation when there is far more
*     memory than necessary  (i.e. when  lena  is huge).

  950 limit  = uspace * lenu + m + n + 1000
      if (lrow .gt. limit) then
         call lu1rec( m, .true., luparm, lrow, lena,
     $                a, indr, lenr, locr )
      end if
      go to 990

*     Not enough storage.

  970 inform = 7

*     Exit.

  990 return

*     end of lu7for
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu7rnk( m, n, jsing,
     $                   lena, luparm, parmlu,
     $                   lenl, lenu, lrow, nrank,
     $                   a, indc, indr, ip, iq, lenr, locc, locr,
     $                   inform, diag )

      implicit           double precision (a-h,o-z)
      integer            luparm(30)
      double precision   parmlu(30), a(lena)
      integer*4          indc(lena), indr(lena), ip(m), iq(n), lenr(m)
      integer            locc(n), locr(m)

*     ------------------------------------------------------------------
*     lu7rnk (check rank) assumes U is currently nrank by n
*     and determines if row nrank contains an acceptable pivot.
*     If not, the row is deleted and nrank is decreased by 1.
*
*     jsing is an input parameter (not altered).  If jsing is positive,
*     column jsing has already been judged dependent.  A substitute
*     (if any) must be some other column.
*
*     -- Jul 1987: First version.
*     09 May 1988: First f77 version.
*     ------------------------------------------------------------------

      intrinsic          abs
      parameter        ( zero = 0.0d+0 )

      utol1    = parmlu(4)
      diag     = zero

*     Find umax, the largest element in row nrank.

      iw       = ip(nrank)
      lenw     = lenr(iw)
      if (lenw .eq. 0) go to 400
      l1       = locr(iw)
      l2       = l1 + lenw - 1
      umax     = zero
      lmax     = l1

      do 100 l = l1, l2
         if (umax .lt. abs( a(l) )) then
             umax   =  abs( a(l) )
             lmax   =  l
         end if
  100 continue

*     Find which column that guy is in (in pivotal order).
*     Interchange him with column nrank, then move him to be
*     the new diagonal at the front of row nrank.

      diag   = a(lmax)
      jmax   = indr(lmax)

      do 300 kmax = nrank, n
         if (iq(kmax) .eq. jmax) go to 320
  300 continue

  320 iq(kmax)  = iq(nrank)
      iq(nrank) = jmax
      a(lmax)   = a(l1)
      a(l1)     = diag
      indr(lmax)= indr(l1)
      indr(l1)  = jmax

*     See if the new diagonal is big enough.

      if (umax .le. utol1) go to 400
      if (jmax .eq. jsing) go to 400

*     ------------------------------------------------------------------
*     The rank stays the same.
*     ------------------------------------------------------------------
      inform = 0
      return

*     ------------------------------------------------------------------
*     The rank decreases by one.
*     ------------------------------------------------------------------
  400 inform = -1
      nrank  = nrank - 1
      if (lenw .gt. 0) then

*        Delete row nrank from U.

         lenu     = lenu - lenw
         lenr(iw) = 0
         do 420 l = l1, l2
            indr(l) = 0
  420    continue

         if (l2 .eq. lrow) then

*           This row was at the end of the data structure.
*           We have to reset lrow.
*           Preceding rows might already have been deleted, so we
*           have to be prepared to go all the way back to 1.

            do 450 l = 1, l2
               if (indr(lrow) .gt. 0) go to 900
               lrow  = lrow - 1
  450       continue
         end if
      end if

  900 return

*     end of lu7rnk
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu7zap( m, n, jzap, kzap,
     $                   lena, lenu, lrow, nrank,
     $                   a, indr, ip, iq, lenr, locr )

      implicit           double precision (a-h,o-z)
      double precision   a(lena)
      integer*4          indr(lena), ip(m), iq(n), lenr(m)
      integer            locr(m)

*     ------------------------------------------------------------------
*     lu7zap  eliminates all nonzeros in column  jzap  of  U.
*     It also sets  kzap  to the position of  jzap  in pivotal order.
*     Thus, on exit we have  iq(kzap) = jzap.
*
*     -- Jul 1987: nrank added.
*     10 May 1988: First f77 version.
*     ------------------------------------------------------------------

      do 100 k  = 1, nrank
         i      = ip(k)
         leni   = lenr(i)
         if (leni .eq. 0) go to 90
         lr1    = locr(i)
         lr2    = lr1 + leni - 1
         do 50 l = lr1, lr2
            if (indr(l) .eq. jzap) go to 60
   50    continue
         go to 90

*        Delete the old element.

   60    a(l)      = a(lr2)
         indr(l)   = indr(lr2)
         indr(lr2) = 0
         lenr(i)   = leni - 1
         lenu      = lenu - 1

*        Stop if we know there are no more rows containing  jzap.

   90    kzap   = k
         if (iq(k) .eq. jzap) go to 800
  100 continue

*     nrank must be smaller than n because we haven't found kzap yet.

      do 200 k = nrank+1, n
         kzap  = k
         if (iq(k) .eq. jzap) go to 800
  200 continue

*     See if we zapped the last element in the file.

  800 if (lrow .gt. 0) then
         if (indr(lrow) .eq. 0) lrow = lrow - 1
      end if

*     end of lu7zap
      end

************************************************************************
*
*     file  lu8a    fortran
*
*     lu8rpc
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine lu8rpc( mode1, mode2, m, n, jrep, v, w,
     $                   lena, luparm, parmlu,
     $                   a, indc, indr, ip, iq,
     $                   lenc, lenr, locc, locr,
     $                   inform, diag, vnorm )

      implicit           double precision(a-h,o-z)
      integer            luparm(30)
      double precision   parmlu(30), a(lena), v(m), w(n)
      integer*4          indc(lena), indr(lena), ip(m), iq(n)
      integer*4          lenc(n), lenr(m)
      integer            locc(n), locr(m)

*     ------------------------------------------------------------------
*     lu8rpc  updates the LU factorization  A = L*U  when column  jrep
*     is replaced by some vector  a(new).
*
*     lu8rpc  is an implementation of the Bartels-Golub update,
*     designed for the case where A is rectangular and/or singular.
*     L is a product of stabilized eliminations (m x m, nonsingular).
*     P U Q is upper trapezoidal (m x n, rank nrank).
*
*     If  mode1 = 0,  the old column is taken to be zero
*                     (so it does not have to be removed from  U).
*
*     If  mode1 = 1,  the old column need not have been zero.
*
*     If  mode2 = 0,  the new column is taken to be zero.
*                     v(*)  is not used or altered.
*
*     If  mode2 = 1,  v(*)  must contain the new column  a(new).
*                     On exit,  v(*)  will satisfy  L*v = a(new).
*
*     If  mode2 = 2,  v(*)  must satisfy  L*v = a(new).
*
*     The array  w(*)  is not used or altered.
*
*     On entry, all elements of  locc  are assumed to be zero.
*     On a successful exit (inform ne 7), this will again be true.
*
*     On exit:
*     inform = -1  if the rank of U decreased by 1.
*     inform =  0  if the rank of U stayed the same.
*     inform =  1  if the rank of U increased by 1.
*     inform =  7  if the update was not completed (lack of storage).
*     inform =  8  if jrep is not between 1 and n.
*
*     -- Jan 1985: Original F66 version.
*     -- Jul 1987: Modified to maintain U in trapezoidal form.
*     10 May 1988: First f77 version.
*     ------------------------------------------------------------------

      intrinsic          abs
      logical            singlr
      parameter        ( zero = 0.0d+0 )

      nout   = luparm(1)
      lprint = luparm(2)
      nrank  = luparm(16)
      lenl   = luparm(23)
      lenu   = luparm(24)
      lrow   = luparm(25)
      utol1  = parmlu(4)
      utol2  = parmlu(5)
      nrank0 = nrank
      diag   = zero
      vnorm  = zero
      if (jrep .lt. 1) go to 980
      if (jrep .gt. n) go to 980

*     ------------------------------------------------------------------
*     If mode1 = 0, there are no elements to be removed from  U
*     but we still have to set  krep  (using a backward loop).
*     Otherwise, use lu7zap to remove column  jrep  from  U
*     and set  krep  at the same time.
*     ------------------------------------------------------------------
      if (mode1 .eq. 0) then
         krep   = n + 1

   10    krep   = krep - 1
         if (iq(krep) .ne. jrep) go to 10
      else
         call lu7zap( m, n, jrep, krep,
     $                lena, lenu, lrow, nrank,
     $                a, indr, ip, iq, lenr, locr )
      end if

*     ------------------------------------------------------------------
*     Insert a new column of u and find klast.
*     ------------------------------------------------------------------

      if (mode2 .eq. 0) then
         klast  = 0
      else
         if (mode2 .eq. 1) then

*           Transform v = a(new) to satisfy  L*v = a(new).

            call lu6sol( 1, m, n, v, w, lena, luparm, parmlu,
     $                   a, indc, indr, ip, iq,
     $                   lenc, lenr, locc, locr, inform )
         end if

*        Insert into  U  any nonzeros in the top of  v.
*        row  ip(klast)  will contain the last nonzero in pivotal order.
*        Note that  klast  will be in the range  ( 0, nrank ).

         call lu7add( m, n, jrep, v,
     $                lena, luparm, parmlu,
     $                lenl, lenu, lrow, nrank,
     $                a, indr, ip, lenr, locr,
     $                inform, klast, vnorm )
         if (inform .eq. 7) go to 970
      end if

*     ------------------------------------------------------------------
*     In general, the new column causes U to look like this:
*
*                 krep        n                 krep  n
*
*                ....a.........          ..........a...
*                 .  a        .           .        a  .
*                  . a        .            .       a  .
*                   .a        .             .      a  .
*        P U Q =     a        .    or        .     a  .
*                    b.       .               .    a  .
*                    b .      .                .   a  .
*                    b  .     .                 .  a  .
*                    b   ......                  ..a...  nrank
*                    c                             c
*                    c                             c
*                    c                             c     m
*
*     klast points to the last nonzero "a" or "b".
*     klast = 0 means all "a" and "b" entries are zero.
*     ------------------------------------------------------------------

      if (mode2 .eq. 0) then
         if (krep .gt. nrank) go to 900
      else if (nrank .lt. m) then

*        Eliminate any "c"s (in either case).
*        Row nrank + 1 may end up containing one nonzero.

         call lu7elm( m, n, jrep, v,
     $                lena, luparm, parmlu,
     $                lenl, lenu, lrow, nrank,
     $                a, indc, indr, ip, iq, lenr, locc, locr,
     $                inform, diag )
         if (inform .eq. 7) go to 970

         if (inform .eq. 1) then

*           The nonzero is apparently significant.
*           Increase nrank by 1 and make klast point to the bottom.

            nrank = nrank + 1
            klast = nrank
         end if
      end if

      if (nrank .lt. n) then

*        The column rank is low.
*
*        In the first case, we want the new column to end up in
*        position nrank, so the trapezoidal columns will have a chance
*        later on (in lu7rnk) pivot in that position.
*
*        Otherwise the new column is not part of the triangle.  We
*        swap it into position nrank so we can judge it for singularity.
*        lu7rnk might choose some other trapezoidal column later.

         if (krep .lt. nrank) then
            klast     = nrank
         else
            iq(krep ) = iq(nrank)
            iq(nrank) = jrep
            krep      = nrank
         end if
      end if

*     ------------------------------------------------------------------
*     If krep .lt. klast, there are some "b"s to eliminate:
*
*                  krep
*
*                ....a.........
*                 .  a        .
*                  . a        .
*                   .a        .
*        P U Q =     a        .  krep
*                    b.       .
*                    b .      .
*                    b  .     .
*                    b   ......  nrank
*
*     If krep .eq. klast, there are no "b"s, but the last "a" still
*     has to be moved to the front of row krep (by lu7for).
*     ------------------------------------------------------------------

      if (krep .le. klast) then

*        Perform a cyclic permutation on the current pivotal order,
*        and eliminate the resulting row spike.  krep becomes klast.
*        The final diagonal (if any) will be correctly positioned at
*        the front of the new krep-th row.  nrank stays the same.

         call lu7cyc( krep, klast, ip )
         call lu7cyc( krep, klast, iq )

         call lu7for( m, n, krep, klast,
     $                lena, luparm, parmlu,
     $                lenl, lenu, lrow,
     $                a, indc, indr, ip, iq, lenr, locc, locr,
     $                inform, diag )
         if (inform .eq. 7) go to 970
         krep   = klast
      end if

*     ------------------------------------------------------------------
*     Test for singularity in column krep (where krep .le. nrank).
*     ------------------------------------------------------------------

      diag   = zero
      iw     = ip(krep)
      singlr = lenr(iw) .eq. 0

      if (.not. singlr) then
         l1     = locr(iw)
         j1     = indr(l1)
         singlr = j1 .ne. jrep

         if (.not. singlr) then
            diag   = a(l1)
            singlr = abs( diag ) .le. utol1          .or.
     $               abs( diag ) .le. utol2 * vnorm
         end if
      end if

      if ( singlr  .and.  krep .lt. nrank ) then

*        Perform cyclic permutations to move column jrep to the end.
*        Move the corresponding row to position nrank
*        then eliminate the resulting row spike.

         call lu7cyc( krep, nrank, ip )
         call lu7cyc( krep, n    , iq )

         call lu7for( m, n, krep, nrank,
     $                lena, luparm, parmlu,
     $                lenl, lenu, lrow,
     $                a, indc, indr, ip, iq, lenr, locc, locr,
     $                inform, diag )
         if (inform .eq. 7) go to 970
      end if

*     Find the best column to be in position nrank.
*     If singlr, it can't be the new column, jrep.
*     If nothing satisfactory exists, nrank will be decreased.

      if ( singlr  .or.  nrank .lt. n ) then
         jsing  = 0
         if ( singlr ) jsing = jrep

         call lu7rnk( m, n, jsing,
     $                lena, luparm, parmlu,
     $                lenl, lenu, lrow, nrank,
     $                a, indc, indr, ip, iq, lenr, locc, locr,
     $                inform, diag )
      end if

*     ------------------------------------------------------------------
*     Set inform for exit.
*     ------------------------------------------------------------------

  900 if (nrank .eq. nrank0) then
         inform =  0
      else if (nrank .lt. nrank0) then
         inform = -1
         if (nrank0 .eq. n) then
            if (lprint .ge. 0) write(nout, 1100) jrep, diag
         end if
      else
         inform =  1
      end if
      go to 990

*     Not enough storage.

  970 inform = 7
      if (lprint .ge. 0) write(nout, 1700) lena
      go to 990

*     jrep  is out of range.

  980 inform = 8
      if (lprint .ge. 0) write(nout, 1800) m, n, jrep

*     Exit.

  990 luparm(10) = inform
      luparm(15) = luparm(15) + 1
      luparm(16) = nrank
      luparm(23) = lenl
      luparm(24) = lenu
      luparm(25) = lrow
      return

 1100 format(/ ' lu8rpc  warning.  Singularity after replacing column.',
     $       '    jrep =', i8, '    diag =', 1p, e12.2 )
 1700 format(/ ' lu8rpc  error...  Insufficient storage.',
     $         '    lena =', i8)
 1800 format(/ ' lu8rpc  error...  jrep  is out of range.',
     $         '    m =', i8, '    n =', i8, '    jrep =', i8)

*     end of lu8rpc
      end
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*     File  mi30spec fortran
*
*     miopt    miopti   mioptr   m3char   m3dflt   m3key
*     opfile   oplook   opnumb   opscan   optokn   opuppr
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine miopt ( buffer, iprint, isumm, inform )

      implicit           double precision (a-h,o-z)
      character*(*)      buffer

*     ------------------------------------------------------------------
*     miopt  decodes the option contained in  buffer.
*
*     The buffer is output to file iprint, minus trailing blanks.
*     Error messages are output to files iprint and isumm.
*     buffer is echoed to iprint but normally not to isumm.
*     It is echoed to isumm before any error msg.
*
*     On entry,
*     iprint is the Print   file.  No output occurs if iprint .le 0.
*     isumm  is the Summary file.  No output occurs if isumm  .le 0.
*     inform is the number of errors so far.
*
*     On exit,
*     inform is the number of errors so far.
*
*     27 Nov 1991: First version.
*     ------------------------------------------------------------------

      character*16       key

      call m3key ( buffer, key, iprint, isumm, inform )

*     end of miopt
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine miopti( buffer, ivalue, iprint, isumm, inform )

      implicit           double precision (a-h,o-z)
      character*(*)      buffer
      integer            ivalue

*     ------------------------------------------------------------------
*     miopti decodes the option contained in  buffer // ivalue.
*     The parameters other than ivalue are as in miopt.
*
*     27 Nov 1991: First version.
*     17 Jan 1992: buff72 needed to comply with f77 standard.
*     ------------------------------------------------------------------

      character*16       key
      character*72       buff72

      write(key, '(i16)') ivalue
      lenbuf = len(buffer)
      buff72 = buffer
      buff72(lenbuf+1:lenbuf+16) = key
      call m3key ( buff72, key, iprint, isumm, inform )

*     end of miopti
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine mioptr( buffer, rvalue, iprint, isumm, inform )

      implicit           double precision (a-h,o-z)
      character*(*)      buffer
      double precision   rvalue

*     ------------------------------------------------------------------
*     mioptr decodes the option contained in  buffer // rvalue.
*     The parameters other than rvalue are as in miopt.
*
*     27 Nov 1991: First version.
*     17 Jan 1992: buff72 needed to comply with f77 standard.
*     ------------------------------------------------------------------

      character*16       key
      character*72       buff72

      write(key, '(1p, e16.8)') rvalue
      lenbuf = len(buffer)
      buff72 = buffer
      buff72(lenbuf+1:lenbuf+16) = key
      call m3key ( buff72, key, iprint, isumm, inform )

*     end of mioptr
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3char( lname, iname )

      character*4        lname
      integer            iname

*     ------------------------------------------------------------------
*     m3char copies lname into iname.
*     lname contains character data in a4 format.
*     ------------------------------------------------------------------

      read (lname, '(a4)') iname

*     end of m3char
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3dflt( mode )

      implicit           double precision (a-h,o-z)

*     ------------------------------------------------------------------
*     If mode = 1, m3dflt sets default values for most of the parameters
*                  that can be altered by opfile via the OPTIONS file.
*     If mode = 2, the parameter values are checked and possibly changed
*                  to reasonable values.
*     If mode = 3  and iprint > 0 and iparm(3) > 0, the parameters are
*                  printed.  (In the OPTIONS file,  Suppress parameters
*                  sets iparm(3) = 0.)
*
*     26 Apr 1992: mode 3 added.
*     ------------------------------------------------------------------

      logical            conv,restrt
      logical            alone, AMPL, GAMS, MINT, page1, page2
      common    /m1env / alone, AMPL, GAMS, MINT, page1, page2
      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      parameter        ( ntime = 5 )
      common    /m1tim / tlast(ntime), tsum(ntime), numt(ntime), ltime
      common    /m1word/ nwordr,nwordi,nwordh
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m2len / mrows,mcols,melms
      common    /m2lu4 / parmlu(30),luparm(30)
      common    /m2mapz/ maxw,maxz
      common    /m2parm/ dparm(30),iparm(30)
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5len / maxr  ,maxs  ,mbs   ,nn    ,nn0   ,nr    ,nx
      common    /m5freq/ kchk,kinv,ksav,klog,ksumm,i1freq,i2freq,msoln
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5prc / nparpr,nmulpr,kprc,newsb
      common    /m5step/ featol, tolx0,tolinc,kdegen,ndegen,
     $                   itnfix, nfix(2)
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m7cg1 / cgbeta,itncg,msgcg,modcg,restrt
      common    /m7cg2 / lcg1,lcg2,lcg3,lcg4,modtcg,nitncg,nsubsp
      common    /m7conv/ etash,etarg,lvltol,nfail,conv(4)
      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /m8al2 / radius,rhsmod,modpen,modrhs
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2
      common    /m8veri/ jverif(4),lverif(2)
      common    /cyclcm/ cnvtol,jnew,materr,maxcy,nephnt,nphant,nprint

      intrinsic          max, min, sqrt

      parameter         (idummy =  -11111,  rdummy = -11111.0d+0,
     $                   zero   =  0.0d+0,  one    =      1.0d+0)

      logical            lincon, linear, nlncon, nonlin, SYSTEM

      character*4        lblank, lmax, lmin, id(6)
      data               lblank /'    '/,
     $                   lmax   /'Max '/,
     $                   lmin   /'Min '/,
     $                   id     /'No  ', 'Yes ', 'Part',
     $                           'Full', ' Den', 'Spar'/

*     SYSTEM means that MINOS is being used "behind the scenes"
*     by some other system such as GAMS or AMPL or ASCEND.
*     Certain defaults should then be different.

      SYSTEM = GAMS .or. AMPL

*     Set some local machine-dependent constants.

      c3     = max( 1.0d-3, eps4 )
      c4     = max( 1.0d-4, eps3 )
      c5     = max( 1.0d-5, eps2 )
      c6     = max( 1.0d-6, eps2 )
      c7     = max( 1.0d-7, eps2 )

*     Set the Reduced-Gradient tolerances.

      xtol(1) = 0.1
      xtol(2) = c6
      ftol(1) = xtol(1)*0.1
      ftol(2) = xtol(2)**2
      gtol(1) = c3
      gtol(2) = c7
      gdummy  = -111111.0d+0

      if (mode .eq. 1) then
*        ---------------------------------------------------------------
*        mode = 1.      Set parameters to default or dummy values.
*     
*        For some keywords like LAGRANGIAN, the Options file doesn't
*        allow the user to set illegal values.  Sensible defaults can
*        therefore be set here, and the final values don't need to be
*        checked later.
*     
*        Some of the options with numerical values do need to be checked
*        and sometimes we need to know if the user set values or not.
*        Such options are set to dummy values here.
*        ---------------------------------------------------------------
*     
*        Options needed by GAMS.
*        Things like file numbers have to be initialized.
*        Genuine options are mostly concerned with solving the problem.
      
         iback  =   0
         idump  =   0
         iload  =   0
         inewb  =   0
         imps   =   0
         insrt  =   0
         ioldb  =   0
         ipnch  =   0
         iprob  =   0
         ireprt =   0
         isoln  =   0
      
         idebug =   0
         itnlim = idummy
         kchk   = idummy
         kinv   = idummy
         klog   = idummy
         ksav   = idummy
         ksumm  = idummy
         kdegen = idummy
         lderiv =   3
         lprint =   0
         prnt0  = .true.
         prnt1  = .false.
         summ0  = .true.
         summ1  = .false.
         lscale = idummy
         ltime  = 3
         lverif(1) =  0
         lverif(2) = -2
         maxr   = idummy
         maxs   = idummy
         minimz =   1
         msoln  =   2
         ncom   = idummy
         nlag   =   1
         nmajor =  50
         nminor =  40
         nmulpr =   1
         nparpr = idummy
      
         do 60 i = 1, 4
            jverif(i) = -1
   60    continue
      
*        iparm(1) = Crash option
*        iparm(2) = Linesearch debug -- starting iteration
*        iparm(3) = Suppress parameters
*        iparm(4) = Scale print
*        iparm(5) = Start assigned nonlinears  (GAMS only)
      
         iparm(1) = 3
         iparm(2) = 9999999
         iparm(3) = 1
         iparm(4) = 0
         iparm(5) = 2
      
*        dparm(1) = Unbounded objective
*        dparm(2) = Unbounded step
*        dparm(3) = Function precision
*        dparm(4) = Major damping parameter
*        dparm(5) = Crash tolerance
*        dparm(6) = Minor damping parameter
*        dparm(7) = Penalty parameter
*        dparm(8) = LU swap tolerance
      
         do 10 i = 1, 8
            dparm(i) = rdummy
   10    continue
      
         difint(1) = rdummy
         difint(2) = rdummy
         etarg     = rdummy
         etash     = rdummy
         parmlu(1) = rdummy
         parmlu(2) = rdummy
         parmlu(4) = rdummy
         parmlu(5) = rdummy
         parmlu(8) = rdummy
         penpar    = rdummy
         radius    = rdummy
         rowtol    = rdummy
         scltol    = rdummy
         toldj(3)  = rdummy
         tolpiv    = rdummy
         tolrow    = c4
         tolx      = rdummy
         wtobj     = rdummy

*        Certain defaults should be different within modeling systems.

         if (SYSTEM) then
            lverif(1) = -1
            msoln     = 0
         end if

*        Options not used by GAMS.
*        These are mostly to do with the MPS file.
      
         aijtol    = 1.0d-10
         bstruc(1) = zero
         bstruc(2) = plinfy
         cnvtol    = zero
      
         i1freq =   0
         i2freq =   0
         mrows  =   0
         mcols  =   0
         melms  =   0
         mer    =  10
         mlst   =   0
         modcg  =  -1
         modtcg =   1
         nitncg =   0
         nden   =   1
         nncon  =   0
         nnjac  =   0
         nnobj  =   0
         nprob  =   0
         maxcy  =   1
         nephnt =   0
         nphant =   0
         nprint =   1
      
         call m3char( lblank, iblank )
         do 20 i = 1, 2
            name(i) = iblank
            mobj(i) = iblank
            mrhs(i) = iblank
            mrng(i) = iblank
            mbnd(i) = iblank
   20    continue
      
      else if (mode .eq. 2) then
*        ---------------------------------------------------------------
*        mode = 2.   Check parameters and assign default values.
*        ---------------------------------------------------------------
      
*        Options and variables needed by GAMS.
      
         if (nncon  .eq. 0) nnjac = 0
         if (nnjac  .eq. 0) nncon = 0
         nn     = max( nnjac, nnobj )
         nncon0 = max( nncon,   1   )
         lincon = nncon .eq. 0
         linear = nn    .eq. 0
         nlncon = nncon .gt. 0
         nonlin = nn    .gt. 0

*        Set unspecified frequencies or silly values to defaults.
      
         if (kchk   .eq. idummy) kchk   =     60
         if (kinv   .le.    0  ) kinv   =    100
         if (klog   .eq. idummy) klog   =    100
         if (ksav   .eq. idummy) ksav   =    100
         if (ksumm  .eq. idummy) ksumm  =    100
         if (kdegen .eq. idummy) kdegen =  10000
      
*        Sometimes, frequency 0 means "almost never".
      
         if (kchk   .le. 0) kchk   = 99999999
         if (klog   .le. 0) klog   = 99999999
         if (ksav   .le. 0) ksav   = 99999999
         if (ksumm  .le. 0) ksumm  = 99999999
         if (kdegen .le. 0) kdegen = 99999999
      
         prnt0  = lprint .eq. 0
         prnt1  = lprint .gt. 0
         if (iprint .le. 0) then
            prnt0  = .false.
            prnt1  = .false.
         end if
         if (isumm  .le. 0) then
            summ0  = .false.
            summ1  = .false.
         end if
      
*        Check Hessian dimension maxr and Superbasics limit maxs.
      
         if ( nonlin ) then
            if (maxr   .gt. 0  .and.  maxs .lt. 0) maxs = maxr
            if (maxs   .gt. 0  .and.  maxr .lt. 0) maxr = maxs
            if (.not. GAMS) then
               if (maxs .lt. 0) maxs = 50
               if (maxr .lt. 0) maxr = 50
            end if
         end if
         if (.not. GAMS) then
            if (maxs   .le. 0   ) maxs = 1
            if (maxr   .lt. 0   ) maxr = 0
            if (maxs   .lt. maxr) maxs = maxr
         end if
         maxr   = min( maxr, maxs )
      
*        Check other options.
      
         if (lscale .lt. 0) then
                        lscale = 2
            if (nonlin) lscale = 1
         end if
         if (ncom   .lt. 0) then
                        ncom   = 1
            if (nlncon) ncom   = 0
         end if
         if (nparpr .le. 0) then
                        nparpr = 10
            if (nonlin) nparpr = 1
         end if
      
*        If the Optimality tolerance was not specified, it should not
*        be smaller than the sqrt of the Function precision.
      
         if (toldj(3) .le. zero) then
            toldj(3) = c6
            if (dparm(3) .gt. zero) toldj(3) = sqrt( dparm(3) )
         end if
      
*        See the list of dparms above.
      
         if (dparm(1) .le. zero) dparm(1) = plinfy
         if (dparm(2) .le. zero) dparm(2) = 1.0d+10
         if (dparm(3) .le. zero) dparm(3) = eps0
         if (dparm(4) .le. zero) dparm(4) = 2.0
         if (dparm(6) .le. zero) dparm(6) = 2.0
         if (penpar   .lt. zero) penpar   = one
         dparm(7) = penpar
         if (dparm(8) .le. zero) dparm(8) = eps4
      
         if (dparm(5) .lt. zero  .or.  dparm(5) .ge. one) dparm(5)= 0.1
         if (etarg    .le. zero  .or.  etarg    .gt. one) etarg   = 0.5
         if (etash    .lt. zero  .or.  etash    .gt. one) etash   = 0.1
      
         if (difint(1).le. zero) difint(1) = sqrt( dparm(3) )
         if (difint(2).le. zero) difint(2) = dparm(3) ** 0.333333
         if (parmlu(1).lt. one ) parmlu(1) = 100.0
         if (parmlu(2).lt. one ) parmlu(2) = 10.0
         if (parmlu(4).le. zero) parmlu(4) = eps1
         if (parmlu(5).le. zero) parmlu(5) = eps1
         if (parmlu(8).le. zero) parmlu(8) = 0.5
         if (radius   .le. eps2) radius    = 0.01
         if (rowtol   .le. eps ) rowtol    = c6
         if (scltol   .le. zero) scltol    = 0.90
         if (tolpiv   .le. zero) tolpiv    = eps1
         if (tolx     .le. zero) tolx      = c6
         if (wtobj    .lt. zero) wtobj     = zero
      
*        Check the Start and Stop column numbers for gradient checking.
      
         if (jverif(1) .lt. 0) jverif(1) = 1
         if (jverif(2) .lt. 0) jverif(2) = nnobj
         if (jverif(3) .lt. 0) jverif(3) = 1
         if (jverif(4) .lt. 0) jverif(4) = nnjac
      

*        Options not used by GAMS.
      
         if (bstruc(1) .gt. bstruc(2)) then
             t          =   bstruc(1)
             bstruc(1)  =   bstruc(2)
             bstruc(2)  =   t
         end if
         if (minimz .gt. 0     ) call m3char( lmin, minmax )
         if (minimz .le. 0     ) call m3char( lmax, minmax )
         if (iback  .eq. inewb ) iback  = 0
         if (mrows  .le. 0     ) mrows  = 100
         if (mcols  .le. 0     ) mcols  = 3*mrows
         if (melms  .le. 0     ) melms  = 5*mcols
         if (itnlim .lt. 0     ) itnlim = 3*mrows + 10*nn
         maxs   = min( maxs, mcols + mrows + 1 )

      else if (mode .eq. 3  .and.  iprint .gt. 0) then
*        ---------------------------------------------------------------
*        mode = 3.  Print the parameters if
*                   Print level > 0 and iparm(3) > 0.
*        ---------------------------------------------------------------
         if (prnt1  .and.  iparm(3) .gt. 0) then
            call m1page( 1 )
            write(iprint, 1000)

            if (SYSTEM) then
*              relax
            else
               write(iprint, 2000) mrows , mlst  , bstruc(1),
     $                             mcols , mer   , bstruc(2),
     $                             melms , nephnt, aijtol
               write(iprint, 2100) imps  , ioldb , iread ,
     $                             isoln , inewb , iprint,
     $                             insrt , iback , ispecs,
     $                             ipnch , iload , idump
            end if                      
         
            write(iprint, 2200) klog     , kchk     , ksav     ,
     $                          ksumm    , kinv     , kdegen
            write(iprint, 2300) lscale   , tolx     , itnlim   ,
     $                          scltol   , toldj(3) , nparpr   ,
     $                          iparm(1) , tolpiv   , nmulpr   ,
     $                          dparm(5) , wtobj
            write(iprint, 2400) nncon    , maxr     , dparm(3) ,
     $                          nnjac    , maxs     , difint(1),
     $                          nnobj    , etash    , difint(2),
     $                          nprob    , etarg    , lderiv   ,
     $                          dparm(1) , dparm(2) , lverif(1)
            write(iprint, 2450) id(4+nden),nmajor   , radius   ,
     $                          id(1+nlag),nminor   , rowtol   ,
     $                          penpar   , id(3+ncom),lprint   ,
     $                          dparm(4) , dparm(6)
            write(iprint, 2500) parmlu(1), maxw     , idebug   ,
     $                          parmlu(2), maxz     , iparm(2) ,
     $                          parmlu(4), eps,    nwordr,nwordi,nwordh,
     $                          dparm(8) , ltime
         end if
      end if
      return

 1000 format(' Parameters' / ' ----------')
 2000 format(' MPS INPUT DATA.'
     $/ ' Row limit..............', i10, 6x,
     $  ' List limit.............', i10, 6x,
     $  ' Lower bound default....', 1p,  e10.2
     $/ ' Column limit...........', i10, 6x,
     $  ' Error message limit....', i10, 6x,
     $  ' Upper bound default....', e10.2
     $/ ' Elements limit ........', i10, 6x,
     $  ' Phantom elements.......', i10, 6x,
     $  ' Aij tolerance..........', e10.2)
 2100 format(/ ' FILES.'
     $/ ' MPS file ..............', i10, 6x,
     $  ' Old basis file ........', i10, 6x,
     $  ' (Card reader)..........', i10
     $/ ' Solution file..........', i10, 6x,
     $  ' New basis file ........', i10, 6x,
     $  ' (Printer)..............', i10
     $/ ' Insert file............', i10, 6x,
     $  ' Backup basis file......', i10, 6x,
     $  ' (Specs file)...........', i10
     $/ ' Punch file.............', i10, 6x,
     $  ' Load file..............', i10, 6x,
     $  ' Dump file..............', i10)
 2200 format(/ ' FREQUENCIES.'
     $/ ' Log frequency..........', i10, 6x,
     $  ' Check row error........', i10, 6x,
     $  ' Save new basis map.....', i10
     $/ ' Summary frequency......', i10, 6x,
     $  ' Factorize basis........', i10, 6x,
     $  ' Expand frequency.......', i10)
 2300 format(/ ' LP PARAMETERS.'
     $/ ' Scale option...........', i10,       6x,
     $  ' Feasibility tolerance..', 1p, e10.2, 6x,
     $  ' Iteration limit........', i10
     $/ ' Scale tolerance........', 0p, f10.3, 6x,
     $  ' Optimality tolerance...', e10.2,     6x,
     $  ' Partial  price.........', i10
     $/ ' Crash option...........', i10,       6x,
     $  ' Pivot tolerance........', 1p, e10.2, 6x,
     $  ' Multiple price.........', i10 
     $/ ' Crash tolerance........', 0p, f10.3, 6x,
     $  ' Weight on objective....', e10.2)
 2400 format(/ ' NONLINEAR PROBLEMS.'
     $/ ' Nonlinear constraints..', i10,       6x,
     $  ' Hessian dimension......', i10,       6x,
     $  ' Function precision.....', 1p, e10.2
     $/ ' Nonlinear Jacobian vars', i10,       6x,
     $  ' Superbasics limit......', i10,       6x,
     $  ' Difference interval....', e10.2
     $/ ' Nonlinear objectiv vars', i10,       6x,
     $  ' Linesearch tolerance...', 0p, f10.5, 6x,
     $  ' Central difference int.', 1p, e10.2
     $/ ' Problem number.........', i10,       6x,
     $  ' Subspace tolerance.....', 0p, f10.5, 6x,
     $  ' Derivative level.......', i10
     $/ ' Unbounded objective val', 1p, e10.2, 6x,
     $  ' Unbounded step size....', e10.2,     6x,
     $  ' Verify level...........', i10)
 2450 format(/ ' AUGMENTED LAGRANGIAN.'
     $/ ' Jacobian...............', 4x, a4, 'se', 6x,
     $  ' Major iterations limit.', i10, 6x,
     $  ' Radius of convergence..', 1p, e10.2
     $/ ' Lagrangian.............', 7x, a3,    6x,
     $  ' Minor iterations limit.', i10,       6x,
     $  ' Row tolerance..........', e10.2
     $/ ' Penalty parameter......', e10.2,     6x,
     $  ' Completion.............', 6x, a4,    6x,
     $  ' Print level..(JFLXB)...', i10
     $/ ' Major damping parameter', e10.2,     6x,
     $  ' Minor damping parameter', e10.2)
 2500 format(/ ' MISCELLANEOUS.'
     $/ ' LU factor tolerance....', f10.2,     6x,
     $  ' Workspace (user).......', i10,       6x,
     $  ' Debug level............', i10
     $/ ' LU update tolerance....', f10.2,     6x,
     $  ' Workspace (total)......', i10,       6x,
     $  ' Linesearch debug after.', i10
     $/ ' LU singularity tol.....', 1p, e10.2, 6x,
     $  ' eps (machine precision)', e10.2,     6x,
     $  ' nwordr, nwordi, nwordh.', 1x, 3i3
     $/ ' LU swap tolerance......', e10.2,     6x,
     $  ' Timing level...........', i10)

*     end of m3dflt
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3key ( buffer, key, lprnt, lsumm, inform )

      implicit           double precision (a-h,o-z)
      character*(*)      buffer
      character*16       key

************************************************************************
*     m3key  decodes the option contained in  buffer  in order to set
*     a parameter value in the relevant common block.
*
*     The buffer is output to file iprint, minus trailing blanks.
*     Error messages are output to files iprint and isumm.
*     buffer is echoed to iprint but normally not to isumm.
*     It is echoed to isumm before any error msg.
*
*     On entry,
*     lprnt  is iprint as given to opfile.
*     lsumm  is isumm  as given to opfile.
*     inform is the number of errors so far.
*
*     On exit,
*     key    is the first keyword contained in buffer.
*     inform is the number of errors so far.
*
*     m3key  calls opnumb and the subprograms
*                 lookup, scannr, tokens, upcase
*     (now called oplook, opscan, optokn, opuppr)
*     supplied by Sterling Software, Palo Alto, California.
*
*     Systems Optimization Laboratory, Stanford University.
*     22 Mar 1988: First MINOS version.
*     10 Nov 1991: inform added to count errors and allow for no output.
************************************************************************

      parameter         (mxparm = 30)
      logical            conv,restrt
      logical            alone, AMPL, GAMS, MINT, page1, page2
      common    /m1env / alone, AMPL, GAMS, MINT, page1, page2
      common    /m1file/ iread,iprint,isumm
      parameter        ( ntime = 5 )
      common    /m1tim / tlast(ntime), tsum(ntime), numt(ntime), ltime
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m2len / mrows,mcols,melms
      common    /m2lu4 / parmlu(30),luparm(30)
      common    /m2mapz/ maxw,maxz
      common    /m2parm/ dparm(30),iparm(30)
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5len / maxr  ,maxs  ,mbs   ,nn    ,nn0   ,nr    ,nx
      common    /m5freq/ kchk,kinv,ksav,klog,ksumm,i1freq,i2freq,msoln
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5prc / nparpr,nmulpr,kprc,newsb
      common    /m5step/ featol, tolx0,tolinc,kdegen,ndegen,
     $                   itnfix, nfix(2)
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m7cg1 / cgbeta,itncg,msgcg,modcg,restrt
      common    /m7cg2 / lcg1,lcg2,lcg3,lcg4,modtcg,nitncg,nsubsp
      common    /m7conv/ etash,etarg,lvltol,nfail,conv(4)
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /m8al2 / radius,rhsmod,modpen,modrhs
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2
      common    /m8veri/ jverif(4),lverif(2)
      common    /cyclcm/ cnvtol,jnew,materr,maxcy,nephnt,nphant,nprint
*-----------------------------------------------------------------------

      external           opnumb
      intrinsic          abs
      logical            more  , number, opnumb, sorted

      parameter         (     maxkey = 38,  maxtie = 40,   maxtok = 10)
      character*16       keys(maxkey), ties(maxtie), token(maxtok)

*     Next 2 lines not needed by GAMS
      parameter         (     mxmkey = 35)
      character*16       mkey(mxmkey)

      character*16       key2, k2save, value

      parameter         (idummy =    -11111,   rdummy = -11111.0d+0,
     $                   sorted =    .true.,
     $                   maxint = 100000000,   zero   =  0.0d+0    )

*     maxint above should be larger than any expected integer value.

*     GAMS recognizes the following keywords.

      data   keys
     $ / 'CHECK           ', 'COMPLETION      ', 'CRASH           ',
     $   'DEBUG           ', 'DEFAULTS        ', 'EXPAND          ',
     $   'FACTORIZATION   ', 'FEASIBILITY     ', 'HESSIAN         ',
     $   'IPARM           ',
     $   'ITERATIONS      ', 'ITERS:ITERATIONS', 'ITNS :ITERATIONS',
     $   'LAGRANGIAN      ', 'LINESEARCH      ', 'LOG             ',
     $   'LU              ', 'MAJOR           ', 'MINOR           ',
     $   'MULTIPLE        ', 'OPTIMALITY      ', 'PARTIAL         ',
     $   'PENALTY         ', 'PIVOT           ', 'PRINT           ',
     $   'RADIUS          ', 'ROWS            ', 'RPARM           ',
     $   'SCALE           ', 'SOLUTION        ', 'START           ',
     $   'SUBSPACE        ', 'SUMMARY         ', 'SUPERBASICS     ',
     $   'TIMING          ', 'UNBOUNDED       ',
     $   'VERIFY          ', 'WEIGHT          '/

      data   ties
     $ / '(TOTAL)         ', '(USER)          ',
     $   'ALL             ', 'BASIC           ', 'COLUMNS         ',
     $   'CONSTRAINTS     ', 'DAMPING         ', 'DEBUG           ',
     $   'DENSE           ', 'DENSITY         ',
     $   'ELEMENTS        ', 'ELIGIBLE        ',
     $   'FACTORIZATION   ', 'FILE            ', 'FREQUENCY       ',
     $   'FULL            ', 'GRADIENTS       ',
     $   'ITERATIONS      ', 'ITERS:ITERATIONS', 'ITNS :ITERATIONS',
     $   'JACOBIAN        ', 'LEVEL           ', 'LIMIT           ',
     $   'LINEAR          ',
     $   'NO              ', 'NONBASIC        ', 'NONLINEAR       ',
     $   'OBJECTIVE       ', 'OPTION          ', 'PARTIAL         ',
     $   'PRINT           ', 'SINGULARITY     ', 'SPARSE          ',
     $   'STEP            ', 'SUPERBASIC      ', 'SWAP            ',
     $   'TOLERANCE       ', 'UPDATE          ',
     $   'VARIABLES       ', 'YES             '/

*    More keywords for MINOS --- not needed by GAMS.

      data   mkey
     $ / 'AIJ             ', 'BACKUP          ', 'BOUNDS          ',
     $   'CENTRAL         ', 'COEFFICIENTS    ', 'COLUMNS         ',
     $   'CYCLE           ', 'DERIVATIVE      ', 'DIFFERENCE      ',
     $   'DUMP            ', 'ELEMENTS        ', 'ERROR           ',
     $   'FUNCTION        ', 'INSERT          ', 'JACOBIAN        ',
     $   'LIST            ', 'LOAD            ', 'LOWER           ',
     $   'MAXIMIZE        ', 'MINIMIZE        ', 'MPS             ',
     $   'NEW             ', 'NONLINEAR       ', 'OBJECTIVE       ',
     $   'OLD             ', 'PHANTOM         ', 'PROBLEM         ',
     $   'PUNCH           ', 'RANGES          ', 'REPORT          ',
     $   'RHS             ', 'SAVE            ',
     $   'STOP            ', 'UPPER           ', 'WORKSPACE       '/
*-----------------------------------------------------------------------

*     iparm(1) = Crash option
*     iparm(2) = Linesearch debug -- starting iteration
*     iparm(3) = Suppress parameters
*     iparm(4) = Scale print
*     iparm(5) = Start assigned nonlinears

*     dparm(1) = Unbounded objective
*     dparm(2) = Unbounded step
*     dparm(3) = Function precision
*     dparm(4) = Major damping parameter
*     dparm(5) = Crash tolerance
*     dparm(6) = Minor damping parameter
*     dparm(7) = Penalty parameter

*     Set lenbuf = length of buffer without trailing blanks.
*     Echo to the print file.

      lenbuf  = 1
      do 10 j = 1, len(buffer)
         if (buffer(j:j) .ne. ' ') lenbuf = j
   10 continue

      if (lprnt .gt. 0) then
         write(lprnt, '(6x, a)') buffer(1:lenbuf)
      end if

*     Set lenb = length of buffer without trailing comments.
*     Eliminate comments and empty lines.
*     A '*' appearing anywhere in buffer terminates the string.

      i      = index( buffer(1:lenbuf), '*' )
      if (i .eq. 0) then
         lenb = lenbuf
      else
         lenb = i - 1
      end if
      if (lenb .le. 0) then
         key = '*'
         go to 900
      end if

*     ------------------------------------------------------------------
*     Extract up to maxtok tokens from the record.
*     ntoken returns how many were actually found.
*     key, key2, are the first tokens if any, otherwise blank.
*     For some values of key (BOUNDS, OBJECTIVE, RANGES, RHS)
*     we have to save key2 before oplook gets a chance to alter it.
*     For example, if the data is     Objective = OBJ
*     oplook will change obj to objective.
*     ------------------------------------------------------------------
      ntoken = maxtok
      call optokn( buffer(1:lenb), ntoken, token )
      key    = token(1)
      key2   = token(2)
      k2save = key2

*     Certain keywords require no action.

      if (key .eq. '   ') go to 900
      if (key .eq. 'END') go to 900

*     Most keywords will have an associated integer or real value,
*     so look for it no matter what the keyword.

      i      = 1
      number = .false.

   50 if (i .lt. ntoken  .and.  .not. number) then
         i      = i + 1
         value  = token(i)
         number = opnumb( value )
         go to 50
      end if

      ivalue = 0
      rvalue = zero
      if ( number ) then
         read  (value, '(bn, e16.0)') rvalue
         if (abs(rvalue) .lt. maxint) ivalue = rvalue
      end if

*     Convert the keywords to their most fundamental form
*     (upper case, no abbreviations).
*     sorted says whether the dictionaries are in alphabetic order.
*     loci   says where the keywords are in the dictionaries.
*     loci = 0 signals that the keyword wasn't there.

      call oplook( maxkey, keys, sorted, key , loc1 )
      call oplook( maxtie, ties, sorted, key2, loc2 )

*     ------------------------------------------------------------------
*     Decide what to do about each keyword.
*     The second keyword (if any) might be needed to break ties.
*     Some seemingly redundant testing of more is used
*     to avoid compiler limits on the number of consecutive else if's.
*     ------------------------------------------------------------------
      more   = .true.
      if (more) then
         more   = .false.
         if      (key .eq. 'CHECK       ') then
            kchk   = ivalue
         else if (key .eq. 'COMPLETION  ') then
              if (key2.eq. 'PARTIAL     ') ncom   = 0
              if (key2.eq. 'FULL        ') ncom   = 1
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'CRASH       ') then
              if (key2.eq. 'OPTION      ') iparm(1) = ivalue
              if (key2.eq. 'TOLERANCE   ') dparm(5) = rvalue
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'DEBUG       ') then
            idebug = ivalue
         else if (key .eq. 'DEFAULTS    ') then
            call m3dflt( 1 )
         else if (key .eq. 'EXPAND      ') then
            kdegen = ivalue
         else if (key .eq. 'FACTORIZATION') then
            kinv   = ivalue
         else if (key .eq. 'FEASIBILITY ') then
            tolx   = rvalue
         else if (key .eq. 'HESSIAN     ') then
            maxr   = ivalue
         else
            more   = .true.
         end if
      end if

      if (more) then
         more   = .false.
         if      (key .eq. 'IPARM       ') then
*           Allow things like  Iparm 21 = 100  to set iparm(21) = 100
            key2   = token(3)
            if (ivalue .ge. 1  .and. ivalue .le. mxparm) then
               read (key2, '(bn, i16)') iparm(ivalue)
            else
               go to 880
            end if
         else if (key .eq. 'ITERATIONS  ') then
            itnlim = ivalue
         else if (key .eq. 'LAGRANGIAN  ') then
              if (key2.eq. 'YES         ') nlag   = 1
              if (key2.eq. 'NO          ') nlag   = 0
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'LINESEARCH  ') then
              if (key2.eq. 'TOLERANCE   ') etash    = rvalue
              if (key2.eq. 'DEBUG       ') iparm(2) = ivalue
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'LOG         ') then
            klog   = ivalue
         else if (key .eq. 'LU          ') then
              if (key2.eq. 'FACTORIZATION')parmlu(1) = rvalue
              if (key2.eq. 'UPDATE      ') parmlu(2) = rvalue
              if (key2.eq. 'DENSITY     ') parmlu(8) = rvalue
              if (key2.eq. 'SINGULARITY ') then
                 parmlu(4) = rvalue
                 parmlu(5) = rvalue
              end if
              if (key2.eq. 'SWAP        ') dparm(8)  = rvalue
              if (loc2.eq.  0            ) go to 820
         else
            more   = .true.
         end if
      end if

      if (more) then
         more   = .false.
         if      (key .eq. 'MAJOR       ') then
              if (key2.eq. 'DAMPING     ') dparm(4) = rvalue
              if (key2.eq. 'ITERATIONS  ') nmajor   = ivalue
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'MINOR       ') then
              if (key2.eq. 'DAMPING     ') dparm(6) = rvalue
              if (key2.eq. 'ITERATIONS  ') nminor   = ivalue
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'MULTIPLE    ') then
            nmulpr = ivalue
         else if (key .eq. 'OPTIMALITY  ') then
            toldj(3) = rvalue
         else if (key .eq. 'PARTIAL     ') then
            nparpr = ivalue
         else if (key .eq. 'PENALTY     ') then
            dparm(7) = rvalue
            penpar   = rvalue
         else if (key .eq. 'PIVOT       ') then
            tolpiv = rvalue
         else if (key .eq. 'PRINT       ') then
              if (key2.eq. 'FILE        ') iprint   = ivalue
              if (key2.eq. 'FREQUENCY   ') klog     = ivalue
              if (key2.eq. 'LEVEL       ') then
                 lprint   = ivalue
              end if
              if (loc2.eq.  0            ) go to 820
         else
            more   = .true.
         end if
      end if

      if (more) then
         more   = .false.
         if      (key .eq. 'RADIUS      ') then
            radius = rvalue
         else if (key .eq. 'ROWS        ') then
*             GAMS and AMPL should recognize Row tolerance
*             but not just          Rows
              if (key2.eq. 'TOLERANCE   ') then
                 rowtol = rvalue
              else
                 if ( .not. (GAMS .or. AMPL) ) mrows = ivalue
              end if
         else if (key .eq. 'RPARM       ') then
*           Allow things like  Rparm 21 = 2  to set dparm(21) = 2.0
            key2   = token(3)
            if (ivalue .ge. 1  .and. ivalue .le. mxparm) then
               read (key2, '(bn, e16.0)') dparm(ivalue)
            else
               go to 880
            end if
         else
            more   = .true.
         end if
      end if

      if (more) then
         more   = .false.
         if      (key .eq. 'SCALE       ') then
              if (key2.eq. 'OPTION      ') then
                 lscale = ivalue
              else
                 if (rvalue .gt. zero       ) scltol   = rvalue
                 if (key2.eq. 'PRINT       ') iparm(4) =  1
                 if (key2.eq. 'ALL         ') lscale   =  2
                 if (key2.eq. 'NONLINEAR   ') lscale   =  2
                 if (key2.eq. 'LINEAR      ') lscale   =  1
                 if (key2.eq. 'NO          ') lscale   =  0
                 if (key2.eq. 'YES         ') lscale   = -1
                 if (key2.eq. '            ') lscale   = -1
                 if (key2.eq. '            ') loc2     =  1
                 if (loc2.eq.  0            ) go to 820
              end if
         else
            more   = .true.
         end if
      end if

      if (more) then
         more   = .false.
         if      (key .eq. 'SOLUTION    ') then
              if (key2.eq. 'YES         ') msoln = 2
              if (key2.eq. 'NO          ') msoln = 0
              if (key2.eq. 'FILE        ') isoln = ivalue
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'START       ') then
              key2   = token(4)
              call oplook( maxtie, ties, sorted, key2, loc2 )
              if (key2.eq. 'SUPERBASIC  ') iparm(5) = 2
              if (key2.eq. 'BASIC       ') iparm(5) = 3
              if (key2.eq. 'NONBASIC    ') iparm(5) = 4
              if (key2.eq. 'ELIGIBLE    ') iparm(5) = 1
              if (key2.eq. 'OBJECTIVE   ') jverif(1) = ivalue
              if (key2.eq. 'CONSTRAINTS ') jverif(3) = ivalue
              if (loc2.eq.  0            ) go to 840
         else
            more   = .true.
         end if
      end if

      if (more) then
         more   = .false.
         if      (key .eq. 'SUBSPACE    ') then
            etarg  = rvalue
         else if (key .eq. 'SUPERBASICS ') then
            maxs   = ivalue
         else if (key .eq. 'SUMMARY     ') then
              if (key2.eq. 'FILE        ') isumm    = ivalue
              if (key2.eq. 'FREQUENCY   ') ksumm    = ivalue
              if (key2.eq. 'LEVEL       ') then
                 summ0    = ivalue .eq. 0
                 summ1    = ivalue .gt. 0
              end if
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'TIMING      ') then
            ltime    = ivalue
         else if (key .eq. 'UNBOUNDED   ') then
              if (key2.eq. 'OBJECTIVE   ') dparm(1) = rvalue
              if (key2.eq. 'STEP        ') dparm(2) = rvalue
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'VERIFY      ') then
              if (key2.eq. 'OBJECTIVE   ') lverif(1) = 1
              if (key2.eq. 'CONSTRAINTS ') lverif(1) = 2
              if (key2.eq. 'GRADIENTS   ') lverif(1) = 3
              if (key2.eq. 'YES         ') lverif(1) = 3
              if (key2.eq. 'NO          ') lverif(1) = 0
              if (key2.eq. 'LEVEL       ') lverif(1) = ivalue
              if (key2.eq. '            ') lverif(1) = 3
              if (key2.eq. '            ') loc2      = 1
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'WEIGHT      ') then
            wtobj  = rvalue
         else
            more   = .true.
         end if
      end if

      if (.not. more) go to 900
      if (      GAMS) go to 800

*     ------------------------------------------------------------------
*     The following keywords are not recognized by GAMS.
*     ------------------------------------------------------------------
      call oplook( mxmkey, mkey, sorted, key , loc1 )

      if (more) then
         more   = .false.
         if      (key .eq. 'AIJ         ') then
            aijtol = rvalue
         else if (key .eq. 'BACKUP      ') then
            iback  = ivalue
         else if (key .eq. 'BOUNDS      ') then
           call m3char( k2save(1:4), mbnd(1) )
           call m3char( k2save(5:8), mbnd(2) )
         else if (key .eq. 'CENTRAL     ') then
            difint(2) = rvalue
         else if (key .eq. 'COEFFICIENTS') then
            melms  = ivalue
         else if (key .eq. 'COLUMNS     ') then
            mcols  = ivalue
         else if (key .eq. 'CYCLE       ') then
              if (key2.eq. 'LIMIT       ') maxcy  = ivalue
              if (key2.eq. 'PRINT       ') nprint = ivalue
              if (key2.eq. 'TOLERANCE   ') cnvtol = rvalue
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'DERIVATIVE  ') then
            lderiv = ivalue
         else
            more   = .true.
         end if
      end if

      if (more) then
         more   = .false.
         if      (key .eq. 'DIFFERENCE  ') then
            difint(1) = rvalue
         else if (key .eq. 'DUMP        ') then
            idump  = ivalue
         else if (key .eq. 'ELEMENTS    ') then
            melms  = ivalue
         else if (key .eq. 'ERROR       ') then
            mer    = ivalue
         else if (key .eq. 'FUNCTION    ') then
            dparm(3) = rvalue
         else if (key .eq. 'INSERT      ') then
            insrt  = ivalue
         else if (key .eq. 'JACOBIAN    ') then
              if (key2.eq. 'DENSE       ') nden = 1
              if (key2.eq. 'SPARSE      ') nden = 2
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'LIST        ') then
            mlst   = ivalue
         else if (key .eq. 'LOAD        ') then
            iload  = ivalue
         else if (key .eq. 'LOWER       ') then
            bstruc(1) = rvalue
         else if (key .eq. 'MAXIMIZE    ') then
            minimz = -1
         else if (key .eq. 'MINIMIZE    ') then
            minimz =  1
         else if (key .eq. 'MPS         ') then
            imps   = ivalue
         else
            more   = .true.
         end if
      end if

      if (more) then
         more   = .false.
         if      (key .eq. 'NEW         ') then
            inewb  = ivalue
         else if (key .eq. 'NONLINEAR   ') then
              if (key2.eq. 'CONSTRAINTS ') nncon = ivalue
              if (key2.eq. 'OBJECTIVE   ') nnobj = ivalue
              if (key2.eq. 'JACOBIAN    ') nnjac = ivalue
              if (key2.eq. 'VARIABLES   ') then
                  nnobj = ivalue
                  nnjac = ivalue
              end if
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'OBJECTIVE   ') then
            call m3char( k2save(1:4), mobj(1) )
            call m3char( k2save(5:8), mobj(2) )
         else if (key .eq. 'OLD         ') then
            ioldb  = ivalue
         else if (key .eq. 'PHANTOM     ') then
              if (key2.eq. 'COLUMNS     ') nphant = ivalue
              if (key2.eq. 'ELEMENTS    ') nephnt = ivalue
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'PROBLEM     ') then
              if (key2.eq. 'FILE        ') ifile  = ivalue
              if (key2.ne. 'FILE        ') nprob  = ivalue
         else if (key .eq. 'PUNCH       ') then
            ipnch  = ivalue
         else if (key .eq. 'RANGES      ') then
            call m3char( k2save(1:4), mrng(1) )
            call m3char( k2save(5:8), mrng(2) )
         else if (key .eq. 'REPORT      ') then
            ireprt = ivalue
         else if (key .eq. 'RHS         ') then
            call m3char( k2save(1:4), mrhs(1) )
            call m3char( k2save(5:8), mrhs(2) )
         else
            more   = .true.
         end if
      end if

      if (more) then
         more   = .false.
         if      (key .eq. 'SAVE        ') then
            ksav   = ivalue
         else if (key .eq. 'STOP        ') then
              if (key2.eq. 'OBJECTIVE   ') jverif(2) = ivalue
              if (key2.eq. 'CONSTRAINTS ') jverif(4) = ivalue
              if (loc2.eq.  0            ) go to 820
         else if (key .eq. 'UPPER       ') then
            bstruc(2) = rvalue
         else if (key .eq. 'WORKSPACE   ') then
              if (key2.eq. '(USER)      ') maxw = ivalue
              if (key2.eq. '(TOTAL)     ') maxz = ivalue
              if (loc2.eq.  0            ) go to 820
         else
            more   = .true.
         end if
      end if

      if (.not. more) go to 900

*     ------------------------------------------------------------------
*     Error messages.
*     ------------------------------------------------------------------
  800 inform = inform + 1
      if (lprnt .gt. 0) then
         write (lprnt, 2300) key
      end if
      if (lsumm .gt. 0) then
         write (lsumm, '(1x, a )') buffer
         write (lsumm, 2300) key
      end if
      return

  820 inform = inform + 1
      if (lprnt .gt. 0) then
         write (lprnt, 2320) key2
      end if
      if (lsumm .gt. 0) then
         write (lsumm, '(1x, a )') buffer
         write (lsumm, 2320) key2
      end if
      return

  840 inform = inform + 1
      if (lprnt .gt. 0) then
         write (lprnt, 2340) key2
      end if
      if (lsumm .gt. 0) then
         write (lsumm, '(1x, a )') buffer
         write (lsumm, 2340) key2
      end if
      return

  880 inform = inform + 1
      if (lprnt .gt. 0) then
         write (lprnt, 2380) ivalue
      end if
      if (lsumm .gt. 0) then
         write (lsumm, '(1x, a )') buffer
         write (lsumm, 2380) ivalue
      end if

  900 return

 2300 format(' XXX  Keyword not recognized:         ', a)
 2320 format(' XXX  Second keyword not recognized:  ', a)
*2330 format(' XXX  Third  keyword not recognized:  ', a)
 2340 format(' XXX  Fourth keyword not recognized:  ', a)
 2380 format(' XXX  The parm subscript is out of range:', i10)

*     end of m3key
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine opfile( ncalls, ioptns, opkey,
     $                   title , iprint, isumm, inform )

      integer            ncalls, ioptns, iprint, isumm, inform
      character*(*)      title
      external           opkey

************************************************************************
*     opfile  reads the options file from unit  ioptns  and loads the
*     relevant options, using opkey to process each line.
*
*     On exit, inform says how many errors were encountered.
*
*     Systems Optimization Laboratory, Stanford University.
*     18-Dec-1985: Original version.
*     20-Mar-1988: First MINOS version -- title added, prnt deleted.
*     10 Nov 1991: Provision made for no output to iprint and isumm.
*     11 Nov 1991: opfile now calls m3dflt( 1 ) to initialize options
*                  only after an Options file has been found.
************************************************************************

      character*16       key   , token(1)
      character*72       buffer
      character*30       dashes
      data               dashes /'=============================='/

*     lprnt and lsumm and local copies of iprint and isumm
*     (which might get changed by m3key).

      lprnt  = iprint
      lsumm  = isumm
      inform = 0

*     Return if the unit number is out of range.

      if (ioptns .lt. 0  .or.  ioptns .gt. 99) then
         inform = 1
         return
      end if

*     ------------------------------------------------------------------
*     Look for  Begin, Endrun  or  Skip.
*     ------------------------------------------------------------------
      nread  = 0
   50    read (ioptns, '(a72)', end = 930) buffer
         nread = nread + 1
         nkey  = 1
         call optokn( buffer, nkey, token )
         key   = token(1)
         if (key .eq. 'ENDRUN') go to 940
         if (key .ne. 'BEGIN' ) then
            if (nread .eq. 1  .and.  key .ne. 'SKIP') then
               inform = inform + 1
               if (lprnt .gt. 0) write (lprnt, 2000) ioptns, buffer
               if (lsumm .gt. 0) write (lsumm, 2000) ioptns, buffer
            end if
            go to 50
         end if

*     ------------------------------------------------------------------
*     BEGIN found.
*     This is taken to be the first line of an OPTIONS file.
*     It is printed without the trailing blanks.
*     ------------------------------------------------------------------
      call m1page(1)
      do 10 j = 1, len(buffer)
         if (buffer(j:j) .ne. ' ') lenbuf = j
   10 continue

      if (lprnt .gt. 0) then
         write (lprnt, '(  9x, a)') ' ', dashes, title, dashes
         write (lprnt, '(/ 6x, a)') buffer(1:lenbuf)
      end if
      if (lsumm .gt. 0) then
         write (lsumm, '(  1x, a)') ' ', dashes, title, dashes
         write (lsumm, '(/ 1x, a)') buffer(1:lenbuf)
      end if

*     Set options to default values.

      call m3dflt( 1 )

*     ------------------------------------------------------------------
*     Read the rest of the file.
*     ------------------------------------------------------------------
*+    while (key .ne. 'END') loop
  100 if    (key .ne. 'END') then
         read  (ioptns, '(a72)', end = 920) buffer
         call opkey ( buffer, key, lprnt, lsumm, inform )
         go to 100
      end if
*+    end while

      return

  920 if (lprnt .gt. 0) write (lprnt, 2200) ioptns
      if (lsumm .gt. 0) write (lsumm, 2200) ioptns
      inform = 2
      return

  930 if (ncalls .le. 1) then
         if (lprnt .gt. 0) write (lprnt, 2300) ioptns
         if (lsumm .gt. 0) write (lsumm, 2300) ioptns
      else
         if (lprnt .gt. 0) write (lprnt, '(a)') ' Endrun'
         if (lsumm .gt. 0) write (lsumm, '(a)') ' Endrun'
      end if
      inform = 3
      return

  940 if (lprnt .gt. 0) write (lprnt, '(/ 6x, a)') buffer
      if (lsumm .gt. 0) write (lsumm, '(/ 1x, a)') buffer
      inform = 4
      return

 2000 format(
     $ //' XXX  Error while looking for an OPTIONS file on unit', I7
     $ / ' XXX  The file should start with Begin, Skip or Endrun'
     $ / ' XXX  but the first record found was the following:'
     $ //' ---->', a
     $ //' XXX  Continuing to look for OPTIONS file...')
 2200 format(//' XXX  End-of-file encountered while processing',
     $         ' an OPTIONS file on unit', I6)
 2300 format(//' XXX  End-of-file encountered while looking for',
     $         ' an OPTIONS file on unit', I6)

*     end of opfile
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE OPLOOK (NDICT, DICTRY, ALPHA, KEY, ENTRY)
C
C
C Description and usage:
C
C       Performs dictionary lookups.  A pointer is returned if a
C    match is found between the input key and the corresponding
C    initial characters of one of the elements of the dictionary.
C    If a "synonym" has been provided for an entry, the search is
C    continued until a match to a primary dictionary entry is found.
C    Cases of no match, or multiple matches, are also provided for.
C
C     Dictionary entries must be left-justified, and may be alphabetized
C    for faster searches.  Secondary entries, if any, are composed of
C    two words separated by one or more characters such as blank, tab,
C    comma, colon, or equal sign which are treated as non-significant
C    by OPSCAN.  The first entry of each such pair serves as a synonym
C    for the second, more fundamental keyword.
C
C       The ordered search stops after the section of the dictionary
C    having the same first letters as the key has been checked, or
C    after a specified number of entries have been examined.  A special
C    dictionary entry, the vertical bar '|', will also terminate the
C    search.  This will speed things up if an appropriate dictionary
C    length parameter cannot be determined.  Both types of search are
C    sequential.  See "Notes" below for some suggestions if efficiency
C    is an issue.
C
C
C Parameters:
C
C    Name    Dimension  Type  I/O/S  Description
C    NDICT               I    I      Number of dictionary entries to be
C                                    examined.
C    DICTRY  NDICT       C    I      Array of dictionary entries,
C                                    left-justified in their fields.
C                                    May be alphabetized for efficiency,
C                                    in which case ALPHA should be
C                                    .TRUE.  Entries with synonyms are
C                                    of the form
C                                    'ENTRY : SYNONYM', where 'SYNONYM'
C                                    is a more fundamental entry in the
C                                    same dictionary.  NOTE: Don't build
C                                    "circular" dictionaries!
C    ALPHA               L    I      Indicates whether the dictionary
C                                    is in alphabetical order, in which
C                                    case the search can be terminated
C                                    sooner.
C    KEY                 C    I/O    String to be compared against the
C                                    dictionary.  Abbreviations are OK
C                                    if they correspond to a unique
C                                    entry in the dictionary.  KEY is
C                                    replaced on termination by its most
C                                    fundamental equivalent dictionary
C                                    entry (uppercase, left-justified)
C                                    if a match was found.
C    ENTRY               I      O    Dictionary pointer.  If > 0, it
C                                    indicates which entry matched KEY.
C                                    In case of trouble, a negative
C                                    value means that a UNIQUE match
C                                    was not found - the absolute value
C                                    of ENTRY points to the second
C                                    dictionary entry that matched KEY.
C                                    Zero means that NO match could be
C                                    found.  ENTRY always refers to the
C                                    last search performed -
C                                    in searching a chain of synonyms,
C                                    a non-positive value will be
C                                    returned if there is any break,
C                                    even if the original input key
C                                    was found.
C
C
C External references:
C
C    Name    Description
C    OPSCAN  Finds first and last significant characters.
C
C
C Environment:  Digital VAX-11/780 VMS FORTRAN (FORTRAN 77).
C               Appears to satisfy the ANSI Fortran 77 standard.
C
C
C Notes:
C
C    (1)  IMPLICIT NONE is non-standard.  (Has been commented out.)
C
C    (2)  We have assumed that the dictionary is not too big.  If
C         many searches are to be done or if the dictionary has more
C         than a dozen or so entries, it may be advantageous to build
C         an index array of pointers to the beginning of the section
C         of the dictionary containing each letter, then pass in the
C         portion of the dictionary beginning with DICTRY (INDEX).
C         (This won't generally work for dictionaries with synonyms.)
C         For very large problems, a completely different approach may
C         be advisable, e.g. a binary search for ordered dictionaries.
C
C    (3)  OPLOOK is case sensitive.  In most applications it will be
C         necessary to use an uppercase dictionary, and to convert the
C         input key to uppercase before calling OPLOOK.  Companion
C         routines OPTOKN and PAIRS, available from the author, already
C         take care of this.
C
C    (4)  The key need not be left-justified.  Any leading (or
C         trailing) characters which are "non-significant" to OPSCAN
C         will be ignored.  These include blanks, horizontal tabs,
C         commas, colons, and equal signs.  See OPSCAN for details.
C
C    (5)  The ASCII collating sequence for character data is assumed.
C         (N.B. This means the numerals precede the alphabet, unlike
C         common practice!)  This should not cause trouble on EBCDIC
C         machines if DICTRY just contains alphabetic keywords.
C         Otherwise it may be necessary to use the FORTRAN lexical
C         library routines to force use of the ASCII sequence.
C
C    (6)  Parameter NUMSIG sets a limit on the length of significant
C         dictionary entries.  Special applications may require that
C         this be increased.  (It is 16 in the present version.)
C
C    (7)  No protection against "circular" dictionaries is provided:
C         don't claim that A is B, and that B is A.  All synonym chains
C         must terminate!  Other potential errors not checked for
C         include duplicate or mis-ordered entries.
C
C    (8)  The handling of ambiguities introduces some ambiguity:
C
C            ALPHA = .TRUE.  A potential problem, when one entry
C                            looks like an abbreviation for another
C                            (eg. does 'A' match 'A' or 'AB'?) was
C                            resolved by dropping out of the search
C                            immediately when an "exact" match is found.
C
C            ALPHA = .FALSE. The programmer must ensure that the above
C                            situation does not arise: each dictionary
C                            entry must be recognizable, at least when
C                            specified to full length.  Otherwise, the
C                            result of a search will depend on the
C                            order of entries.
C
C
C Author:  Robert Kennelly, Informatics General Corporation.
C
C
C Development history:
C
C    24 Feb. 1984  RAK/DAS  Initial design and coding.
C    25 Feb. 1984    RAK    Combined the two searches by suitable
C                           choice of terminator FLAG.
C    28 Feb. 1984    RAK    Optional synonyms in dictionary, no
C                           longer update KEY.
C    29 Mar. 1984    RAK    Put back replacement of KEY by its
C                           corresponding entry.
C    21 June 1984    RAK    Corrected bug in error handling for cases
C                           where no match was found.
C    23 Apr. 1985    RAK    Introduced test for exact matches, which
C                           permits use of dictionary entries which
C                           would appear to be ambiguous (for ordered
C                           case).  Return -I to point to the entry
C                           which appeared ambiguous (had been -1).
C                           Repaired loop termination - had to use
C                           equal length strings or risk quitting too
C                           soon when one entry is an abbreviation
C                           for another.  Eliminated HIT, reduced
C                           NUMSIG to 16.
C    15 Nov. 1985    MAS    Loop 20 now tests .LT. FLAG, not .LE. FLAG.
C                           If ALPHA is false, FLAG is now '|', not '{'.
C    26 Jan. 1986    PEG    Declaration of FLAG and TARGET modified to
C                           conform to ANSI-77 standard.
C-----------------------------------------------------------------------


C     Variable declarations.
C     ----------------------

*     IMPLICIT NONE

C     Parameters.

      INTEGER
     $   NUMSIG
      CHARACTER
     $   BLANK, VBAR
      PARAMETER
     $   (BLANK = ' ', VBAR = '|', NUMSIG = 16)

C     Variables.

      LOGICAL
     $   ALPHA
      INTEGER
     $   ENTRY, FIRST, I, LAST, LENGTH, MARK, NDICT
*     CHARACTER
*    $   DICTRY (NDICT) * (*), FLAG * (NUMSIG),
*    $   KEY * (*), TARGET * (NUMSIG)
      CHARACTER
     $   DICTRY (NDICT) * (*), FLAG * 16,
     $   KEY * (*), TARGET * 16

C     Procedures.

      EXTERNAL
     $   OPSCAN


C     Executable statements.
C     ----------------------

      ENTRY = 0

C     Isolate the significant portion of the input key (if any).

      FIRST = 1
      LAST  = MIN( LEN(KEY), NUMSIG )
      CALL OPSCAN (KEY, FIRST, LAST, MARK)

      IF (MARK .GT. 0) THEN
         TARGET = KEY (FIRST:MARK)

C        Look up TARGET in the dictionary.

   10    CONTINUE
            LENGTH = MARK - FIRST + 1

C           Select search strategy by cunning choice of termination test
C           flag.  The vertical bar is just about last in both the
C           ASCII and EBCDIC collating sequences.

            IF (ALPHA) THEN
               FLAG = TARGET
            ELSE
               FLAG = VBAR
            END IF


C           Perform search.
C           ---------------

            I = 0
   20       CONTINUE
               I = I + 1
               IF (TARGET (1:LENGTH) .EQ. DICTRY (I) (1:LENGTH)) THEN
                  IF (ENTRY .EQ. 0) THEN

C                    First "hit" - must still guard against ambiguities
C                    by searching until we've gone beyond the key
C                    (ordered dictionary) or until the end-of-dictionary
C                    mark is reached (exhaustive search).

                     ENTRY = I

C                    Special handling if match is exact - terminate
C                    search.  We thus avoid confusion if one dictionary
C                    entry looks like an abbreviation of another.
C                    This fix won't generally work for un-ordered
C                    dictionaries!

                     FIRST = 1
                     LAST = NUMSIG
                     CALL OPSCAN (DICTRY (ENTRY), FIRST, LAST, MARK)
                     IF (MARK .EQ. LENGTH) I = NDICT
                  ELSE


C                    Oops - two hits!  Abnormal termination.
C                    ---------------------------------------

                     ENTRY = -I
                     RETURN
                  END IF
               END IF

C           Check whether we've gone past the appropriate section of the
C           dictionary.  The test on the index provides insurance and an
C           optional means for limiting the extent of the search.

            IF (DICTRY (I) (1:LENGTH) .LT. FLAG  .AND.  I .LT. NDICT)
     $         GO TO 20


C           Check for a synonym.
C           --------------------

            IF (ENTRY .GT. 0) THEN

C              Look for a second entry "behind" the first entry.  FIRST
C              and MARK were determined above when the hit was detected.

               FIRST = MARK + 2
               CALL OPSCAN (DICTRY (ENTRY), FIRST, LAST, MARK)
               IF (MARK .GT. 0) THEN

C                 Re-set target and dictionary pointer, then repeat the
C                 search for the synonym instead of the original key.

                  TARGET = DICTRY (ENTRY) (FIRST:MARK)
                  ENTRY = 0
                  GO TO 10

               END IF
            END IF

      END IF
      IF (ENTRY .GT. 0) KEY = DICTRY (ENTRY)


C     Normal termination.
C     -------------------

      RETURN

C     End of OPLOOK
      END
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      FUNCTION OPNUMB( STRING )

      LOGICAL          OPNUMB
      CHARACTER*(*)    STRING

************************************************************************
*     Description and usage:
*
*        A simple(-minded) test for numeric data is implemented by
*        searching an input string for legitimate characters:
*                digits 0 to 9, D, E, -, + and .
*        Insurance is provided by requiring that a numeric string
*        have at least one digit, at most one D, E or .
*        and at most two -s or +s.  Note that a few ambiguities remain:
*
*           (a)  A string might have the form of numeric data but be
*                intended as text.  No general test can hope to detect
*                such cases.
*
*           (b)  There is no check for correctness of the data format.
*                For example a meaningless string such as 'E1.+2-'
*                will be accepted as numeric.
*
*        Despite these weaknesses, the method should work in the
*        majority of cases.
*
*
*     Parameters:
*
*        Name    Dimension  Type  I/O/S  Description
*        OPNUMB              L      O    Set .TRUE. if STRING appears
*                                        to be numerical data.
*        STRING              C    I      Input data to be tested.
*
*
*     Environment:  ANSI FORTRAN 77.
*
*
*     Notes:
*
*        (1)  It is assumed that STRING is a token extracted by
*             OPTOKN, which will have converted any lower-case
*             characters to upper-case.
*
*        (2)  OPTOKN pads STRING with blanks, so that a genuine
*             number is of the form  '1234        '.
*             Hence, the scan of STRING stops at the first blank.
*
*        (3)  COMPLEX data with parentheses will not look numeric.
*
*
*     Systems Optimization Laboratory, Stanford University.
*     12 Nov  1985    Initial design and coding, starting from the
*                     routine ALPHA from Informatics General, Inc.
************************************************************************

      LOGICAL         NUMBER
      INTEGER         J, LENGTH, NDIGIT, NEXP, NMINUS, NPLUS, NPOINT
      CHARACTER*1     ATOM

      NDIGIT = 0
      NEXP   = 0
      NMINUS = 0
      NPLUS  = 0
      NPOINT = 0
      NUMBER = .TRUE.
      LENGTH = LEN (STRING)
      J      = 0

   10    J    = J + 1
         ATOM = STRING (J:J)
         IF      (ATOM .GE. '0'  .AND.  ATOM .LE. '9') THEN
            NDIGIT = NDIGIT + 1
         ELSE IF (ATOM .EQ. 'D'  .OR.   ATOM .EQ. 'E') THEN
            NEXP   = NEXP   + 1
         ELSE IF (ATOM .EQ. '-') THEN
            NMINUS = NMINUS + 1
         ELSE IF (ATOM .EQ. '+') THEN
            NPLUS  = NPLUS  + 1
         ELSE IF (ATOM .EQ. '.') THEN
            NPOINT = NPOINT + 1
         ELSE IF (ATOM .EQ. ' ') THEN
            J      = LENGTH
         ELSE
            NUMBER = .FALSE.
         END IF

         IF (NUMBER  .AND.  J .LT. LENGTH) GO TO 10

      OPNUMB = NUMBER
     $         .AND.  NDIGIT .GE. 1
     $         .AND.  NEXP   .LE. 1
     $         .AND.  NMINUS .LE. 2
     $         .AND.  NPLUS  .LE. 2
     $         .AND.  NPOINT .LE. 1

      RETURN

*     End of OPNUMB
      END
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE OPSCAN (STRING, FIRST, LAST, MARK)
C
C
C Description and usage:
C
C       Looks for non-blank fields ("tokens") in a string, where the
C    fields are of arbitrary length, separated by blanks, tabs, commas,
C    colons, or equal signs.  The position of the end of the 1st token
C    is also returned, so this routine may be conveniently used within
C    a loop to process an entire line of text.
C
C       The procedure examines a substring, STRING (FIRST : LAST), which
C    may of course be the entire string (in which case just call OPSCAN
C    with FIRST <= 1 and LAST >= LEN (STRING) ).  The indices returned
C    are relative to STRING itself, not the substring.
C
C
C Parameters:
C
C    Name    Dimension  Type  I/O/S  Description
C    STRING              C    I      Text string containing data to be
C                                    scanned.
C    FIRST               I    I/O    Index of beginning of substring.
C                                    If <= 1, the search begins with 1.
C                                    Output is index of beginning of
C                                    first non-blank field, or 0 if no
C                                    token was found.
C    LAST                I    I/O    Index of end of substring.
C                                    If >= LEN (STRING), the search
C                                    begins with LEN (STRING).  Output
C                                    is index of end of last non-blank
C                                    field, or 0 if no token was found.
C    MARK                I      O    Points to end of first non-blank
C                                    field in the specified substring.
C                                    Set to 0 if no token was found.
C
C
C Environment:  Digital VAX-11/780 VMS FORTRAN (FORTRAN 77).
C               ANSI Fortran 77, except for the tab character HT.
C
C Notes:
C
C    (1)  IMPLICIT NONE is non-standard.  Constant HT (Tab) is defined
C         in a non-standard way:  the CHAR function is not permitted
C         in a PARAMETER declaration (OK on VAX, though).  For Absoft
C         FORTRAN 77 on 68000 machines, use HT = 9.  In other cases, it
C         may be best to declare HT as a variable and assign
C         HT = CHAR(9) on ASCII machines, or CHAR(5) for EBCDIC.
C
C    (2)  The pseudo-recursive structure was chosen for fun.  It is
C         equivalent to three DO loops with embedded GO TOs in sequence.
C
C    (3)  The variety of separators recognized limits the usefulness of
C         this routine somewhat.  The intent is to facilitate handling
C         such tokens as keywords or numerical values.  In other
C         applications, it may be necessary for ALL printing characters
C         to be significant.  A simple modification to statement
C         function SOLID will do the trick.
C
C
C Author:  Robert Kennelly, Informatics General Corporation.
C
C
C Development history:
C
C    29 Dec. 1984    RAK    Initial design and coding, (very) loosely
C                           based on SCAN_STRING by Ralph Carmichael.
C    25 Feb. 1984    RAK    Added ':' and '=' to list of separators.
C    16 Apr. 1985    RAK    Defined SOLID in terms of variable DUMMY
C                           (previous re-use of STRING was ambiguous).
C
C-----------------------------------------------------------------------


C     Variable declarations.
C     ----------------------

*     IMPLICIT NONE

C     Parameters.

      CHARACTER
     $   BLANK, EQUAL, COLON, COMMA, HT
      PARAMETER
     $   (BLANK = ' ', EQUAL = '=', COLON = ':', COMMA = ',')

C     Variables.

      LOGICAL
     $   SOLID
      INTEGER
     $   BEGIN, END, FIRST, LAST, LENGTH, MARK
      CHARACTER
     $   DUMMY, STRING * (*)

C     Statement functions.

      SOLID (DUMMY) = (DUMMY .NE. BLANK) .AND.
     $                (DUMMY .NE. COLON) .AND.
     $                (DUMMY .NE. COMMA) .AND.
     $                (DUMMY .NE. EQUAL) .AND.
     $                (DUMMY .NE. HT)


C     Executable statements.
C     ----------------------

****  HT     = CHAR(9) for ASCII machines, CHAR(5) for EBCDIC.
      HT     = CHAR(9)
      MARK   = 0
      LENGTH = LEN (STRING)
      BEGIN  = MAX (FIRST, 1)
      END    = MIN (LENGTH, LAST)

C     Find the first significant character ...

      DO 30 FIRST = BEGIN, END, +1
         IF (SOLID (STRING (FIRST : FIRST))) THEN

C           ... then the end of the first token ...

            DO 20 MARK = FIRST, END - 1, +1
               IF (.NOT.SOLID (STRING (MARK + 1 : MARK + 1))) THEN

C                 ... and finally the last significant character.

                  DO 10 LAST = END, MARK, -1
                     IF (SOLID (STRING (LAST : LAST))) THEN
                        RETURN
                     END IF
   10             CONTINUE

C                 Everything past the first token was a separator.

                  LAST = LAST + 1
                  RETURN
               END IF
   20       CONTINUE

C           There was nothing past the first token.

            LAST = MARK
            RETURN
         END IF
   30 CONTINUE

C     Whoops - the entire substring STRING (BEGIN : END) was composed of
C     separators !

      FIRST = 0
      MARK = 0
      LAST = 0
      RETURN

C     End of OPSCAN
      END
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE OPTOKN (STRING, NUMBER, LIST)
C
C
C Description and usage:
C
C       An aid to parsing input data.  The individual "tokens" in a
C    character string are isolated, converted to uppercase, and stored
C    in an array.  Here, a token is a group of significant, contiguous
C    characters.  The following are NON-significant, and hence may
C    serve as separators:  blanks, horizontal tabs, commas, colons,
C    and equal signs.  See OPSCAN for details.  Processing continues
C    until the requested number of tokens have been found or the end
C    of the input string is reached.
C
C
C Parameters:
C
C    Name    Dimension  Type  I/O/S  Description
C    STRING              C    I      Input string to be analyzed.
C    NUMBER              I    I/O    Number of tokens requested (input)
C                                    and found (output).
C    LIST    NUMBER      C      O    Array of tokens, changed to upper
C                                    case.
C
C
C External references:
C
C    Name    Description
C    OPSCAN  Finds positions of first and last significant characters.
C    OPUPPR  Converts a string to uppercase.
C
C
C Environment:  Digital VAX-11/780 VMS FORTRAN (FORTRAN 77).
C               Appears to satisfy the ANSI Fortran 77 standard.
C
C
C Notes:
C
C    (1)  IMPLICIT NONE is non-standard.  (Has been commented out.)
C
C
C Author:  Robert Kennelly, Informatics General Corporation.
C
C
C Development history:
C
C    16 Jan. 1984    RAK    Initial design and coding.
C    16 Mar. 1984    RAK    Revised header to reflect full list of
C                           separators, repaired faulty WHILE clause
C                           in "10" loop.
C    18 Sep. 1984    RAK    Change elements of LIST to uppercase one
C                           at a time, leaving STRING unchanged.
C
C-----------------------------------------------------------------------


C     Variable declarations.
C     ----------------------

*     IMPLICIT NONE

C     Parameters.

      CHARACTER
     $   BLANK
      PARAMETER
     $   (BLANK = ' ')

C     Variables.

      INTEGER
     $   COUNT, FIRST, I, LAST, MARK, NUMBER
      CHARACTER
     $   STRING * (*), LIST (NUMBER) * (*)

C     Procedures.

      EXTERNAL
     $   OPUPPR, OPSCAN


C     Executable statements.
C     ----------------------

C     WHILE there are tokens to find, loop UNTIL enough have been found.

      FIRST = 1
      LAST = LEN (STRING)

      COUNT = 0
   10 CONTINUE

C        Get delimiting indices of next token, if any.

         CALL OPSCAN (STRING, FIRST, LAST, MARK)
         IF (LAST .GT. 0) THEN
            COUNT = COUNT + 1

C           Pass token to output string array, then change case.

            LIST (COUNT) = STRING (FIRST : MARK)
            CALL OPUPPR (LIST (COUNT))
            FIRST = MARK + 2
            IF (COUNT .LT. NUMBER) GO TO 10

         END IF


C     Fill the rest of LIST with blanks and set NUMBER for output.

      DO 20 I = COUNT + 1, NUMBER
         LIST (I) = BLANK
   20 CONTINUE

      NUMBER = COUNT


C     Termination.
C     ------------

      RETURN

C     End of OPTOKN
      END
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE OPUPPR(STRING)
C
C ACRONYM:  UPper CASE
C
C PURPOSE:  This subroutine changes all lower case letters in the
C           character string to upper case.
C
C METHOD:   Each character in STRING is treated in turn.  The intrinsic
C           function INDEX effectively allows a table lookup, with
C           the local strings LOW and UPP acting as two tables.
C           This method avoids the use of CHAR and ICHAR, which appear
C           be different on ASCII and EBCDIC machines.
C
C ARGUMENTS
C    ARG       DIM     TYPE I/O/S DESCRIPTION
C  STRING       *       C   I/O   Character string possibly containing
C                                 some lower-case letters on input;
C                                 strictly upper-case letters on output
C                                 with no change to any non-alphabetic
C                                 characters.
C
C EXTERNAL REFERENCES:
C  LEN    - Returns the declared length of a CHARACTER variable.
C  INDEX  - Returns the position of second string within first.
C
C ENVIRONMENT:  ANSI FORTRAN 77
C
C DEVELOPMENT HISTORY:
C     DATE  INITIALS  DESCRIPTION
C   06/28/83   CLH    Initial design.
C   01/03/84   RAK    Eliminated NCHAR input.
C   06/14/84   RAK    Used integer PARAMETERs in comparison.
C   04/21/85   RAK    Eliminated DO/END DO in favor of standard code.
C   09/10/85   MAS    Eliminated CHAR,ICHAR in favor of LOW, UPP, INDEX.
C
C AUTHOR: Charles Hooper, Informatics General, Palo Alto, CA.
C
C-----------------------------------------------------------------------

      CHARACTER      STRING * (*)
      INTEGER        I, J
      character*1    C
      character*26   LOW, UPP
      data           LOW /'abcdefghijklmnopqrstuvwxyz'/,
     $               UPP /'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/

      DO 10 J = 1, LEN(STRING)
         C    = STRING(J:J)
         IF (C .GE. 'a'  .AND.  C .LE. 'z') THEN
            I           = INDEX( LOW, C )
            IF (I .GT. 0) STRING(J:J) = UPP(I:I)
         END IF
   10 CONTINUE

*     End of OPUPPR
      END
************************************************************************
*
*     File  mi35inpt fortran.
*
*     m3getp   m3hash   m3imov
*     m3inpt   m3mpsa   m3mpsb   m3mpsc   m3read
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3getp( maxm, lenh )

*     ------------------------------------------------------------------
*     m3getp finds a prime number lenh suitably larger than maxm.
*     It is used as the length of the hash table for the MPS row names.
*
*     20 Apr 1992: Implemented for use in m3inpt and matmps.
*     ------------------------------------------------------------------

      lenh   = maxm*2
      lenh   = max( lenh, 100 )
      lenh   = (lenh/2)*2 - 1
      k      = lenh/20 + 6

  100 k      = k + 1
      lenh   = lenh + 2
      do 120 i = 3, k, 2
         if (mod(lenh,i) .eq. 0) go to 100
  120 continue

*     end of m3getp
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3hash( len  , nen  , ncoll,
     $                   key1 , key2 , mode , keytab,
     $                   name1, name2, ka   , found )

      integer            keytab(len), name1(nen), name2(nen)
      logical            found

*     ------------------------------------------------------------------
*     m3hash looks up and/or inserts integer keys in a table.
*     Reference:  R.P. Brent, CACM 16,2 (Feb 1973), pp. 105-109.
*     This version is simplified for the case where no entries are
*     deleted.
*     keytab is used as an index into a consecutive list of unique
*     identifiers name1 and name2.
*     Each pair   name1(i), name2(i)  is two 4-character identifiers,
*     treated as one 8-character identifier.
*     ------------------------------------------------------------------

      len2 = len - 2
      ic   = -1

*     Compute address of first probe (ir) and increment (iq).
*     ******************************************************************
*     NOTE -- the next statements are machine dependent.  The aim
*     is to produce a valid positive integer  key  out of the
*     two words  key1  and  key2 .  The latter contain four
*     characters left-justified (read under a4 format).  These
*     may turn on the sign bit (e.g. IBM 360/370) or give a
*     non-zero floating point exponent (e.g. Burroughs B6700).
*     ******************************************************************
*---+ DEC VAX
      k1  = abs(key1)
      k2  = abs(key2)

*---- On CDC systems (and possibly CRAY systems), change the previous 2
*-    lines to comments, and uncomment the next 3 lines.
*-    decode( 4, 10, key1 ) k1
*-    decode( 4, 10, key2 ) k2
*- 10 format( r4 )

      key = abs(k1 - k2)
      iq  = mod(key, len2) + 1
      ir  = mod(key, len)  + 1
      ka  = ir

*     Look in the table.
   20 kt  = keytab(ka)

*     Check for an empty space or a match.
      if (kt .eq. 0) go to 30
      if (key1 .eq. name1(kt)  .and.  key2 .eq. name2(kt)) go to 60
      ic    = ic + 1
      ncoll = ncoll + 1

*     Compute address of next probe.
      ka    = ka + iq
      if (ka .gt. len) ka = ka - len

*     See if whole table has been searched.
      if (ka .ne. ir ) go to 20

*     The key is not in the table.
   30 found = .false.

*     Return with KA = 0 unless an entry has to be made.
      if ((mode .eq. 2)  .and.  (ic .le. len2)) go to 70
      ka    = 0
      return

   60 found = .true.
      return

*     Look for the best way to make an entry.
   70 if (ic .le. 0) return
      ia  = ka
      is  = 0

*     Compute the maximum length to search along current chain.
   80 ix  = ic - is
      kt  = keytab(ir)

*     Compute increment JQ for current chain.
*     ******************************************************************
*     NOTE -- the next statements are machine dependent.  The same
*     transformation as discussed in the note above should be
*     applied to  name1(kt)  and  name2(kt)  to produce an integer  key.
*     ******************************************************************
*---+ DEC VAX
      k1  = abs(name1(kt))
      k2  = abs(name2(kt))

*---- On CDC systems (and possibly CRAY systems), change the previous 2
*-    lines to comments, and uncomment the next 2 lines.
*-    decode( 4, 10, name1(kt) ) k1
*-    decode( 4, 10, name2(kt) ) k2

      key = abs(k1 - k2)
      jq  = mod(key, len2) + 1
      jr  = ir

*     Look along the chain.
   90 jr  = jr + jq
      if (jr .gt. len) jr = jr - len

*     Check for a hole.
      if (keytab(jr) .eq. 0) go to 100
      ix  = ix - 1
      if (ix .gt. 0) go to 90
      go to 110

*     Save location of hole.
  100 ia  = jr
      ka  = ir
      ic  = ic - ix

*     Move down to the next chain.
  110 is  = is + 1
      ir  = ir + iq
      if (ir .gt. len) ir = ir - len

*     Go back if a better hole might still be found.
      if (ic .gt. is ) go to 80

*     If necessary move an old entry.
      if (ia .ne. ka ) keytab(ia) = keytab(ka)

*     end of m3hash
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3imov( lennm, lrow, m, n, name )

      integer            name(lennm)

*     ------------------------------------------------------------------
*     m3imov is needed by m3inpt and matmps to move the row names
*     to be contiguous with the column names inside the integer array
*     name(*), once the true number of columns is known.
*     On entry, the column names begin at name(1) and the row names
*               begin at name(lrow).
*     On exit,  the row names begin at name(n+1).
*
*     20 Apr 1992: m3imov implemented to help matmps.
*     ------------------------------------------------------------------

      if (lrow .gt. n+1) then
         call icopy ( m, name(lrow), 1, name(n+1), 1 )
      end if

*     end of m3imov
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3inpt( objadd, z, nwcore )

      implicit           double precision (a-h,o-z)
      double precision   z(nwcore)

*     ------------------------------------------------------------------
*     m3inpt  inputs constraint data in MPS format, and sets up
*     various quantities as follows:
*
*     objadd     (output) is minus the coefficient in row iobj of the
*                RHS section (zero by default).  MINOS adds it to the
*                objective function.
*               
*     m, n, ne   are the number of rows, columns and elements in A.
*
*     iobj       is the row number for the linear objective (if any).
*                It must come after any nonlinear rows.
*                iobj = 0 if there is no linear objective.
*
*     a, ha, ka  is the matrix a stored in z at locations la, lha, lka.
*     bl, bu     are the bounds  stored in z at locations lbl, lbu.
*     hs, xn     are states and values   stored in z at   lhs, lxn.
*
*     hs(j)      is set to  0, 1  to indicate a plausible initial state
*                (at lo or up bnd) for each variable j  (j = 1 to nb).
*                If crash is to be used, i.e., crash option gt 0 and
*                if no basis file will be supplied, the initial bounds
*                set may initialize hs(j) as follows to assist crash:
*
*     -1      if column or row j is likely to be in the optimal basis,
*      4      if column j is likely to be nonbasic at its lower bound,
*      5      if column j is likely to be nonbasic at its upper bound,
*      2      if column or row j should initially be superbasic,
*      0 or 1 otherwise.
*
*     xn(j)      is a corresponding set of initial values.
*                Safeguards are applied later by m4chek, so the
*                values of hs and xn are not desperately critical.
*
*     The arrays name(*), mobj(*), mrhs(*), mrng(*), mbnd(*) are loaded
*     with the appropriate names in 2a4 format.
*
*     m3inpt (and hence m3hash, m3mpsa, m3mpsb, m3mpsc and m3read)
*     may be replaced by routines that output the same information.
*
*     31 Oct 1991: Modified to be compatible with subroutine minoss.
*     10 Apr 1992: objadd added as an output parameter.
*     04 May 1992: m3mpsb now outputs pi.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m2len / mrows ,mcols ,melms
      common    /m2mapa/ ne    ,nka   ,la    ,lha   ,lka
      common    /m2mapz/ maxw  ,maxz
      common    /m3len / m     ,n     ,nb    ,nscl
      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m3mps1/ lname1,lname2,lkeynm,nname
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /m5len / maxr  ,maxs  ,mbs   ,nn    ,nn0   ,nr    ,nx
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m5log1/ idebug,ierr,lprint
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m7loc / lgobj ,lgobj2
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold

      intrinsic          max, mod

      integer            ncard(6)
      logical            allnn
      character*4        key
      character*4        lenda
      data               lenda /'ENDA'/

*     ------------------------------------------------------------------
*     Here's a kludge to let users say they want all columns to be
*     nonlinear.  They should specify
*          Columns              n
*          Nonlinear variables  n
*     for any large enough n (ge the true n).
*     here we just test if the two n's are the same.
*     Later we reset nn once we know the true n.
*     ------------------------------------------------------------------
      allnn  = mcols .eq. nn

*     We may come back here to try again with more workspace.
*     key    retains the first 4 characters of the NAME, ROWS, COLUMNS
*            RHS, RANGES and BOUNDS cards.
*     ncard  counts the number of data records in each section.
*     m3getp finds a prime number for the length of the row hash table.

   10 ierr   = 0
      ncoll  = 0
      key    = '    '
      call iload1( 6, 0, ncard, 1 )
      call m3getp( mrows, lenh )

      call m2core( 2, mincor )
      if (maxz .lt. mincor) go to 600

*     ------------------------------------------------------------------
*     Input ROWS.
*     lrow   is the location of the first rowname in name1, name2.
*     lennm  is the initial length of name1, name2,
*            i.e. the maximum no. of names allowed for.
*     ------------------------------------------------------------------
      lrow   = mcols + 1
      lennm  = mcols + mrows
      call m3mpsa( mrows, mcols, melms, ncoll, m,
     $             lrow, lennm, lenh, nn, nncon, key, ncard,
     $             z(lhrtyp), z(lname1), z(lname2), z(lkeynm) )
      if (ierr .eq. 40) go to 400
      if (ierr .eq. 41) go to 500

*     ------------------------------------------------------------------
*     m  is now known.
*     Input COLUMNS, RHS, RANGES.
*     ------------------------------------------------------------------
      mrows  = m
      call m3mpsb( mcols, melms, lrow, lennm, lenh, ncoll, objadd,
     $             m, n, nb, ne, nka,
     $             nn, nncon, nnjac, nnobj, njac, key, ncard,
     $             z(lhrtyp), z(lname1), z(lname2), z(lkeynm),
     $             z(lka), z(lha), z(la), z(lbl), z(lbu),
     $             z(lkb), z(lpi) )
      if (ierr .eq. 40) go to 400
      if (ierr .eq. 41) go to 510

*     ------------------------------------------------------------------
*     n  and  ne  are now known.
*     Move the row names to be contiguous with the column names.
*     Input BOUNDS.
*     ------------------------------------------------------------------
      call m3imov( lennm, lrow, m, n, z(lname1) )
      call m3imov( lennm, lrow, m, n, z(lname2) )
      mcols  = n
      melms  = ne
      np1    = n + 1
      nb     = n + m
      if (maxs .gt. np1) maxs = np1
      if (maxr .gt. np1) maxr = np1
      if (nn   .ge. nb ) nn   = nb
      if (    allnn    ) then
          nn     = n
          if (nnobj .gt. 0) nnobj = n
          if (nnjac .gt. 0) nnjac = n
      end if

      call m3mpsc( m, n, nb, ne, ns, lennm,
     $             key, ncard, z(lname1), z(lname2),
     $             z(lbl), z(lbu), z(lhs), z(lxn) )

      if (iprint .gt. 0) then
                            write(iprint, '(/)')
         if (lprint .gt. 0) write(iprint, 1300) lenh, ncoll
         if (na0    .gt. 0) write(iprint, 1320) na0
         if (nncon  .gt. 0) write(iprint, 1350) njac
         if (nncon  .gt. 0  .or.  ncard(5) .gt. 0)
     $                      write(iprint, 1400) ncard(5)
         if (nn     .gt. 0  .or.  ncard(6) .gt. 0)
     $                      write(iprint, 1420) ncard(6), ns
      end if

*     ------------------------------------------------------------------
*     Compress storage, now that we know the size of everything.
*     ------------------------------------------------------------------

*     Save current positions of  bl, bu, etc.

      kha    = lha
      kka    = lka
      kbl    = lbl
      kbu    = lbu
      kn1    = lname1
      kn2    = lname2
      khs    = lhs
      kxn    = lxn
      kpi    = lpi

*     Redefine addresses in  z  in terms of the known dimensions.

      call m2core( 3, mincor )
      if (maxz .lt. mincor) go to 800

*     Move bl, bu, etc. into their final positions.

      call hcopy ( ne, z(kha), 1, z(lha), 1 )
      call icopy ( nka,z(kka), 1, z(lka), 1 )
      call dcopy ( nb, z(kbl), 1, z(lbl), 1 )
      call dcopy ( nb, z(kbu), 1, z(lbu), 1 )
      if (nname .eq. nb) then
         call icopy ( nb, z(kn1), 1, z(lname1), 1 )
         call icopy ( nb, z(kn2), 1, z(lname2), 1 )
      end if
      call hcopy ( nb, z(khs), 1, z(lhs), 1 )
      call dcopy ( nb, z(kxn), 1, z(lxn), 1 )
      call dcopy ( m , z(kpi), 1, z(lpi), 1 )
      go to 900

*     ---------------------------
*     Fatal error in MPS file.
*     ---------------------------
  400 call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1100)
      if (isumm  .gt. 0) write(isumm , 1100)
      go to 700

*     ---------------------------
*     Too many rows.
*     ---------------------------
  500 mrows  = m
      go to 520

*     -----------------------------
*     Too many columns or elements.
*     -----------------------------
  510 mcols  = n
      melms  = ne

*     Try again.

  520 if (imps .ne. iread  .and.  imps .ne. ispecs) then
         rewind imps
         go to 10
      end if

*     ---------------------------------
*     Not enough core to read MPS file.
*     ---------------------------------
  600 call m1page(2)
      if (iprint .gt. 0) write(iprint, 1110)
      if (isumm  .gt. 0) write(isumm , 1110)
      ierr   = 41

*     ------------------------------------
*     Flush MPS file to the ENDATA card
*     if it is the same as the SPECS file.
*     ------------------------------------
  700 if (imps .eq. ispecs) then
         do 750 idummy = 1, 100000
            if (key .eq. lenda) go to 900
            read(imps, 1700, end=900) key
  750    continue
      end if
      go to 900

*     -------------------------------------
*     Not enough core to solve the problem.
*     -------------------------------------
  800 call m1page(2)
      if (iprint .gt. 0) write(iprint, 1120) mincor
      if (isumm  .gt. 0) write(isumm , 1120) mincor
      ierr   = 42

*     Exit.

  900 if (imps .ne. iread  .and.  imps .ne. ispecs) rewind imps
      return

 1100 format(' EXIT -- fatal errors in the MPS file')
 1110 format(' EXIT -- not enough storage to read the MPS file')
 1120 format(' EXIT -- not enough storage to start solving',
     $       ' the problem'
     $    // ' Workspace (total)  should be significantly',
     $       ' more than', i8)
 1300 format(/ ' Length of row-name hash table  ', i12
     $       / ' Collisions during table lookup ', i12)
 1320 format(  ' No. of rejected coefficients   ', i12)
 1350 format(  ' No. of Jacobian entries specified', i10)
 1400 format(  ' No. of LAGRANGE entries specified', i10)
 1420 format(  ' No. of INITIAL  bounds  specified', i10
     $       / ' No. of superbasics specified   ', i12)
 1700 format(a4)

*     end of m3inpt
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3mpsa( mrows, mcols, melms, ncoll, m,
     $                   lrow, lennm, lenh, nn, nncon, key, ncard,
     $                   hrtype, name1, name2, keynam )

      implicit           double precision (a-h,o-z)
      character*4        key
      integer*4          hrtype(mrows)
      integer            ncard(6), name1(lennm), name2(lennm),
     $                   keynam(lenh)

*     ------------------------------------------------------------------
*     m3mpsa  inputs the name and rows sections of an MPS file.
*
*     Original version written by Keith Morris, Wellington, 1973.
*     1975: Use a hash table for the row names.
*     1979: Treat the (nncon by nnjac) principal submatrix
*           as a Jacobian for nonlinear constraints.
*     1980: Add Phantom columns to the end of  A.
*     1982: Store the rhs as bounds on the logicals,
*           instead of the last column of  A.  The constraints now have
*           the form    A*x  +  I*s  =  0,    bl .le. (x, s) .le. bu,
*           where A has m rows, n columns and ne nonzero elements.
*     1982  Treat * in column 1 correctly, and retry if the MPS file
*           is on disk and mrows, mcols or melms are too small.
*     Apr 1984: Added check for duplicate row entries in columns.
*     Mar 1985: Changes made to handle characters as in Fortran 77.
*     Oct 1985: m3mps split into m3mpsa, m3mpsb, m3mpsc.
*               Revisions made to handle characters more efficiently.
*     Oct 1991: More f77.  Row names now at end of name1, name2.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3mps5/ aelem(2), id(6), iblank
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint

      logical            found,gotnm
      character*4        a4   ,lblank,
     $                   lname,lrows,lcolu,
     $                   lex,lgx,llx,lnx,lxe,lxg,lxl,lxn
      data               a4   ,lblank      /'(a4)','    '/
      data               lname,lrows,lcolu /'NAME','ROWS','COLU'       /
      data               lex,lgx,llx,lnx   /' E  ',' G  ',' L  ',' N  '/
      data               lxe,lxg,lxl,lxn   /'  E ','  G ','  L ','  N '/
*     ------------------------------------------------------------------

      call m1page( 1 )
      if (iprint .gt. 0) write(iprint, 1000)
      read (lblank, '(a4)') iblank
      inform = 0
      iobj   = 0
      line   = 0
      m      = 0
      gotnm  = mobj(1) .ne. iblank
      call iload1( 20  , 0, ier   , 1 )
      call iload1( lenh, 0, keynam, 1 )

*     Look for the NAME card.

   10 call m3read( 1, imps, line, 5, key, inform )
      if (key .ne. lname) then
         if (ier(1) .eq. 0) then
             ier(1) = 1
             if (iprint .gt. 0) write(iprint, 1100)
             if (isumm  .gt. 0) write(isumm , 1100)
         end if
         go to 10
      end if

      name(1) = id(3)
      name(2) = id(4)
      if (isumm  .gt. 0) write(isumm, 5000) name

*     Look for the ROWS card.

      call m3read( 1, imps, line, 5, key, inform )
      inform = 0
      if (key .ne. lrows) then
         ier(1) = ier(1) + 1
         if (iprint .gt. 0) write(iprint, 1120)
         if (isumm  .gt. 0) write(isumm , 1120)
         go to 35
      end if

*     ==================================================================
*     Read the row names and check if the relationals are valid.
*     ==================================================================
   30 call m3read( 1, imps, line, mlst, key, inform )
      if (inform .ne. 0) go to 110

   35 if      (key .eq. lgx  .or.  key .eq. lxg) then
         it  = -1
      else if (key .eq. lex  .or.  key .eq. lxe) then
         it  =  0
      else if (key .eq. llx  .or.  key .eq. lxl) then
         it  =  1
      else if (key .eq. lnx  .or.  key .eq. lxn) then
         it  =  2

*        Record objective name if we don't already have one.

         if (iobj .eq. 0) then
            if (.not. gotnm) then
               mobj(1) = id(1)
               mobj(2) = id(2)
               if (nn .gt. 0) then
                  if (iprint .gt. 0) write(iprint, 1170) mobj
                  if (isumm  .gt. 0) write(isumm , 1170) mobj
               end if
            end if

            if (id(1) .eq. mobj(1) .and. id(2) .eq. mobj(2)) then
               iobj     = m + 1
               ncard(1) = ncard(1) + 1
            end if
         end if
      else
         ier(3) = ier(3) + 1
         if (ier(3) .le. mer) then
            if (iprint .gt. 0) write(iprint, 1160) line,key,id(1),id(2)
            if (isumm  .gt. 0) write(isumm , 1160) line,key,id(1),id(2)
         end if
         go to 30
      end if

*     ..................................................................
*     Look up the row name  id(1), id(2)  in the hash table.
*     ..................................................................
      call m3hash( lenh, mrows, ncoll, id(1), id(2), 2,
     $             keynam, name1(lrow), name2(lrow), ia, found )

*     Error if the row name was already there.
*     Otherwise, enter the new name into the hash table.

      if (found) then
         ier(4) = ier(4) + 1
         if (ier(4) .le. mer) then
            if (iprint .gt. 0) write(iprint, 1200) id(1), id(2)
            if (isumm  .gt. 0) write(isumm , 1200) id(1), id(2)
         end if
      else
         m      = m + 1
         if (m .le. mrows) then
            jrow        = mcols + m
            keynam(ia)  = m
            name1(jrow) = id(1)
            name2(jrow) = id(2)
            hrtype(m)   = it
         end if
      end if
      go to 30

*     ==================================================================
*     Should be COLUMNS card.
*     ==================================================================
  110 if (key .ne. lcolu) then
         ier(1) = ier(1) + 1
         if (iprint .gt. 0) write(iprint, 1130)
         if (isumm  .gt. 0) write(isumm , 1130)
      end if

*     Error if no rows or too many rows.

      if (m .le. 0) then
         if (iprint .gt. 0) write(iprint, 1300)
         if (isumm  .gt. 0) write(isumm , 1300)
         ier(1) = ier(1) + 1
         ierr   = 40
         return
      else if (m .gt. mrows) then
         if (iprint .gt. 0) write(iprint, 3030) mrows, m
         if (isumm  .gt. 0) write(isumm , 3030) mrows, m
         ier(1) = ier(1) + 1
         ierr   = 41
         return
      end if

*     Warning if no objective row found.
*     Error if linear objective is ahead of nonlinear rows.

      if (iobj .eq. 0) then
         if (iprint .gt. 0) write(iprint, 1600)
         if (isumm  .gt. 0) write(isumm , 1600)
      else if (iobj .le. nncon) then
         if (iprint .gt. 0) write(iprint, 1180) mobj
         if (isumm  .gt. 0) write(isumm , 1180) mobj
         ierr   = 40
         return
      end if

      if (isumm  .gt. 0) write(isumm, 5100) m
      return

 1000 format(' MPS file' / ' --------')
 1100 format(' XXXX  Garbage before NAME card')
 1120 format(' XXXX  ROWS card not found')
 1130 format(' XXXX  COLUMNS card not found')
 1160 format(' XXXX  Illegal row type at line', i7, '... ', 3a4)
 1170 format(' XXXX  Note:  row  ', 2a4,
     $   '  selected as linear part of objective.')
 1180 format(/ ' XXXX  The linear objective card      N ', 2a4
     $       / ' XXXX  is out of place.    Nonlinear constraints'
     $       / ' XXXX  must be listed first in the ROWS section.')
 1200 format(' XXXX  Duplicate row name --', 2a4, ' -- ignored')
 1300 format(' XXXX  No rows specified')
 1600 format(' XXXX  Warning - no linear objective selected')
 2000 format(2a4)
 3030 format(' XXXX  Too many rows.  Limit was', i8,
     $   4x, '  Actual number is', i8)
 5000 format(' Name   ', 2a4)
 5100 format(' Rows   ',  i8)

*     end of m3mpsa
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3mpsb( mcols, melms, lrow, lennm, lenh, ncoll, objadd,
     $                   m, n, nb, ne, nka,
     $                   nn, nncon, nnjac, nnobj, njac, key, ncard,
     $                   hrtype, name1, name2, keynam,
     $                   ka, ha, a, bl, bu, kb, pi )

      implicit           double precision (a-h,o-z)
      character*4        key
      integer*4          hrtype(m), ha(melms)
      integer            ncard(6) , name1(lennm), name2(lennm),
     $                   keynam(lenh), ka(nka)  , kb(m)
      double precision   a(melms) , bl(nb), bu(nb)
      double precision   pi(m)

*     ------------------------------------------------------------------
*     m3mpsb inputs the COLUMNS, RHS and RANGES sections of an MPS file.
*     10 Apr 1992: objadd added as an output parameter.  It is minus
*                  the coefficient in row iobj of the RHS.
*     04 May 1992: pi(m)  added as an output parameter to return the
*                  special RHS called LAGRANGE.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3mps5/ aelem(2), id(6), iblank
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /cyclcm/ cnvtol,jnew,materr,maxcy,nephnt,nphant,nprint

      intrinsic          abs
      parameter        ( zero = 0.0d+0 )

      logical            dense, found, gotnm
      character*4        lrhs , lrhsx, lrang,
     $                   llagr, lange
      data               lrhs , lrhsx, lrang /'RHS ','RHS''','RANG'/
      data               llagr, lange        /'LAGR','ANGE'/
*     ------------------------------------------------------------------

      read (llagr , '(a4)') ilagr
      read (lange , '(a4)') iange
      objadd = zero
      dense  = nden .eq. 1
      bplus  = plinfy
      bminus = - bplus
      nmcol1 = 1234
      nmcol2 = 5678
      n      = 0
      na0    = 0
      ne     = 0
      ne1    = -1
      njac   = 0
      inform = 0
      call iload1( m,    0, kb, 1 )
      call dload ( m, zero, pi, 1 )

*     ==================================================================
*     Read the next columns card.
*     ==================================================================
  210 call m3read( 2, imps, line, mlst, key, inform )
      if (inform .ne. 0) go to 310

  220 if (id(1) .ne. nmcol1  .or.  id(2) .ne. nmcol2) then

*        Start a new column.

         if (ne .le. ne1) go to 310
         n         = n + 1
         ne1       = ne
         nmcol1    = id(1)
         nmcol2    = id(2)
         if (n .le. mcols) then
            ka(n)    = ne + 1
            name1(n) = nmcol1
            name2(n) = nmcol2

*           Make room for a dense Jacobian column.

            if (nncon .gt. 0) then
               ljac   = ne
               if (dense  .and.  n .le. nnjac) then
                  ne     = ne + nncon
                  if (ne .le. melms) then
                     ne     = ne - nncon
                     do 225 i  = 1, nncon
                        ne     = ne + 1
                        ha(ne) = i
                        a(ne)  = zero
  225                continue
                  end if
               end if
            end if
         end if
      end if

*     Process two row names and values.

      do 260 i  = 1, 2

*        Check for only one on the card.

         k      = i + i
         id1    = id(k+1)
         id2    = id(k+2)
         if (id1 .ne. iblank) go to 230
         if (id2 .eq. iblank) go to 260

*        Look up the row name.

  230    call m3hash( lenh, m, ncoll, id1, id2, 1,
     $                keynam, name1(lrow), name2(lrow), ia, found )

         if ( found ) then
            aij    = aelem(i)
            irow   = keynam(ia)

*           Test for a duplicate entry.

            if (kb(irow) .eq. n) then
               ier(8) = ier(8) + 1
               if (iprint .gt. 0  .and.  ier(8) .le. mer)
     $         write(iprint, 1420) nmcol1, nmcol2, id1, id2, aij, line
               go to 260
            end if

            kb(irow) = n
            if (irow .le. nncon  .and.  n .le. nnjac) then

*              Deal with Jacobian elements.

               njac   = njac + 1
               if ( dense ) then
                  a(ne1 + irow) = aij
                  go to 260
               end if

*              Sparse Jacobian -- make sure the new element is
*              squeezed in ahead of any linear-constraint elements.

               ljac   = ljac + 1
               if (ljac .le. ne) then
                  aij      = a(ljac)
                  irow     = ha(ljac)
                  a(ljac)  = aelem(i)
                  ha(ljac) = keynam(ia)
               end if

            else if (abs( aij ) .lt. aijtol) then

*              Ignore small aijs.

               na0    = na0 + 1
               go to 260
            end if

*           Pack the nonzero.

            ne     = ne + 1
            if (ne .le. melms) then
               ha(ne) = irow
               a(ne)  = aij
            end if
         else
            ier(5) = ier(5) + 1
            if (iprint .gt. 0  .and.  ier(5) .le. mer)
     $         write(iprint, 1400) id1, id2, line
         end if
  260 continue
      go to 210

*     Test for an empty column.

  310 if (ne .le. ne1) then

*        Column with no rows.   Warning unless variable is nonlinear.
*        Insert dummy column with zero in first row.

         if (n .gt. nn) then
            ier(6) = ier(6) + 1
            if (iprint .gt. 0  .and.  ier(6) .le. mer)
     $         write(iprint, 1500) nmcol1, nmcol2
         end if

         ne     = ne + 1
         if (ne .le. melms) then
            ha(ne) = 1
            a(ne)  = zero
         end if
         if (inform .eq. 0) go to 220
      end if

*     ==================================================================
*     See if we have hit the RHS.
*     ==================================================================
      if (key .ne. lrhs  .and.  key .ne. lrhsx) then

*        Nope sumpins rong.
*        Terminate the COLUMNS section anyway.

         ier(7) = ier(7) + 1
         if (iprint .gt. 0) write(iprint, 1140)
         if (isumm  .gt. 0) write(isumm , 1140)
      end if

*     Are there any columns at all?
*     Or too many columns or elements?
*     Include phantom columns and elements too.

      nephnt = max( nephnt, nphant )
      if (n .le. 0) then
         if (iprint .gt. 0) write(iprint, 1610)
         if (isumm  .gt. 0) write(isumm , 1610)
         ier(2) = ier(2) + 1
         ierr   = 40
         return
      else if (n + nphant .gt. mcols) then
         n      = n + nphant
         if (iprint .gt. 0) write(iprint, 3040) mcols, n
         if (isumm  .gt. 0) write(isumm , 3040) mcols, n
         ier(2) = ier(2) + 1
         ierr   = 41
         return
      else if (ne + nephnt .gt. melms) then
         ne     = ne + nephnt
         if (iprint .gt. 0) write(iprint, 3050) melms, ne
         if (isumm  .gt. 0) write(isumm , 3050) melms, ne
         ier(2) = ier(2) + 1
         ierr   = 41
         return
      end if

*     ------------------------------------------------------------------
*     Input the RHS.
*     ------------------------------------------------------------------
      if (isumm  .gt. 0) write(isumm, 5200) n, ne

*     Insert phantom columns for cycling algorithm.
*     Give them names  PHNT   1, PHNT   2, ...
*     (Trouble if more than 9999 phantom columns.)

      if (nphant .gt. 0) then
         do 402 i = 1, nephnt
            j     = ne + i
            ha(j) = 1
            a(j)  = zero
  402    continue

         do 403 k = 1, nphant
            n     = n    + 1
            ne    = ne   + 1
            write(name1(n), '(a4)') 'PHNT'
            write(name2(n), '(i4)') k
            ka(n) = ne
            if (k .eq. 1) ne = ne + nephnt - nphant
  403    continue
      end if

*     We finally know how big the problem is.

      ka(n+1) = ne + 1

*     Set bounds to default values.

      call dload ( n, bstruc(1), bl, 1 )
      call dload ( n, bstruc(2), bu, 1 )

      do 408 i  = 1, m
         k      = hrtype(i)
         jslack = n + i
         if (k .lt. 0) bl(jslack) = bminus
         if (k .le. 0) bu(jslack) = zero
         if (k .ge. 0) bl(jslack) = zero
         if (k .gt. 0) bu(jslack) = bplus
         if (k .eq. 2) bl(jslack) = bminus
  408 continue

*     Check for no RHS.

      if (key .ne. lrhs  .and.  key .ne. lrhsx) go to 600
      gotnm  = mrhs(1) .ne. iblank
      inform = 0

*     ==================================================================
*     Read next RHS card and see if it is the one we want.
*     ==================================================================
  410 call m3read( 2, imps, line, mlst, key, inform )
      if (inform .ne. 0) go to 600

*     A normal RHS is terminated if LAGRANGE is found.

      if (id(1) .eq. ilagr  .and.  id(2) .eq. iange) go to 490

      if (.not. gotnm) then
         gotnm   = .true.
         mrhs(1) = id(1)
         mrhs(2) = id(2)
      end if                                      

      if (id(1) .eq. mrhs(1)  .and.  id(2) .eq. mrhs(2)) then

*        Look at both halves of the record.

         do 440 i  = 1, 2
            k      = i + i
            id1    = id(k+1)
            id2    = id(k+2)
            if (id1 .eq. iblank  .and.  id2 .eq. iblank) go to 440
            call m3hash( lenh, m, ncoll, id1, id2, 1,
     $                   keynam, name1(lrow), name2(lrow), ia,found )

            if ( found ) then
               ncard(2) = ncard(2) + 1
               bnd      = aelem(i)
               irow     = keynam(ia)
               jslack   = n + irow
               k        = hrtype(irow)
               if (irow .eq. iobj) then
                  objadd = - bnd
               else if (k .ne. 2) then
                  if (k .le. 0) bu(jslack) = - bnd
                  if (k .ge. 0) bl(jslack) = - bnd
               end if
            else
               ier(5) = ier(5) + 1
               if (iprint .gt. 0  .and.  ier(5) .le. mer)
     $            write(iprint, 1400) id1, id2, line
            end if
  440    continue
      end if
      go to 410

*     LAGRANGE RHS found.

  490 if (ncard(2) .eq. 0) then
         mrhs(1) = iblank
         mrhs(2) = iblank
         if (iprint .gt. 0) write(iprint, 1720)
      end if
      go to 520

*     ==================================================================
*     Read next RHS card and see if it is a LAGRANGE one.
*     ==================================================================
  510 call m3read( 2, imps, line, mlst, key, inform )
      if (inform .ne. 0) go to 600

      if (id(1) .ne. ilagr  .or.   id(2) .ne. iange) go to 510

*     Find which row.
*     Look at both halves of the record.

  520 do 540 i  = 1, 2
         k      = i + i
         id1    = id(k+1)
         id2    = id(k+2)
         if (id1 .eq. iblank  .and.  id2 .eq. iblank) go to 540
         call m3hash( lenh, m, ncoll, id1, id2, 1,
     $                keynam, name1(lrow), name2(lrow), ia,found )
         
         if ( found ) then
            ncard(5) = ncard(5) + 1
            irow     = keynam(ia)
            pi(irow) = aelem(i)
         else
            ier(5) = ier(5) + 1
            if (iprint .gt. 0  .and.  ier(5) .le. mer)
     $         write(iprint, 1400) id1, id2, line
         end if
  540 continue
      go to 510

*     ------------------------------------------------------------------
*     RHS has been input.
*     ------------------------------------------------------------------
  600 if (ncard(2) .eq. 0) then
         if (iprint .gt. 0) write(iprint, 1620)
         if (isumm  .gt. 0) write(isumm , 1620)
      end if

      if (objadd .ne. zero) then
         if (iprint .gt. 0) write(iprint, 1630) objadd
         if (isumm  .gt. 0) write(isumm , 1630) objadd
      end if

*     ------------------------------------------------------------------
*     Input RANGES.
*     ------------------------------------------------------------------

*     Check for no RANGES.

      if (key .ne. lrang) go to 800
      gotnm  = mrng(1) .ne. iblank
      inform = 0
                                  
*     ==================================================================
*     Read card and see if it is the range we want.
*     ==================================================================
  610 call m3read( 2, imps, line, mlst, key, inform )
      if (inform .ne. 0) go to 800

      if (.not. gotnm) then
         gotnm   = .true.
         mrng(1) = id(1)
         mrng(2) = id(2)
      end if

      if (id(1) .eq. mrng(1)  .and.  id(2) .eq. mrng(2)) then

*        Look at both halves of the record.

         do 640 i  = 1, 2
            k      = i + i
            id1    = id(k+1)
            id2    = id(k+2)
            if (id1 .eq. iblank  .and.  id2 .eq. iblank) go to 640
            call m3hash( lenh, m, ncoll, id1, id2, 1,
     $                   keynam, name1(lrow), name2(lrow), ia,found )

            if ( found ) then
            ncard(3) = ncard(3)+1
            brng     = aelem(i)
            arng     = abs( brng )
            irow     = keynam(ia)
            jslack   = n + irow
            k        = hrtype(irow)
            if (k .ne. 2) then
               if (k .lt. 0) bl(jslack) = bu(jslack) - arng
               if (k .gt. 0) bu(jslack) = bl(jslack) + arng
               if (k .eq. 0) then
                  if (brng .gt. zero) bl(jslack) = bu(jslack) - arng
                  if (brng .lt. zero) bu(jslack) = bl(jslack) + arng
               end if
            end if
         else
            ier(5) = ier(5) + 1
            if (iprint .gt. 0  .and.  ier(5) .le. mer)
     $         write(iprint, 1400) id1, id2, line
            end if
  640    continue
      end if
      go to 610

*     RANGES have been input.

  800 return

 1140 format(' XXXX  RHS card not found')
 1400 format(' XXXX  Non-existent row    specified -- ', 2a4,
     $   ' -- entry ignored in line', i7)
 1420 format(' XXXX  Column  ', 2a4, '  has more than one entry',
     $       ' in row  ', 2a4
     $     / ' XXXX  Coefficient', 1p, e15.5, '  ignored in line', i10)
 1500 format(' XXXX  No valid row entries in column  ', 2a4)
 1610 format(' XXXX  No columns specified')
 1620 format(' XXXX  Warning - the RHS is zero')
 1630 format(' XXXX  Note:  constant', 1p, e15.7,
     $   '  is added to the objective.')
 1720 format(' XXXX  Warning - first RHS is LAGRANGE.',
     $   '   Other RHS''s will be ignored.')
 2000 format(2a4)
 3040 format(' XXXX  Too many columns.   The limit was', i8,
     $   4x, '  Actual number is', i8)
 3050 format(' XXXX  Too many elements.  The limit was', i8,
     $   4x, '  Actual number is', i8)
 5200 format(' Columns', i8 / ' Elements', i7)

*     end of m3mpsb
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3mpsc( m, n, nb, ne, ns, lennm,
     $                   key, ncard, name1, name2,
     $                   bl, bu, hs, xn )

      implicit           double precision (a-h,o-z)
      character*4        key
      integer*4          hs(nb)
      integer            ncard(6), name1(lennm), name2(lennm)
      double precision   bl(nb), bu(nb)
      double precision   xn(nb)

*     ------------------------------------------------------------------
*     m3mpsc inputs the BOUNDS section of an MPS file.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3mps5/ aelem(2), id(6), iblank
      common    /m5log1/ idebug,ierr,lprint
      common    /cyclcm/ cnvtol,jnew,materr,maxcy,nephnt,nphant,nprint

      intrinsic          abs, max, min
      parameter        ( zero = 0.0d+0 )

      logical            gotnm, ignore
      character*4        a4,
     $                   lboun, lenda,
     $                   lfr  , lfx, llo, lmi, lpl, lup,
     $                   linit, lial
      data               a4                   /'(a4)'/
      data               lboun, lenda         /'BOUN','ENDA'/
      data               lfr  , lfx, llo      /' FR ',' FX ',' LO '/
      data               lmi  , lpl, lup      /' MI ',' PL ',' UP '/
      data               linit, lial          /'INIT', 'IAL '/
*     ------------------------------------------------------------------

      read (linit , '(a4)') iinit
      read (lial  , '(a4)') iial
      inform = 1
      bplus  = plinfy
      bminus = -bplus

*     Fix phantom variables at zero.

      j1     = n - nphant + 1
      call dload ( nphant, zero, bl(j1), 1 )
      call dload ( nphant, zero, bu(j1), 1 )

*     Check for no BOUNDS.

      if (key .ne. lboun) go to 700
      gotnm  = mbnd(1). ne. iblank
      inform = 0
      jmark  = 1

*     ==================================================================
*     Read and check BOUNDS cards.  Notice the double plural.
*     ==================================================================
  610 call m3read( 3, imps, line, mlst, key, inform )
      if (inform .ne. 0) go to 700

*     A normal bounds set is terminated if INITIAL is found.

      bnd     = aelem(1)
      if (id(1) .eq. iinit  .and.  id(2) .eq. iial ) go to 690

      if (.not. gotnm) then
         gotnm   = .true.
         mbnd(1) = id(1)
         mbnd(2) = id(2)
      end if

      if (id(1) .ne. mbnd(1)  .or.  id(2) .ne. mbnd(2)) go to 610

*     Find which column.

      call m4name( n, name1, name2, id(3), id(4),
     $             line, ier(10), 0, 1, n, jmark, j )

      if (j .le. 0) then
         if (iprint .gt. 0  .and.  ier(10) .le. mer)
     $      write(iprint, 1400) id(3), id(4), line
      else

*        Select bound type for column j.

         ncard(4) = ncard(4) + 1
         if      (key .eq. lup) then
            bu(j)  = bnd
         else if (key .eq. llo) then
            bl(j)  = bnd
         else if (key .eq. lfx) then
            bu(j)  = bnd
            bl(j)  = bnd
         else if (key .eq. lfr) then
            bu(j)  = bplus
            bl(j)  = bminus
         else if (key .eq. lmi) then
            if (bu(j) .ge. bplus) bu(j)  = zero
            bl(j)  = bminus
         else if (key .eq. lpl) then
            bu(j)  = bplus
*---        bl(j)  = zero
*---        31-oct-1989: John Stone suggests delete this line
*---                     and add the "if" above.
         else
*           This lad didn't even make it to Form 1.

            ier(11) = ier(11) + 1
            if (iprint .gt. 0  .and.  ier(11) .le. mer)
     $         write(iprint, 1700) line, key, (id(i), i=1,4)
         end if
      end if
      go to 610

*     INITIAL bounds set found.

  690 if (ncard(4) .eq. 0) then
         mbnd(1) = iblank
         mbnd(2) = iblank
         if (iprint .gt. 0) write(iprint, 1720)
      end if

*     ------------------------------------------------------------------
*     End of normal bounds.
*     ------------------------------------------------------------------
  700 ns     = 0
      bplus  = 0.9*bplus
      bminus =   - bplus

*     Set variables to be nonbasic at zero (as long as that's feasible).

      do 706 j = 1, nb
         xn(j) = max( zero , bl(j) )
         xn(j) = min( xn(j), bu(j) )
         hs(j) = 0
         if (xn(j) .eq. bu(j)) hs(j) = 1
  706 continue

*     Ignore INITIAL bounds if a basis will be loaded.

      if (inform .ne. 0) go to 790
      ignore = ioldb .gt. 0  .or.  insrt .gt. 0  .or.  iload .gt. 0
      if (.not. ignore ) then
         jmark  = 1
         go to 720
      end if

*     ==================================================================
*     Read INITIAL bounds set.
*     ==================================================================
  710 call m3read( 3, imps, line, mlst, key, inform )
      if (inform .ne. 0) go to 790

      bnd    = aelem(1)
      if (ignore  .or.  id(1).ne.iinit  .or.  id(2).ne.iial) go to 710

*     Find which column.

  720 call m4name( n, name1, name2, id(3), id(4),
     $             line, ier(12), 0, 1, n, jmark, j )

      if (j .le. 0) then
         if (iprint .gt. 0  .and.  ier(12) .le. mer)
     $      write(iprint, 1400) id(3), id(4), line
      else

*        Select bound type for column j.

         ncard(6) = ncard(6)+1
         if      (key .eq. lfr) then
            js  = -1
         else if (key .eq. lfx) then
            js  =  2
            ns  = ns + 1
         else if (key .eq. llo) then
            js  =  4
            bnd = bl(j)
         else if (key .eq. lup) then
            js  =  5
            bnd = bu(j)
         else if (key .eq. lmi) then
            js  =  4
         else if (key .eq. lpl) then
            js  =  5
         else
            ier(13) = ier(13) + 1
            if (iprint .gt. 0  .and.  ier(13) .le. mer)
     $         write(iprint, 1700) line, key, (id(i), i=1,4)
            go to 710
         end if
      end if

      if (abs( bnd ) .ge. bplus) bnd = zero
      xn(j)  = bnd
      hs(j)  = js
      go to 710

*     Should be ENDATA card.

  790 if (key .ne. lenda) then
         ier(14) = 1
         if (iprint .gt. 0) write(iprint, 1150)
         if (isumm  .gt. 0) write(isumm , 1150)
      end if

*     ------------------------------------------------------------------
*     Pass the buck - not got to Truman yet.
*     ------------------------------------------------------------------

*     Check that  bl .le. bu

      do 802 j = 1, n
         b1    = bl(j)
         b2    = bu(j)
         if (b1 .gt. b2) then
            ier(20) = ier(20) + 1
            if (iprint .gt. 0  .and.  ier(20) .le. mer)
     $         write(iprint, 1740) j, b1, b2
            bl(j) = b2
            bu(j) = b1
         end if
  802 continue

*     Count the errors.

      k      = 0
      do 804 i = 1, 20
         k     = k + ier(i)
  804 continue
      if (k .gt. 0) then
         if (iprint .gt. 0) write(iprint, 1900) k
         if (isumm  .gt. 0) write(isumm , 1900) k
      end if
      if (iprint .gt. 0) then
         write(iprint, 2100) mobj, minmax, ncard(1),
     $                       mrhs, ncard(2),
     $                       mrng, ncard(3),
     $                       mbnd, ncard(4)
      end if
      return

 1150 format(' XXXX  ENDATA card not found')
 1400 format(' XXXX  Non-existent column specified -- ', 2a4,
     $   ' -- entry ignored in line', i7)
 1700 format(' XXXX  Illegal bound type at line', i7, '... ',
     $   a1, a2, a1, 2a4, 2x, 2a4)
 1720 format(' XXXX  Warning - first bounds set is  INITIAL .',
     $   '   Other bounds will be ignored.')
 1740 format(/' XXXX  Bounds back to front on column', i6,' :',
     $   1p, 2e15.5)
 1900 format(/' XXXX  Total no. of errors in MPS file', i6)
 2100 format(///
     $   ' Names selected' /
     $   ' --------------' /
     $   ' Objective', 6x, 2a4, ' (', a3, ')', i8 /
     $   ' RHS      ', 6x, 2a4, i14 /
     $   ' RANGES   ', 6x, 2a4, i14 /
     $   ' BOUNDS   ', 6x, 2a4, i14)

*     end of m3mpsc
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m3read( mode, imps, line, mxlist, key, inform )

      implicit           double precision (a-h,o-z)
      character*4        key

*     ------------------------------------------------------------------
*     m3read  reads data from file imps and prints a listing on file
*     iprint.  The data is assumed to be in MPS format, with items of
*     interest in the following six fields...
*
*     Field:     1         2         3         4         5         6
*
*     Columns: 01-04     05-12     15-22     25-36     40-47     50-61
*
*     Format:    a4       2a4       2a4      e12.0      2a4      e12.0
*
*     Data:     key     id(1-2)   id(3-4)  aelem(1)   id(5-6)  aelem(2)
*
*
*     Comments may contain a * in column 1 and anything in columns 2-61.
*     They are listed and then ignored.
*
*
*     On entry,  mode    specifies which fields are to be processed.
*     On exit ,  inform  is set to 1 if column 1 is not blank.
*
*     04 Oct 1985: First version.
*     27 Sep 1991: More f77.  Each line read into a character buffer
*                  to simplify handling of comments and key.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m3mps5/ aelem(2), id(6), iblank

      character*61       buffer
      character*1        buff1
      character*1        lblank      , lstar
      data               lblank /' '/, lstar /'*'/

*     ------------------------------------------------------------------
*     Read a data card and look for keywords and comments.
*     ------------------------------------------------------------------
   10 read (imps, 1000) buffer
      buff1  = buffer(1:1)
      line   = line + 1

*     Print the buffer if column 1 is nonblank
*     or if a listing is wanted.

      if (buff1 .ne. lblank  .or.  line .le. mxlist) then

*        Find the last nonblank character.

         do 20 last = 61, 2, -1
            if (buffer(last:last) .ne. lblank) go to 30
   20    continue
         last   = 1

   30    if (iprint .gt. 0) write(iprint, 2000) line, buffer(1:last)
      end if

*     Ignore comments.

      if (buff1 .eq. lstar ) go to 10

*     If column 1 is nonblank, load key and exit.
*     The NAME card is unusual in having some data in field 3.
*     We have to load it into id(3)-id(4).

      if (buff1 .ne. lblank) then
         read(buffer, 1100) key, id(1), id(2), id(3), id(4)
         inform = 1
         return
      end if

*     ------------------------------------------------------------------
*     Process normal data cards.
*     ------------------------------------------------------------------
      if (mode .eq. 1) then

*        NAME or ROWS sections.

         read(buffer, 1100) key, id(1), id(2), id(3), id(4)
      else if (mode .eq. 2) then

*        COLUMNS, RHS or RANGES sections.

         read(buffer, 1100) key,   id(1), id(2),
     $                      id(3), id(4), aelem(1),
     $                      id(5), id(6), aelem(2)
      else

*        BOUNDS section.

         read(buffer, 1100) key, id(1), id(2), id(3), id(4), aelem(1)
      end if

      return

 1000 format(a61)
 1100 format(a4, 2a4, 2x, 2a4, 2x, e12.0, 3x, 2a4, 2x, e12.0)
 2000 format(i7, 4x, a)

*     end of m3read
      end
************************************************************************
*
*     File  mi40bfil fortran.
*
*     m4getb   m4chek   m4id     m4name   m4inst   m4load   m4oldb
*     m4savb   m4dump   m4newb   m4pnch   m4rc     m4infs
*     m4rept   m4soln   m4solp   m4stat
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4getb( ncycle, istart, m, mbs, n, nb, nn, nname, nscl,
     $                   lcrash, ns,
     $                   ne, nka, a, ha, ka,
     $                   hrtype, hs, kb, ascale, bl, bu,
     $                   pi, xn, y, y2, name1, name2, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hrtype(mbs), hs(nb)
      integer            ka(nka), kb(mbs), name1(nname), name2(nname)
      double precision   a(ne), ascale(nscl), bl(nb), bu(nb)
      double precision   pi(m), xn(nb), y(m), y2(m), z(nwcore)

*     ------------------------------------------------------------------
*     m4getb is called with ncycle = 0 before the first cycle.
*     A basis file is loaded (if any exists).
*
*     m4getb is called with ncycle > 0 before every cycle.
*     The Jacobian is evaluated and stored in a (unscaled).
*     If gotscl is false, m2scal is called to compute scales.
*     If relevant, the scales are applied to a, bl, bu, xn, pi, fcon.
*     If gotbas is false, m2crsh is called to initialize hs.
*     (kb, y, y2  are used as workspace by m2crsh.)
*
*     In both cases, lcrash is an output parameter to tell m5solv
*     if further calls to crash are needed.
*
*     14 May 1992: pi(1:nncon) assumed to be initialized on entry.
*                  xn passed to m2crsh.
*     04 Jun 1992: lcrash added as an output parameter.
*     09 Jul 1992: istart added as an input parameter.
*                  Used when ncycle = 0 to load a basis file only if
*                  it is a cold start.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m2parm/ dparm(30),iparm(30)
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2
      common    /m8save/ vimax ,virel ,maxvi ,majits,minits,nssave
      logical            gotbas,gotfac,gothes,gotscl
      common    /cycle1/ gotbas,gotfac,gothes,gotscl

      intrinsic          max, min
      equivalence      ( iparm(1), icrash )
      logical            gotjac
      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

      lcrash = 0

      if (ncycle .eq. 0) then
*        ---------------------------------------------------------------
*        This call is made before the cycle loop.
*        ---------------------------------------------------------------

*        Initialize  lvldif = 1, nfcon(*) = 0, nfobj(*) = 0.
*        Also set linear pi(i) = 0 in case they are printed.
*        We have to do this before scales are applied anyway.
*        One day the basis files might load nonlinear pi(i).

         lvldif = 1
         call iload1( 4, 0, nfcon, 1 )
         call iload1( 4, 0, nfobj, 1 )
         if (nncon .lt. m) call dload ( m-nncon, zero, pi(nncon+1), 1 )

*        Load a basis file if one exists and istart = 0 (Cold start).

         if (istart .eq. 0) then
            if      (ioldb .gt. 0) then
               call m4oldb( m, n, nb, ns, hs, bl, bu, xn )
            else if (insrt .gt. 0) then
               call m4inst( m, n, nb, ns, hs, bl, bu, xn, name1, name2 )
            else if (iload .gt. 0) then
               call m4load( m, n, nb, ns, hs, bl, bu, xn, name1, name2 )
            end if
            nssave = ns
         end if

*        Make sure the nonlinear variables are within bounds
*        (for the gradient checkers if nothing else).

         do 160 j = 1, nn
            xn(j) = max( xn(j), bl(j) )
            xn(j) = min( xn(j), bu(j) )
  160    continue

      else
*        ---------------------------------------------------------------
*        ncycle > 0.  This call is made every cycle.
*        ---------------------------------------------------------------
         if (nncon .gt. 0) then

*           Evaluate the Jacobian at the initial  xn
*           and store it in  a(*)  for the first linearization.
*           We may have to copy some constant (unscaled) Jacobian
*           elements from gcon2 into gcon.
*           Also, we have to disable scaling temporarily, since if this
*           is the first cycle, the scales have not yet been computed.

            if (lscale .eq. 2  .and.  lderiv .ge. 2) then
               call dcopy ( njac, z(lgcon2), 1, z(lgcon), 1 )
            end if
            lssave = lscale
            lscale = 0
            gotjac = .false.
            call m8ajac( gotjac, nncon, nnjac, njac,
     $                   ne, nka, a, ha, ka,
     $                   z(lfcon), z(lfcon2), z(lgcon), z(lgcon2),
     $                   xn, y, z, nwcore )
            lscale = lssave
            if (ierr .ne. 0) go to 900
         end if

*        Compute scales from  a, bl, bu  (unless we already have them).
*        Then apply them to   a, bl, bu, pi, xn and fcon.

         if (lscale .gt. 0) then
            if (.not. gotscl) then
               gotscl = .true.
               call m2scal( m, n, nb, ne, nka, nn, nncon, nnjac,
     $                      hrtype, ha, ka, a, ascale, bl, bu, y, y2 )
            end if

            call m2scla( 1, m, n, nb, ne, nka,
     $                   ha, ka, a, ascale, bl, bu, pi, xn )

            if (lscale .eq. 2  .and.  nncon .gt. 0) then
               call dddiv ( nncon, ascale(n+1), 1, z(lfcon), 1 )
            end if
         end if

*        Initialize xlam from pi.  These are Lagrange multipliers
*        for the nonlinear constraints.
*        Do it at the very start, or on later cycles if the present
*        solution is feasible.  (ninf = 0 in both cases.
*        Keep the previous xlam if solution is infeasible.)
*        First change the sign of pi if maximizing.
*
*        If pi(1:nncon) seems ridiculously big, assume that it
*        has not been correctly initialized and just use xlam = 0.
                          
         if (ninf .eq. 0  .and.  nncon .gt. 0) then
            if (minimz .lt. 0) then
               call dscal ( nncon, -one, pi, 1 )
            end if
            toobig = 1.0d+10
            imax   = idamax( nncon, pi, 1 )
            xlmax  = abs( pi(imax) )
            if (xlmax .lt. toobig) then
               call dcopy ( nncon, pi, 1, z(lxlam), 1 )
            else
               call dload ( nncon,  zero, z(lxlam), 1 )
               if (iprint .gt. 0) write(iprint, 1000)
               if (isumm  .gt. 0) write(isumm , 1000)
            end if
         end if

*        ---------------------------------------------------------------
*        If there was no basis file, find an initial basis via Crash.
*        This works best with A scaled (hence much of the complication).
*        xn(1:n) is input.  xn(n+1:nb) is initialized by m2crsh.
*        ---------------------------------------------------------------
         if (.not. gotbas) then
            gotbas = .true.
            if (icrash .gt. 0  .and.  icrash .le. 3) lcrash = icrash

            call m2crsh( lcrash, m, n, nb, nn,
     $                   ne, nka, a, ha, ka,
     $                   kb, hs, hrtype, bl, bu, xn, z, nwcore )
         end if
      end if

*     Exit.

  900 return

 1000 format(' XXX  pi(1:nncon) is big  (perhaps not initialized).'
     $     / ' XXX  Will set lambda = 0 for nonlinear rows.')
*     end of m4getb
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4chek( m, maxs, mbs, n, nb, ns,
     $                   hs, kb, bl, bu, xn )

      implicit           double precision (a-h,o-z)
      integer*4          hs(nb)
      integer            kb(mbs)
      double precision   bl(nb), bu(nb)
      double precision   xn(nb)

*     ------------------------------------------------------------------
*     m4chek  takes hs and xn and checks they contain reasonable values.
*     The entries hs(j) = 2 are used to set ns and nssave and possibly
*     the list of superbasic variables kb(m+1) thru kb(m+ns).
*     Scaling, if any, has taken place by this stage.
*
*     If gotbas and gothes are both true, nssave and the superbasic kb's
*     are assumed to be set.  It must be a Hot start, or ncycle > 1.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m8save/ vimax ,virel ,maxvi ,majits,minits,nssave
      logical            gotbas,gotfac,gothes,gotscl
      common    /cycle1/ gotbas,gotfac,gothes,gotscl

      intrinsic          abs, max, min
      logical            setkb
      parameter        ( zero = 0.0d+0,  tolb = 1.0d-4 )

*     Make sure hs(j) = 0, 1, 2 or 3 only.

      do 5 j = 1, nb
         js  = hs(j)
         if (js .lt. 0) hs(j) = 0
         if (js .ge. 4) hs(j) = js - 4
    5 continue

      setkb  = .not. (gotbas .and. gothes)

*     ------------------------------------------------------------------
*     Make sure the objective is basic and free.
*     Then count the basics and superbasics, making sure they don't
*     exceed m and maxs respectively.  Also, set ns and possibly
*     kb(m+1) thru kb(m+ns) to define the list of superbasics.
*     Mar 1988: Loop 100 now goes backwards to make sure we grab obj.
*     Apr 1992: Backwards seems a bit silly in the documentation.
*               We now go forward through the slacks,
*               then forward through the columns.
*     ------------------------------------------------------------------
   10 nbasic = 0
      ns     = 0
      if (iobj .gt. 0) then
          jobj     =   n + iobj
          hs(jobj) =   3
          bl(jobj) = - plinfy
          bu(jobj) =   plinfy
      end if

*     If too many basics or superbasics, make them nonbasic.
*     Do slacks first to make sure we grab the objective slack.

      j = n

      do 100 jj = 1, nb
         j      = j + 1
         if (j .gt. nb) j = 1
         js     = hs(j)
         if (js .eq. 2) then
            ns     = ns + 1
            if (ns .le. maxs) then
               if ( setkb ) kb(m + ns) = j
            else
               hs(j) = 0
            end if

         else if (js .eq. 3) then
            nbasic = nbasic + 1
            if (nbasic .gt. m) hs(j) = 0
         end if
  100 continue

*     Proceed if the superbasic kbs were reset, or if ns seems to
*     agree with nssave from the previous cycle.
*     Otherwise, give up trying to save the projected Hessian, and
*     reset the superbasic kbs after all.

      if (setkb) then
*        ok
      else if (ns .ne. nssave) then
         setkb  = .true.
         gothes = .false.
         if (iprint .gt. 0) write(iprint, 1000) ns, nssave
         if (isumm  .gt. 0) write(isumm , 1000) ns, nssave
         go to 10
      end if

*     Check the number of basics.

      ns     = min( ns, maxs )
      nssave = ns
      if (nbasic .ne. m ) then
         gothes = .false.
         if (iprint .gt. 0) write(iprint, 1100) nbasic, m
         if (isumm  .gt. 0) write(isumm , 1100) nbasic, m
      end if

*     -----------------------------------------------------------
*     On all cycles, set each nonbasic xn(j) to be exactly on its
*     nearest bound if it is within tolb of that bound.
*     -----------------------------------------------------------
      bplus  = 0.1*plinfy
      do 300 j = 1, nb
         xj    = xn(j)
         if (abs( xj ) .ge.  bplus) xj = zero
         if (hs(j)     .le.  1    ) then
            b1    = bl(j)
            b2    = bu(j)
            xj    = max( xj, b1 )
            xj    = min( xj, b2 )
            if ((xj - b1) .gt. (b2 - xj)) b1 = b2
            if (abs(xj - b1)  .le.  tolb) xj = b1
            hs(j) = 0
            if (xj .gt. bl(j)) hs(j) = 1
         end if
         xn(j) = xj
  300 continue

      return

 1000 format(/ ' WARNING:', i6, ' superbasics in hs(*);',
     $         ' previously ns =', i6, '.  Hessian not saved')
 1100 format(/ ' WARNING:', i7, ' basics specified;',
     $         ' preferably should have been', i7)

*     end of m4chek
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4id  ( j, m, n, nb, nname, name1, name2, id1, id2 )

      integer            name1(nname), name2(nname)

*     ------------------------------------------------------------------
*     m4id   returns a name id1-id2 for the j-th variable.
*     If nname = nb, the name is already in name1, name2.
*     Otherwise nname = 1. Some generic column or row name is cooked up
*     ------------------------------------------------------------------

      character*7        f1
      character*5        f2
      character*1        cname
      character*1        rname
      character*8        gname
      data               f1    /'(a1,i7)'/
      data               f2    /'(2a4)'/
      data               cname /'x'/
      data               rname /'r'/

      if (nname .eq. nb) then
         id1 = name1(j)
         id2 = name2(j)
      else if (j .le. n) then
         write(gname, f1) cname, j
         read (gname, f2) id1, id2
      else
         i   = j - n
         write(gname, f1) rname, i
         read (gname, f2) id1, id2
      end if

*     end of m4id
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4name( n, name1, name2, id1, id2,
     $                   ncard, notfnd, maxmsg, j1, j2, jmark, jfound )

      implicit           double precision (a-h,o-z)
      integer            name1(n), name2(n)

*     ------------------------------------------------------------------
*     m4name searches for name1-name2 in arrays name1-2(j), j = j1, j2.
*     jmark  will probably speed the search on the next entry.
*     Used by subroutines m3mpsc, m4inst, m4load.
*
*     Left-justified alphanumeric data is being tested for a match.
*     On Burroughs B6700-type systems, one could replace .eq. by .is.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm

      do 50 j = jmark, j2
         if (id1 .eq. name1(j)  .and.  id2 .eq. name2(j)) go to 100
   50 continue

      do 60 j = j1, jmark
         if (id1 .eq. name1(j)  .and.  id2 .eq. name2(j)) go to 100
   60 continue

*     Not found.

      jfound = 0
      jmark  = j1
      notfnd = notfnd + 1
      if (notfnd .le. maxmsg) then
         if (iprint .gt. 0) write(iprint, 1000) ncard, id1, id2
      end if
      return

*     Got it.

  100 jfound = j
      jmark  = j
      return

 1000 format(' XXX  Line', i6, '  --  name not found:', 8x, 2a4)

*     end of m4name
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4inst( m, n, nb, ns,
     $                   hs, bl, bu, xn, name1, name2 )

      implicit           double precision (a-h,o-z)
      integer*4          hs(nb)
      integer            name1(nb), name2(nb)
      double precision   bl(nb), bu(nb)
      double precision   xn(nb)

*     ------------------------------------------------------------------
*     This impression of INSERT reads a file produced by  m4pnch.
*     It is intended to read files similar to those produced by
*     standard MPS systems.  It recognizes SB as an additional key.
*     Also, values are extracted from columns 25--36.
*
*     17 May 1992: John Stone (Ketron) mentioned trouble if rows and
*                  columns have the same name.  The quick fix is to
*                  search column names from the beginning always,
*                  rather than from position jmark.  Just set jmark = 1.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj

      intrinsic          abs
      integer            id(5)
      character*4        key
      character*4        lll         , lul         , lxl         ,
     $                   lxu         , lsb         , lend
      data               lll /' LL '/, lul /' UL '/, lxl /' XL '/,
     $                   lxu /' XU '/, lsb /' SB '/, lend/'ENDA'/

      bplus  = 0.9*plinfy
      if (iprint .gt. 0) write(iprint, 1999) insrt
      if (isumm  .gt. 0) write(isumm , 1999) insrt
                         read (insrt , 1000) id
      if (iprint .gt. 0) write(iprint, 2000) id
      l1    = n + 1

*     Make logicals basic.

      do 20 j  = l1, nb
         hs(j) = 3
   20 continue

      ignord = 0
      nbs    = 0
      ns     = 0
      notfnd = 0
      ncard  = 0
*     jmark  = 1
      lmark  = l1
      ndum   = n + 100000

*     Read names until ENDATA

      do 300 nloop = 1, ndum
         read(insrt, 1020) key, name1c, name2c, name1r, name2r, xj
         if (key .eq. lend) go to 310

*        Look for name1.  It may be a column or a row,
*        since a superbasic variable could be either.
*        17 May 1992: Set jmark = 1 so columns are searched first.
*        This avoids trouble when columns and rows have the same name.

         ncard  = nloop
         jmark  = 1
         call m4name( nb, name1, name2, name1c, name2c,
     $                ncard, notfnd, mer, 1, nb, jmark, j )
         if (   j  .le. 0) go to 300
         if (hs(j) .gt. 1) go to 290

         if (key .eq. lxl  .or.  key .eq. lxu) then
*           ------------------------------------------------------------
*           XL, XU (exchange card) -- make col j basic, row l nonbasic.
*           ------------------------------------------------------------
                  
*           Look for name2.  It has to be a row.
         
            call m4name( nb, name1, name2, name1r, name2r,
     $                   ncard, notfnd, mer, l1, nb, lmark, l )
            if (l  .le.  0  ) go to 300
            if (l  .eq. jobj) go to 290
            if (hs(l) .ne. 3) go to 290

            nbs    = nbs + 1
            hs(j)  = 3
            if (key .eq. lxl) then
               hs(l) = 0
               if (bl(l) .gt. -bplus) xn(l) = bl(l)
            else
               hs(l) = 1
               if (bu(l) .lt.  bplus) xn(l) = bu(l)
            end if

*        ---------------------------------------------------------------
*        else LL, UL, SB  --  only  j  and  xj  are relevant.
*        ---------------------------------------------------------------
         else if (key .eq. lll) then
            hs(j) = 0
         else if (key .eq. lul) then
            hs(j) = 1
         else if (key .eq. lsb) then
            hs(j) = 2
            ns    = ns + 1
         else
            go to 290
         end if

*        Save xj.

         if (abs(xj) .lt. bplus) xn(j) = xj
         go to 300

*        Card ignored.

  290    ignord = ignord + 1
         if (iprint .gt. 0  .and.  ignord .le. mer) then
            write(iprint, 2010) ncard, key, name1c,name2c,name1r,name2r
         end if
  300 continue

  310 ignord = ignord + notfnd
      if (iprint .gt. 0) write(iprint, 2050) ncard, ignord, nbs, ns
      if (isumm  .gt. 0) write(isumm , 2050) ncard, ignord, nbs, ns
      if (insrt  .ne. iread) rewind insrt
      return

 1000 format(14x, 2a4, 2x, 3a4)
 1005 format(2a4)
 1020 format(3a4, 2x, 2a4, 2x, e12.5)
 1999 format(/ ' INSERT file to be input from file', i4)
 2000 format(/ ' NAME', 10x, 2a4, 2x, 3a4)
 2010 format(' XXX  Line', i6, '  ignored:', 8x, 3a4, 2x, 2a4)
 2050 format(/ ' No. of lines read      ', i6, '  Lines ignored', i6
     $       / ' No. of basics specified', i6, '  Superbasics  ', i6)

*     end of m4inst
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4load( m, n, nb, ns,
     $                   hs, bl, bu, xn, name1, name2 )

      implicit           double precision (a-h,o-z)
      integer*4          hs(nb)
      integer            name1(nb), name2(nb)
      double precision   bl(nb), bu(nb)
      double precision   xn(nb)

*     ------------------------------------------------------------------
*     m4load  inputs a load file, which may contain a full or partial
*     list of row and column names and their states and values.
*     Valid keys are   BS, LL, UL, SB.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m3mps3/ aijtol,bstruc(2),mlst,mer,
     $                   aijmin,aijmax,na0,line,ier(20)
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj

      intrinsic          abs
      integer            id(5)
      character*4        key
      character*4        lbs         , lll         , lul         ,
     $                   lsb         , lend
      data               lbs /' BS '/, lll /' LL '/, lul /' UL '/,
     $                   lsb /' SB '/, lend/'ENDA'/

      bplus  = 0.9*plinfy
      if (iprint .gt. 0) write(iprint, 1999) iload
      if (isumm  .gt. 0) write(isumm , 1999) iload
                         read (iload , 1000) id
      if (iprint .gt. 0) write(iprint, 2000) id
      l1     = n + 1
      ignord = 0
      nbs    = 0
      ns     = 0
      notfnd = 0
      ncard  = 0
      jmark  = 1
      ndum   = n + 100000

*     Read names until ENDATA is found.

      do 300 nloop = 1, ndum
         read (iload, 1020) key, id1, id2, xj
         if (key .eq. lend) go to 310

         ncard  = nloop
         call m4name( nb, name1, name2, id1, id2,
     $                ncard, notfnd, mer, 1, nb, jmark, j )
         if (j .le. 0) go to 300

*        The name id1-id2 belongs to the j-th variable.

         if (hs(j) .gt. 1) go to 290
         if (j   .eq.jobj) go to  90
         if (key .eq. lbs) go to  90
         if (key .eq. lll) go to 100
         if (key .eq. lul) go to 150
         if (key .eq. lsb) go to 200
         go to 290

*        Make basic.

   90    nbs    = nbs + 1
         hs(j)  = 3
         go to 250

*        LO or UP.

  100    hs(j)  = 0
         go to 250

  150    hs(j)  = 1
         go to 250

*        Make superbasic.

  200    ns     = ns + 1
         hs(j)  = 2

*        Save  x  values.

  250    if (abs(xj) .lt. bplus) xn(j) = xj
         go to 300

*        Card ignored.

  290    ignord = ignord + 1
         if (ignord .le. mer) then
            if (iprint .gt. 0) write(iprint, 2010) ncard, key, id1, id2
         end if
  300 continue

  310 ignord = ignord + notfnd
      if (iprint .gt. 0) write(iprint, 2050) ncard, ignord, nbs, ns
      if (isumm  .gt. 0) write(isumm , 2050) ncard, ignord, nbs, ns

*     Make sure the linear objective is basic.

      if (iobj  .gt. 0  .and.  hs(jobj) .ne. 3) then
         hs(jobj) = 3

*        Swap obj with last basic variable.

         do 850 j = nb, 1, -1
            if (hs(j) .eq. 3) go to 860
  850    continue

  860    hs(j)  = 0
      end if

      if (iload .ne. iread) rewind iload
      return

 1000 format(14x, 2a4, 2x, 3a4)
 1005 format(2a4)
 1020 format(3a4, 12x, e12.5)
 1999 format(/ ' LOAD file to be input from file', i4)
 2000 format(/ ' NAME', 10x, 2a4, 2x, 3a4)
 2010 format(' XXX  Line', i6, '  ignored:', 8x, 3a4, 2x, 2a4)
 2050 format(/ ' No. of lines read      ', i6, '  Lines ignored', i6
     $       / ' No. of basics specified', i6, '  Superbasics  ', i6)

*     end of m4load
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4oldb( m, n, nb, ns,
     $                   hs, bl, bu, xn )

      implicit           double precision (a-h,o-z)
      integer*4          hs(nb)
      double precision   bl(nb), bu(nb)
      double precision   xn(nb)

*     ------------------------------------------------------------------
*     m4oldb  inputs a compact basis file from file  ioldb.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m5log1/ idebug,ierr,lprint

      character*80       id

      bplus  = 0.9*plinfy
      if (iprint .gt. 0) write(iprint, 1999) ioldb
      if (isumm  .gt. 0) write(isumm , 1999) ioldb
         read (ioldb , 1000) id
      if (iprint .gt. 0) then
         write(iprint, 2000) id
      end if

         read (ioldb , 1005) id(1:52), newm, newn, ns
      if (iprint .gt. 0) then
         write(iprint, 2005) id(1:52), newm, newn, ns
      end if

      if (newm .ne. m  .or.  newn .ne. n) go to 900
      read (ioldb , 1010) hs

*     Set values for nonbasic variables.

      do 200 j = 1, nb
         js    = hs(j)
         if (js .le. 1) then
            if (js .eq. 0) xj = bl(j)
            if (js .eq. 1) xj = bu(j)
            if (abs(xj) .lt. bplus) xn(j) = xj
         end if
  200 continue

*     Load superbasics.

      ns     = 0
      ndummy = m + n + 10000

      do 300 idummy = 1, ndummy
         read(ioldb, 1020, end=310) j, xj
         if (j .le.  0) go to  310
         if (j .le. nb) then
            xn(j)  = xj
            if (hs(j) .eq. 2) ns = ns + 1
         end if
  300 continue

  310 if (ns .gt. 0) then
         if (iprint .gt. 0) write(iprint, 2010) ns
         if (isumm  .gt. 0) write(isumm , 2010) ns
      end if
      go to 990

*     Error exits.

  900 call m1page( 1 )
      if (iprint .gt. 0) write(iprint, 3000)
      if (isumm  .gt. 0) write(isumm , 3000)
      ierr   = 30

  990 if (ioldb .ne. iread) rewind ioldb
      return

 1000 format(a80)
 1005 format(a52, 2x, i7, 3x, i7, 4x, i5)
 1010 format(80i1)
 1020 format(i8, e24.14)
 1999 format(/ ' OLD BASIS file to be input from file', i4)
 2000 format(1x, a80)
 2005 format(1x, a52, 'M=', i7, ' N=', i7, ' SB=', i5)
 2010 format(' No. of superbasics loaded', i7)
 3000 format(' EXIT -- the basis file dimensions do not match',
     $   ' this problem')

*     end of m4oldb
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4savb( mode, m, mbs, n, nb, nn, nname, nscl, msoln,ns,
     $                   ne, nka, a, ha, ka,
     $                   hs, kb, ascale, bl, bu,
     $                   name1, name2, pi, rc, xn, y, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hs(nb)
      integer            ka(nka), kb(mbs), name1(nname), name2(nname)
      double precision   a(ne), ascale(nscl), bl(nb), bu(nb)
      double precision   pi(m), rc(nb), xn(nb), y(m), z(nwcore)

*     ------------------------------------------------------------------
*     m4savb  saves basis files  and/or  prints the solution.
*
*     If mode = 1, the problem is first unscaled, then from 0 to 4 files
*     are saved (PUNCH file, DUMP file, SOLUTION file, REPORT file,
*     in that order).
*     A new BASIS file, if any, will already have been saved by m5solv.
*     A call with mode = 1 must precede a call with mode = 2.
*
*     If mode = 2, the solution is printed under the control of msoln
*     (which is set by the Solution keyword in the SPECS file).
*
*     18 Nov 1991: Scaled pinorm saved for use in m4soln.
*     25 Nov 1991: rc added as parameter to return reduced costs.
*     31 Jan 1991: Call m4rc   to get the reduced costs.
*     18 Dec 1992: Maximum primal and dual infeasibilities computed
*                  and printed here.
*     ------------------------------------------------------------------

      logical            alone, AMPL, GAMS, MINT, page1, page2
      common    /m1env / alone, AMPL, GAMS, MINT, page1, page2
      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5inf / prinf, duinf, jprinf, jduinf
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m5step/ featol, tolx0,tolinc,kdegen,ndegen,
     $                   itnfix, nfix(2)
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m7loc / lgobj ,lgobj2
      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold
      common    /m8save/ vimax ,virel ,maxvi ,majits,minits,nssave

      intrinsic          max
      character*12       istate
      logical            feasbl, prnt
      save               pnorm1, pnorm2
      parameter        ( one = 1.0d+0 )

      feasbl = ninf .eq. 0
      ms     = m + ns
      k      = ierr + 1
      call m4stat( k, istate )

      if (mode .eq. 1) then
*        ---------------------------------------------------------------
*        mode = 1.
*        Compute rc and unscale everything.
*        Then save basis files.
*        ---------------------------------------------------------------

*        Compute reduced costs rc(*) for all columns and rows.
*        Find the maximum primal and dual infeasibilities.

         call m4rc  ( feasbl, featol, minimz,
     $                m, n, nb, nnobj, nnobj0,
     $                ne, nka, a, ha, ka,
     $                hs, bl, bu, z(lgobj), pi, rc, xn )
         call m4infs( nb, jobj, bl, bu, rc, xn )

*        Unscale a, bl, bu, pi, xn, rc, fcon, gobj and xnorm, pinorm.
*        (m4soln uses scaled pinorm, so save it.)

         pnorm1 = pinorm
         xnorm1 = xnorm
         jpinf1 = jprinf
         jdinf1 = jduinf
         prinf1 = prinf
         duinf1 = duinf

         if (lscale .gt. 0) then
            call m2scla( 2, m, n, nb, ne, nka,
     $                   ha, ka, a, ascale, bl, bu, pi, xn )

            call dddiv ( nb, ascale, 1, rc, 1 )

            if (lscale .eq. 2) then
               if (nncon .gt. 0)
     $         call ddscl ( nncon, ascale(n+1), 1, z(lfcon), 1 )

               if (nnobj .gt. 0)
     $         call dddiv ( nnobj, ascale     , 1, z(lgobj), 1 )
            end if

*           Call m5setp to redefine pinorm.  y is not used.

            xnorm  = dnorm1( nb, xn, 1 )
            call m5setp( 3, m, y, pi, z, nwcore )
            call m4infs( nb, jobj, bl, bu, rc, xn )
         end if
         pnorm2 = pinorm

*        ---------------------------------------------------------------
*        Print various scaled and unscaled norms.
*        ---------------------------------------------------------------
         if (lscale .gt. 0) then
            if (iprint .gt. 0) write(iprint, 1010) xnorm1, pnorm1
            if (isumm  .gt. 0) write(isumm , 1010) xnorm1, pnorm1
         end if
            if (iprint .gt. 0) write(iprint, 1020) xnorm , pinorm
            if (isumm  .gt. 0) write(isumm , 1020) xnorm , pinorm
         if (lscale .gt. 0) then
            if (iprint .gt. 0) write(iprint, 1030) jpinf1, prinf1,
     $                                             jdinf1, duinf1
            if (isumm  .gt. 0) write(isumm , 1030) jpinf1, prinf1,
     $                                             jdinf1, duinf1
         end if
            if (iprint .gt. 0) write(iprint, 1040) jprinf, prinf,
     $                                             jduinf, duinf
            if (isumm  .gt. 0) write(isumm , 1040) jprinf, prinf,
     $                                             jduinf, duinf

*        Change the sign of pi and rc if feasible and maximizing.

         if (ninf .eq. 0  .and.  minimz .lt. 0) then
            call dscal ( m , -one, pi, 1 )
            call dscal ( nb, -one, rc, 1 )
         end if

*        Compute nonlinear constraint violations.

         if (nncon .gt. 0) then
            call m8viol( n, nb, nncon,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, z(lfcon), xn, y, z, nwcore )
            if (iprint .gt. 0) write(iprint, 1080) vimax, virel
            if (isumm  .gt. 0) write(isumm , 1080) vimax, virel
         end if

*        ---------------------------------------------------------------
*        Output PUNCH, DUMP, SOLUTION and/or REPORT files.
*        ---------------------------------------------------------------
         if (ipnch .gt. 0)
     $   call m4pnch( ipnch, n, nb, hs, bl, bu, xn,
     $                name1, name2 )

         if (idump .gt. 0)
     $   call m4dump( idump, n, nb, hs, bl, bu, xn,
     $                name1, name2 )

         pinorm = pnorm1
         if (isoln .gt. 0)
     $      call m4soln( .true., m, n, nb, nname, nscl,
     $                nn, nnobj, nnobj0, ns,
     $                ne, nka, a, ha, ka,
     $                hs, ascale, bl, bu,
     $                z(lgobj), pi, rc, xn, y,
     $                name1, name2, istate, z, nwcore )

         if (ireprt .gt. 0)
     $   call m4rept( .true., m, n, nb, nname, nscl,
     $                nn, nnobj, nnobj0, ns,
     $                ne, nka, a, ha, ka,
     $                hs, ascale, bl, bu,
     $                z(lgobj), pi, rc, xn, y,
     $                name1, name2, istate, z, nwcore )
         pinorm = pnorm2
      else
*        ---------------------------------------------------------------
*        mode = 2.    Print solution if requested.
*
*        msoln  = 0   means   no
*               = 1   means   if optimal, infeasible or unbounded
*               = 2   means   yes
*               = 3   means   if error condition
*        ---------------------------------------------------------------
         prnt   = iprint .gt. 0  .and.  msoln .gt. 0
         if ((msoln .eq. 1  .and.  ierr .gt. 2)  .or.
     $       (msoln .eq. 3  .and.  ierr .le. 2)) prnt = .false.
         if ( prnt ) then
            pinorm = pnorm1
            call m4soln( .false., m, n, nb, nname, nscl,
     $                   nn, nnobj, nnobj0, ns,
     $                   ne, nka, a, ha, ka,
     $                   hs, ascale, bl, bu,
     $                   z(lgobj), pi, rc, xn, y,
     $                   name1, name2, istate, z, nwcore )
            pinorm = pnorm2
            if (isumm  .gt. 0) write(isumm, 1200) iprint

         else if (.not. (GAMS .or. AMPL)) then
            if (isumm  .gt. 0) write(isumm, 1300)
         end if
      end if

      return

 1010 format(  ' Norm of x  (scaled)  ', 1p, e16.3,
     $     2x, ' Norm of pi (scaled)  ',     e16.3)
 1020 format(  ' Norm of x ', 1p, e27.3,
     $     2x, ' Norm of pi',     e27.3)
 1030 format(  ' Primal inf (scaled)', i7, 1p, e11.1,
     $     2x, ' Dual inf   (scaled)', i7,     e11.1)
 1040 format(  ' Primal infeas      ', i7, 1p, e11.1,
     $     2x, ' Dual infeas        ', i7,     e11.1)
 1080 format(  ' Constraint violation ', 1p, e16.3,
     $     2x, ' Normalized           ',     e16.3)
 1100 format(2a4)
 1200 format(/ ' Solution printed on file', i4)
 1300 format(/ ' Solution not printed')

*     end of m4savb
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4dump( idump, n, nb, hs, bl, bu, xn, name1, name2 )

      implicit           double precision (a-h,o-z)
      integer*4          hs(nb)
      integer            name1(nb), name2(nb)
      double precision   bl(nb), bu(nb)
      double precision   xn(nb)

*     ------------------------------------------------------------------
*     m4dump outputs basis names in a format compatible with m4load.
*     This file is normally easier to modify than a punch file.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax

      character*4        key(4)
      data               key   /' LL ', ' UL ', ' SB ', ' BS '/

      write(idump, 2000) name

      do 500 j = 1, nb
         k     = hs(j) + 1
         write(idump, 2100) key(k), name1(j), name2(j), xn(j)
  500 continue

      write(idump , 2200)
      if (iprint .gt. 0) write(iprint, 3000) idump
      if (isumm  .gt. 0) write(isumm , 3000) idump
      if (idump .ne. iprint) rewind idump
      return

 2000 format('NAME', 10x, 2a4, 2x, '   DUMP/LOAD')
 2100 format(3a4, 12x, 1p, e12.5)
 2200 format('ENDATA')
 3000 format(/ ' DUMP file saved on file', i4)

*     end of m4dump
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4newb( mode, inewb, m, n, nb, nn, ns, ms, nscl, fsub,
     $                   kb, hs, ascale, bl, bu, x, xn, istate )

      implicit           double precision (a-h,o-z)
      integer*4          hs(nb)
      integer            kb(ms)
      double precision   ascale(nscl), bl(nb), bu(nb)
      double precision   x(ms), xn(nb)
      character*12       istate

*     ------------------------------------------------------------------
*     m4newb  saves a compact basis on file inewb.  Called from m5solv.
*     If mode = 1, the save is a periodic one due to the save frequency.
*     If mode = 2, m5solv has just finished the current problem.
*                 (Hence, mode 2 calls occur at the end of every cycle.)
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m8len / njac  ,nncon ,nncon0,nnjac

      logical            scaled

      scaled = lscale .gt. 0
      obj    = sinf
      if (ninf .eq. 0) obj = minimz * fsub

*     Output header cards and the state vector.

      write(inewb, 1000) name, itn, istate, ninf, obj
      write(inewb, 1005) mobj, mrhs, mrng, mbnd, m, n, ns
      write(inewb, 1010) hs

*     Output the superbasic variables.

      do 50 k = m + 1, ms
         j    = kb(k)
         xj   = x(k)
         if (scaled) xj = xj * ascale(j)
         write(inewb, 1020) j, xj
   50 continue

*     If there are nonlinear constraints,
*     output the values of all other (non-sb) nonlinear variables.

      do 100 j = 1, nnjac
         if (hs(j) .ne. 2) then
            xj    = xn(j)
            if (scaled) xj = xj * ascale(j)
            write(inewb, 1020) j, xj
         end if
  100 continue

*     Output nonbasic variables that are not at a bound.
*     Ignore ones that the EXPAND anti-cycling procedure
*     has put slightly outside their bound.

      do 250 j = nnjac + 1, nb
         if (hs(j) .le. 1 ) then
            xj    = xn(j)
            if (xj .le. bl(j)) go to 250
            if (xj .ge. bu(j)) go to 250
            if (scaled) xj = xj * ascale(j)
            write(inewb, 1020) j, xj
         end if
  250 continue

*     Terminate the list with a zero.

      j     = 0
      write(inewb, 1020) j
      if (inewb .ne. iprint) rewind inewb
      if (iprint .gt. 0) write(iprint, 1030) inewb, itn
      if (isumm  .gt. 0) write(isumm , 1030) inewb, itn
      return

 1000 format(2a4, '  ITN', i8, 4x, a12, '  NINF', i7,
     $       '      OBJ', 1p, e21.12)
 1005 format('OBJ=', 2a4, ' RHS=', 2a4, ' RNG=', 2a4, ' BND=', 2a4,
     $       ' M=', i7,  ' N=', i7, ' SB=', i5)
 1010 format(80i1)
 1020 format(i8, 1p, e24.14)
 1030 format(/ ' NEW BASIS file saved on file', i4, '    itn =', i7)

*     end of m4newb
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4pnch( ipnch, n, nb, hs, bl, bu, xn, name1, name2 )

      implicit           double precision (a-h,o-z)
      integer*4          hs(nb)
      integer            name1(nb), name2(nb)
      double precision   bl(nb), bu(nb)
      double precision   xn(nb)

*     ------------------------------------------------------------------
*     m4pnch  outputs a PUNCH file (list of basis names, states and
*     values) in a format that is compatible with MPS/360.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax

      parameter        ( zero = 0.0d+0 )
      character*4        key(5), ibl
      data               key    /' LL ', ' UL ', ' SB ', ' XL ', ' XU '/
      data               ibl    /'    '/

      write(ipnch, 2000) name
      irow   = n

      do 500  j = 1, n
         id1    = name1(j)
         id2    = name2(j)
         k      = hs(j)

         if (k .eq. 3) then

*           Basics -- find the next row that isn't basic.

  300       irow   = irow + 1
            if (irow .le. nb) then
               k      = hs(irow)
               if (k .eq. 3) go to 300

               if (k .eq. 2) k = 0
               write(ipnch, 2100) key(k+4), id1, id2,
     $                            name1(irow), name2(irow), xn(j)
            end if
         else

*           Skip nonbasic variables with zero lower bounds.

            if (k .le. 1) then
               if (bl(j) .eq. zero  .and.  xn(j) .eq. zero) go to 500
            end if
            write(ipnch, 2100) key(k+1), id1, id2, ibl, ibl, xn(j)
         end if
  500 continue

*     Output superbasic slacks.

      do 700 j = n + 1, nb
         if (hs(j) .eq. 2)
     $   write(ipnch, 2100) key(3), name1(j), name2(j), ibl, ibl, xn(j)
  700 continue

      write(ipnch , 2200)
      if (iprint .gt. 0) write(iprint, 3000) ipnch
      if (isumm  .gt. 0) write(isumm , 3000) ipnch
      if (ipnch .ne. iprint) rewind ipnch
      return

 2000 format('NAME', 10x, 2a4, 2x, 'PUNCH/INSERT')
 2100 format(3a4, 2x, 2a4, 2x, 1p, e12.5)
 2200 format('ENDATA')
 3000 format(/ ' PUNCH file saved on file', i4)

*     end of m4pnch
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4rc  ( feasbl, featol, minimz,
     $                   m, n, nb, nnobj, nnobj0,
     $                   ne, nka, a, ha, ka,
     $                   hs, bl, bu, gobj, pi, rc, xn )

      implicit           double precision (a-h,o-z)
      logical            feasbl
      integer*4          ha(ne), hs(nb)
      integer            ka(nka)
      double precision   a(ne), bl(nb), bu(nb)
      double precision   gobj(nnobj0), pi(m), rc(nb), xn(nb)

*     ------------------------------------------------------------------
*     m4rc   computes reduced costs rc(*) for all columns of ( A  I ).
*     If xn is feasible, the true nonlinear objective gradient gobj(*)
*     is used (not the gradient of the augmented Lagrangian).
*     Otherwise, the Phase-1 objective is included.
*
*     m4rc   is called by m4savb BEFORE unscaling.
*     External values of hs(*) are used (0, 1, 2, 3),
*     but internal would be ok too since we only test for > 1.
*
*     31 Jan 1992: First version.
*     ------------------------------------------------------------------

      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

      do 300 j = 1, n
         dj    = zero
         do 200 k = ka(j), ka(j+1) - 1
            i     = ha(k)
            dj    = dj  +  pi(i) * a(k)
  200    continue
         rc(j) = - dj
  300 continue

      do 320 i = 1, m
         rc(n+i) = - pi(i)
  320 continue

      if (feasbl  .and.  nnobj .gt. 0) then

*        Include the nonlinear objective gradient.

         sgnobj = minimz
         call daxpy ( nnobj, sgnobj, gobj, 1, rc, 1 )
      else

*        Include the Phase 1 objective.
*        Only basics and superbasics can be infeasible.

         do 500 j = 1, nb
            if (hs(j) .gt. 1) then
               d1 = bl(j) - xn(j)
               d2 = xn(j) - bu(j)
               if (d1 .gt. featol) rc(j) = rc(j) - one
               if (d2 .gt. featol) rc(j) = rc(j) + one
            end if
  500    continue
      end if

*     end of m4rc
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4infs( nb, jobj, bl, bu, rc, xn )

      implicit           double precision (a-h,o-z)
      double precision   bl(nb), bu(nb), rc(nb), xn(nb)

*     ------------------------------------------------------------------
*     m4infs computes the maximum primal and dual infeasibilities,
*     using bl, bu, rc and xn.
*
*     m4infs is called by m4savb before and after unscaling.
*
*     18 Dec 1992: First version.
*                  prinf , duinf  return the max primal and dual infeas.
*                  jprinf, jduinf return the corresponding column nos.
*     ------------------------------------------------------------------

      common    /m5inf / prinf, duinf, jprinf, jduinf
      parameter        ( zero = 0.0d+0 )

      jprinf = 0
      jduinf = 0
      prinf  = zero
      duinf  = zero
      
      do 500 j = 1, nb
         d1  = bl(j) - xn(j)
         d2  = xn(j) - bu(j)
         if (prinf .lt. d1) then
             prinf  =   d1
             jprinf =   j
         end if
         if (prinf .lt. d2) then
             prinf  =   d2
             jprinf =   j
         end if

         if (   j  .eq. jobj ) go to 500
         if (bl(j) .eq. bu(j)) go to 500

         dj     = rc(j)
         if      (d1 .ge. zero) then
            dj  = - dj
         else if (d2 .le. zero) then
*           dj  = + dj
         else
            dj  = abs( dj )
         end if
         
         if (duinf .lt. dj) then
             duinf   =  dj
             jduinf  =  j
         end if
  500 continue

*     end of m4infs
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4rept( ondisk, m, n, nb, nname, nscl,
     $                   nn, nnobj, nnobj0, ns,
     $                   ne, nka, a, ha, ka,
     $                   hs, ascale, bl, bu, gobj, pi, rc, xn, y,
     $                   name1, name2, istate, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            ondisk
      integer*4          ha(ne), hs(nb)
      integer            ka(nka), name1(nname), name2(nname)
      character*12       istate
      double precision   a(ne), ascale(nscl), bl(nb), bu(nb)
      double precision   gobj(nnobj0), pi(m), rc(nb), xn(nb), y(m),
     $                   z(nwcore)

*     ------------------------------------------------------------------
*     m4rept  has the same parameter list as m4soln, the routine that
*     prints the solution.  It will be called if the SPECS file
*     specifies  REPORT file  n  for some positive value of  n.
*
*     pi contains the unscaled dual solution.
*     xn contains the unscaled primal solution.  There are n + m = nb
*        values (n structural variables and m slacks, in that order).
*     y  contains the true slack values for nonlinear constraints
*        in its first nncon components (computed by m8viol).
*
*     This version of m4rept does nothing.    Added for PILOT, Oct 1985.
*     31 Oct 1991: Name changed from "report" to "m4rept".
*                  Parameters altered to allow for MPS or generic names.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /cycle2/ objtru,suminf,numinf
*     ------------------------------------------------------------------

      if (iprint .gt. 0) write(iprint, 1000)
      if (isumm  .gt. 0) write(isumm , 1000)
      return

 1000 format(/ ' XXX Report file requested.  m4rept does nothing.')

*     end of m4rept
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4soln( ondisk, m, n, nb, nname, nscl,
     $                   nn, nnobj, nnobj0, ns,
     $                   ne, nka, a, ha, ka,
     $                   hs, ascale, bl, bu, gobj, pi, rc, xn, y,
     $                   name1, name2, istate, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            ondisk
      integer*4          ha(ne), hs(nb)
      integer            ka(nka), name1(nname), name2(nname)
      character*12       istate
      double precision   a(ne), ascale(nscl), bl(nb), bu(nb)
      double precision   gobj(nnobj0), pi(m), rc(nb), xn(nb), y(m),
     $                   z(nwcore)

*     ------------------------------------------------------------------
*     m4soln  is the standard output routine for printing the solution.
*
*     On entry,
*     pi    contains the dual solution.
*     xn    contains the primal solution.  There are n + m = nb values
*           (n structural variables and m slacks, in that order).
*     rc    contains reduced costs for all variables:
*           rc(1:n)    =  gobj + c - A'pi  for structurals,
*           rc(n+1:nb) = -pi  for slacks.
*           pi and rc corresond to the Phase-1 objective
*           if the solution is infeasible.
*     y     contains the true slack values for nonlinear constraints
*           in its first nncon components (computed by m8viol).
*
*     All quantities a, bl, bu, pi, rc, xn, y, fcon, gobj are unscaled
*     and adjusted in sign if maximizing.  (fcon is not used here.)  
*
*     If ondisk is true, the solution is output to the solution file.
*     Otherwise, it is output to the printer.
*
*     31 Jan 1991: rc is now an input parameter.  It is set in m4savb.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /cycle2/ objtru,suminf,numinf

      intrinsic          abs
      logical            feasbl, infsbl, maximz, scaled
      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )
*     ------------------------------------------------------------------

      bplus  = 0.1*plinfy
      scale  = one
      feasbl = ninf   .eq. 0
      infsbl = .not. feasbl
      maximz = minimz .lt. 0
      scaled = lscale .gt. 0
      lpr    = iprint
      if (ondisk) lpr = isoln

      call m1page( 1 )
      if (infsbl) write(lpr, 1000) name, ninf, sinf
      if (feasbl) write(lpr, 1002) name, objtru
      write(lpr, 1004) istate, itn, ns
      write(lpr, 1005) mobj, minmax, mrhs, mrng, mbnd
      write(lpr, 1010)
**    tolfea = 0.1 * tolx
**    tolopt = 0.1 * toldj(3) * pinorm
** 05 Oct 1991: Might as well flag according to the tolerances
*               actually used.
      tolfea = tolx
      tolopt = toldj(3) * pinorm

*     ------------------------------------------------------------------
*     Output the ROWS section.
*     ------------------------------------------------------------------
      do 300 iloop = 1, m
         i      = iloop
         j      = n + i
         if (scaled) scale = ascale(j)
         js     = hs(j)
         b1     = bl(j)
         b2     = bu(j)
         xj     = xn(j)
         py     = pi(i)
         dj     = rc(j)

*        Define row and slack activities.

         if (i .le. nncon) xj = y(i)
         row    = - xj
         d1     =   b1 - xj
         d2     =   xj - b2
         slk    = - d1
         if (abs( d1  )  .gt.  abs( d2 )) slk =   d2
         if (abs( slk )  .ge.  bplus    ) slk = - row
         d1     = d1 / scale
         d2     = d2 / scale
         djtest = dj * scale
         if (feasbl) then
            if (   maximz   ) djtest =  - djtest
         end if

*        Change slacks into rows.

         if (js .le. 1) js = 1 - js
         b1     = - b2
         b2     = - bl(j)

         call m4id  ( j, m, n, nb, nname, name1, name2, id1, id2 )
         call m4solp( ondisk, bplus, tolfea, tolopt,
     $                js,  d1,  d2, djtest,
     $                j , id1, id2, row, slk, b1, b2, py, i )
  300 continue

*     ------------------------------------------------------------------
*     Output the COLUMNS section.
*     ------------------------------------------------------------------
      call m1page( 1 )
      write(lpr, 1020)

      do 400 jloop = 1, n
         j      = jloop
         if (scaled) scale = ascale(j)
         js     = hs(j)
         b1     = bl(j)
         b2     = bu(j)
         xj     = xn(j)
         cj     = zero
         dj     = rc(j)

         do 320 k = ka(j), ka(j+1) - 1
            ir    = ha(k)
            if (ir .eq. iobj) cj = a(k)
  320    continue

         d1     =   (b1 - xj) / scale
         d2     =   (xj - b2) / scale
         djtest = - dj * scale
         if (feasbl) then
            if (j .le. nnobj) cj     =    cj + gobj(j)
            if (   maximz   ) djtest =  - djtest
         end if

         call m4id  ( j, m, n, nb, nname, name1, name2, id1, id2 )
         call m4solp( ondisk, bplus, tolfea, tolopt,
     $                js,  d1,  d2, djtest,
     $                j , id1, id2, xj, cj, b1, b2, dj, m+j )
  400 continue

      if (ondisk) then
         if (isoln .ne. iprint) rewind isoln
         if (iprint .gt. 0) write(iprint, 1400) isoln
         if (isumm  .gt. 0) write(isumm , 1400) isoln
      end if
      return

 1000 format(' NAME', 11x, 2a4, 13x,
     $   ' INFEASIBILITIES', i7, 1p, e16.4)
 1002 format(' NAME', 11x, 2a4, 13x,
     $   ' OBJECTIVE VALUE', 1p, e23.10)
 1004 format(/ ' STATUS', 9x, a12, 9x,
     $   ' ITERATION', i7, '    SUPERBASICS', i7)
 1005 format(/
     $   ' OBJECTIVE', 6x, 2a4, ' (', a3, ')' /
     $   ' RHS      ', 6x, 2a4 /
     $   ' RANGES   ', 6x, 2a4 /
     $   ' BOUNDS   ', 6x, 2a4)
 1010 format(/ ' SECTION 1 - ROWS' //
     $   '  NUMBER  ...ROW.. STATE  ...ACTIVITY...  SLACK ACTIVITY',
     $   '  ..LOWER LIMIT.  ..UPPER LIMIT.  .DUAL ACTIVITY    ..I' /)
 1020 format(  ' SECTION 2 - COLUMNS' //
     $   '  NUMBER  .COLUMN. STATE  ...ACTIVITY...  .OBJ GRADIENT.',
     $   '  ..LOWER LIMIT.  ..UPPER LIMIT.  REDUCED GRADNT    M+J' /)
 1400 format(/ ' SOLUTION file saved on file', i4)

*     end of m4soln
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4solp( ondisk, bplus, tolfea, tolopt,
     $                   js,  d1,  d2, djtest,
     $                   j , id1, id2, xj, cj, b1, b2, dj, k )

      implicit           double precision (a-h,o-z)
      logical            ondisk

*     ------------------------------------------------------------------
*     m4solp  prints one line of the Solution file.
*
*     The following conditions are marked by key:
*
*        D  degenerate basic or superbasic variable.
*        I  infeasible basic or superbasic variable.
*        A  alternative optimum      (degenerate nonbasic dual).
*        N  nonoptimal superbasic or nonbasic (infeasible dual).
*
*     Tests for these conditions are performed on scaled quantities
*     d1, d2, djtest,
*     since the correct indication is then more likely to be given.
*     On badly scaled problems, the unscaled solution may then appear
*     to be flagged incorrectly, but this is just an illusion.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt

      intrinsic          abs
      character*4        jstat, jstate(6)
      character*1        key, lblank, laltop, ldegen, linfea, lnotop
      data               jstate /' LL ', ' UL ', 'SBS ',
     $                           ' BS' , ' EQ' , ' FR '/
      data               lblank /' '/, laltop /'A'/, ldegen /'D'/,
     $                   linfea /'I'/, lnotop /'N'/
*     ------------------------------------------------------------------

      key    = lblank
      if (js .le. 1) then

*        Set key for nonbasic variables.

         if (b1 .eq. b2) js = 4
         if (- d1 .gt. tolfea  .and.  - d2 .gt. tolfea) js = 5
         if (js .eq. 1 ) djtest = - djtest
         if (js .ge. 4 ) djtest =   abs( djtest )
         if (             abs( djtest ) .le. tolopt) key = laltop
         if (js .ne. 4  .and.  djtest   .gt. tolopt) key = lnotop
      else

*        Set key for basic and superbasic variables.

         if (abs(   d1   ) .le. tolfea  .or.
     $       abs(   d2   ) .le. tolfea) key = ldegen
         if (           js .eq. 2       .and.
     $       abs( djtest ) .gt. tolopt) key = lnotop
         if (           d1 .gt. tolfea  .or.
     $                  d2 .gt. tolfea) key = linfea
      end if

*     Select format for printing.

      jstat   = jstate(js + 1)
      if (ondisk) then
               write(isoln,  1000) j,id1,id2,key,jstat,xj,cj,b1,b2,dj,k
      else
         if (b2 .lt. bplus) then
            if (b1 .gt. - bplus) then
               write(iprint, 1200) j,id1,id2,key,jstat,xj,cj,b1,b2,dj,k
            else
               write(iprint, 1300) j,id1,id2,key,jstat,xj,cj,   b2,dj,k
            end if
         else
            if (b1 .gt. - bplus) then
               write(iprint, 1400) j,id1,id2,key,jstat,xj,cj,b1,   dj,k
            else
               write(iprint, 1500) j,id1,id2,key,jstat,xj,cj,      dj,k
            end if
         end if
      end if
      return

 1000 format(i8, 2x, 2a4, 1x, a1, 1x, a3, 1p, e16.6, 4e16.6, i7)
 1200 format(i8, 2x, 2a4, 1x, a1, 1x, a3, 5f16.5, i7)
 1300 format(i8, 2x, 2a4, 1x, a1, 1x, a3, 2f16.5,
     $   '           NONE ', f16.5, f16.5, i7)
 1400 format(i8, 2x, 2a4, 1x, a1, 1x, a3, 2f16.5,
     $   f16.5, '           NONE ', f16.5, i7)
 1500 format(i8, 2x, 2a4, 1x, a1, 1x, a3, 2f16.5,
     $   '           NONE ', '           NONE ', f16.5, i7)

*     end of m4solp
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m4stat( k, istate )

      integer            k
      character*12       istate

*     ------------------------------------------------------------------
*     m4stat loads istate with words describing the current state.
*     ------------------------------------------------------------------

      intrinsic          min
      character*12       c(0:5)
      data               c     /'PROCEEDING  ',
     $                          'OPTIMAL SOLN',
     $                          'INFEASIBLE  ',
     $                          'UNBOUNDED   ',
     $                          'EXCESS ITNS ',
     $                          'ERROR CONDN '/

      j      = min( k, 5 )
      istate = c(j)

*     end of m4stat
      end
************************************************************************
*
*     file  mi50lp   fortran.
*
*     m5bsx    m5chzr   m5dgen   m5frmc   m5hs     m5log    m5lpit
*     m5pric   m5rc     m5setp   m5setx   m5solv
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5bsx ( mode, ms, nb, kb, x, xn )

      implicit           double precision (a-h,o-z)
      integer            kb(ms)
      double precision   x(ms), xn(nb)

*     ------------------------------------------------------------------
*     m5bsx   copies basic and superbasic components from  x  into  xn
*     or vice versa, depending on whether  mode = 1 or 2.
*     ------------------------------------------------------------------

      if (mode .eq. 1) then
         do 10  k = 1, ms
            j     = kb(k)
            xn(j) = x(k)
   10    continue
      else
         do 30  k = 1, ms
            j     = kb(k)
            x(k)  = xn(j)
   30    continue
      end if

*     end of m5bsx
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5chzr( ms    , stepmx, plinfy, tolpiv,
     $                   hrtype, bbl   , bbu   , x     , y,
     $                   hitlow, move  , onbnd , unbndd,
     $                   jp    , bound , exact , alpha )

      implicit           double precision (a-h,o-z)
      integer*4          hrtype(ms)
      double precision   bbl(ms), bbu(ms), x(ms), y(ms)
      logical            hitlow , move   , onbnd, unbndd

*     ------------------------------------------------------------------
*     m5chzr  finds a step  alpha  such that the point  x + alpha*y
*     reaches one of the bounds on  x.
*
*     In this version of chuzr, when x is infeasible, the number of
*     infeasibilities will never increase.  If the number stays the
*     same, the sum of infeasibilities will decrease.
*     If the number decreases by one or more,
*     the sum of infeasibilities will usually decrease also, but
*     occasionally it will increase after the step  alpha  is taken.
*     (Convergence is still assured because the number has decreased.)
*
*     Two possible steps are computed as follows:
*
*     alphaf = the maximum step that can be taken without violating
*              one of the bounds that are currently satisfied.
*
*     alphai = the maximum (nonzero) step that has the property of
*              reaching a bound that is currently violated,
*              subject to the pivot being reasonably close to the
*              maximum pivot among infeasible variables.
*              (alphai is not defined if x is feasible.)
*
*     alphai is needed occasionally when infeasible, to prevent
*     going unnecessarily far when alphaf is quite large.  It will
*     always come into effect when x is about to become feasible.
*     The sum of infeasibilities will decrease initially as alpha
*     increases from zero, but may start increasing for larger steps.
*     choosing a large alphai allows several elements of  x  to
*     become feasible at the same time.
*
*     In the end, we take  alpha = alphaf  if x is feasible, or if
*     alphai > alphap (where alphap is the perturbed step from pass 1).
*     Otherwise,  we take  alpha = alphai.
*
*     Input parameters
*     ----------------
*     ms     is  m + 1  for m5lpit,  m + ns  for m7rgit.
*     stepmx defines what should be treated as an unbounded step.
*     plinfy provides insurance for detecting unboundedness.
*            if alpha reaches a bound as large as plinfy, it is
*            classed as an unbounded step.
*     tolpiv is a tolerance to exclude negligible elements of y.
*     featol (in common) is the current feasibility tolerance used by
*            m5frmc.  typically in the range 0.5*tolx to 0.99*tolx,
*            where tolx is the featol specified by the user.
*     tolinc (in common) is used to determine stepmn (see below),
*            the minimum positive step.
*     hrtype is set by  m5frmc  as follows:
*            hrtype(j) = -2  if x(j) .lt. bl(j) - featol
*                      =  0  if x(j)  is feasible
*                      = +2  if x(j) .gt. bu(j) + featol
*     bbl    the lower bounds on the basic and superbasic variables.
*     bbu    the upper bounds on ditto.
*     x      the values of       ditto.
*     y      the search direction.
*
*
*     Output parameters
*     -----------------
*     hitlow  = true  if a lower bound restricted alpha.
*             = false otherwise.
*     move    = true  if exact ge stepmn (defined at end of code).
*     onbnd   = true  if alpha =  exact.
*                     this means that the step alpha moves x(jp)
*                     exactly onto one of its bounds, namely bound.
*             = false if the exact step would be too small
*                     ( exact lt stepmn ).
*               (with these definitions,  move = onbnd).
*     unbndd  = true  if alpha = stepmx.  jp may possibly be zero.
*               the parameters hitlow, move, onbnd, bound and exact
*               should not be used.
*     jp      = the index (if any) such that x(jp) reaches a bound.
*     bound   = the bound value bbl(jp) or bbu(jp) corresponding
*               to hitlow.
*     exact   = the step that would take x(jp) exactly onto bound.
*     alpha   = an allowable, positive step.
*               if unbndd is true,  alpha = stepmx.
*               otherwise,          alpha = max( stepmn, exact ).
*
*
*
*
*     Original version written November 1981.
*     The two-pass approach used follows Paula Harris (1973).
*
*     September 1986:  Modified to deal with x + alpha*y only, and to
*     return a few extra parameters.
*
*     20 aug 1987:  EXPAND procedure implemented to deal with
*     degeneracy in a more rigorous way.  The step alphaf is chosen
*     the same way as Harris, but since this may be negative, this
*     version insists on returning a positive step, alpha.
*     Two features make this possible:
*
*        1. featol increases slightly each iteration.
*
*        2. The blocking variable, when made nonbasic, is allowed to
*           retain the value x(jp) + alpha * y(jp), even if this is
*           not exactly on the blocking bound.
*
*     15 feb 1988:  Modified to prevent a very small pivot being
*     selected in connection with alphai.  For infeasible variables
*     moving towards their bound, we now require the chosen pivot
*     to be at least gamma times as large as the biggest available.
*     This still gives us freedom in pass 2.
*     gamma = 0.1 and 0.01 seemed to inhibit phase 1 somewhat.
*     gamma = 0.001 seems to be safe.
*
*     Note: if the weight on the objective is positive, there is still a
*     danger of small pivots in phase 1.  We have to let an "unbounded"
*     exit occur.  m5solv will set wtobj = zero and try again.
*     ------------------------------------------------------------------

      common    /m5step/ featol, tolx0,tolinc,kdegen,ndegen,
     $                   itnfix, nfix(2)

      intrinsic          abs, max
      logical            blockf, blocki

      parameter        ( zero = 0.0d+00,  gamma = 0.001d+00 )

*     ------------------------------------------------------------------
*     First pass.
*     For feasible variables, find the step alphap that reaches the
*     nearest perturbed (expanded) bound.  alphap will be slight larger
*     than the step to the nearest true bound.
*     For infeasible variables, find the maximum pivot pivmxi.
*     ------------------------------------------------------------------

      delta  = featol
      alphap = stepmx
      pivmxi = zero

      do 200 j  = 1, ms
         pivot  = y(j)
         pivabs = abs( pivot )
         if (pivabs .le. tolpiv) go to 200
         jtype  = hrtype(j)
         if (pivot  .gt. zero  ) go to 150

*        x  is decreasing.
*        Test for smaller alphap if lower bound is satisfied.

         if (jtype .lt. 0) go to 200
         res    = x(j) - bbl(j) + delta
         if (alphap*pivabs .gt. res) alphap = res / pivabs

*        Test for bigger pivot if upper bound is violated.

         if (jtype .gt. 0) pivmxi = max( pivmxi, pivabs )
         go to 200

*        x  is increasing.
*        Test for smaller alphap if upper bound is satisfied.

  150    if (jtype .gt. 0) go to 200
         res    = bbu(j) - x(j) + delta
         if (alphap*pivabs .gt. res) alphap = res / pivabs

*        Test for bigger pivot if lower bound is violated.

         if (jtype .lt. 0) pivmxi = max( pivmxi, pivabs )
  200 continue

*     ------------------------------------------------------------------
*     Second pass.
*     For feasible variables, recompute steps without perturbation.
*     Choose the largest pivot element subject to the step being
*     no greater than alphap.
*     For infeasible variables, find the largest step subject to the
*     pivot element being no smaller than gamma * pivmxi.
*     ------------------------------------------------------------------

      alphai = zero
      pivmxf = zero
      pivmxi = gamma * pivmxi
      jhitf  = 0
      jhiti  = 0

      do 400 j  = 1, ms
         pivot  = y(j)
         pivabs = abs( pivot )
         if (pivabs .le. tolpiv) go to 400
         jtype  = hrtype(j)
         if (pivot  .gt. zero  ) go to 350

*        x  is decreasing.
*        Test for bigger pivot if lower bound is satisfied.

         if (jtype    .lt.     0   ) go to 400
         if (pivabs   .le.   pivmxf) go to 340
         res    = x(j) - bbl(j)
         if (alphap*pivabs .lt. res) go to 340
         pivmxf = pivabs
         jhitf  = j

*        Test for bigger alphai if upper bound is violated.

  340    if (jtype    .eq.     0   ) go to 400
         if (pivabs   .lt.   pivmxi) go to 400
         res    = x(j) - bbu(j)
         if (alphai*pivabs .ge. res) go to 400
         alphai = res / pivabs
         jhiti  = j
         go to 400

*        x  is increasing.
*        Test for bigger pivot if upper bound is satisfied.

  350    if (jtype    .gt.     0   ) go to 400
         if (pivabs   .le.   pivmxf) go to 360
         res    = bbu(j) - x(j)
         if (alphap*pivabs .lt. res) go to 360
         pivmxf = pivabs
         jhitf  = j

*        Test for bigger alphai if lower bound is violated.

  360    if (jtype    .eq.     0   ) go to 400
         if (pivabs   .lt.   pivmxi) go to 400
         res    = bbl(j) - x(j)
         if (alphai*pivabs .ge. res) go to 400
         alphai = res / pivabs
         jhiti  = j
  400 continue

*     ------------------------------------------------------------------
*     See if a feasible and/or infeasible variable blocks.
*     ------------------------------------------------------------------
      blockf = jhitf .gt. 0
      blocki = jhiti .gt. 0
      unbndd = .not. ( blockf  .or.  blocki )
      if (       unbndd ) go to 900
      if ( .not. blockf ) go to 500

*     ------------------------------------------------------------------
*     A variable hits a bound for which it is currently feasible.
*     the corresponding step alphaf is not used, so no need to get it,
*     but we know that alphaf .le. alphap, the step from pass 1.
*     ------------------------------------------------------------------
      jp     = jhitf
      pivot  = y(jp)
      hitlow = pivot .lt. zero

*     If there is a choice between alphaf and alphai, it is probably
*     best to take alphai (so we can kick the infeasible variable jhiti
*     out of the basis).
*     However, we can't if alphai is bigger than alphap.

      if (   .not. blocki   ) go to 600
      if (alphai .gt. alphap) go to 600

*     ------------------------------------------------------------------
*     An infeasible variable reaches its violated bound.
*     ------------------------------------------------------------------
  500 jp     = jhiti
      pivot  = y(jp)
      hitlow = pivot .gt. zero

*     ------------------------------------------------------------------
*     Try to step exactly onto bound, but make sure the exact step
*     is sufficiently positive.  (exact will be alphaf or alphai.)
*     Since featol increases by tolinc each iteration, we know that
*     a step as large as stepmn (below) will not cause any feasible
*     variables to become infeasible (where feasibility is measured
*     by the current featol).
*     ------------------------------------------------------------------
  600 if ( hitlow ) then
         bound = bbl(jp)
      else
         bound = bbu(jp)
      end if
      unbndd = abs( bound ) .ge. plinfy
      if ( unbndd ) go to 900

      stepmn = tolinc / abs( pivot )
      exact  = (bound - x(jp)) / pivot
      alpha  = max( stepmn, exact )
      onbnd  = alpha .eq. exact
      move   = exact .ge. stepmn
      if (.not. move) ndegen = ndegen + 1
      return

*     ------------------------------------------------------------------
*     Unbounded.
*     ------------------------------------------------------------------
  900 alpha  = stepmx
      move   = .true.
      onbnd  = .false.

*     end of m5chzr
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5dgen( mode, m, n, nb, ms, inform,
     $                   ne, nka, a, ha, ka,
     $                   hs, kb, bl, bu, x, xn, y, y2, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hs(nb)
      integer            ka(nka), kb(ms)
      double precision   a(ne), bl(nb), bu(nb)
      double precision   x(ms), xn(nb), y(m), y2(m), z(nwcore)

*     ------------------------------------------------------------------
*     m5dgen performs most of the manoeuvres associated with degeneracy.
*     The degeneracy-resolving strategy operates in the following way.
*
*     Over a cycle of iterations, the feasibility tolerance featol
*     increases slightly (from tolx0 to tolx1 in steps of tolinc).
*     This ensures that all steps taken will be positive.
*
*     After kdegen consecutive iterations, nonbasic variables within
*     featol of their bounds are set exactly on their bounds and the
*     basic variables are recomputed to satisfy ax = b.
*     featol is then reduced to tolx0 for the next cycle of iterations.
*
*
*     If mode = 1, m5dgen initializes the parameters in
*     common block m5step:
*
*     featol  is the current feasibility tolerance.
*     tolx0   is the minimum feasibility tolerance.
*     tolx1   is the maximum feasibility tolerance.
*     tolinc  is the increment to featol.
*     kdegen  is the expand frequency (specified by the user).
*             it is the frequency of resetting featol to tolx0.
*     ndegen  counts the number of degenerate steps (incremented
*             by m5chzr).
*     itnfix  is the last iteration at which a mode 2 or 3 entry
*             set nonbasics onto their bound.
*     nfix(j) counts the number of times a mode 3 entry has
*             set nonbasics onto their bound,
*             where j=1 if infeasible, j=2 if feasible.
*
*     tolx0 and tolx1 are both close to the feasibility tolerance tolx
*     specified by the user.  (They must both be less than tolx.)
*
*
*     If mode = 2,  m5dgen has been called after a cycle of kdegen
*     iterations.  Nonbasic xn(j)s are examined to see if any are
*     outside their bounds. (It will never be as much as featol).
*     inform returns how many.  Deviations as small as tolz
*     (e.g. 1.0d-11) are not counted.
*     If inform is positive, the basic variables are recomputed.
*     It is assumed that m5solv will then continue iterations.
*
*
*     If mode = 3,  m5dgen is being called after a subproblem has been
*     judged optimal, infeasible or unbounded.  Nonbasic xn(j)s are
*     examined as above.
*
*     First version: August 1987.
*     22 Sep 1987: itnfix, nfix(j) and maxfix introduced to allow
*                  more than one mode 3 call at the end of a run.
*     14 Oct 1991: a, ha, ka passed in to help with minoss.
*     08 Apr 1992: Internal values of hs(*) now used.
*     04 Jun 1992: mode 2 and 3 fixed to allow for hs(j) = 4.
*     24 Feb 1994: Bug fixed.  Have to treat hs(j)=4 separately.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5step/ featol, tolx0,tolinc,kdegen,ndegen,
     $                   itnfix, nfix(2)
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m8len / njac  ,nncon ,nncon0,nnjac
*     ------------------------------------------------------------------

      parameter        ( zero = 0.0d+0 )

      inform = 0
      if (mode .eq. 1) then

*        mode = 1.
*        Initialize at the start of each major iteration.
*        kdegen is the expand frequency      and
*        tolx   is the feasibility tolerance
*        (specified by the user).  They are not changed.

         ndegen  = 0
         itnfix  = 0
         nfix(1) = 0
         nfix(2) = 0
         tolx0   = tolx  * 0.5
         tolx1   = tolx  * 0.99
         if (kdegen .lt. 99999999) then
            tolinc = (tolx1 - tolx0) / kdegen
         else
            tolinc = 0.0
         end if
         featol  = tolx0
      else

*        mode = 2 or 3.
*        Initialize local variables maxfix and tolz.

         maxfix = 2
         tolz   = eps1
         if (mode .eq. 3) then

*           mode = 3.
*           Return with inform = 0 if the last call was at the
*           same itn, or if there have already been maxfix calls
*           with the same state of feasibility.

            if (itnfix .eq. itn   ) return
            if (ninf   .gt.   0   ) j = 1
            if (ninf   .eq.   0   ) j = 2
            if (nfix(j).ge. maxfix) return
            nfix(j) = nfix(j) + 1
         end if

*        Set nonbasics on their nearest bound if they are within
*        the current featol of that bound.  hs(j) = 0, 1 or 4.
*        To avoid go to's we test on diff for all hs(j).

         itnfix = itn

         do 250 j = 1, nb
            diff  = zero
            js    = hs(j)

            if      (js .eq. 0) then
               bnd   = bl(j)
               diff  = bnd - xn(j)
            else if (js .eq. 1) then
               bnd   = bu(j)
               diff  = xn(j) - bnd
            else if (js .eq. 4) then
               bnd   = bu(j)
               diff  = abs( xn(j) - bnd )
            end if

            if (diff .gt. zero) then
               if (diff .gt. tolz) inform = inform + 1
               xn(j) = bnd
            end if
  250    continue

*        Reset featol to its minimum value.

         featol = tolx0
         if (inform .gt. 0) then

*           Reset the basic variables.
*           Set ninf positive to make sure m5frmc tests for feasibility.

            call m5setx( 1, m, n, nb, ms, kb,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, x, xn, y, y2, z, nwcore )
            ninf   = 1
            if (prnt1  .or.  nncon .eq. 0) then
               if (iprint .gt. 0) write(iprint, 1000) itn, inform
               if (isumm  .gt. 0) write(isumm , 1000) itn, inform
            end if
         end if
      end if

      return

 1000 format(' Itn', i7, ' --', i7,
     $       '  nonbasics set on bound, basics recomputed')

*     end of m5dgen
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5frmc( n, nb, nn, ns, ms, maxs,
     $                   lcrash, first, fsub, featol, objadd,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, bbl, bbu, hrtype, hs, kb,
     $                   grd2, x, xn, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            first
      integer*4          ha(ne), hrtype(ms), hs(nb)
      integer            ka(nka), kb(ms)
      double precision   a(ne), bl(nb), bu(nb)
      double precision   bbl(ms), bbu(ms),
     $                   grd2(ms), x(ms), xn(nb), z(nwcore)

*     ------------------------------------------------------------------
*     m5frmc  sets up a vector in  grd2  to be used to compute  pi.
*     It also defines  hrtype  to be used in  m5chzr.
*     After  m2bfac,  or at a newly feasible point, the subproblem
*     objective function is evaluated.
*
*     lcrash is used at a first feasible point to print a suitable msg.
*
*     10 Apr 1992: objadd added as an input parameter.
*     04 Jun 1992: lcrash added as an input parameter.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5lp2 / invrq,invitn,invmod
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2

      logical            kludge, feasbl, infsbl,
     $                   linear, nonlin, lincon, nlncon

      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

*     ------------------------------------------------------------------

      feasbl = ninf .eq. 0
      linear = nn   .eq. 0
      if ( first ) feasbl = .false.
      infsbl = .not. feasbl
      nonlin = .not. linear

      call dload ( ms, zero, grd2, 1 )
      if (iobj .ne. 0) grd2(iobj) = - minimz * sclobj

*     Revert to the simplex method if the number of superbasics
*     has just decreased to zero.
*     Note that it is ok to do phase 1 simplex on nonlinear problems
*     as long as we are at a vertex.

      if (ns .eq. 0) then
         if ( infsbl ) then
            nphs = 1
         else if ( linear ) then
            nphs = 2
         end if
      end if

*     Exit if the previous iteration was feasible and the
*     basis has not just been refactorized.

      if (feasbl  .and.  invitn .gt. 0) return

*     ------------------------------------------------------------------
*     Find the current number and sum of infeasibilities.
*     ------------------------------------------------------------------
      lincon = nncon  .eq. 0
      nlncon = nncon  .gt. 0
      numinf = 0
      suminf = zero

      do 100 k = 1, ms
         hrtype(k) = 0
         xk        = x(k)
         res       = bbl(k) - xk
         if (res .le. featol) go to 50
         grd2(k)   = - one
         hrtype(k) = - 2
         go to 60

   50    res       = xk - bbu(k)
         if (res .le. featol) go to 100
         grd2(k)   = one
         hrtype(k) = 2

   60    numinf    = numinf + 1
         suminf    = suminf + res
  100 continue

      sinf   = suminf
      ninf   = numinf

      if (numinf .gt. 0) then
*        ---------------------------------------------------------------
*        Infeasible.
*        ---------------------------------------------------------------
                             
*        If first iteration, set  nphs.
      
         if (first) then
            nphs   = 1
            if (ns .gt. 0) nphs = 4
         end if
      
*        Set  grd2(iobj)  to allow for a composite objective.
      
         if (nphs .eq. 2) nphs = 1
         kmodpi = 1
         fsub   = sinf
         fobj   = zero
         if (iobj .ne. 0) grd2(iobj) = - minimz * wtobj * sclobj
      
*        Print something if the basis has just been refactorized.
      
         if (prnt1  .and.  invitn .eq. 0)
     $       write(iprint, 1010) itn, numinf, suminf

      else
*        ---------------------------------------------------------------
*        Feasible.
*        ---------------------------------------------------------------
           
*        Reset  nphs  if the previous itn was infeasible.
      
         if (first .or. infsbl) then
            nphs   = 2
            if ( nonlin  ) nphs = 3
            if (ns .gt. 0) nphs = 4
         end if
      
*        Set up the feasible objective value.
      
         kmodpi = 1
         flin   = zero
         if (iobj .ne. 0) flin = - x(iobj) * sclobj
         fsub   = minimz * flin

         if (lcrash .eq. 3) then

*           Only the linear equalities (E rows) have been satisfied.
*           We don't want to evaluate the functions yet.
*           Print something and quit.

            if (iprint .gt. 0) then
               if (prnt1 .or. lincon) then
                  write(iprint, 1000)
                  write(iprint, 1015) itn, fsub
               end if
            end if
            if (isumm  .gt. 0) then
               if (summ1 .or. lincon) write(isumm , 1015) itn, fsub
            end if
            return
         end if

         if ( nonlin ) then
            modefg = 2
            call m6fun ( 0, modefg, n, nb, ms, fsub,
     $                   ne, nka, a, ha, ka,
     $                   x, xn, z, nwcore )
            call m6fun ( 1, modefg, n, nb, ms, fsub,
     $                   ne, nka, a, ha, ka,
     $                   x, xn, z, nwcore )
            if (ierr .ne. 0) return
         end if
      
         wtobj  = zero
         obj    = minimz * fsub  +  objadd
         objtru = flin   + fobj  +  objadd
      
*        Print something unless Print level = 0 and there are
*        nonlinear constraints.
      
         if (iprint .gt. 0) then
            if (prnt1  .or.  (lincon  .and.  infsbl)) then
               if (infsbl) write(iprint, 1000)
               if (lincon) write(iprint, 1020) itn, obj
               if (nlncon) write(iprint, 1030) itn, objtru, obj
               if (infsbl) write(iprint, 1000)
            end if
         end if
      
         if (isumm  .gt. 0) then
            if (infsbl) then
               if (lincon            ) write(isumm , 1020) itn, obj
               if (nlncon .and. summ1) write(isumm , 1040) itn, obj
            end if
         end if
      end if

      return

 1000 format(' ')
 1010 format(' Itn', i7, ' -- infeasible.  Num =', i5,
     $   '   Sum =', 1p, e18.9)
 1015 format(' Itn', i7, ' -- linear E rows feasible.   Obj =',
     $   1p, e18.9)
 1020 format(' Itn', i7, ' -- feasible solution.  Objective =',
     $   1p, e18.9)
 1030 format(' Itn', i7, ' -- feasible subproblem.  True obj =',
     $   1p, e17.9, 4x, ' Auglag obj =', e17.9)
 1040 format(' Itn', i7, ' -- feasible subproblem.',
     $   1p,        4x, ' Auglag obj =', e17.9)

*     end of m5frmc
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5hs  ( mode, nb, bl, bu, hs, xn )

      implicit           double precision (a-h,o-z)
      character*8        mode
      double precision   bl(nb), bu(nb)
      integer*4          hs(nb)
      double precision   xn(nb)

*     ------------------------------------------------------------------
*     m5hs   is called from m5solv and m8setj.
*     if mode = 'Internal', m5hs sets hs(j) = -1 or 4 for certain
*        nonbasic variables.  This allows m5pric to operate more
*        efficiently.  The internal values of hs are now as follows:
*
*        hs(j) = -1  Nonbasic between bounds (bl     <  x <  bu    )
*        hs(j) =  0  Nonbasic on lower bound (bl-tol <  x <= bl    )
*        hs(j) =  1  Nonbasic on upper bound (bu     <= x <  bu+tol)
*        hs(j) =  2  Superbasic
*        hs(j) =  3  Basic
*        hs(j) =  4  Nonbasic and fixed      (bl     = x  =  bu    )
*
*        where 0 <= tol < the feasibility tolerance.
*
*     if mode = 'External', m5hs changes -1 or 4 values to hs(j) = 0,
*        ready for basis saving and the outside world.
*
*     08 Apr 1992: First version.
*     ------------------------------------------------------------------

      if (mode .eq. 'Internal') then
*        ---------------------------------------------------------------
*        Change nonbasic hs(j) to internal values (including 4 and -1).
*        This may change existing internal values if bl and bu have been
*        changed -- e.g. at the start of each major iteration.
*        ---------------------------------------------------------------
         do 100 j = 1, nb
            if (hs(j) .le. 1) then
               if (bl(j) .eq. bu(j)) then
                  hs(j) =  4
               else if (xn(j) .le. bl(j)) then
                  hs(j) =  0
               else if (xn(j) .ge. bu(j)) then
                  hs(j) =  1
               else
                  hs(j) = -1
               end if
            end if
  100    continue

      else
*        ---------------------------------------------------------------
*        Change hs to external values.
*        Some nonbasic hs(j) may be 4 or -1.  Change them to 0.
*        ---------------------------------------------------------------
         do 200 j = 1, nb
            if (hs(j) .eq. 4  .or.  hs(j) .eq. -1) hs(j) = 0
  200    continue
      end if

*     end of m5hs
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5log ( m, maxs, mbs, n, nb, nn, ns, nx, fsub, objadd,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, hs, kb, x, xn, y, z, nwcore )

      implicit           double precision (a-h,o-z)
      double precision   a(ne), bl(nb), bu(nb)
      integer*4          ha(ne), hs(nb)
      integer            ka(nka), kb(mbs)
      double precision   x(nx), xn(nb), y(nx), z(nwcore)

*     ------------------------------------------------------------------
*     m5log  prints the iteration log.
*     Normally the only parameters used are nn, ns and fsub.
*     The others are there to allow monitoring of various items for
*     experimental purposes.
*     mbs = m + maxs  .ge.  m + ns,  and   nx = max( mbs, nn ).
*     The array  y(nx)  is available for workspace.
*
*     The print controls are as follows:
*
*     prnt0 is true if print level = 0.
*     For lp and lc, a brief log is output every  k  minor iterations,
*     where  k  is the log frequency.
*     For nlc, a brief log is output by m8setj every major iteration.
*
*     prnt1 is true if print level > 0.
*     A fuller log is output every  k  minor iterations.
*
*     summ0 and summ1 are the same as prnt0 and prnt1, but are false
*     if there is no summary file.  summary frequency defines  k.
*
*     newhed is true if a new heading is required for some reason other
*     than frequency (e.g., after a basis factorization).
*
*     lines1 and lines2 count how many lines have been output to the
*     print and summary files respectively since the last heading.
*     They too may force a new heading.
*
*     10 Apr 1992: objadd added as an input parameter.
*     ------------------------------------------------------------------

      logical            conv
      common    /m1file/ iread,iprint,isumm
      common    /m2lu3 / lenl,lenu,ncp,lrow,lcol
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5freq/ kchk,kinv,ksav,klog,ksumm,i1freq,i2freq,msoln
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m5log2/ jq1,jq2,jr1,jr2,lines1,lines2
      common    /m5log3/ djq,theta,pivot,cond,nonopt,jp,jq,modr1,modr2
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5lp2 / invrq,invitn,invmod
      common    /m5prc / nparpr,nmulpr,kprc,newsb
      common    /m7conv/ etash,etarg,lvltol,nfail,conv(4)
      common    /m7phes/ rgmin1,rgnrm1,rgnrm2,jz1,jz2,labz,nfullz,mfullz
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2
      common    /m8save/ vimax ,virel ,maxvi ,majits,minits,nssave

      intrinsic          mod
      logical            first , newset, phead , pline, hshort, lshort
      parameter        ( zero = 0.0d+0 )
      character*4        label(2), l
      data               label(1)/' '/, label(2)/'r'/
*     ------------------------------------------------------------------

      first  = minits .le. 1
      hshort = ns     .eq. 0  .and.  nn .eq. 0
      lshort = ns     .eq. 0
      it     = mod( itn, 1000000 )
      ms     = m + ns
      if (ninf .gt. 0) then
         if (iobj .gt. 0) then
            obj = - x(iobj) * sclobj
         else
            obj =   zero
         end if
      else
         obj    = fsub
      end if
      obj    = minimz * obj  +  objadd

      if (prnt0  .and.  nncon .eq. 0) then
*        ---------------------------------------------------------------
*        Terse print.
*        If there are nonlinear constraints, this is handled by m8setj.
*        ---------------------------------------------------------------
         lu     = lenl + lenu
         newset = lines1  .ge.  40
         pline  = first   .or.  mod( itn, klog ) .eq. 0
         phead  = pline  .and. ( newhed .or. newset )

         if ( phead ) then
            newhed = .false.
            lines1 = 0
            if ( hshort ) then
               write(iprint, 1000)
            else
               write(iprint, 1100)
            end if
         end if

         if ( pline ) then
            lines1 = lines1 + 1
            if ( lshort ) then
               write(iprint, 1200) it, djq, ninf, sinf, obj, lu
            else
               write(iprint, 1200) it, djq, ninf, sinf, obj, lu,
     $                             nfobj(1), ns, cond
            end if
         end if
      end if

      if (summ0  .and.  nncon .eq. 0) then
*        ---------------------------------------------------------------
*        Output to the summary file.
*        ---------------------------------------------------------------
         newset = lines2  .ge.  10
         pline  = first   .or.  mod( itn, ksumm ) .eq. 0
         phead  = pline  .and. ( first .or. newset )

         if ( phead ) then
            lines2 = 0
            if ( hshort ) then
               write(isumm , 2000)
            else
               write(isumm , 2100)
            end if
         end if

         if ( pline ) then
            lines2 = lines2 + 1
            if ( lshort ) then
               write(isumm , 2200) it, djq, ninf, sinf, obj
            else
               write(isumm , 2200) it, djq, ninf, sinf, obj, nfobj(1),ns
            end if
         end if
      end if

      if ( prnt1 ) then
*        ---------------------------------------------------------------
*        Detailed print.
*        ---------------------------------------------------------------
         l      = label(labz)
         newset = lines1  .ge.  40
         pline  = first   .or.  mod( itn, klog ) .eq. 0
         phead  = pline  .and. ( newhed .or. newset )

         if ( phead ) then
            newhed = .false.
            lines1 = 0
            if ( hshort ) then
               write(iprint, 3000)
            else
               write(iprint, 3100)
            end if
         end if

         if ( pline ) then
            lines1 = lines1 + 1
            if ( ninf .gt. 0 ) then
               sumobj = sinf
            else
               sumobj = obj
            end if

            if ( lshort ) then
               write(iprint, 3200) it, l, nphs, kprc, djq, jq2, jr2,jr1,
     $            theta, pivot, ninf, sumobj, lenl, lenu, ncp
            else
               write(iprint, 3200) it, l, nphs, kprc, djq, jq2, jr2,jr1,
     $            theta, pivot, ninf, sumobj, lenl, lenu, ncp,
     $            nfobj(1), nfcon(1), ns, modr1, modr2, cond, conv
            end if

*           Special output if Hessian dimension is less than the current
*           number of superbasics, and if variable  jz2  has just been
*           moved from set  z2  into set  z1  in place of variable  jz1.

            if ( jz2 .gt. 0 ) then
               write(iprint, 3300) rgnrm2, jz2, jz1, rgmin1
            end if
         end if
      end if

      if ( summ1 ) then
*        ---------------------------------------------------------------
*        Output to the summary file.
*        ---------------------------------------------------------------
         newset = lines2  .ge.  10
         pline  = first   .or.  mod( itn, ksumm ) .eq. 0
         phead  = pline  .and. ( first .or. newset )

         if ( phead ) then
            lines2 = 0
            if ( hshort ) then
               write(isumm , 4000)
            else
               write(isumm , 4100)
            end if
         end if

         if ( pline ) then
            lines2 = lines2 + 1
            if ( lshort ) then
               write(isumm , 4200) it, djq, ninf, sinf, obj
            else
               write(isumm , 4200) it, djq, ninf, sinf, obj,
     $                             nfobj(1), nfcon(1), ns
            end if
         end if
      end if

*     ------------------------------------------------------------------
*     Debug output.
*     ------------------------------------------------------------------
      if (idebug .eq. 100) then
         write(iprint, 5000) (kb(k), x(k), k = 1, ms)
      end if

*     Exit.

      return

 1000 format(/ '    Itn       dj  ninf      sinf       objective',
     $         '     LU')
 1100 format(/ '    Itn       rg  ninf      sinf       objective',
     $         '     LU  nobj  nsb  cond(H)')
 1200 format(1p, i7, e9.1, i6, e10.3, e16.8, i7, i6, i5, e8.1)
 2000 format(/ '    Itn       dj  ninf      sinf       objective')
 2100 format(/ '    Itn       rg  ninf      sinf       objective',
     $         '  nobj  nsb')
 2200 format(1p, i7, e9.1, i6, e10.3, e16.8, i6, i5)
 3000 format(/ '    Itn ph pp     dj    +sbs  -sbs   -bs',
     $   '  step    pivot   ninf  sinf,objective     L     U ncp')
 3100 format(/ '    Itn ph pp     rg    +sbs  -sbs   -bs',
     $   '  step    pivot   ninf  sinf,objective     L     U ncp',
     $   '  nobj  ncon  nsb Hmod cond(H) conv')
 3200 format(1p, i7, 1x, a1, i1, i3, e9.1, 3i6,
     $   e8.1, e9.1, i5, e16.8, 2i6, i4,
     $   2i6, i5, i3, i2, e8.1, 1x, 4l1)
 3300 format(1p, 17x, e9.1, 2i5, e9.1)
 4000 format(/ '    Itn       dj  ninf      sinf       objective')
 4100 format(/ '    Itn       rg  ninf      sinf       objective',
     $         '  nobj  ncon  nsb')
 4200 format(1p, i7, e9.1, i6, e10.3, e16.8, 2i6, i5)
 5000 format(/ ' BS and SB values...' / (5(i7, g17.8)))

*     end of m5log
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5lpit( m, m1, n, nb, incres,
     $                   ne, nka, a, ha, ka,
     $                   hrtype, hs, kb,
     $                   bl, bu, bbl, bbu, x, xn, y, y2, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            incres
      integer*4          ha(ne), hrtype(m1), hs(nb)
      integer            ka(nka), kb(m1)
      double precision   a(ne), bl(nb), bu(nb)
      double precision   bbl(m1), bbu(m1),
     $                   x(m1), xn(nb), y(m1), y2(m), z(nwcore)

*     ------------------------------------------------------------------
*     m5lpit performs an iteration of the primal simplex method.
*     jq is the variable entering the basis and djq is its reduced cost.
*
*     00 Aug 1987  This version allows the variable leaving the basis
*                  to be up to featol outside its bounds.
*     26 Mar 1992  incres now passed in as a parameter from m5pric.
*                  We used to say  incres = djq .lt. zero  to mean that
*                  the new variable x(jq) wants to increase, but that
*                  didn't allow a rigorous way of eliminating nonbasics
*                  that are between their bounds with djq = zero.
*     08 Apr 1992  Internal values of hs(*) mean hs(jq) and hs(jr)
*                  must be set more carefully.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m5log2/ jq1,jq2,jr1,jr2,lines1,lines2
      common    /m5log3/ djq,theta,pivot,cond,nonopt,jp,jq,modr1,modr2
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5lp2 / invrq,invitn,invmod
      common    /m5step/ featol, tolx0,tolinc,kdegen,ndegen,
     $                   itnfix, nfix(2)
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm

      logical            hitlow, move, onbnd, unbndd
      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )
*     ------------------------------------------------------------------

      jq2        = jq
      jr2        = jq
      hrtype(m1) = 0
      bbl(m1)    = bl(jq)
      bbu(m1)    = bu(jq)
      x(m1)      = xn(jq)

*     Unpack column jq into  y2  and solve  B*y = y2.
*     y2  will be altered, and is needed later to modify L and U.

      call m2unpk( jq, m, n, ne, nka, a, ha, ka, y2 )
      call m2bsol( 2, m, y2, y, z, nwcore )

*     Select a variable to be dropped from B.
*     m5chzr  uses the m+1-th element of hrtype, bbl, bbu, x and y.

      if (incres) then
         call dscal ( m, (- one), y, 1 )
         y(m1) =   one
      else
         y(m1) = - one
      end if
      stepmx = 1.0d+11

      call m5chzr( m1    , stepmx, plinfy, tolpiv,
     $             hrtype, bbl   , bbu   , x     , y,
     $             hitlow, move  , onbnd , unbndd,
     $             jp    , bound , exact , theta )

      if (unbndd) go to 800

*     See if there is a basis change.

      jr      = kb(jp)
      if (jp .eq. m1) then

*        Variable jq reaches its opposite bound.

         if (incres) then
            hs(jq)  = 1
         else
            hs(jq)  = 0
         end if
         pivot   = zero
         kmodlu  = 0
         if (ninf .eq. 0) kmodpi = 0
      else

*        Variable jq replaces jr, the jp-th variable of  B.
*        If jr is a fixed variable, its new state is hs(jr) = 4.

         jq1     = jq
         jr1     = jr
         hs(jq)  = 3
         if (bbl(jp) .eq. bbu(jp)) then
            hs(jr) = 4
         else if (hitlow) then
            hs(jr) = 0
         else
            hs(jr) = 1
         end if
         bbl(jp) = bbl(m1)
         bbu(jp) = bbu(m1)
         pivot   = - y(jp)
      end if

*     Update the basic variables x and copy them into xn.

      call daxpy ( m1, theta, y, 1, x, 1 )
      call m5bsx ( 1, m1, nb, kb, x, xn )
      kb(jp) = jq
      x(jp)  = x(m1)
      if (onbnd) xn(jr) = bound
      go to 900

*     The solution is apparently unbounded.

  800 if (iprint .gt. 0) then
         if (incres) then
            write(iprint, 1000) jq
         else
            write(iprint, 1100) jq
         end if
      end if
      ierr   = 2

*     Exit.

  900 return

 1000 format(' Variable', i6, '  can increase indefinitely')
 1100 format(' Variable', i6, '  can decrease indefinitely')

*     end of m5lpit
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5pric( m, mbs, n, nb, nn, nn0, ns, maxr, maxs, incres,
     $                   ne, nka, a, ha, ka,
     $                   hs, kb, bl, bu, gsub,
     $                   pi, rc, rg )

      implicit           double precision (a-h,o-z)
      logical            incres
      integer*4          ha(ne), hs(nb)
      integer            ka(nka), kb(mbs)
      double precision   a(ne), bl(nb), bu(nb)
      double precision   gsub(nn0), pi(m), rc(nb), rg(maxs)

*     ------------------------------------------------------------------
*     m5pric  selects a nonbasic variable to enter the basis,
*     using the reduced gradients  dj = gsub(j) - pi(t)*a(j).
*
*     This version does partial pricing on both structurals and slacks.
*     Also, multiple price no longer cancels partial price.
*     Dynamic tolerances are used if partial price is in effect.
*
*     Partial pricing here means sectional pricing, because the
*     columns of  A  and  I  are both sliced up into nparpr sections
*     of equal size.  (The last section of each may be a little bigger,
*     since nparpr is unlikely to divide evenly into  n  or  m.)
*
*     Input    gsub   = gradient for nonlinear variables
*                       (for the subproblem objective function).
*              pi     = pricing vector.
*              kprc   = the no. of the section where  m5pric  last found
*                       a useful dj.
*                       (kprc = 0 at the start of each major iteration.)
*              toldj(1-2) hold the current told if partial pricing, for
*                       phase 1 and 2 respectively.  told is used to
*                       determine if a dj is significant.
*              toldj(3) holds the specified optimality tolerance.
*              biggst   keeps track of the biggest dj found during the
*                       last scan of all sections of ( A  I ).
*
*     Output   kprc   = the last section scanned.
*              nonopt = the no. of useful djs found in that section.
*              jq     = best column found.
*              djq    = best dj.
*              newsb  = no. of new superbasics selected, if nonopt > 0.
*              toldj(1-2) save the current told if partial pricing.
*              incres   says if variable jq should increase or decrease.
*              kb(ns+1), ... kb(ns+newsb) = superbasics selected,
*              rg(ns+1), ... rg(ns+newsb) = their reduced gradients.
*
*     In the code below,
*     the next section of  A  contains npr1 structurals (j1+1 thru k1),
*     the next section of  I  contains npr2 slacks      (j2+1 thru k2).
*     If  nparpr  is rather large, either npr1 or npr2 could be zero,
*     but not both.
*
*     Original version written September 1981.
*
*     00 sep 1986: Modified to allow nonbasic variables to have
*     any value xn(j) between bl(j) and bu(j).  Variables that are
*     strictly between their bounds are allowed to go either way.
*
*x    00 aug 1987: Modified to let nonbasic variables go either way
*x    only if they are greater than bl(j) + featol
*x                    and less than bu(j) - featol,
*x    where featol (in common) is the current feasibility tolerance.
*x
*x    01 sep 1987: Modified to ignore nonbasics whose bounds are
*x    featol or less apart.  this prevents foolish data such as
*x                      0 .le. xn(j) .le. 0.0000001
*x    from causing lots of bound swaps.  m4chek has done its best to
*x    keep xn(j) at zero in such cases.  The final solution may appear
*x    to be non-optimal if xn(j) should really be at the other bound.
*
*     17 sep 1987: Previous two mods undone.  The preceding one allows
*     nonbasic xn(j)'s to be frozen at either 0 or 0.0000001
*     (in that example), and it may then be impossible to make
*     certain basic variables feasible.  pilotja strikes again!
*
*     We now price as follows.  Fixed variables are ignored as always.
*     Nonbasics that are on or outside a bound are allowed to move only
*     one way, towards the opposite bound.
*     Nonbasics strictly between their bounds are allowed to move
*     either way.
*
*     In the latter case, the worry is that rounding error in previous
*     steps (xn = xn + theta*y) might have put a blocking variable
*     very slightly inside one of its bounds.  Letting it move towards
*     that bound will give a very small (and essentially wasted) step.
*     To guard against this, m5chzr chooses a step that puts the
*     blocking variable exactly onto a bound if possible, and now
*     returns the logical onbnd and the relevant value bound so that
*     m5lpit and m7rgit will know whether to set xn(jr) = bound exactly.
*
*     16 Sep 1988: Partial pricing strategy altered to reduce told more
*     systematically.  biggst keeps track of the biggest dj encountered
*     during a scan from section 1 to section nparpr.  In general, told
*     is held fixed until the problem is optimal to within that
*     tolerance.  told is then reduced to reduce * told.
*     (This strategy abandoned April 1992.)
*
*     31 Jan 1992: m5rc   used to compute reduced costs.
*     26 Mar 1992: incres added as output parameter.
*                  Floating nonbasics treated more rigorously
*                  (i.e. nonbasics with djq = 0 or very small
*                  that we want to move towards a bound).
*     08 Apr 1992: Internal values of hs(*) now used (-1 and 4).
*                  xn, bl and bu are no longer needed.
*     09 Apr 1992: Initialize toldj(1-2) = plinfy in m5solv, so told
*                  is always set on the first iteration of Phase 1 or 2.
*     09 Apr 1992: We have had reduce = 0.2 for a while.
*                  Tried 0.1, 0.25 and 0.5.  Decided to go with 0.25.
*                  It still gives a moderate number of reductions
*                  to told, and did a bit better on STAIR and ETAMACRO.
*     11 Aug 1992: Went back to 0.2.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m5log3/ djq,theta,pivot,cond,nonopt,jp,jq,modr1,modr2
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5prc / nparpr,nmulpr,kprc,newsb
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg

      intrinsic          abs, max, min
      logical            feasbl, mulprc, normal
      parameter        ( zero  = 0.0d+0, reduce = 0.2d+0 )
*     ------------------------------------------------------------------

      maxmp  = min( nmulpr, maxs - ns )
      feasbl = ninf   .eq. 0
      mulprc = maxmp  .gt. 1
      normal = .not. mulprc
      djmax  = - plinfy
      djq    = zero
      jq     = 0
      jfree  = 0
      ms     = m + ns
      ncan   = 0
      newsb  = 1
      nonopt = 0
      nprc   = 0
      nparp  = nparpr
      npr1   = n / nparp
      npr2   = m / nparp
      if (max( npr1, npr2 ) .le. 0) nparp = 1

*     Set the tolerance for a significant dj.

      tolmin = toldj(3) * pinorm
      lvldj  = 1
      if ( feasbl )  lvldj = 2
      told   = toldj(lvldj)
      if (nparp .eq. 1) told = tolmin

*     Set pointers to the next section of  A  and  I.
*     nprc counts how many sections have been scanned in this call.
*     kprc keeps track of which one to start with.

  100 nprc = nprc + 1
      kprc = kprc + 1
      if (kprc .gt. nparp) kprc = 1

      npr1 = n / nparp
      j1   = (kprc - 1)*npr1
      k1   = j1 + npr1
      if (kprc .eq. nparp) k1 = n
      npr1 = max( 0, k1-j1 )

      npr2 = m / nparp
      j2   = n + (kprc - 1)*npr2
      k2   = j2 + npr2
      if (kprc .eq. nparp) k2 = nb
      npr2 = max( 0, k2-j2 )

*     ------------------------------------------------------------------
*     Main loops for partial pricing (or full pricing).
*     Compute reduced costs rc(*)
*     for the kprc-th section of structurals
*     and the kprc-th section of slacks.
*     ------------------------------------------------------------------
      call m5rc  ( j1+1, k1, feasbl,
     $             m, n, nn, nn0,
     $             ne, nka, a, ha, ka,
     $             hs, gsub, pi, rc )

      do 200 j = j2+1, k2
         rc(j) = - pi(j - n)
  200 continue

*     ------------------------------------------------------------------
*     Main loop for testing rc(*).
*     dj is rc(j), the reduced cost.
*     d  is -dj or +dj, depending on which way x(j) can move.
*     We are looking for the largest d (which will be positive).
*     ------------------------------------------------------------------
      np   = npr1 + npr2
      j    = j1
      jslk = npr1 + 1

      if ( normal ) then
*        ---------------------------------------------------------------
*        Regular case -- no multiple pricing.
*        ---------------------------------------------------------------
         do 400 jj = 1, np
            if (jj .eq. jslk) j = j2
            j      = j + 1
            js     = hs(j)

            if (js .le. 1) then
               dj     = rc(j)

               if      (js .eq. 0) then
*                 xj  is allowed to increase.
                  d      = - dj
               else if (js .eq. 1) then
*                 xj  is allowed to decrease.
                  d      =   dj
               else
*                 xj  is free to move either way.
*                 Remember him as jfree in case he is the only one.
                  d      = abs( dj )
                  jfree  = j
               end if

*              See if this dj is significant.
*              Also see if it is the biggest dj so far.

               if (d  .gt. told) nonopt = nonopt + 1
               if (djmax .lt. d) then
                  djmax  = d
                  djq    = dj
                  jq     = j
                  kpsav  = kprc
               end if
            end if
  400    continue

      else
*        ---------------------------------------------------------------
*        Multiple pricing.  The first part of the loop is the same.
*        ---------------------------------------------------------------
         do 500 jj = 1, np
            if (jj .eq. jslk) j = j2
            j      = j + 1
            js     = hs(j)

            if (js .le. 1) then
               dj     = rc(j)

               if      (js .eq. 0) then
*                 xj  is allowed to increase.
                  d      = - dj
               else if (js .eq. 1) then
*                 xj  is allowed to decrease.
                  d      =   dj
               else
*                 xj  is free to move either way.
*                 Remember him as jfree in case he is the only one.
                  d      = abs( dj )
                  jfree  = j
               end if

*              See if this dj is significant.
*              Also see if it is the biggest dj so far.

               if (d  .gt. told) nonopt = nonopt + 1
               if (djmax .lt. d) then
                  djmax  = d
                  djq    = dj
                  jq     = j
                  kpsav  = kprc
               end if

*              ========================================
*              Extra stuff for multiple pricing.
*              We have to build the list of candidates.
*              ========================================
               if (d .gt. tolmin) then

*                 Search the list backwards,
*                 starting with smallest rg values.

                  do 410 i  = ncan, 1, -1
                     msi    = ms + i
                     nsi    = ns + i

*                    See if this list element will stay put.
*                    If not, make sure we have space to move it.

                     if (d .le. abs( rg(nsi) )) go to 420

                     if (i .lt. maxmp) then
*                       This element lost the contest but is
*                       staying in the list.
                        kb(msi + 1) = kb(msi)
                        rg(nsi + 1) = rg(nsi)
                     end if
  410             continue

*                 This is the biggest dj so far.

                  i     = 0

*                 If room, put the new candidate into the list.

  420             if (i .lt. maxmp) then
                     i          = i + 1
                     kb(ms + i) = j
                     rg(ns + i) = dj
                     ncan       = min( ncan + 1, maxmp )
                  end if
               end if
            end if
  500    continue
      end if
*     ------------------------------------------------------------------
*     End of loop looking for biggest dj in the kprc-th section.
*     ------------------------------------------------------------------

      if (nonopt .eq. 0) then
         if (nparp .gt. 1) then
*           ============================================================
*           No significant dj has been found.  (All are less than told.)
*           Price the next section, if any remain.
*           ============================================================
            if (nprc .lt. nparp) go to 100

*           ============================================================
*           All sections have been scanned.  Reduce told
*           and grab the best dj if it is bigger than tolmin.
*           ============================================================
            if (djmax .gt. tolmin) then
               nonopt = 1
               kprc   = kpsav
               told   = max( reduce * djmax, tolmin  )
               toldj(lvldj) = told
               if (itn    .gt. 0) wtobj = reduce * wtobj
               if (idebug .ge. 1) then
                  if (iprint .gt. 0)
     $            write(iprint, 1000) itn, told, pinorm, wtobj
                  if (isumm  .gt. 0)
     $            write(isumm , 1000) itn, told, pinorm, wtobj
               end if
            end if
         end if
      end if

*     Finish up multiple pricing.
*     The new dj's are now in descending order, with the biggest
*     in rg(ns+1).  Truncate the list if any are much smaller.

      if ( mulprc ) then
         d      = reduce * djmax
         do 550 j = 2, ncan
            if (abs( rg(ns+j) ) .ge. d) newsb = newsb + 1
  550    continue
      end if

*     -----------------------------------------------------------------
*     Finish if we found someone nonoptimal (nonopt .gt. 0)
*     or if there's a nonbasic floating free
*     between its bounds                    (jfree  .eq. 0)
*     or if the problem is nonlinear        (nn     .gt. 0)
*     or there are some superbasics         (ns     .gt. 0).
*     -----------------------------------------------------------------
      incres = djq .lt. zero
      if (nonopt .gt. 0) go to 900
      if (jfree  .eq. 0) go to 900
      if ( feasbl ) then
         if (nn  .gt. 0) go to 900
         if (ns  .gt. 0) go to 900
      end if

*     -----------------------------------------------------------------
*     jfree > 0.
*     We prefer not to end an LP problem (or an infeasible problem)
*     with some nonbasic variables floating free between their bounds
*     (hs(j) = -1).  Their dj's will be essentially zero
*     but we might as well jam them into the basis.
*
*     We leave true free variables alone -- they will probably be zero.
*     (To be rigorous, we should test if they ARE zero and move them
*     towards zero if not.  This has yet to be done...)
*     -----------------------------------------------------------------
      do 600 j = 1, nb
         if (hs(j) .eq. -1) then
            b1     = bl(j)
            b2     = bu(j)
            if (b1 .gt. -plinfy  .or.  b2 .lt. plinfy) then
      
*              We just found a floating variable with finite bounds.
*              Ask for a move towards the bound nearest zero.
      
               incres = abs(b1) .ge. abs(b2)
               nonopt = 1
               jq     = j
               djq    = rc(j)
               go to 900
            end if
         end if
  600 continue

*     Exit.

  900 kb(ms + 1) = jq
      rg(ns + 1) = djq
      return

 1000 format(' Itn', i7, ' -- toldj =', 1p, e8.1,
     $       '    Norm pi =', e8.1, '    wtobj = ', e8.1)

*     end of m5pric
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5rc  ( j1, j2, feasbl,
     $                   m, n, nn, nn0,
     $                   ne, nka, a, ha, ka,
     $                   hs, gsub, pi, rc )

      implicit           double precision (a-h,o-z)
      logical            feasbl
      integer*4          ha(ne), hs(n)
      integer            ka(nka)
      double precision   a(ne)
      double precision   gsub(nn0), pi(m), rc(n)

*     ------------------------------------------------------------------
*     m5rc   computes reduced costs rc(j) for nonbasic columns of A
*     in the range j = j1 to j2.  It is called by m5pric.
*
*     The loop for computing dj for each column could conceivably be
*     optimized on some machines.  However, there are seldom more than
*     5 or 10 entries in a column.
*
*     Note that we could skip fixed variables by passing in the bounds
*     and testing if bl(j) .eq. bu(j), but these are relatively rare.
*     But see comment for 08 Apr 1992.
*
*     31 Jan 1992: First version
*     08 Apr 1992: Internal values of hs(j) are now used, so fixed
*                  variables (hs(j) = 4) are skipped as we would like.
*     ------------------------------------------------------------------

      parameter        ( zero = 0.0d+0 )

      do 300 j = j1, j2
         if (hs(j) .le. 1) then
            dj    = zero

            do 200 i = ka(j), ka(j+1) - 1
               ir    = ha(i)
               dj    = dj  +  pi(ir) * a(i)
  200       continue

            rc(j) = - dj
         end if
  300 continue

*     Include the nonlinear objective gradient if relevant.

      if (feasbl  .and.  j1 .le. nn) then
         do 350 j = j1, min( j2, nn )
            if (hs(j) .le. 1) rc(j) = rc(j) + gsub(j)
  350    continue
      end if

*     end of m5rc
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5setp( mode, m, rhs, pi, z, nwcore )

      implicit           double precision (a-h,o-z)
      double precision   pi(m), rhs(m), z(nwcore)

*     ------------------------------------------------------------------
*     m5setp solves for pi or finds pinorm (or does both).
*     mode = 1     means solve B'pi = rhs and get pinorm.
*     mode = 2     means solve B'pi = rhs.
*     mode = 3     means                 just get pinorm.
*
*     Beware -- in mode 1 and 2, rhs is altered by m2bsol.
*
*     10 Mar 1992: Until now we have required pinorm ge 1.
*                  However, many users have very small objectives,
*                  and the effect is to cause premature termination.
*                  Using pimin < 1 should help fix this.
*     16 Mar 1992: mode used more fully as described above.
*     ------------------------------------------------------------------

      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg

      intrinsic          max
      parameter        ( pimin = 1.0d-2 )

      if (mode .eq. 1  .or.  mode .eq. 2) then
         call m2bsol( 3, m, rhs, pi, z, nwcore )
      end if

      if (mode .eq. 1  .or.  mode .eq. 3) then
         pinorm = dnorm1( m, pi, 1 )
         pinorm = max( pinorm, pimin )
      end if

*     end of m5setp
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5setx( mode, m, n, nb, ms, kb,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, x, xn, y, y2, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka), kb(ms)
      double precision   a(ne), bl(nb), bu(nb)
      double precision   x(ms), xn(nb), y(m), y2(m), z(nwcore)

*     ------------------------------------------------------------------
*     m5setx performs the following functions:
*
*     If mode = 0, the slacks are set to satisfy Ax + s = rhs
*                  if possible.  (Thus s = rhs - Ax, but s may
*                  have to be moved inside its bounds.)
*                  Called (perhaps) from the top of m5solv.
*
*     If mode = 1, the basic components of x are computed to satisfy
*                  Ax + s = rhs; that is  (A I)*xn = rhs.
*                  Then a row check is performed to see how well
*                  (A I)*xn = rhs is satisfied.
*                  y is set to be the row residuals, y = rhs - Ax - s,
*                  and the row error is norm(y).
*                  Called from m2bfac, m5dgen, m5solv.
*
*     If mode = 2  or more, a row check is performed.
*                  Called from m5solv.
*
*     14 Oct 1991: a, ha, ka added as parameters to help with minoss.
*     06 Dec 1991: mode 0 added.  n, bl, bu added as parameters.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m5log1/ idebug,ierr,lprint
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold

      intrinsic          abs
      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

*     ------------------------------------------------------------------
*     Compute row residuals  y  =  rhs  -  A*x.
*     ------------------------------------------------------------------
      if (nncon .gt. 0) call dcopy ( nncon, z(lrhs), 1, y, 1 )
      if (nncon .lt. m) call dload ( m-nncon, zero, y(nncon+1), 1 )
      call m2aprd( 5, xn, n, y, m,
     $             ne, nka, a, ha, ka,
     $             z, nwcore )

      if (mode .eq. 0) then
*        ---------------------------------------------------------------
*        Set the slacks.
*        s = y in principle, but we have to make s feasible.
*        Any elements CLOSE to a bound are moved exactly onto the bound.
*        ---------------------------------------------------------------
         tolb   = eps1
         do 10  i = 1, m
            j     = n + i
            b1    = bl(j)
            b2    = bu(j)
            s     = max( y(i), b1 )
            s     = min( s   , b2 )
            if ((s - b1) .gt. (b2 - s)) b1 = b2
            if (abs(s - b1) .le.  tolb) s  = b1
            xn(j) = s
   10    continue

      else
*        ---------------------------------------------------------------
*        Do a row check, perhaps after recomputing the basic x.
*        ---------------------------------------------------------------

*        Get the full row residuals  y  =  rhs  -  (A  I)*xn.

         call daxpy ( m, (- one), xn(n+1), 1, y, 1 )

         if (mode .eq. 1) then
*           ===============================================
*           Extract the basic and superbasic x from xn.
*           See if iterative refinement is worth doing.
*           ===============================================
            call m5bsx ( 2, ms, nb, kb, x, xn )
            ir     = idamax( m, y, 1 )
            rmax   = abs( y(ir) )
            ir     = idamax( m, x, 1 )
            xnorm  = abs( x(ir) )
            rowerr = rmax / (one + xnorm)

            if (rowerr .gt. eps0) then
*              ===============================================
*              Compute a correction to basic x from  B*y2 = y.
*              Set basic x = x + y2.
*              Store the new basic x into xn.
*              ===============================================
               call m2bsol( 2, m, y, y2, z, nwcore )
               call daxpy ( m, one, y2, 1, x, 1 )
               call m5bsx ( 1, m , nb, kb, x, xn )
         
*              Compute  y  =  rhs  -  (A  I)*xn  again for the new xn.
         
               if (nncon .gt. 0) call dcopy ( nncon, z(lrhs), 1, y, 1 )
               if (nncon .lt. m) call dload ( m-nncon, zero,
     $                                                  y(nncon+1), 1 )
               call m2aprd( 5, xn, n, y, m,
     $                      ne, nka, a, ha, ka,
     $                      z, nwcore )
               call daxpy ( m, (- one), xn(n+1), 1, y, 1 )
            end if
         end if

*        Find the norm of x(BS), the basic and superbasic x.
*        Allow for the rare case  x = 0.

         xnorm  = dnorm1( ms, x, 1 )
         if (xnorm .le. eps4) xnorm = one

*        Find the maximum row residual.

         ir     = idamax( m, y, 1 )
         rmax   = abs( y(ir) )
         rowerr = rmax / (one + xnorm)
         if (rowerr .gt. tolrow) ierr = 10
         if (rowerr .gt. tolrow  .or.  idebug .ge. 2) then
            if (iprint .gt. 0) write(iprint, 1000) itn, rmax, ir, xnorm
         end if
      end if
      return

 1000 format(' Itn', i7, ' -- row check',
     $   '.   Max residual =', 1p, e8.1, '  on row', i5,
     $   '.   Norm x =', e9.2)

*     end of m5setx
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m5solv( m, maxr, maxs, mbs, n, nb, nn, nn0, nr,
     $                   lcrash, ns, nscl, nx, objadd,
     $                   ne, nka, a, ha, ka,
     $                   hrtype, hs, kb, ascale, bl, bu,
     $                   bbl, bbu, fsub, gsub, grd, grd2,
     $                   pi, r, rc, rg, rg2,
     $                   x, xn, y, y2, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hrtype(mbs), hs(nb)
      integer            ka(nka), kb(mbs)
      double precision   a(ne), ascale(nscl), bl(nb), bu(nb)
      double precision   bbl(mbs), bbu(mbs),
     $                   gsub(nn0), grd(mbs), grd2(mbs),
     $                   pi(m), r(nr), rc(nb), rg(maxs), rg2(maxs),
     $                   x(nx), xn(nb), y(nx), y2(nx), z(nwcore)

*     ------------------------------------------------------------------
*     m5solv  solves the current problem.  A basis is assumed to be
*     specified by ns, hs(*), xn(*) and the superbasic parts of kb(*).
*     In particular, there must be ns values hs(j) = 2, and the
*     corresponding j's must be listed in kb(m+1) thru kb(m+ns).
*     The ordering in kb matches the projected Hessian R (if any).
*
*     GOTFAC and GOTHES are input and output variables.
*     On entry they say whether to preserve LU and R.  For the first
*              cycle they will normally be false.
*     On exit  they say whether LU and R exist, for possible use on
*              on later cycles.
*     gotlam   no longer exists.  Instead, initial estimates of the
*              Lagrange multipliers for the nonlinear constraints are
*              always extracted from pi.
*
*     On exit, if  ierr .lt. 30  it is safe to save the final
*     basis files and print the solution.  Otherwise, a fatal error
*     condition exists and numerous items will be undefined.
*     The last basis map saved (if any) retains the only useful
*     information.
*
*     31 Jan 1992: rc(*) introduced as workspace for m5pric.
*     06 Mar 1992: Major iteration 1 now just satisfies the linear
*                  constraints and bounds.  It may take as long as it
*                  likes -- the minor iterations limit does not apply.
*     17 Mar 1992: m7fixb implemented to try and fix an ill-conditioned
*                  basis by swapping a column B with a column of S.
*                  In this version, we consider it only at the start of
*                  a major iteration.
*                  fixb   says when we want to try.
*     08 Apr 1992: m5hs   implemented to switch nonbasic hs(j) between
*                  external and internal values to help speed up m5pric.
*     10 Apr 1992: objadd added as an input parameter.
*     04 Jun 1992: Crash level 3 implemented.
*     07 Oct 1992: Minor iterations limit now restricts each major itn
*                  to nminor FEASIBLE iterations.
*     ------------------------------------------------------------------

      logical            conv,restrt
      logical            alone, AMPL, GAMS, MINT, page1, page2
      common    /m1env / alone, AMPL, GAMS, MINT, page1, page2
      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2file/ iback,idump,iload,imps,inewb,insrt,
     $                   ioldb,ipnch,iprob,iscr,isoln,ispecs,ireprt
      common    /m2lu3 / lenl,lenu,ncp,lrow,lcol
      common    /m2parm/ dparm(30),iparm(30)
      common    /m3mps2/ lcnam1,lrnam1,lcnam2,lrnam2,lxs,lxl,lfree
      common    /m3mps4/ name(2),mobj(2),mrhs(2),mrng(2),mbnd(2),minmax
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5freq/ kchk,kinv,ksav,klog,ksumm,i1freq,i2freq,msoln
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m5log2/ jq1,jq2,jr1,jr2,lines1,lines2
      common    /m5log3/ djq,theta,pivot,cond,nonopt,jp,jq,modr1,modr2
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5lp2 / invrq,invitn,invmod
      common    /m5prc / nparpr,nmulpr,kprc,newsb
      common    /m5step/ featol, tolx0,tolinc,kdegen,ndegen,
     $                   itnfix, nfix(2)
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m7loc / lgobj ,lgobj2
      common    /m7cg1 / cgbeta,itncg,msgcg,modcg,restrt
      common    /m7conv/ etash,etarg,lvltol,nfail,conv(4)
      common    /m7phes/ rgmin1,rgnrm1,rgnrm2,jz1,jz2,labz,nfullz,mfullz
      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /m8al2 / radius,rhsmod,modpen,modrhs
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2
      common    /m8save/ vimax ,virel ,maxvi ,majits,minits,nssave
      logical            gotbas,gotfac,gothes,gotscl
      common    /cycle1/ gotbas,gotfac,gothes,gotscl
      common    /cycle2/ objtru,suminf,numinf

      character*12       istate
      logical            first , fixb  , incres,
     $                   feasbl, infsbl, jstfes,
     $                   linear, nonlin, lincon, nlncon,
     $                   major1, convgd, optsub

      intrinsic          abs, min, max, mod

      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

*     ------------------------------------------------------------------

*     Initialize scalars (in alphabetical order).

      fixb   = .false.
      linear = nn     .eq. 0
      nonlin = nn     .gt. 0
      lincon = nncon  .eq. 0
      nlncon = nncon  .gt. 0

      cond   = zero
      flin   = zero
      fobj   = zero
      fsub   = zero
      fxprev = zero

      ierr   = 0
      invitn = 0
      invrq  = 0
      itn    = 0
      lines1 = 0
      lines2 = 0
      majits = 0
      minits = 0
      minfes = 0
      modwt  = 0
      m1     = m + 1
      nfac   = 0
      nfail  = 0
      ninf   = 0
      nphs   = 1
      nsamef = 0
      msamef = max( 2*nb, 200 )
      msamef = min( msamef, 1000 )
      nssave = ns

      objtru = zero
      penpar = dparm(7)
      pinorm = one
      rgnorm = zero
      rgtest = zero
      sinf   = zero
      tolz   = eps0
      xnorm  = one
      do 30 i = 1, 4
         conv(i) = .false.
   30 continue

      newhed = .true.
      if (.not. gothes) r(1) = zero

      if ( lincon ) then

*        Set nonbasic hs(j) to the correct internal values.

         call m5hs  ( 'Internal', nb, bl, bu, hs, xn )
      else
*        ( nlncon )
         infsub = 0
         lcstat = 0
         nomove = 0
         nreduc = 0
         gotfac = .false.
         optsub = .false.
      end if

*     ------------------------------------------------------------------
*     Start of a Major Iteration.
*     "first" says we've just started, and tells m5frmc to set nphs.
*     If the problem is linearly constrained, there is only one
*     major iteration (only one subproblem to be solved).
*     ------------------------------------------------------------------
   90 majits = majits + 1
      major1 = majits .eq. 1
      first  = .true.
      ierr   = 0
      kprc   = 0
      lvldif = 1
      lvltol = 1
      ms     = m + ns
      nfail  = 0
      nfullz = 0
      nonopt = 0
      toldj(1) = plinfy
      toldj(2) = plinfy
      tolrg  = zero
      call m5dgen( 1, m, n, nb, ms, inform,
     $             ne, nka, a, ha, ka,
     $             hs, kb, bl, bu, x, xn, y, y2, z, nwcore )

      if (nlncon) then
         call m8setj( lcstat, m, n, nb, nn, nncon, nnjac, njac, nscl,
     $                lcrash, ns,
     $                nconvg, nomove, nreduc, optsub, convgd,
     $                ne, nka, a, ha, ka, hs, ascale, bl, bu,
     $                z(lblslk),z(lbuslk),z(lfcon), z(lfcon2),
     $                z(lfdif), z(lfold), z(lgcon), z(lgcon2), z(lxlam),
     $                z(lrhs ), z(lxdif), z(lxold),
     $                pi, xn, y, z, nwcore )

         minits = 0
         minfes = 0
         fixb   = ns .gt. 0
         call m1envt( 999 )
         if (ierr   .ne.   0   ) go to 900
         if (      convgd      ) go to 800
         if (majits .gt. nmajor) go to 825
         if (itn    .ge. itnlim) go to 830
         if (nomove .ge.   4   ) go to 850
         optsub = .false.
      end if

*--   if (major1) then

*        Set the slacks s using the current structurals x.
*        If lincon, try and satisfy Ax + s = 0.
*        If nlncon, try and satisfy Ax + s = rhs.

*--      call m5setx( 0, m, n, nb, ms, kb,
*--  $                ne, nka, a, ha, ka,
*--  $                bl, bu, x, xn, y, y2, z, nwcore )
*--   end if

*     ------------------------------------------------------------------
*     Factorize the basis ( B = L*U ).  If gotfac is true on entry to
*     m5solv, the first call to m2bfac will try to use existing factors.
*     ------------------------------------------------------------------
  100 if (first) then
*        relax
      else
*        Safeguard against system error: If there have been no
*        iterations since the last factorize, something is haywire.
         if (invitn .eq. 0) go to 865
      end if

  105 call m2bfac( gotfac, nfac, m, mbs, n, nb, nr, nn, ns,
     $             lcrash, fsub, objadd,
     $             ne, nka, a, ha, ka,
     $             kb, hs, bl, bu, bbl, bbu,
     $             r, grd, x, xn, y, y2, z, nwcore )
      if (ierr .gt. 0) go to 860

      nssave = ns
      lusiz0 = lenl + lenu
      lumax  = 2*lusiz0
      kmodpi = 1
      restrt = .true.
      if (mod(lprint,10) .gt. 0) newhed = .true.

*     Under certain circumstances, see if B is ill-conditioned
*     and perhaps improve it by swapping in a superbasic.

  110 if ( fixb ) then
         fixb   = .false.
         ms     = m + ns
         call m7fixb( m, maxr, ms, n, nb, nr, ns, nx, inform,
     $                ne, nka, a, ha, ka,
     $                hs, kb, bl, bu, bbl, bbu,
     $                r, x, y, y2, z, nwcore )

*        Refactorize if m7fixb tried to update B = L*U and failed.

         if (inform .eq. 5) go to 105
      end if

*     ------------------------------------------------------------------
*     MAIN ITERATION LOOP.
*     ------------------------------------------------------------------
  200 ierr   = 0
      ms     = m + ns
      ninf0  = ninf
      sinf0  = sinf

*     Increment featol every iteration.
*     Every kdegen iterations, reset featol and move nonbasic variables
*     onto their bounds if they are currently very close.

      featol = featol + tolinc
      if (mod( minits, kdegen ) .eq. 0) then
         if (minits .gt. 0) then
            call m5dgen( 2, m, n, nb, ms, inform,
     $                   ne, nka, a, ha, ka,
     $                   hs, kb, bl, bu, x, xn, y, y2, z, nwcore )
         end if
      end if

*     ------------------------------------------------------------------
*     Check feasibility.
*     ------------------------------------------------------------------
      call m5frmc( n, nb, nn, ns, ms, maxs,
     $             lcrash, first, fsub, featol, objadd,
     $             ne, nka, a, ha, ka,
     $             bl, bu, bbl, bbu, hrtype, hs, kb,
     $             grd2, x, xn, z, nwcore )

      if (ierr .ne. 0) go to 900
      feasbl =  ninf  .eq. 0
      infsbl = .not. feasbl
      jstfes =  feasbl  .and.  (first  .or.  ninf0 .gt. 0)
      first  = .false.

      if (infsbl) then

*        Reduce wtobj if sinf has increased noticeably.

         if (wtobj .gt. zero) then
            if (itn .gt. 0) then
               if (sinf  .gt.  sinf0 * 1.1) go to 655
            end if
         end if
      end if

*     ------------------------------------------------------------------
*     See if the next phase of Crash is needed.
*     ------------------------------------------------------------------
      if (jstfes  .and.  lcrash .gt. 0) then
         if      (lcrash .eq. 1) then

*           All constraints have now been satisfied.
*           Major1 will be terminated below after pi is set.

            lcrash = 0

         else if (lcrash .eq. 2  .or.  lcrash .eq. 4) then

*           Linear constraints have been satisfied.
*           Major1 will be terminated and Major2 will
*           call Crash on nonlinear rows.

            if (lincon) lcrash = 0
            if (nlncon) lcrash = 5

         else if (lcrash .eq. 3) then
            if (nncon .lt. m) then

*              So far the linear inequality constraints (if any)
*              have been ignored.  Do another Crash and continue Major1.
*              m2amat sets row-types.
            
               lcrash = 4
               call m5hs  ( 'External', nb, bl, bu, hs, xn )
               call m2amat( 2, m, n, nb,
     $                      ne, nka, a, ha, ka,
     $                      bl, bu, hrtype )
               call m2crsh( lcrash, m, n, nb, nn,
     $                      ne, nka, a, ha, ka,
     $                      kb, hs, hrtype, bl, bu, xn, z, nwcore )
               call m5hs  ( 'Internal', nb, bl, bu, hs, xn )
               ninf   = 1
               go to 105

            else
               lcrash = 5
            end if
         end if
      end if

*     ------------------------------------------------------------------
*     Compute pi.
*     ------------------------------------------------------------------
      if (kmodpi .gt. 0) then
         if (feasbl  .and.  nonlin) then
            call m6grd ( ms, nb, nn, gsub, grd2,
     $                   ne, nka, a, ha, ka,
     $                   xn, z, nwcore )
            if (ierr .ne. 0) go to 900
         end if
         call dcopy ( ms, grd2, 1, grd, 1 )
         call m5setp( 1, m, grd2, pi, z, nwcore )
      end if

*     ------------------------------------------------------------------
*     If we have just got feasible, reset a few things.
*     If there are nonlinear constraints and it's Major1,
*     we have satisfied the linear constraints for the first time.
*     That is enough for now --- time to relinearize.
*     ------------------------------------------------------------------
      if (jstfes) then
          lvltol = 1
          tolrg  = zero
          if (nlncon  .and.  major1) go to 500
      end if

*     ------------------------------------------------------------------
*     Compute rg = reduced gradient.
*     ------------------------------------------------------------------
      rgnorm = zero
      if (nphs .gt. 2) then
         if (ns .gt. 0) then
            call m7rg  ( m, ms, ns, grd, pi, rg, rgnorm,
     $                   ne, nka, a, ha, ka,
     $                   z, nwcore )
         end if
         if (tolrg .eq. zero) tolrg = etarg * rgnorm
      end if

*     ------------------------------------------------------------------
*     Proceed with the next iteration.
*     ------------------------------------------------------------------
  250 kmodlu = 1
      kmodpi = 1
      modr1  = 0
      modr2  = 0
      jq1    = 0
      jq2    = 0
      jr1    = 0
      jr2    = 0
      jz2    = 0
      labz   = 1
      nxtphs = nphs
      if (jobj .gt. 0) flin = - xn(jobj) * sclobj  +  objadd
      objtru = flin + fobj

      if (nphs .le. 3) then

*        Phase 1, 2 or 3.
*        PRICE:
*        Select one or more nonbasic variables to become superbasic.

         if (ns .eq. maxs) go to 840

         call m5pric( m, mbs, n, nb, nn, nn0, ns, maxr, maxs, incres,
     $                ne, nka, a, ha, ka,
     $                hs, kb, bl, bu, gsub,
     $                pi, rc, rg )

         if (nonopt .eq. 0) go to 400
         if (newsb  .gt. 1) nphs   = 3
         if (   nlncon    ) nconvg = 0
         lvldif = 1
         lvltol = 1
         nsubsp = 0
      end if

*     Test for excess iterations or time.

  260 if (lincon) then
         if (itn    .ge. itnlim) go to 830
         if (mod(itn,10) .eq. 0) call m1envt( 999 )
      else
         if (itn    .ge. itnlim) go to 500

*        The first major iteration may take as long as it likes
*        to satisfy the linear constraints.  Later major iterations
*        are terminated by the minor iterations limit.
*        07 Oct 1992: Let all majors take as long as they like
*                     to get feasible.  Use minfes instead of minits.

         if (minfes .ge. nminor) go to 500
      end if
      if (ierr .gt. 0) go to 880

*     ==================================================================
*     Perform a minor iteration (either LP or reduced-gradient method).
*     ==================================================================
  280 if (nphs .le. 2) then
         call m5lpit( m, m1, n, nb, incres,
     $                ne, nka, a, ha, ka,
     $                hrtype, hs, kb,
     $                bl, bu, bbl, bbu, x, xn, y, y2, z, nwcore )
      else
         if (nphs .eq. 4  .and.  ns .le. 0) then
            nphs = 3
            go to 250
         end if

         call m7rgit( m, maxr, maxs, mbs, n, nb, incres,
     $                nn, nn0, nr, ns, nx, inform, nxtphs,
     $                ne, nka, a, ha, ka,
     $                hrtype, hs, kb, bl, bu, bbl, bbu,
     $                fsub, gsub, grd, grd2,
     $                pi, r, rg, rg2, x, xn, y, y2, z, nwcore )

*        For certain values of  inform  we want to try again.
*        If inform = 6, m7fixb has been requested (to fix R).

         if (inform .eq. 6) then
            fixb   = .true.
            if (invitn .gt. 0) go to 105
            go to 110
         end if

*        If inform = 5, m2bfac has been requested
*                       (part of error recovery).
*        If inform = 4, we want a better  gsub  and  pi  using
*                       central differences (non-derivative case only).
*        If inform = 1, we want to call  m5pric.

         if (inform .eq. 5) go to 100
         if (inform .eq. 4) go to 200
         if (inform .eq. 1) go to 250

*        Otherwise, m7rgit exits with a good pi and grd
*        (unless ierr gt 0).
*        Set kmodpi to indicate that we don't need a new pi.

         kmodpi = 0
      end if

*     ==================================================================
*     Test for error condition and/or frequency interrupts.
*     ==================================================================
  300 ms     = m + ns
      if (ierr .eq. 2) go to 460
      if (ierr .gt. 0) go to 680
      itn    = itn    + 1
      invitn = invitn + 1
      minits = minits + 1
      if (ninf .eq. 0) minfes = minfes + 1
      if (linear  .and.  iobj .ne. 0) fsub = - minimz * x(iobj) * sclobj

*     Print the iteration log.

      call m5log ( m, maxs, mbs, n, nb, nn, ns, nx, fsub, objadd,
     $             ne, nka, a, ha, ka,
     $             bl, bu, hs, kb, x, xn, y, z, nwcore )

*     Check for the dreaded infinite loop
*     by seeing if the objective has stayed the same much too long.

      nsamef = nsamef + 1
      fx     = sinf
      if (     feasbl       ) fx = fsub
      if (abs( fxprev - fx )  .gt.  (one + abs( fx )) * eps1) nsamef = 0
      if (nsamef .ge. msamef) go to 835

*     Press on.

      fxprev = fx
      nphs   = nxtphs
      if (nlncon) then
         if (infsbl  .or.  rgnorm .gt. rgtest) nconvg = 0
      end if

*     Save a basis map (frequency controlled).
*     We have to convert hs(*) to External and then back to Internal.

      if (mod(itn,ksav) .eq. 0) then
         if (inewb .gt. 0  .and.  itn .lt. itnlim) then
            call m5hs  ( 'External', nb, bl, bu, hs, xn )
            call m4stat( 0, istate )
            call m4newb( 1, inewb, m, n, nb, nn, ns, ms, nscl, fsub,
     $                   kb, hs, ascale, bl, bu, x, xn, istate )
            if (iback .gt. 0)
     $      call m4newb( 1, iback, m, n, nb, nn, ns, ms, nscl, fsub,
     $                   kb, hs, ascale, bl, bu, x, xn, istate )
            call m5hs  ( 'Internal', nb, bl, bu, hs, xn )
         end if
      end if

*     Refactorize the basis if it has been modified a lot.

      if (invmod .ge. kinv - 1) then
         invrq  = 1
         go to 100
      end if

      lusize = lenl + lenu
      if (invmod .ge. 20  .and.  lusize .gt. lumax) then
         bgrwth = lusize
         bold   = lusiz0
         bgrwth = bgrwth / bold
         if (prnt1) write(iprint, 1030) bgrwth
         invrq  = 2
         go to 100
      end if

*     Update the LU factors of the basis if requested.

      if (kmodlu .eq. 1) call m2bsol( 4, m, y2, y, z, nwcore )
      if (invrq  .ne. 0) go to 100

*     Check row error (frequency controlled).

      if (mod(invitn,kchk) .eq. 0) then
         call m5setx( 2, m, n, nb, ms, kb,
     $                ne, nka, a, ha, ka,
     $                bl, bu, x, xn, y, y2, z, nwcore )
         if (ierr .gt. 0) then
            invrq = 10
            go to 100
         end if
      end if

      go to 200

*     ------------------------------------------------------------------
*     END OF MAIN ITERATION LOOP.
*     ------------------------------------------------------------------

*     The solution is apparently optimal -- check rgtols, etc.

  400 if ( infsbl ) then
*        =============================
*        Optimal but still infeasible.
*        =============================
         if (wtobj  .ne. zero ) go to 655

*        See if any nonbasics have to be set back on their bound.

         call m5dgen( 3, m, n, nb, ms, inform,
     $                ne, nka, a, ha, ka,
     $                hs, kb, bl, bu, x, xn, y, y2, z, nwcore )
         if (inform .gt.   0  ) go to 200

         if (ns     .eq.   0  ) go to 410
         if (lvltol .eq.   2  ) go to 410
         lvltol = 2
         tolrg  = toldj(3) * pinorm
         if (rgnorm .le. tolrg) go to 410

         nphs   = 4
         if (prnt1) write(iprint, 1400) tolrg, lvltol
         go to 280

*        The subproblem is infeasible.
*        Stop if the constraints are linear
*        or if this is the first major iteration
*        (which could not satisfy the linear constraints).

  410    lcstat = 1
         if (lincon  .or.  major1) go to 810
         if (modrhs .eq. 0) infsub = infsub + 1
         if (infsub .gt. 5) go to 810

*        Slightly relax the bounds on the linearized constraints.

         call m8augl( 5, m, n, nb, ns, inform,
     $                ne, nka, a, ha, ka,
     $                hs, bl, bu, xn, z, nwcore )

         if (inform .eq. 0) then

*           Reset nonbasic hs(j) to internal values.
*           Reset the basic x's and check the row error.

            kmodpi = 1
            call m5hs  ( 'Internal', nb, bl, bu, hs, xn )
            call m5setx( 1, m, n, nb, ms, kb,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, x, xn, y, y2, z, nwcore )
            if (ierr .gt. 0) then
               invrq = 10
               go to 100
            end if
            go to 200
         end if

*        The rhs has been relaxed often enough for this subproblem.
*        Keep going if it's the first time or something is happening.

         if (infsub .le. 1  .or.  minits .gt. 0) go to 500
         go to 810
      else
*        =============================================
*        Feasible and maybe optimal.
*        m5pric says there are no nonbasics to bring in.
*        We can stop if rgnorm is suitably small.
*        For Partial Completion (ncom = 0), a looser
*        measure of small will do.
*        =============================================
         if (ns   .eq. 0) go to 420
         if (ncom .eq. 0) then
            tolmin = 1.0d-2   * pinorm
         else
            tolmin = toldj(3) * pinorm
         end if
         if (rgnorm .le. tolmin) go to 420
         if (lvltol .eq.    2  ) go to 420
         if (nfail  .gt. 0  .and.
     $       nfullz .le.    1  ) go to 420

         nfail  = 0
         nphs   = 4
         tolrg  = 0.1 * min( tolrg, rgnorm )
         if (tolrg  .le. tolmin) then
            lvltol = 2
            tolrg  = tolmin
         end if
         if (prnt1) write(iprint, 1400) tolrg, lvltol
         go to 280

*        The rgtols are small enough.
*        See if any nonbasics have to be set back on their bound.

  420    call m5dgen( 3, m, n, nb, ms, inform,
     $                ne, nka, a, ha, ka,
     $                hs, kb, bl, bu, x, xn, y, y2, z, nwcore )
         if (inform .gt. 0) go to 200

*        Check residuals before terminating.

         djq    = minimz * djq
         if (prnt1) write(iprint, 1410) djq, jq, rgnorm, pinorm

         if (invitn .gt. 0) then
            call m5setx( 3, m, n, nb, ms, kb,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, x, xn, y, y2, z, nwcore )
            if (ierr .gt. 0) then
               invrq = 10
               go to 100
            end if
         end if

*        The subproblem is optimal.

         optsub = .true.
         lcstat = 0
         if ( nlncon ) then
            if (prnt1 ) write(iprint, 1510) majits, minits, itn
            if (summ1 ) write(isumm , 1511) minits, itn
            rgtest = 100.0 * toldj(3) * pinorm
            if (modrhs .eq. 0) infsub = 0
            if (modrhs .gt. 0) lcstat = 1
            nphs   = 4
            go to 90
         end if
         go to 800
      end if

*     The subproblem is unbounded.
*     Reduce wtobj or increase the penalty parameter.

  460 if (infsbl  .and.  wtobj .gt. zero) go to 650
      call m5dgen( 3, m, n, nb, ms, inform,
     $             ne, nka, a, ha, ka,
     $             hs, kb, bl, bu, x, xn, y, y2, z, nwcore )
      if (inform .gt. 0) go to 200

      lcstat = 2
      if ( nlncon ) then
         call m8augl( 4, m, n, nb, ns, inform,
     $                ne, nka, a, ha, ka,
     $                hs, bl, bu, xn, z, nwcore )
         if (inform .eq. 0) then
            kmodpi = 1
            ninf   = 1
            go to 200
         end if
      end if
      go to 820

*     Minor iterations terminated.

  500 lcstat = 3
      if (prnt1) write(iprint, 1500) majits, minits, itn
      if (summ1) write(isumm , 1501) minits, itn
      nphs   = 4
      go to 90

*     Reset or reduce wtobj.

  650 wtobj  = zero
  655 modwt  = modwt + 1
      wtobj  = 0.1 * wtobj
      if (modwt .gt. 5) wtobj = zero
      if (prnt1 .or. idebug .ge. 1) write(iprint, 1200) wtobj
      go to 200

*     Error condition.
*     If it was a linesearch failure, keep the major iterations
*     going if this one did at least one minor iteration.
*     Otherwise, print something helpful and terminate.
*     t1  is something that ought to be small at a solution.
*     t2  is what we consider 'reasonably small'.

  680 if (ierr .ne. 9) go to 900
      t1     = rgnorm / pinorm
      t2     = 0.1
      if ( nlncon ) then
         if (minits .gt. 0) go to 500
         t1  = max( t1, virel )
      end if

      if (iprint .gt. 0) write(iprint, 1104)
      if (isumm  .gt. 0) write(isumm , 1104)
      if (lderiv .gt. 0  .and.  .not. GAMS) then
         if (iprint .gt. 0) write(iprint, 1105)
         if (isumm  .gt. 0) write(isumm , 1105)
      end if
      if (t1 .lt. t2) then
         if (iprint .gt. 0) write(iprint, 1114)
         if (isumm  .gt. 0) write(isumm , 1114)
      else
         if (iprint .gt. 0) write(iprint, 1115)
         if (isumm  .gt. 0) write(isumm , 1115)
      end if
      go to 850

*     ------------------------------------------------------------------
*     Exit.
*     m1page( ) decides whether to do a page eject before the EXIT msg.
*     m1page(2) also calls m1envt(1) to print '=1' for GAMS
*     ------------------------------------------------------------------

*     Optimal.

  800 call m1page( 2 )
      rgtest = rgnorm / pinorm
      if (rgtest .lt. 0.1) then
         if (iprint .gt. 0) write(iprint, 1800)
         if (isumm  .gt. 0) write(isumm , 1800)
      else
         if (iprint .gt. 0) write(iprint, 1802)
         if (isumm  .gt. 0) write(isumm , 1802)
         ierr   = 13
      end if
      go to 900

*     Infeasible.

  810 call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1810)
      if (isumm  .gt. 0) write(isumm , 1810)
      ierr   = 1
      go to 900

*     Unbounded.

  820 call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1820)
      if (isumm  .gt. 0) write(isumm , 1820)
      go to 900

*     Too many iterations.

  825 call m1envt( 1 )
      if (iprint .gt. 0) write(iprint, 1825)
      call m1envt( 2 )
  830 call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1830)
      if (isumm  .gt. 0) write(isumm , 1830)
      ierr   = 3
      go to 900

*     The objective hasn't changed for ages.

  835 call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1835) msamef
      if (isumm  .gt. 0) write(isumm , 1835) msamef
      ierr   = 4
      go to 900

*     maxs  too small.

  840 call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1840) maxs
      if (isumm  .gt. 0) write(isumm , 1840) maxs
      ierr   = 5
      go to 900

*     Linesearch failure.

  850 call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1850)
      if (isumm  .gt. 0) write(isumm , 1850)
      ierr   = 9
      go to 900

*     Fatal error after LU factorization.

  860 if (ierr .ne. 10) go to 900
      call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1860)
      if (isumm  .gt. 0) write(isumm , 1860)
      go to 900

*     m2bfac called twice in a row.

  865 call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1865)
      if (isumm  .gt. 0) write(isumm , 1865)
      ierr   = 12
      go to 900

*     Resource interrupt (too much time, etc.)

  880 call m1envt( 1 )
      if (iprint .gt. 0) write(iprint, 1880)
      if (isumm  .gt. 0) write(isumm , 1880)
      ierr   = 19
      go to 900

*     ------------------------------------------------------------------
*     Set output variables and print a summary of the final solution.
*     M1ENVT( 2 ) prints '=2' for GAMS.
*     ------------------------------------------------------------------

  900 call m1envt( 2 )
      gotfac = ierr .le. 9
      gothes = r(1) .ne. zero
      degen  = 100.0 * ndegen / max( itn, 1 )
      infsbl = ninf .gt. 0
      suminf = sinf
      numinf = ninf
      rgtest = rgnorm / pinorm
      xnorm  = dnorm1( nb, xn, 1 )

*     Count basic nonlinear variables (not used for anything).

      nnb    = 0
      do 910 j = 1, nn
         if (hs(j) .eq. 3) nnb = nnb + 1
  910 continue

*     Restore nonlinear constraints bounds, in case they were relaxed.
*     Maybe reduce majits for printing purposes.

      if (nlncon) then
         call m8augl( 6, m, n, nb, ns, inform,
     $                ne, nka, a, ha, ka,
     $                hs, bl, bu, xn, z, nwcore )

         if (minits .eq. 0) majits = majits - 1
      end if

*     Set nonbasic hs(j) to external values.
*     Save the final basis file.

      call m5hs  ( 'External', nb, bl, bu, hs, xn )
      if (inewb .gt. 0  .and.  ierr .lt. 20) then
         k = ierr + 1
         call m4stat( k, istate )
         call m4newb( 2, inewb, m, n, nb, nn, ns, ms, nscl, fsub,
     $                kb, hs, ascale, bl, bu, x, xn, istate )
      end if

*     Print statistics.

      if (iprint .gt. 0) then
                     write(iprint, 1900) name, itn, objtru
         if (infsbl) write(iprint, 1910) ninf, sinf
         if (nonlin) write(iprint, 1920) majits, flin, penpar, fobj
         if (nonlin) write(iprint, 1950) nfobj(1), nfcon(1)
         if (nonlin  .and.  lderiv .lt. 3)
     $               write(iprint, 1960) (nfobj(i), nfcon(i), i=2,4)
         if (nonlin  .or.  ns .gt. 0)
     $               write(iprint, 1970) ns, rgnorm, nnb, rgtest
                     write(iprint, 1975) ndegen, degen
      end if

      if (isumm  .gt. 0) then
                     write(isumm , 1900) name, itn, objtru
         if (infsbl) write(isumm , 1910) ninf, sinf
         if (nonlin) write(isumm , 1920) majits, flin, penpar, fobj
         if (nonlin) write(isumm , 1950) nfobj(1), nfcon(1)
         if (nonlin  .and.  lderiv .lt. 3)
     $               write(isumm , 1960) (nfobj(i), nfcon(i), i=2,4)
         if (nonlin  .or.  ns .gt. 0)
     $               write(isumm , 1970) ns, rgnorm, nnb, rgtest
                     write(isumm , 1975) ndegen, degen
      end if

  950 return


 1030 format(  ' LU file has increased by a factor of', f6.1)
 1104 format(/ ' XXX  The functions are non-smooth or very nonlinear')
 1105 format(  ' XXX  MAKE SURE THE GRADIENTS ARE CODED CORRECTLY')
 1114 format(  ' XXX  The final point is REASONABLY NEAR AN OPTIMUM' /)
 1115 format(  ' XXX  The final point is NOT CLOSE TO AN OPTIMUM' /)
 1200 format(/ ' Weight on obj reduced to', 1p, e12.2)
 1400 format(  ' tolrg  reduced to', 1p, e11.3, 5x, ' lvltol =', i2)
 1410 format(/ ' Biggest dj =', 1p, e11.3, ' (variable', i7, ')',
     $   '   norm rg =', e11.3, '   norm pi =', e11.3)
 1500 format(/ ' End of major itn', i4,
     $   '  -  minor itns terminated at', i4,
     $   '  -  total itns =', i7)
 1501 format(' Minor itns terminated at', i4,
     $   '  -  total itns =', i7)
 1510 format(/ ' End of major itn', i4,
     $   '  -  optimal subproblem at minor itn', i4,
     $   '  -  total itns =', i7)
 1511 format(' Optimal subproblem at minor itn', i4,
     $   '  -  total itns =', i7)

 1800 format(' EXIT -- optimal solution found')
 1802 format(' EXIT -- near-optimal solution found'
     $  /' XXX WARNING: reduced gradient is large --',
     $   ' solution is not really optimal')
 1810 format(' EXIT -- the problem is infeasible')
 1820 format(' EXIT -- the problem is unbounded ',
     $   ' (or badly scaled).')
 1825 format(' XXX  Major iterations terminated before convergence')
 1830 format(' EXIT -- too many iterations')
 1835 format(' EXIT -- the objective has not changed',
     $   ' for the last', i8, '  iterations')
 1840 format(' EXIT -- the superbasics limit is too small:', i7)
 1850 format(' EXIT -- the current point cannot be improved upon')
 1860 format(' EXIT -- numerical error.  General constraints',
     $   ' cannot be satisfied accurately')
 1865 format(' EXIT -- basis factorization requested',
     $   ' twice in a row')
 1880 format(' EXIT -- resource interrupt')

 1900 format(/ 1x, 2a4
     $       / ' No. of iterations', i20,
     $     2x, ' Objective value', 1p, e22.10)
 1910 format(  ' No. of infeasibilities', i15,
     $     2x, ' Sum of infeas', 1p, e24.10)
 1920 format(  ' No. of major iterations', i14,
     $     2x, ' Linear objective', 1p, e21.10
     $       / ' Penalty parameter', 0p, f20.6,
     $     2x, ' Nonlinear objective', 1p, e18.10)
 1950 format(  ' No. of calls to funobj', i15,
     $     2x, ' No. of calls to funcon', i15)
 1960 format(  ' Calls with mode=2 (f, known g)', i7,
     $     2x, ' Calls with mode=2 (f, known g)', i7
     $       / ' Calls for forward differencing', i7,
     $     2x, ' Calls for forward differencing', i7
     $       / ' Calls for central differencing', i7,
     $     2x, ' Calls for central differencing', i7)
 1970 format(  ' No. of superbasics', i19,
     $     2x, ' Norm of reduced gradient', 1p, e13.3
     $       / ' No. of basic nonlinears', i14,
     $     2x, ' Norm rg / Norm pi', e20.3)
 1975 format(  ' No. of degenerate steps', i14,
     $     2x, ' Percentage', f27.2)

*     end of m5solv
      end   
****************************************************************
*
*     file  mi60srch fortran.
*
*     m6dmmy   m6fcon   m6fobj   m6fun    m6fun1   m6grd    m6grd1
*     m6dobj   m6dcon   m6srch   srchc    srchq
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6dmmy( n, g )

      implicit           double precision (a-h,o-z)
      double precision   g(n)

*     ------------------------------------------------------------------
*     m6dmmy  sets the elements of  g  to a dummy value.
*     When  funcon  or  funobj  are subsequently called, we can
*     then determine which elements of  g  are known.
*     ------------------------------------------------------------------

      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)

      call dload ( n, gdummy, g, 1 )

*     end of m6dmmy
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6fcon( modefg, nncon, nnjac, njac, f, g,
     $                   ne, nka, ha, ka,
     $                   x, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   f(nncon), g(njac), x(nnjac), z(nwcore)

*     ------------------------------------------------------------------
*     m6fcon  calls the user-written routine  funcon  to evaluate
*     the nonlinear constraints and possibly their gradients.
*
*     08-Jun-1987: Rewritten for GAMS during Tony Brooke's visit.
*     20-Mar-1988: Tony found it hadn't been installed (gasp!).
*     12 Sep 1991: More if-then-elses.
*     19 Nov 1991: ha, ka passed in as parameters to help with minoss.
*     11 Jul 1992: Clock 4 added.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      parameter        ( ntime = 5 )
      common    /m1tim / tlast(ntime), tsum(ntime), numt(ntime), ltime
      common    /m3len / m     ,n     ,nb    ,nscl
      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m5log1/ idebug,ierr,lprint
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2

      logical            scaled

      if (ltime  .ge. 2) call m1time( 4,0 )
      scaled   = lscale .eq. 2
      mode     = modefg
      nfcon(1) = nfcon(1) + 1
      if (mode   .eq. 2) nfcon(2) = nfcon(2) + 1

*     ------------------------------------------------------------------
*     On first entry, count how many Jacobian elements are known.
*     ------------------------------------------------------------------
      if (nstat2 .eq. 1) then
         call m6dmmy( njac, g )
         call funcon( mode, nncon, nnjac, njac, x, f, g,
     $                nstat2, nprob, z, nwcore )
         nstat2    = 0
         if (mode .lt. 0) go to 900

         ngrad     = 0
         do 100 l  = 1, njac
            if (g(l) .ne. gdummy) ngrad  = ngrad + 1
  100    continue

         knowng(2) = ngrad
         if (iprint .gt. 0) then
            write(iprint, 1100)
            write(iprint, 2000) ngrad, njac
         end if
         if (isumm  .gt. 0) then
            write(isumm , 2000) ngrad, njac
         end if
         if (ngrad .eq. njac) go to 990

*        Some Jacobian elements are missing.
*        If the Jacobian is supposedly known (lderiv .ge. 2), the
*        missing elements are assumed to be constant, and must be
*        restored.  They have been saved from the initial a(*) by
*        m8augl( 1, ... ) and saved in gcon2.

         if (lderiv .ge. 2) call dcopy ( njac, z(lgcon2), 1, g, 1 )
         nfcon(2) = nfcon(2) + 1
         go to 400
      end if

*     Test for last entry.
*     Scaling will have been disabled, so it's ok to
*     skip the subsequent part.

      if (nstat2 .ge. 2) then
         if (iprint .gt. 0) write(iprint, 3000) nstat2
         if (isumm  .gt. 0) write(isumm , 3000) nstat2
         go to 400
      end if

*     ------------------------------------------------------------------
*     Normal entry.
*     If the Jacobian is known and scaled, and if there are some
*     constant elements saved in gcon2, we have to copy them into g.
*     ------------------------------------------------------------------

      if (scaled) then
         call dcopy ( nnjac, x        , 1, z(lx2), 1 )
         call ddscl ( nnjac, z(lascal), 1, x,      1 )
         if (lderiv .ge. 2  .and.  knowng(2) .lt. njac
     $                      .and.  modefg    .eq. 2   ) then
            call dcopy ( njac, z(lgcon2), 1, g, 1 )
         end if
      end if

  400 call funcon( mode, nncon, nnjac, njac, x, f, g,
     $             nstat2, nprob, z, nwcore )

      if (scaled) then
         call dcopy ( nnjac, z(lx2)     , 1, x, 1 )
         call dddiv ( nncon, z(lascal+n), 1, f, 1 )
         if (modefg .eq. 2) then
            call m8sclj( 2, nncon, nnjac, njac, n, nb, ne, nka,
     $                   z(lascal), ha, ka, g )
         end if
      end if

      if (mode .lt. 0) go to 900
      go to 990

*     -----------------------
*     The user wants to stop.
*     -----------------------
  900 ierr   = 6
      call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1000)
      if (isumm  .gt. 0) write(isumm , 1000)

  990 if (ltime  .ge. 2) call m1time(-4,0 )
      return

 1000 format(' EXIT -- Termination requested by User',
     $   ' in subroutine funcon')
 1100 format(//)
 2000 format(' funcon  sets', i8, '   out of', i8,
     $   '   constraint gradients.')
 3000 format(/ ' funcon called with nstate =', i4)

*     end of m6fcon
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6fobj( modefg, n, f, g, x, z, nwcore )

      implicit           double precision (a-h,o-z)
      double precision   g(n), x(n), z(nwcore)

*     ------------------------------------------------------------------
*     m6fobj  calls the user-written routine  funobj  to evaluate
*     the nonlinear objective function and possibly its gradient.
*     12 Sep 1991: More if-then-elses.
*     11 Jul 1992: Clock 5 added.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      parameter        ( ntime = 5 )
      common    /m1tim / tlast(ntime), tsum(ntime), numt(ntime), ltime
      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m5log1/ idebug,ierr,lprint
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2

      logical            scaled

      if (ltime  .ge. 2) call m1time( 5,0 )
      scaled   = lscale .eq. 2
      mode     = modefg
      nfobj(1) = nfobj(1) + 1
      if (mode   .eq. 2) nfobj(2) = nfobj(2) + 1

*     Test for first or last entry.

      if (nstat1 .eq. 1) then
         call m6dmmy( n, g )
      else if (nstat1 .ge. 2) then
         if (iprint .gt. 0) write(iprint, 3000) nstat1
         if (isumm  .gt. 0) write(isumm , 3000) nstat1
      end if

*     Normal entry.

      if (scaled) then
         call dcopy ( n, x        , 1, z(lx2), 1 )
         call ddscl ( n, z(lascal), 1, x,      1 )
      end if

      call funobj( mode, n, x, f, g, nstat1, nprob, z, nwcore )

      if (scaled) then
         call dcopy ( n, z(lx2), 1, x, 1 )
         if (modefg .eq. 2) then
            call m8sclj( 1, 1, n, n, 1, n, 1, 1, z(lascal), z, z, g )
         end if
      end if

      if (mode .lt. 0) go to 900

*     ------------------------------------------------------------------
*     On first entry, count components to be estimated by differences.
*     ------------------------------------------------------------------
      if (nstat1 .eq. 1) then
         nstat1 = 0
         ngrad  = 0
         do 100 l = 1, n
            if (g(l) .ne. gdummy) ngrad = ngrad + 1
  100    continue

         knowng(1) = ngrad
         if (iprint .gt. 0) then
            write(iprint, 1100)
            write(iprint, 2000) ngrad, n
         end if
         if (isumm  .gt. 0) then
            write(isumm , 2000) ngrad, n
         end if

         if (ngrad .lt. n) then
            if (lderiv .eq. 1  .or.  lderiv .eq. 3) then
               lderiv = lderiv - 1
               if (iprint .gt. 0) write(iprint, 2200) lderiv
               if (isumm  .gt. 0) write(isumm , 2200) lderiv
            end if
         end if
      end if

      go to 990

*     -----------------------
*     The user wants to stop.
*     -----------------------
  900 ierr   = 6
      call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1000) nfobj(1)
      if (isumm  .gt. 0) write(isumm , 1000) nfobj(1)

  990 if (ltime  .ge. 2) call m1time(-5,0 )
      return

 1000 format(' EXIT -- Termination requested by User',
     $   ' in subroutine funobj after', i8, '  calls')
 1100 format(//)
 2000 format(' funobj  sets', i8, '   out of', i8,
     $   '   objective  gradients.')
 2200 format(/ ' Derivative Level  now reduced to', i3)
 3000 format(/ ' funobj called with nstate =', i4)

*     end of m6fobj
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6fun ( mode, modefg, n, nb, ms, fsub,
     $                   ne, nka, a, ha, ka,
     $                   x, xn, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne), x(ms), xn(nb), z(nwcore)

*     ------------------------------------------------------------------
*     m6fun  is called mainly by the linesearch, m6srch,
*     to compute the subproblem function value fsub.
*     It is also called by m5frmc when a feasible point is first found,
*     and by m6rgit when the linesearch says the max step is too small.
*
*     If mode = 0, m6fun  sets objective and constraint gradients to
*     dummy values (if necessary) in preparation for calls with mode=1.
*
*     If mode = 1, m6fun  returns the subproblem function value in fsub
*     for the current value of the basic and superbasic vars in  x.
*     The user functions funcon and/or funobj are called
*     via m6fcon and m6fobj, using modefg to control the gradients
*     as follows:
*
*        If modefg = 2, gradients will be requested.
*        If modefg = 0, gradients will NOT be requested.  This is used
*        by m6srch during a function-only linesearch.  An extra call
*        with modefg = 2 is needed once the linesearch finds a good
*        point, but this may be cheaper than getting gradients every
*        time, e.g. if the user estimates gradients by differencing.
*     
*     12 Sep 1991: More if-then-elses.
*     02 Oct 1991: modefg added following request from Hilary Hoynes,
*                  Hoover Institution (Hoynes@Hoover.bitnet, 723-9511).
*     14 Oct 1991: a, ha, ka added as parameters.
*     ------------------------------------------------------------------

      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m5len / maxr  ,maxs  ,mbs   ,nn    ,nn0   ,nr    ,nx
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m7loc / lgobj ,lgobj2
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)

      logical            grdcon, grdobj

      if (mode .eq. 0) then
*        ---------------------------------------------------------------
*        mode = 0.  See if any gradients will have to be estimated.
*        This call is from m5frmc after a refactorize,
*        or from m6srch at the beginning of a linesearch.
*        ---------------------------------------------------------------
         grdcon = nncon .eq. 0  .or.  lderiv .ge. 2  .or.  nlag   .eq. 0
         grdobj = nnobj .eq. 0  .or.  lderiv .eq. 1  .or.  lderiv .eq. 3
         if (.not. grdcon) call m6dmmy( njac , z(lgcon) )
         if (.not. grdobj) call m6dmmy( nnobj, z(lgobj) )
      else
*        ---------------------------------------------------------------
*        mode = 1.  Evaluate the subproblem objective.
*        Copy the free variables  x  into  xn  first.
*        ---------------------------------------------------------------
         call m5bsx ( 1, ms, nb, z(lkb), x, xn )

         call m6fun1( modefg, ms, n, nn,
     $                ne, nka, a, ha, ka,
     $                nncon, nncon0, nnobj, nnobj0, nnjac, njac,
     $                z(lfcon), z(lgcon), fobj, z(lgobj), fsub,
     $                z(lfdif), z(lfold), z(lxlam),
     $                x, xn, z(lxdif), z(lxold), z, nwcore )
      end if

*     end of m6fun
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6fun1( modefg, ms, n, nn,
     $                   ne, nka, a, ha, ka,
     $                   nncon, nncon0, nnobj, nnobj0, nnjac, njac,
     $                   fcon, gcon, fobj, gobj, fsub,
     $                   fdif, fold, xlam,
     $                   x, xn, xdif, xold, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne),
     $                   fcon(nncon0), gcon(njac  ), gobj(nnobj0),
     $                   fdif(nncon0), fold(nncon0), xlam(nncon0),
     $                   x(ms), xn(nn), xdif(nn), xold(nn), z(nwcore)

*     ------------------------------------------------------------------
*     m6fun1  calls the constraint and objective functions, and then
*     computes the subproblem objective function from them.
*
*     03 Sep 1992: Penalty parameter (penpar) is now used as a
*                  multiple of a default value, pendef = 100 / nncon.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor

      logical            lagrng
      parameter        ( zero = 0.0d+0,  dhalf = 0.5d+0 )

*     Evaluate the functions  (constraints first, then objective).
*     If Lagrangian = No, we only evaluate the true objective.

      lagrng = nnjac .gt. 0  .and.  nlag .gt. 0

      if (lagrng) then
         call m6fcon( modefg, nncon, nnjac, njac, fcon, gcon,
     $                ne, nka, ha, ka,
     $                xn, z, nwcore )
         if (ierr  .ne. 0) return
      end if

      fobj   = zero
      if (nnobj .gt. 0) then
         call m6fobj( modefg, nnobj, fobj, gobj, xn, z, nwcore )
         if (ierr  .ne. 0) return
      end if

*     Include the linear objective.

      fsub   = fobj
      if (iobj   .ne. 0) fsub =   fsub  -  x(iobj) * sclobj
      if (minimz .lt. 0) fsub = - fsub

*     Compute the augmented Lagrangian terms:
*        fdif  =  (fcon - fold)  -  (Jacobian)*(xn - xold).

      if (lagrng) then
         do 110  i  = 1, nncon
            fdif(i) = fcon(i) - fold(i)
  110    continue

         do 120  j  = 1, nnjac
            xdif(j) = xn(j) - xold(j)
  120    continue

         call m2aprd( 7, xdif, nnjac, fdif, nncon,
     $                ne, nka, a, ha, ka,
     $                z, nwcore )

         fsub   = fsub  -  ddot( nncon, xlam, 1, fdif, 1 )
         if (penpar .gt. zero) then
            pendef = 100.0d+0 / nncon
            rho    = penpar * pendef
            fdnorm = dnrm2( nncon, fdif, 1 )
            fsub   = fsub  +  dhalf * rho * fdnorm**2
         end if
      end if

  900 return

*     end of m6fun1
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6grd ( ms, nb, nn, gsub, grd,
     $                   ne, nka, a, ha, ka,
     $                   xn, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne), gsub(nn), grd(ms), xn(nb), z(nwcore)

*     ------------------------------------------------------------------
*     m6grd   returns the gradient of the subproblem objective in  gsub.
*
*     14 Oct 1991: a, ha, ka added as parameters.
*     ------------------------------------------------------------------

      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      common    /m7loc / lgobj ,lgobj2
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold

      call m6grd1( n, nn, fobj,
     $             nncon, nncon0, nnobj, nnobj0, nnjac, njac,
     $             ne, nka, a, ha, ka,
     $             z(lfcon), z(lfcon2), z(lfdif ), z(lxlam),
     $             z(lgcon), z(lgcon2), z(lgobj ), z(lgobj2), gsub,
     $             xn, z(ly), z, nwcore )

      call m7bsg ( ms, nn, z(lkb), gsub, grd )

*     end of m6grd
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6grd1( n, nn, fobj,
     $                   nncon, nncon0, nnobj, nnobj0, nnjac, njac,
     $                   ne, nka, a, ha, ka,
     $                   fcon, fcon2, fdif, xlam,
     $                   gcon, gcon2, gobj, gobj2, gsub,
     $                   xn, y, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne), fcon(nncon0), fcon2(nncon0),
     $                   fdif(nncon0), xlam (nncon0),
     $                   gcon(njac  ), gcon2(njac  ),
     $                   gobj(nnobj0), gobj2(nnobj0), gsub(nn),
     $                   xn(nn), y(nncon0), z(nwcore)

*     ------------------------------------------------------------------
*     m6grd1 computes gsub(j), the gradient of the subproblem objective.
*
*     NOTE --  m6dcon overwrites the first  nncon  elements of  y
*     if central differences are needed.
*
*     03 Sep 1992: Penalty parameter (penpar) is now used as a
*                  multiple of a default value, pendef = 100 / nncon.
*     ------------------------------------------------------------------

      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)

      logical            grdcon, grdobj, lagrng
      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

      grdcon = nncon  .eq. 0  .or.  lderiv .ge. 2  .or.  nlag   .eq. 0
      grdobj = nnobj  .eq. 0  .or.  lderiv .eq. 1  .or.  lderiv .eq. 3
      lagrng = nnjac  .gt. 0 .and.  nlag   .gt. 0

*     ------------------------------------------------------
*     Fill in any missing constraint or objective gradients.
*     ------------------------------------------------------

      if (.not. grdcon) then
         call m6dcon( nncon, nnjac, njac,
     $                ne, nka, ha, ka,
     $                fcon, fcon2, gcon, gcon2, xn, y, z, nwcore )
         if (ierr .ne. 0) return
      end if

      if (.not. grdobj) then
         call m6dobj( nnobj, fobj, gobj, gobj2, xn, z, nwcore )
         if (ierr .ne. 0) return
      end if

*     -----------------------------------------
*     Compute the required elements of  gsub.
*     -----------------------------------------

*     Set  gsub = ( gobj, 0 )  and allow for the sign of the objective.

      l      = nn - nnobj
      if (nnobj  .gt. 0) call dcopy ( nnobj , gobj, 1, gsub, 1 )
      if (l      .gt. 0) call dload ( l     , zero, gsub(nnobj+1), 1 )
      if (minimz .lt. 0) call dscal ( nnobj0, (- one), gsub, 1 )

*     Compute the augmented Lagrangian terms.
*     m6fun1  has set up   fdif  =  (fcon - fold)  -  Jac * (x - xold).
*     To save work, set    fdif  =  rho * fdif  -  xlam.

      if (lagrng) then
         pendef = 100.0d+0 / nncon
         rho    = penpar * pendef
         call dscal ( nncon,  rho, fdif, 1 )
         call daxpy ( nncon, (- one), xlam, 1, fdif, 1 )

         l      = 0
         do 600 j = 1, nnjac
            g     = gsub(j)
            k1    = ka(j)
            k2    = ka(j+1) - 1

            do 500 k = k1, k2
               ir    = ha(k)
               if (ir .gt. nncon) go to 590
               l     = l  +  1
               g     = g  +  (gcon(l) - a(k)) * fdif(ir)
  500       continue

  590       gsub(j) = g
  600    continue
      end if

*     end of m6grd1
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6dobj( nnobj, fobj, gobj, gobj2, xn, z, nwcore )

      implicit           double precision (a-h,o-z)
      double precision   gobj(nnobj), gobj2(nnobj)
      double precision   xn(nnobj), z(nwcore)

*     ------------------------------------------------------------------
*     m6dobj  estimates missing objective gradients
*     using finite differences of the objective function  fobj.
*     ------------------------------------------------------------------

      common    /m5log1/ idebug,ierr,lprint
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2

      intrinsic          abs

      logical            centrl
      parameter        ( one = 1.0d+0 )

      centrl = lvldif .eq. 2
      delta  = difint(lvldif)
      numf   = 0
      fback  = fobj

      do 200 j  = 1, nnobj
         if (gobj(j) .eq. gdummy) then
            xj     = xn(j)
            dx     = delta*(one + abs( xj ))
            xn(j)  = xj + dx
            numf   = numf + 1
            call m6fobj( 0, nnobj, fforwd, gobj2, xn, z, nwcore )
            if (ierr .ne. 0) go to 900

            if (centrl) then
               xn(j)  = xj - dx
               dx     = dx + dx
               numf   = numf + 1
               call m6fobj( 0, nnobj, fback, gobj2, xn, z, nwcore )
               if (ierr .ne. 0) go to 900
            end if

            gobj(j) = (fforwd - fback) / dx
            xn(j)   = xj
         end if
  200 continue

  900 l        = lvldif + 2
      nfobj(l) = nfobj(l) + numf

*     end of m6dobj
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6dcon( nncon, nnjac, njac,
     $                   ne, nka, ha, ka,
     $                   fcon, fcon2, gcon, gcon2, xn, y, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   fcon(nncon), fcon2(nncon),
     $                   gcon(njac ), gcon2(njac ),
     $                   xn(nnjac), y(nncon), z(nwcore)

*     ------------------------------------------------------------------
*     m6dcon  estimates missing elements in the columns of the
*     Jacobian (the first  nncon  rows and  nnjac  cols of  A)
*     using finite differences of the constraint functions  fcon.
*     ------------------------------------------------------------------

      common    /m5log1/ idebug,ierr,lprint
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2

      intrinsic          abs

      logical            centrl
      parameter        ( one = 1.0d+0 )

      centrl = lvldif .eq. 2
      delta  = difint(lvldif)
      numf   = 0
      l      = 0

*     If central differences are needed, we first save  fcon  in  y.
*     When  fcon  is later loaded with  f(x - h)  for each pertbn  h,
*     it can be used the same as when forward differences are in effect.

      if (centrl) call dcopy ( nncon, fcon, 1, y, 1 )

      do 200 j  = 1, nnjac

*        Look for the first missing element in this column.

         lsave  = l
         k1     = ka(j)
         k2     = ka(j+1) - 1
         do 100 k = k1, k2
            ir    = ha(k)
            if (ir .gt. nncon) go to 200
            l     = l + 1
            if (gcon(l) .eq. gdummy) go to 120
  100    continue
         go to 200

*        Found one. Approximate it and any others by finite differences.

  120    l      = lsave
         xj     = xn(j)
         dx     = delta * (one + abs( xj ))
         xn(j)  = xj + dx
         numf   = numf + 1
         call m6fcon( 0, nncon, nnjac, njac, fcon2, gcon2,
     $                ne, nka, ha, ka,
     $                xn, z, nwcore )
         if (ierr .ne. 0) go to 900

         if (centrl) then
            xn(j)  = xj - dx
            dx     = dx + dx
            numf   = numf + 1
            call m6fcon( 0, nncon, nnjac, njac, fcon, gcon2, 
     $                   ne, nka, ha, ka,
     $                   xn, z, nwcore )
            if (ierr .ne. 0) go to 900
         end if

         do 150 k = k1, k2
            ir    = ha(k)
            if (ir .gt. nncon) go to 190
            l     = l + 1
            if (gcon(l) .eq. gdummy) then
                gcon(l) = (fcon2(ir) - fcon(ir)) / dx
            end if
  150    continue
  190    xn(j)  = xj
  200 continue

  900 if (centrl) call dcopy ( nncon, y, 1, fcon, 1 )
      l        = lvldif + 2
      nfcon(l) = nfcon(l) + numf

*     end of m6dcon
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6srch( ms, ns, n, nb, nn, itn, inform, 
     $                   debug, fonly, switch,
     $                   ne, nka, a, ha, ka,
     $                   alfa, alfmax, difint, epsmch, epsrf, eta,
     $                   fsub, gnorm , pnorm , xnorm,
     $                   gsub, grd, p, x, x2, xn, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            debug, fonly, switch
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne), gsub(nn), grd(ms),
     $                   p(ms), x(ms), x2(ms), xn(nb), z(nwcore)

*     ------------------------------------------------------------------
*     m6srch  finds a step-length  alfa  along the search direction  p,
*     such that the (subproblem) objective function  F  is sufficiently
*     reduced:    F(x + alpha*p)  <  F(x).
*
*     Input parameters...
*
*     alfa        Initial estimate of the step.
*     alfmax      Maximum allowable step.
*     difint      A difference interval that has been used for forward
*                 differences (not central differences) to get p.
*                 (difint is used if switch is true).
*     epsmch      Relative machine precision.
*     epsrf       Relative function precision.
*     eta         Linesearch accuracy parameter in the range (0,1).
*                 0.001 means very accurate.   0.99 means quite sloppy.
*     fonly       true if function-only search should be used.
*     switch      true if there is a possibility of switching to
*                 central differences to get a better p.
*     fsub        Current value of subproblem objective, F(x).
*     gsub(*)     Current gradient of subproblem objective.
*     grd(*)      gsub  reordered like x and p  (free variables only).
*     p(*)        Search direction for the free variables.
*     x(*)        Current free variables (basic and superbasic).
*
*     Output parameters...
*
*     alfa
*     fsub        New value of the subproblem objective, F(x + alfa*p).
*     gsub(*)     New gradient of the subproblem objective.
*     grd(*)      New  gsub  reordered parallel to  x  and  p.
*     x           New free variables (basic and superbasic).
*     inform = -1 if the user wants to stop.
*            =  1 to 8 -- see below.
*
*     02 Oct 1991: fonly search uses modefg = 0 during the search,
*                  and modefg = 2 after the search.
*     04 Oct 1991: switch added to suppress request for central diffs
*                  when fonly = true but all gradients are really known.
*     16 Oct 1991: getptc, getptq replaced by srchc, srchq.
*                  inform values altered.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m5log1/ idebug,ierr,lprint

      intrinsic          abs   , max  
      logical            done  , first , imprvd
      parameter        ( zero = 0.0d+0,  one = 1.0d+0,  two = 2.0d+0 )
      parameter        ( rmu  = 1.0d-4  )

*     ------------------------------------------------------------------
*     Set the input parameters for  srchc  or  srchq.
*
*     alfsml  is used by srchq.  If  alfa  would be less than  alfsml,
*             the search will be terminated early.
*             If  p  was computed using forward or backward differences,
*             alfsml  should be positive (and related to the difference
*             interval used).
*             If  p  was computed using central differences (lvldif = 2)
*             alfsml  should be zero.
*
*     epsrf   is the relative function precision (specified by user).
*
*     epsaf   is the absolute function precision. If F(x1) and F(x2) are
*             as close as  epsaf,  we cannot safely conclude from the
*             function values alone which of x1 or x2 is a better point.
*
*     tolabs  is an estimate of the absolute spacing between points
*             along  p.  This step should produce a perturbation
*             of  epsaf  in the subproblem objective function.
*
*     tolrel  is an estimate of the relative spacing between points
*             along  p.
*
*     toltny  is the minimum allowable absolute spacing between points
*             along  p.
*     ------------------------------------------------------------------
      first  = .true.
      maxf   = 10
      if (fonly) maxf = 15
      nout   = iprint

      alfsml = zero
      if (fonly  .and.  switch) then
         alfsml = difint * (one + xnorm) / pnorm
      end if

      epsaf  = max( epsrf, epsmch ) * (one + abs( fsub ))
      oldf   = fsub
      oldg   = ddot ( ms, grd, 1, p, 1 )
      eps0   = epsmch**0.8
      tolax  = eps0
      tolrx  = eps0
      tolrel = max( tolrx, epsmch )
      t      = xnorm * tolrx  +  tolax
      tolabs = alfmax
      if (t .lt. pnorm * alfmax) tolabs = t / pnorm

      t      = zero
      do 10 j = 1, ms
         s    = abs( p(j) )
         q    = abs( x(j) )*tolrx + tolax
         if (s .gt. q*t) t = s / q
   10 continue

      if (t*tolabs .gt. one) then
         toltny = one / t
      else
         toltny = tolabs
      end if

      alfbst = zero
      fbest  = zero
      gbest  = (one - rmu)*oldg
      targtg = (rmu - eta)*oldg
      g0     = gbest

      if (debug) write(nout, 1000) itn, pnorm, gnorm

*     ------------------------------------------------------------------
*     Commence main loop, entering srchc or srchq two or more times.
*     first = true for the first entry,  false for subsequent entries.
*     done  = true indicates termination, in which case the value of
*     inform gives the result of the search.
*     inform = 1 if the search is successful and alfa < alfmax.
*            = 2 if the search is successful and alfa = alfmax.
*            = 3 if a better point was found but too many functions
*                were needed (not sufficient decrease).
*            = 4 if alfmax < tolabs (too small to do a search).
*            = 5 if alfa < alfsml (srchq only -- maybe want to switch
*                to central differences to get a better direction).
*            = 6 if the search found that there is no useful step.
*                The interval of uncertainty is less than 2*tolabs.
*                The minimizer is very close to alfa = zero
*                or the gradients are not sufficiently accurate.
*            = 7 if there were too many function calls.
*            = 8 if the input parameters were bad
*                (alfmax le toltny  or  oldg ge 0).
*     ------------------------------------------------------------------
*     Start loop
  100    if (fonly) then
            call srchq ( first , debug , done  , imprvd, inform,
     $                   maxf  , numf  , nout  , 
     $                   alfmax, alfsml, epsaf , 
     $                   g0    , targtg, ftry  ,         
     $                   tolabs, tolrel, toltny,
     $                   alfa  , alfbst, fbest  )
         else
            call srchc ( first , debug , done  , imprvd, inform,
     $                   maxf  , numf  , nout  ,
     $                   alfmax,         epsaf , 
     $                   g0    , targtg, ftry  , gtry  , 
     $                   tolabs, tolrel, toltny,
     $                   alfa  , alfbst, fbest , gbest )
         end if

         if (.not. done ) then
*           ------------------------------------------------------------
*           Compute the function value fsub at the new trial point
*           x + alfa*p.
*           ------------------------------------------------------------
            do 180 j = 1, ms
               x2(j) = x(j) + alfa*p(j)
  180       continue

*           Check for first function request.
*           Maybe set dummy gradients.

            if (numf .eq. 0) then
               modefg = 2
               if ( fonly ) then
                  modefg = 0
                  call m6fun ( 0, modefg, n, nb, ms, fsub,
     $                         ne, nka, a, ha, ka,
     $                         x2, xn, z, nwcore )
               end if
            end if

            call m6fun ( 1, modefg, n, nb, ms, fsub,
     $                   ne, nka, a, ha, ka,
     $                   x2, xn, z, nwcore )
            if (ierr .ne. 0) go to 999
         
            ftry  = fsub - oldf - rmu*oldg*alfa 

*           If gradients are being used, also get the
*           directional derivative gtry.
         
            if (.not. fonly) then
               call m6grd ( ms, nb, nn, gsub, grd,
     $                      ne, nka, a, ha, ka,
     $                      xn, z, nwcore )
               gtp   = ddot( ms, grd, 1, p, 1 )
               gtry  = gtp - rmu*oldg
            end if
         
            go to 100
         end if
*     end loop

*     ------------------------------------------------------------------
*     The search has finished.
*     If some function evaluations were made, 
*     set alfa = alfbst  and  x = x + alfa*p.
*     ------------------------------------------------------------------
      if (inform .le. 7) then
         alfa   = alfbst
         call daxpy ( ms, alfa, p, 1, x, 1 )

*        If only function values were used in the search,
*        or if the last call to m6fun was not at the best point,
*        we have to recompute the function and gradient there
*        (and reset grd).
*        Beware: if central differences are used, m6grd overwrites p.
      
         if (fonly  .or.  .not. imprvd) then
            modefg = 2
            call m6fun ( 1, modefg, n, nb, ms, fsub,
     $                   ne, nka, a, ha, ka,
     $                   x, xn, z, nwcore )
            if (ierr .ne. 0) go to 999
      
            call m6grd ( ms, nb, nn, gsub, grd,
     $                   ne, nka, a, ha, ka,
     $                   xn, z, nwcore )
            if (ierr .ne. 0) go to 999
         end if
      end if

*     ------------------------------------------------------------------
*     Print a few error msgs if necessary.
*     ------------------------------------------------------------------
      if (inform .ge. 7) then
         if (inform .eq. 7) then
*-          if (iprint .gt. 0) write(iprint, 1700) numf
*-          if (isumm  .gt. 0) write(isumm , 1700) numf
         else if (oldg .ge. zero) then
*-          if (iprint .gt. 0) write(iprint, 1600) oldg
         end if

         if (.not. debug) then
            if (iprint .gt. 0) then
               write(iprint, 1800) alfmax, pnorm, gnorm, oldg, numf
            end if
         end if
      end if

      return

*     The user wants to stop.

  999 inform = -1
      return

 1000 format(// ' --------------------------------------------'
     $       /  ' Output from m6srch following iteration', i7,
     $      5x, ' Norm p =', 1p, e11.2, 5x, ' Norm g =', e11.2)
*1600 format(/ ' XXX  The search direction is uphill.   gtp =', 1p,e9.1)
*1700 format(  ' XXX  The linesearch has evaluated the functions', i5,
*    $   '  times')
 1800 format(' alfmax =', 1p, e11.2, '    pnorm  =', e11.2,
     $    '    gnorm  =',     e11.2, '    g(t)p  =', e11.2,
     $    '    numf =', i3)

*     end of m6srch
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine srchc ( first , debug , done  , imprvd, inform,
     $                   maxf  , numf  , nout  ,
     $                   alfmax,         epsaf , 
     $                   g0    , targtg, ftry  , gtry  , 
     $                   tolabs, tolrel, toltny,
     $                   alfa  , alfbst, fbest , gbest )

      implicit           double precision (a-h,o-z)
      logical            first , debug , done  , imprvd

************************************************************************
*     srchc  finds a sequence of improving estimates of a minimizer of
*     the univariate function f(alpha) in the interval (0,alfmax].
*     f(alpha) is a smooth function such that  f(0) = 0  and  f'(0) < 0.
*     srchc  requires both  f(alpha)  and  f'(alpha)  to be evaluated at
*     points in the interval.  Estimates of the minimizer are computed
*     using safeguarded cubic interpolation.
*
*     Reverse communication is used to allow the calling program to
*     evaluate f and f'.  Some of the parameters must be set or tested
*     by the calling program.  The remainder would ordinarily be local
*     variables.
*
*     Input parameters (relevant to the calling program)
*     --------------------------------------------------
*
*     first         must be true on the first entry. It is subsequently
*                   altered by srchc.
*
*     debug         specifies whether detailed output is wanted.
*
*     maxf          is an upper limit on the number of times srchc is to
*                   be entered consecutively with done = false
*                   (following an initial entry with first = true).
*
*     alfa          is the first estimate of a minimizer.  alfa is
*                   subsequently altered by srchc (see below).
*
*     alfmax        is the upper limit of the interval to be searched.
*
*     epsaf         is an estimate of the absolute precision in the
*                   computed value of f(0).
*
*     ftry, gtry    are the values of f, f'  at the new point
*                   alfa = alfbst + xtry.
*
*     g0            is the value of f'(0).  g0 must be negative.
*
*     tolabs,tolrel define a function tol(alfa) = tolrel*alfa + tolabs
*                   such that if f has already been evaluated at alfa,
*                   it will not be evaluated closer than tol(alfa).
*                   These values may be reduced by srchc.
*
*     targtg        is the target value of abs(f'(alfa)). The search
*                   is terminated when 
*                    abs(f'(alfa)) le targtg and f(alfa) lt 0.
*
*     toltny        is the smallest value that tolabs is allowed to be
*                   reduced to.
*
*     Output parameters (relevant to the calling program)
*     ---------------------------------------------------
*
*     imprvd        is true if the previous alfa was the best point so
*                   far.  Any related quantities should be saved by the
*                   calling program (e.g., gradient arrays) before
*                   paying attention to the variable done.
*
*     done = false  means the calling program should evaluate
*                      ftry = f(alfa),  gtry = f'(alfa)
*                   for the new trial alfa, and re-enter srchc.
*
*     done = true   means that no new alfa was calculated.  The value
*                   of inform gives the result of the search as follows
*
*                   inform = 1 means the search has terminated
*                              successfully with alfbst < alfmax.
*
*                   inform = 2 means the search has terminated
*                              successfully with alfbst = alfmax.
*
*                   inform = 3 means that the search failed to find a 
*                              point of sufficient decrease in maxf
*                              functions, but a lower point was found.
*
*                   inform = 4 means alfmax is so small that a search
*                              should not have been attempted.
*
*                   inform = 5 is never set by srchc.
*
*                   inform = 6 means the search has failed to find a
*                              useful step.  The interval of uncertainty 
*                              is [0,b] with b < 2*tolabs. A minimizer
*                              lies very close to alfa = 0, or f'(0) is
*                              not sufficiently accurate.
*
*                   inform = 7 if no better point could be found after 
*                              maxf  function calls.
*
*                   inform = 8 means the input parameters were bad.
*                              alfmax le toltny  or g0 ge zero.
*                              No function evaluations were made.
*
*     numf          counts the number of times srchc has been entered
*                   consecutively with done = false (i.e., with a new
*                   function value ftry).
*
*     alfa          is the point at which the next function ftry and
*                   derivative gtry must be computed.
*
*     alfbst        should be accepted by the calling program as the
*                   approximate minimizer, whenever srchc returns
*                   inform = 1 or 2 (and possibly 3).
*
*     fbest, gbest  will be the corresponding values of f, f'.
*
*
*     The following parameters retain information between entries
*     -----------------------------------------------------------
*
*     braktd        is false if f and f' have not been evaluated at
*                   the far end of the interval of uncertainty.  In this
*                   case, the point b will be at alfmax + tol(alfmax).
*
*     crampd        is true if alfmax is very small (le tolabs).  If the
*                   search fails, this indicates that a zero step should
*                   be taken.
*
*     extrap        is true if xw lies outside the interval of
*                   uncertainty.  In this case, extra safeguards are
*                   applied to allow for instability in the polynomial
*                   fit.
*
*     moved         is true if a better point has been found, i.e.,
*                   alfbst gt 0.
*
*     wset          records whether a second-best point has been
*                   determined it will always be true when convergence
*                   is tested.
*
*     nsamea        is the number of consecutive times that the 
*                   left-hand end point of the interval of uncertainty
*                   has remained the same.
*
*     nsameb        similarly for the right-hand end.
*
*     a, b, alfbst  define the current interval of uncertainty. 
*                   A minimizer lies somewhere in the interval 
*                   [alfbst + a, alfbst + b].
*
*     alfbst        is the best point so far.  It is always at one end 
*                   of the interval of uncertainty.  hence we have
*                   either  a lt 0,  b = 0  or  a = 0,  b gt 0.
*
*     fbest, gbest  are the values of f, f' at the point alfbst.
*
*     factor        controls the rate at which extrapolated estimates 
*                   of alfa may expand into the interval of uncertainty.
*                   factor is not used if a minimizer has been bracketed
*                   (i.e., when the variable braktd is true).
*
*     fw, gw        are the values of f, f' at the point alfbst + xw.
*                   they are not defined until wset is true.
*
*     xtry          is the trial point in the shifted interval (a, b).
*
*     xw            is such that  alfbst + xw  is the second-best point.
*                   it is not defined until  wset  is true.
*                   in some cases,  xw  will replace a previous  xw  
*                   that has a lower function but has just been excluded
*                   from the interval of uncertainty.
*
*
*     Systems Optimization Laboratory, Stanford University, California.
*     Original version February 1982.  Rev. May 1983.
*     Original f77 version 22-August-1985.
*     This version of srchc dated  24-Oct-91.
************************************************************************

      logical            braktd, crampd, extrap, moved , wset
      save               braktd, crampd, extrap, moved , wset

      save               nsamea, nsameb
      save               a     , b     , factor
      save               xtry  , xw    , fw    , gw    , tolmax

      logical            badfun, closef, found 
      logical            quitF , quitI
      logical            fitok , setxw 
      intrinsic          abs   , sqrt

      parameter        ( zero  =0.0d+0, point1 =0.1d+0, half   =0.5d+0 )
      parameter        ( one   =1.0d+0, two    =2.0d+0, three  =3.0d+0 )
      parameter        ( five  =5.0d+0, ten    =1.0d+1, eleven =1.1d+1 )

*     ------------------------------------------------------------------
*     Local variables
*     ===============
*
*     closef     is true if the new function ftry is within epsaf of
*                fbest (up or down).
*
*     found      is true if the sufficient decrease conditions hold at
*                alfbst.
*
*     quitF      is true when  maxf  function calls have been made.
*
*     quitI      is true when the interval of uncertainty is less than
*                2*tol.
*  ---------------------------------------------------------------------

      badfun = .false.
      quitF  = .false.
      quitI  = .false.
      imprvd = .false.

      if (first) then
*        ---------------------------------------------------------------
*        First entry.  Initialize various quantities, check input data
*        and prepare to evaluate the function at the initial alfa.
*        ---------------------------------------------------------------
         first  = .false.
         numf   = 0
         alfbst = zero
         badfun = alfmax .le. toltny  .or.  g0 .ge. zero
         done   = badfun
         moved  = .false.

         if (.not. done) then
            braktd = .false.
            crampd = alfmax .le. tolabs
            extrap = .false.
            wset   = .false.
            nsamea = 0
            nsameb = 0

            tolmax = tolabs + tolrel*alfmax
            a      = zero
            b      = alfmax + tolmax
            factor = five
            tol    = tolabs
            xtry   = alfa
            if (debug) then
               write (nout, 1000) g0    , tolabs, alfmax, 
     $                            targtg, tolrel, epsaf , crampd
            end if
         end if
      else
*        ---------------------------------------------------------------
*        Subsequent entries. The function has just been evaluated at
*        alfa = alfbst + xtry,  giving ftry and gtry.
*        ---------------------------------------------------------------
         if (debug) write (nout, 1100) alfa, ftry, gtry

         numf   = numf   + 1
         nsamea = nsamea + 1
         nsameb = nsameb + 1

         if (.not. braktd) then
            tolmax = tolabs + tolrel*alfmax
            b      = alfmax - alfbst + tolmax
         end if

*        See if the new step is better.  If alfa is large enough that
*        ftry can be distinguished numerically from zero,  the function
*        is required to be sufficiently negative.

         closef = abs( ftry - fbest ) .le.  epsaf
         if (closef) then
            imprvd =  abs( gtry ) .le. abs( gbest )
         else
            imprvd = ftry .lt. fbest
         end if

         if (imprvd) then

*           We seem to have an improvement.  The new point becomes the
*           origin and other points are shifted accordingly.

            fw     = fbest
            fbest  = ftry
            gw     = gbest
            gbest  = gtry
            alfbst = alfa
            moved  = .true.

            a      = a    - xtry
            b      = b    - xtry
            xw     = zero - xtry
            wset   = .true.
            extrap =       xw .lt. zero  .and.  gbest .lt. zero
     $               .or.  xw .gt. zero  .and.  gbest .gt. zero

*           Decrease the length of the interval of uncertainty.

            if (gtry .le. zero) then
               a      = zero
               nsamea = 0
            else
               b      = zero
               nsameb = 0
               braktd = .true.
            end if
         else

*           The new function value is not better than the best point so
*           far.  The origin remains unchanged but the new point may
*           qualify as xw.  xtry must be a new bound on the best point.

            if (xtry .le. zero) then
               a      = xtry
               nsamea = 0
            else
               b      = xtry
               nsameb = 0
               braktd = .true.
            end if

*           If xw has not been set or ftry is better than fw, update the
*           points accordingly.

            if (wset) then
               setxw = ftry .lt. fw  .or.  .not. extrap
            else
               setxw = .true.
            end if

            if (setxw) then
               xw     = xtry
               fw     = ftry
               gw     = gtry
               wset   = .true.
               extrap = .false.
            end if
         end if

*        ---------------------------------------------------------------
*        Check the termination criteria.  wset will always be true.
*        ---------------------------------------------------------------
         tol    = tolabs + tolrel*alfbst
         truea  = alfbst + a
         trueb  = alfbst + b

         found  = abs(gbest) .le. targtg
         quitF  = numf       .ge. maxf
         quitI  = b - a      .le. tol + tol

         if (quitI  .and. .not. moved) then

*           The interval of uncertainty appears to be small enough,
*           but no better point has been found.  Check that changing 
*           alfa by b-a changes f by less than epsaf.

            tol    = tol/ten
            tolabs = tol
            quitI  = abs(fw) .le. epsaf  .or.  tol .le. toltny
         end if

         done  = quitF  .or.  quitI  .or.  found

         if (debug) then
            write (nout, 1200) truea    , trueb , b - a , tol   ,
     $                         nsamea   , nsameb, numf  , 
     $                         braktd   , extrap, closef, imprvd,
     $                         found    , quitI ,
     $                         alfbst   , fbest , gbest ,
     $                         alfbst+xw, fw    , gw
         end if

*        ---------------------------------------------------------------
*        Proceed with the computation of a trial steplength.
*        The choices are...
*        1. Parabolic fit using derivatives only, if the f values are
*           close.
*        2. Cubic fit for a minimizer, using both f and f'.
*        3. Damped cubic or parabolic fit if the regular fit appears to
*           be consistently overestimating the distance to a minimizer.
*        4. Bisection, geometric bisection, or a step of  tol  if
*           choices 2 or 3 are unsatisfactory.
*        ---------------------------------------------------------------
         if (.not. done) then
            xmidpt = half*(a + b)
            s      = zero
            q      = zero

            if (closef) then
*              ---------------------------------------------------------
*              Fit a parabola to the two best gradient values.
*              ---------------------------------------------------------
               s      = gbest
               q      = gbest - gw
               if (debug) write (nout, 2200)
            else
*              ---------------------------------------------------------
*              Fit cubic through  fbest  and  fw.
*              ---------------------------------------------------------
               if (debug) write (nout, 2100)
               fitok  = .true.
               r      = three*(fbest - fw)/xw + gbest + gw
               absr   = abs( r )
               s      = sqrt( abs( gbest ) ) * sqrt( abs( gw ) )

*              Compute  q =  the square root of  r*r - gbest*gw.
*              The method avoids unnecessary underflow and overflow.

               if ((gw .lt. zero  .and.  gbest .gt. zero) .or.
     $             (gw .gt. zero  .and.  gbest .lt. zero)) then
                  scale  = absr + s
                  if (scale .eq. zero) then
                     q  = zero
                  else
                     q  = scale*sqrt( (absr/scale)**2 + (s/scale)**2 )
                  end if
               else if (absr .ge. s) then
                  q     = sqrt(absr + s)*sqrt(absr - s)
               else
                  fitok = .false.
               end if

               if (fitok) then

*                 Compute a minimizer of the fitted cubic.

                  if (xw .lt. zero) q = - q
                  s  = gbest -  r - q
                  q  = gbest - gw - q - q
               end if
            end if
*           ------------------------------------------------------------
*           Construct an artificial interval  (artifa, artifb)  in which
*           the new estimate of a minimizer must lie.  Set a default
*           value of xtry that will be used if the polynomial fit fails.
*           ------------------------------------------------------------
            artifa = a
            artifb = b
            if (.not. braktd) then

*              A minimizer has not been bracketed.  Set an artificial
*              upper bound by expanding the interval  xw  by a suitable
*              factor.

               xtry   = - factor*xw
               artifb =   xtry
               if (alfbst + xtry .lt. alfmax) factor = five*factor

            else if (extrap) then

*              The points are configured for an extrapolation.
*              Set a default value of  xtry  in the interval  (a, b)
*              that will be used if the polynomial fit is rejected.  In
*              the following,  dtry  and  daux  denote the lengths of
*              the intervals  (a, b)  and  (0, xw)  (or  (xw, 0),  if
*              appropriate).  The value of  xtry is the point at which
*              the exponents of  dtry  and  daux  are approximately
*              bisected.

               daux = abs( xw )
               dtry = b - a
               if (daux .ge. dtry) then
                  xtry = five*dtry*(point1 + dtry/daux)/eleven
               else
                  xtry = half * sqrt( daux ) * sqrt( dtry )
               end if
               if (xw .gt. zero)   xtry = - xtry
               if (debug) write (nout, 2400) xtry, daux, dtry

*              Reset the artificial bounds.  If the point computed by
*              extrapolation is rejected,  xtry will remain at the
*              relevant artificial bound.

               if (xtry .le. zero) artifa = xtry
               if (xtry .gt. zero) artifb = xtry
            else

*              The points are configured for an interpolation.  The
*              default value xtry bisects the interval of uncertainty.
*              the artificial interval is just (a, b).

               xtry   = xmidpt
               if (debug) write (nout, 2300) xtry
               if (nsamea .ge. 3  .or.  nsameb .ge. 3) then

*                 If the interpolation appears to be overestimating the
*                 distance to a minimizer,  damp the interpolation.

                  factor = factor / five
                  s      = factor * s
               else
                  factor = one
               end if
            end if
*           ------------------------------------------------------------
*           The polynomial fits give  (s/q)*xw  as the new step.
*           Reject this step if it lies outside  (artifa, artifb).
*           ------------------------------------------------------------
            if (q .ne. zero) then
               if (q .lt. zero) s = - s
               if (q .lt. zero) q = - q
               if (s*xw .ge. q*artifa  .and.  s*xw .le. q*artifb) then

*                 Accept the polynomial fit.

                  if (abs( s*xw ) .ge. q*tol) then
                     xtry = (s/q)*xw
                  else
                     xtry = zero
                  end if
                  if (debug) write (nout, 2500) xtry
               end if
            end if
         end if
      end if

*     ==================================================================

      if (.not. done) then
         alfa  = alfbst + xtry
         if (braktd  .or.  alfa .lt. alfmax - tolmax) then

*           The function must not be evaluated too close to a or b.
*           (It has already been evaluated at both those points.)

            if (xtry .le. a + tol  .or.  xtry .ge. b - tol) then
               if (half*(a + b) .le. zero) then
                  xtry = - tol
               else
                  xtry =   tol
               end if
               alfa = alfbst + xtry
            end if
         else

*           The step is close to, or larger than alfmax, replace it by
*           alfmax to force evaluation of  f  at the boundary.

            braktd = .true.
            xtry   = alfmax - alfbst
            alfa   = alfmax
         end if
      end if

*     ------------------------------------------------------------------
*     Exit.
*     ------------------------------------------------------------------
      if (done) then
         if      (badfun) then
            inform = 8
         else if (found) then
            if (alfbst .lt. alfmax) then
               inform = 1
            else
               inform = 2
            end if
         else if (moved ) then
            inform = 3
         else if (quitF) then
            inform = 7
         else if (crampd) then
            inform = 4
         else
            inform = 6
         end if
      end if

      if (debug) write (nout, 3000)
      return

 1000 format(/'     g0  tolabs  alfmax        ', 1p, 2e22.14,   e16.8
     $       /' targtg  tolrel   epsaf        ', 1p, 2e22.14,   e16.8
     $       /' crampd                        ',  l3)
 1100 format(/' alfa    ftry    gtry          ', 1p, 2e22.14,   e16.8)
 1200 format(/' a       b       b - a   tol   ', 1p, 2e22.14,  2e16.8
     $       /' nsamea  nsameb  numf          ', 3i3
     $       /' braktd  extrap  closef  imprvd', 4l3
     $       /' found   quitI                 ', 2l3
     $       /' alfbst  fbest   gbest         ', 1p, 3e22.14          
     $       /' alfaw   fw      gw            ', 1p, 3e22.14)
 2100 format( ' Cubic.   ')
 2200 format( ' Parabola.')
 2300 format( ' Bisection.              xmidpt', 1p,  e22.14)
 2400 format( ' Geo. bisection. xtry,daux,dtry', 1p, 3e22.14)
 2500 format( ' Polynomial fit accepted.  xtry', 1p,  e22.14)
 3000 format( ' ----------------------------------------------------'/)

*     End of  srchc .
      end
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine srchq ( first , debug , done  , imprvd, inform,
     $                   maxf  , numf  , nout  , 
     $                   alfmax, alfsml, epsaf , 
     $                   g0    , targtg, ftry  , 
     $                   tolabs, tolrel, toltny,
     $                   alfa  , alfbst, fbest  )

      implicit           double precision (a-h,o-z)
      logical            first , debug , done  , imprvd

************************************************************************
*     srchq  finds a sequence of improving estimates of a minimizer of
*     the univariate function f(alpha) in the interval (0,alfmax].
*     f(alpha) is a smooth function such that  f(0) = 0  and  f'(0) < 0.
*     srchq  requires  f(alpha) (but not f'(alpha)) to be evaluated
*     in the interval.  New estimates of a minimizer are computed using
*     safeguarded quadratic interpolation.
*
*     Reverse communication is used to allow the calling program to
*     evaluate f.  Some of the parameters must be set or tested by the
*     calling program.  The remainder would ordinarily be local
*     variables.
*
*     Input parameters (relevant to the calling program)
*     --------------------------------------------------
*
*     first         must be true on the first entry.  It is subsequently
*                   altered by srchq.
*
*     debug         specifies whether detailed output is wanted.
*
*     maxf          is an upper limit on the number of times srchq is to
*                   be entered consecutively with done = false 
*                   (following an initial entry with first = true).
*
*     alfa          is the first estimate of a minimizer.  alfa is
*                   subsequently altered by srchq (see below).
*
*     alfmax        is the upper limit of the interval to be searched.
*
*     alfsml        is intended to prevent inefficiency when a minimizer
*                   is very small, for cases where the calling program
*                   would prefer to redefine f'(alfa).  alfsml is
*                   allowed to be zero.  Early termination will occur if
*                   srchq determines that a minimizer lies somewhere in
*                   the interval [0, alfsml) (but not if alfmax is 
*                   smaller that alfsml).
*
*     epsaf         is an estimate of the absolute precision in the
*                   computed value of f(0).
*
*     ftry          the value of f at the new point
*                   alfa = alfbst + xtry.
*
*     g0            is the value of f'(0).  g0 must be negative.
*
*     tolabs,tolrel define a function tol(alfa) = tolrel*alfa + tolabs
*                   such that if f has already been evaluated at alfa,
*                   it will not be evaluated closer than tol(alfa).
*                   These values may be reduced by srchc.
*
*     targtg        is the target value of abs(f'(alfa)). The search
*                   is terminated when 
*                    abs(f'(alfa)) le targtg and f(alfa) lt 0.
*
*     toltny        is the smallest value that tolabs is allowed to be
*                   reduced to.
*
*     Output parameters (relevant to the calling program)
*     ---------------------------------------------------
*
*     imprvd        is true if the previous alfa was the best point so
*                   far.  Any related quantities should be saved by the
*                   calling program (e.g., arrays) before paying
*                   attention to the variable done.
*
*     done = false  means the calling program should evaluate ftry
*                   for the new trial step alfa, and reenter srchq.
*
*     done = true   means that no new alfa was calculated.  The value
*                   of inform gives the result of the search as follows
*
*                   inform = 1 means the search has terminated 
*                              successfully with alfbst < alfmax.
*
*                   inform = 2 means the search has terminated
*                              successfully with alfbst = alfmax.
*
*                   inform = 3 means that the search failed to find a 
*                              point of sufficient decrease in maxf
*                              functions, but a lower point was found.
*
*                   inform = 4 means alfmax is so small that a search
*                              should not have been attempted.
*
*                   inform = 5 means that the search was terminated
*                              because of alfsml (see above).
*
*                   inform = 6 means the search has failed to find a
*                              useful step.  The interval of uncertainty 
*                              is [0,b] with b < 2*tolabs. A minimizer
*                              lies very close to alfa = 0, or f'(0) is
*                              not sufficiently accurate.
*
*                   inform = 7 if no better point could be found after 
*                              maxf  function calls.
*
*                   inform = 8 means the input parameters were bad.
*                              alfmax le toltny  or  g0 ge zero.
*                              No function evaluations were made.
*
*     numf          counts the number of times srchq has been entered
*                   consecutively with done = false (i.e., with a new
*                   function value ftry).
*
*     alfa          is the point at which the next function ftry must 
*                   be computed.
*
*     alfbst        should be accepted by the calling program as the
*                   approximate minimizer, whenever srchq returns
*                   inform = 1, 2 or 3.
*
*     fbest         will be the corresponding value of f.
*
*     The following parameters retain information between entries
*     -----------------------------------------------------------
*
*     braktd        is false if f has not been evaluated at the far end
*                   of the interval of uncertainty.  In this case, the
*                   point b will be at alfmax + tol(alfmax).
*
*     crampd        is true if alfmax is very small (le tolabs).  If the
*                   search fails, this indicates that a zero step should
*                   be taken.
*
*     extrap        is true if alfbst has moved at least once and xv 
*                   lies outside the interval of uncertainty.  In this
*                   case, extra safeguards are applied to allow for
*                   instability in the polynomial fit.
*
*     moved         is true if a better point has been found, i.e., 
*                   alfbst gt 0.
*
*     vset          records whether a third-best point has been defined.
*
*     wset          records whether a second-best point has been 
*                   defined.  It will always be true by the time the
*                   convergence test is applied.
*
*     nsamea        is the number of consecutive times that the
*                   left-hand end point of the interval of uncertainty
*                   has remained the same.
*
*     nsameb        similarly for the right-hand end.
*
*     a, b, alfbst  define the current interval of uncertainty.
*                   A minimizer lies somewhere in the  interval
*                   [alfbst + a, alfbst + b].
*
*     alfbst        is the best point so far.  It lies strictly within
*                   [atrue,btrue]  (except when alfbst has not been
*                   moved, in which case it lies at the left-hand end
*                   point).  Hence we have a .le. 0 and b .gt. 0.
*
*     fbest         is the value of f at the point alfbst.
*
*     fa            is the value of f at the point alfbst + a.
*
*     factor        controls the rate at which extrapolated estimates of
*                   alfa  may expand into the interval of uncertainty.
*                   Factor is not used if a minimizer has been bracketed
*                   (i.e., when the variable braktd is true).
*
*     fv, fw        are the values of f at the points alfbst + xv  and
*                   alfbst + xw.  They are not defined until  vset  or
*                   wset  are true.
*
*     xtry          is the trial point within the shifted interval
*                   (a, b).  The new trial function value must be
*                   computed at the point alfa = alfbst + xtry.
*
*     xv            is such that alfbst + xv is the third-best point. 
*                   It is not defined until vset is true.
*
*     xw            is such that alfbst + xw is the second-best point. 
*                   It is not defined until wset is true.  In some
*                   cases,  xw will replace a previous xw that has a
*                   lower function but has just been excluded from 
*                   (a,b).
*
*     Systems Optimization Laboratory, Stanford University, California.
*     Original version February 1982.  Rev. May 1983.
*     Original F77 version 22-August-1985.
*     This version of srchq dated  24-Oct-91.
************************************************************************

      logical            braktd, crampd, extrap, moved , vset  , wset
      save               braktd, crampd, extrap, moved , vset  , wset

      save               nsamea, nsameb
      save               a     , b     , fa    , factor
      save               xtry  , xw    , fw    , xv    , fv    , tolmax

      logical            badfun, closef, found 
      logical            quitF , quitFZ, quitI , quitS 
      logical            setxv , xinxw
      intrinsic          abs   , sqrt

      parameter        ( zero  =0.0d+0, point1 =0.1d+0, half   =0.5d+0 )
      parameter        ( one   =1.0d+0, two    =2.0d+0, five   =5.0d+0 )
      parameter        ( ten   =1.0d+1, eleven =1.1d+1                 )

*     ------------------------------------------------------------------
*     Local variables
*     ===============
*
*     closef     is true if the worst function fv is within epsaf of
*                fbest (up or down).
*
*     found      is true if the sufficient decrease conditions holds at
*                alfbst.
*
*     quitF      is true when  maxf  function calls have been made.
*
*     quitFZ     is true when the three best function values are within
*                epsaf of each other, and the new point satisfies
*                fbest le ftry le fbest+epsaf.
*
*     quitI      is true when the interval of uncertainty is less than
*                2*tol.
*
*     quitS      is true as soon as alfa is too small to be useful;
*                i.e., btrue le alfsml.
*
*     xinxw      is true if xtry is in (xw,0) or (0,xw).
*     ------------------------------------------------------------------

      imprvd = .false.
      badfun = .false.
      quitF  = .false.
      quitFZ = .false.
      quitS  = .false.
      quitI  = .false.

      if (first) then
*        ---------------------------------------------------------------
*        First entry.  Initialize various quantities, check input data
*        and prepare to evaluate the function at the initial step alfa.
*        ---------------------------------------------------------------
         first  = .false.
         numf   = 0
         alfbst = zero
         badfun = alfmax .le. toltny  .or.  g0 .ge. zero
         done   = badfun
         moved  = .false.

         if (.not. done) then
            braktd = .false.
            crampd = alfmax .le. tolabs
            extrap = .false.
            vset   = .false.
            wset   = .false.
            nsamea = 0
            nsameb = 0

            tolmax = tolrel*alfmax + tolabs
            a      = zero
            b      = alfmax + tolmax
            fa     = zero
            factor = five
            tol    = tolabs
            xtry   = alfa
            if (debug) then
               write (nout, 1000) g0    , tolabs, alfmax, 
     $                            targtg, tolrel, epsaf , crampd
            end if
         end if
      else
*        ---------------------------------------------------------------
*        Subsequent entries.  The function has just been evaluated at
*        alfa = alfbst + xtry,  giving ftry.
*        ---------------------------------------------------------------
         if (debug) write (nout, 1100) alfa, ftry

         numf   = numf   + 1
         nsamea = nsamea + 1
         nsameb = nsameb + 1

         if (.not. braktd) then
            tolmax = tolabs + tolrel*alfmax
            b      = alfmax - alfbst + tolmax
         end if

*        Check if xtry is in the interval (xw,0) or (0,xw).

         if (wset) then
            xinxw =        zero .lt. xtry  .and.  xtry .le. xw
     $               .or.    xw .le. xtry  .and.  xtry .lt. zero
         else
            xinxw = .false.
         end if

         imprvd = ftry .lt. fbest
         if (vset) then
            closef = abs( fbest - fv ) .le. epsaf
         else
            closef = .false.
         end if

         if (imprvd) then

*           We seem to have an improvement.  The new point becomes the
*           origin and other points are shifted accordingly.

            if (wset) then
               xv     = xw - xtry
               fv     = fw
               vset   = .true.
            end if

            xw     = zero - xtry
            fw     = fbest
            wset   = .true.
            fbest  = ftry
            alfbst = alfa
            moved  = .true.

            a      = a    - xtry
            b      = b    - xtry
            extrap = .not. xinxw

*           Decrease the length of (a,b).

            if (xtry .ge. zero) then
               a      = xw
               fa     = fw
               nsamea = 0
            else
               b      = xw
               nsameb = 0
               braktd = .true.
            end if
         else if (closef  .and.  ftry - fbest .lt. epsaf) then

*           Quit if there has been no progress and ftry, fbest, fw
*           and fv are all within epsaf of each other.

            quitFZ = .true.
         else

*           The new function value is no better than the current best
*           point.  xtry must an end point of the new (a,b).

            if (xtry .lt. zero) then
               a      = xtry
               fa     = ftry
               nsamea = 0
            else
               b      = xtry
               nsameb = 0
               braktd = .true.
            end if

*           The origin remains unchanged but xtry may qualify as xw.

            if (wset) then
               if (ftry .lt. fw) then
                  xv     = xw
                  fv     = fw
                  vset   = .true.

                  xw     = xtry
                  fw     = ftry
                  if (moved) extrap = xinxw
               else if (moved) then
                  if (vset) then
                     setxv = ftry .lt. fv  .or.  .not. extrap
                  else
                     setxv = .true.
                  end if

                  if (setxv) then
                     if (vset  .and.  xinxw) then
                        xw = xv
                        fw = fv
                     end if
                     xv   = xtry
                     fv   = ftry
                     vset = .true.
                  end if
               else
                  xw  = xtry
                  fw  = ftry
               end if
            else
               xw     = xtry
               fw     = ftry
               wset   = .true.
            end if
         end if

*        ---------------------------------------------------------------
*        Check the termination criteria.
*        ---------------------------------------------------------------
         tol    = tolabs + tolrel*alfbst
         truea  = alfbst + a
         trueb  = alfbst + b

         found  = moved  .and.  abs(fa - fbest) .le. -a*targtg
         quitF  = numf  .ge. maxf
         quitI  = b - a .le. tol + tol
         quitS  = trueb .le. alfsml

         if (quitI  .and.  .not. moved) then

*           The interval of uncertainty appears to be small enough,
*           but no better point has been found.  Check that changing 
*           alfa by b-a changes f by less than epsaf.

            tol    = tol/ten
            tolabs = tol
            quitI  = abs(fw) .le. epsaf  .or.  tol .le. toltny
         end if

         done  = quitF  .or.  quitFZ  .or.  quitS  .or.  quitI
     $                  .or.  found

         if (debug) then
            write (nout, 1200) truea    , trueb , b-a   , tol   ,
     $                         nsamea   , nsameb, numf  ,
     $                         braktd   , extrap, closef, imprvd,
     $                         found    , quitI , quitFZ, quitS ,
     $                         alfbst   , fbest ,
     $                         alfbst+xw, fw
            if (vset) then
               write (nout, 1300) alfbst + xv, fv
            end if
         end if

*        ---------------------------------------------------------------
*        Proceed with the computation of an estimate of a minimizer.
*        The choices are...
*        1. Parabolic fit using function values only.
*        2. Damped parabolic fit if the regular fit appears to be
*           consistently overestimating the distance to a minimizer.
*        3. Bisection, geometric bisection, or a step of tol if the
*           parabolic fit is unsatisfactory.
*        ---------------------------------------------------------------
         if (.not. done) then
            xmidpt = half*(a + b)
            s      = zero
            q      = zero

*           ============================================================
*           Fit a parabola.
*           ============================================================
*           See if there are two or three points for the parabolic fit.

            gw = (fw - fbest)/xw
            if (vset  .and.  moved) then

*              Three points available.  Use fbest, fw and fv.

               gv = (fv - fbest)/xv
               s  = gv - (xv/xw)*gw
               q  = two*(gv - gw)
               if (debug) write (nout, 2200)
            else

*              Only two points available.  Use fbest, fw and g0.

               if (moved) then
                  s  = g0 - two*gw
               else
                  s  = g0
               end if
               q = two*(g0 - gw)
               if (debug) write (nout, 2100)
            end if

*           ------------------------------------------------------------
*           Construct an artificial interval (artifa, artifb) in which 
*           the new estimate of the steplength must lie.  Set a default
*           value of  xtry  that will be used if the polynomial fit is
*           rejected. In the following, the interval (a,b) is considered
*           the sum of two intervals of lengths  dtry  and  daux, with
*           common end point the best point (zero).  dtry is the length
*           of the interval into which the default xtry will be placed
*           and endpnt denotes its non-zero end point.  The magnitude of
*           xtry is computed so that the exponents of dtry and daux are
*           approximately bisected.
*           ------------------------------------------------------------
            artifa = a
            artifb = b
            if (.not. braktd) then

*              A minimizer has not yet been bracketed.  
*              Set an artificial upper bound by expanding the interval
*              xw  by a suitable factor.

               xtry   = - factor*xw
               artifb =   xtry
               if (alfbst + xtry .lt. alfmax) factor = five*factor
            else if (vset .and. moved) then

*              Three points exist in the interval of uncertainty.
*              Check if the points are configured for an extrapolation
*              or an interpolation.

               if (extrap) then

*                 The points are configured for an extrapolation.

                  if (xw .lt. zero) endpnt = b
                  if (xw .gt. zero) endpnt = a
               else

*                 If the interpolation appears to be overestimating the
*                 distance to a minimizer,  damp the interpolation step.

                  if (nsamea .ge. 3  .or.   nsameb .ge. 3) then
                     factor = factor / five
                     s      = factor * s
                  else
                     factor = one
                  end if

*                 The points are configured for an interpolation.  The
*                 artificial interval will be just (a,b).  Set endpnt so
*                 that xtry lies in the larger of the intervals (a,b) 
*                 and  (0,b).

                  if (xmidpt .gt. zero) then
                     endpnt = b
                  else
                     endpnt = a
                  end if

*                 If a bound has remained the same for three iterations,
*                 set endpnt so that  xtry  is likely to replace the
*                 offending bound.

                  if (nsamea .ge. 3) endpnt = a
                  if (nsameb .ge. 3) endpnt = b
               end if

*              Compute the default value of  xtry.

               dtry = abs( endpnt )
               daux = b - a - dtry
               if (daux .ge. dtry) then
                  xtry = five*dtry*(point1 + dtry/daux)/eleven
               else
                  xtry = half*sqrt( daux )*sqrt( dtry )
               end if
               if (endpnt .lt. zero) xtry = - xtry
               if (debug) write (nout, 2500) xtry, daux, dtry

*              If the points are configured for an extrapolation set the
*              artificial bounds so that the artificial interval lies
*              within (a,b).  If the polynomial fit is rejected,  xtry 
*              will remain at the relevant artificial bound.

               if (extrap) then
                  if (xtry .le. zero) then
                     artifa = xtry
                  else
                     artifb = xtry
                  end if
               end if
            else

*              The gradient at the origin is being used for the
*              polynomial fit.  Set the default xtry to one tenth xw.

               if (extrap) then
                  xtry = - xw
               else
                  xtry   = xw/ten
               end if
               if (debug) write (nout, 2400) xtry
            end if

*           ------------------------------------------------------------
*           The polynomial fits give (s/q)*xw as the new step.  Reject
*           this step if it lies outside (artifa, artifb).
*           ------------------------------------------------------------
            if (q .ne. zero) then
               if (q .lt. zero) s = - s
               if (q .lt. zero) q = - q
               if (s*xw .ge. q*artifa   .and.   s*xw .le. q*artifb) then
 
*                 Accept the polynomial fit.

                  if (abs( s*xw ) .ge. q*tol) then
                     xtry = (s/q)*xw
                  else
                     xtry = zero
                  end if
                  if (debug) write (nout, 2600) xtry
               end if
            end if
         end if
      end if
*     ==================================================================

      if (.not. done) then
         alfa  = alfbst + xtry
         if (braktd  .or.  alfa .lt. alfmax - tolmax) then

*           The function must not be evaluated too close to a or b.
*           (It has already been evaluated at both those points.)

            xmidpt = half*(a + b)
            if (xtry .le. a + tol  .or.  xtry .ge. b - tol) then
               if (xmidpt .le. zero) then
                  xtry = - tol
               else
                  xtry =   tol
               end if
            end if

            if (abs( xtry ) .lt. tol) then
               if (xmidpt .le. zero) then
                  xtry = - tol
               else
                  xtry =   tol
               end if
            end if
            alfa  = alfbst + xtry
         else

*           The step is close to or larger than alfmax, replace it by
*           alfmax to force evaluation of the function at the boundary.

            braktd = .true.
            xtry   = alfmax - alfbst
            alfa   = alfmax
         end if
      end if
*     ------------------------------------------------------------------
*     Exit.
*     ------------------------------------------------------------------
      if (done) then
         if      (badfun) then
            inform = 8
         else if (quitS ) then
            inform = 5
         else if (found) then
            if (alfbst .lt. alfmax) then
               inform = 1
            else
               inform = 2
            end if
         else if (moved ) then
            inform = 3
         else if (quitF) then
            inform = 7
         else if (crampd) then
            inform = 4
         else
            inform = 6
         end if
      end if

      if (debug) write (nout, 3000)
      return

 1000 format(/'     g0  tolabs  alfmax        ', 1p, 2e22.14,   e16.8
     $       /' targtg  tolrel   epsaf        ', 1p, 2e22.14,   e16.8
     $       /' crampd                        ',  l3)
 1100 format(/' alfa    ftry                  ', 1p,2e22.14          )
 1200 format(/' a       b       b - a   tol   ', 1p,2e22.14,   2e16.8
     $       /' nsamea  nsameb  numf          ', 3i3
     $       /' braktd  extrap  closef  imprvd', 4l3
     $       /' found   quitI   quitFZ  quitS ', 4l3
     $       /' alfbst  fbest                 ', 1p,2e22.14          
     $       /' alfaw   fw                    ', 1p,2e22.14)
 1300 format( ' alfav   fv                    ', 1p,2e22.14 /)
 2100 format( ' Parabolic fit,    two points. ')
 2200 format( ' Parabolic fit,  three points. ')
 2400 format( ' Exponent reduced.  Trial point', 1p,  e22.14)
 2500 format( ' Geo. bisection. xtry,daux,dtry', 1p, 3e22.14)
 2600 format( ' Polynomial fit accepted.  xtry', 1p,  e22.14)
 3000 format( ' ----------------------------------------------------'/)

*     End of  srchq .
      end
************************************************************************
*
*     File  mi65rmod fortran.
*
*     m6bfgs   m6bswp   m6radd   m6rcnd   m6rdel
*     m6rmod   m6rset   m6rsol   m6swap
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6bfgs( maxr, n, nr, r, g, g2, p, v,
     $                   step, told, tolz, inform )

      implicit           double precision (a-h,o-z)
      double precision   r(nr), g(n), g2(n), p(n), v(n)

*     ------------------------------------------------------------------
*     m6bfgs  applies the BFGS update to the upper-triangular matrix  r,
*     which holds the cholesky factor of the quasi-newton approximation
*     of the (projected) hessian.
*
*     r  contains a triangle of size  ncolr = min( n, maxr ).
*     If  n .gt. maxr,  r  also contains a diagonal of size  n - maxr.
*
*     p       holds the search direction.  It is overwritten.
*     v       must satisfy  r(t)*v = g.    It is overwritten.
*
*     On exit,
*     inform = 0  if no update was performed,
*            = 1  if the update was successful,
*            = 2  if it was nearly singular.
*     ------------------------------------------------------------------

      intrinsic          abs, min, sqrt
      parameter        ( one = 1.0d+0 )

      inform = 0
      ncolr  = min( n, maxr )
      gtp    = ddot  ( n, g , 1, p, 1 )
      gtp2   = ddot  ( n, g2, 1, p, 1 )
      if (gtp2 .le. 0.91*gtp) return

      delta1 = one / sqrt( abs( gtp ) )
      delta2 = one / sqrt( step*(gtp2 - gtp) )

*     Normalize  v  and change its sign.

      call dscal ( n, (- delta1), v, 1 )

*     Avoid cancellation error in forming the new vector  p.

      if ( abs( delta1/delta2 - one ) .ge. 0.5d+0) then
         do 100 j = 1, n
            p(j)  = delta2*( g2(j) - g(j) )  +  delta1*g(j)
  100    continue
      else
         d      = delta1 - delta2
         do 120 j = 1, n
            p(j)  = delta2*g2(j)  +  d*g(j)
  120    continue
      end if

*     Find the last nonzero in  v.

      do 450 ii = 1, ncolr
         lastv  = ncolr + 1 - ii
         if (abs( v(lastv) ) .gt. tolz) go to 500
  450 continue
      lastv = 1

*     Triangularize   r  +  v * p(t).

  500 call m6rmod( ncolr, nr, r, v, p, lastv, told, tolz, inform )

*     Deal with surplus diagonals of  r.

      if (n .gt. maxr) then
         j1     = maxr + 1
         l      = maxr*j1/2
         do 650 j = j1, n
            l     = l + 1
            r(l)  = r(l) + v(j)*p(j)
  650    continue
      end if

*     end of m6bfgs
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6bswp( n, nr, r, v, w, lastv, told, tolz, inform )

      implicit           double precision (a-h,o-z)
      double precision   r(nr), v(n), w(n)

*     ------------------------------------------------------------------
*     m6bswp  modifies the upper-triangular matrix  r
*     to account for a basis exchange in which the  lastv-th
*     superbasic variable becomes basic.  r  is changed to
*     r  +  v*w(t),  which is triangularized by  r1mod,
*     where  v  is the  lastv-th column of  r,  and  w  is input.
*     ------------------------------------------------------------------

*     Set  v  =  lastv-th column of  r  and find its norm.

      l      = (lastv - 1)*lastv/2
      call dcopy ( lastv, r(l+1), 1, v, 1 )
      vnorm  = dasum ( lastv, v, 1 )
      call m6rmod( n, nr, r, v, w, lastv,
     $             (vnorm*told), (vnorm*tolz), inform )

*     end of m6bswp
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6radd( maxr, nr, ns, r )

      implicit           double precision (a-h,o-z)
      double precision   r(nr)

*     ------------------------------------------------------------------
*     m6radd  adds column  ns  to the upper triangular matrix  r.
*     In this version (Sep 1984) it is just a unit vector.
*
*     Modified Jan 1983 to add a diagonal only, if  ns .gt. maxr.
*     ------------------------------------------------------------------

      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

      if (ns .eq. 1) then
         r(1)      = one
      else if (ns .le. maxr) then
         ldiag1    = (ns - 1)*ns/2
         ldiag2    = ldiag1 + ns
         call dload ( ns, zero, r(ldiag1+1), 1 )
         r(ldiag2) = one
      else
         ldiag2    = maxr*(maxr + 1)/2  +  (ns - maxr)
         r(ldiag2) = one
      end if

*     end of m6radd
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6rcnd( maxr, nr, ns, r, dmax, dmin, cond )

      implicit           double precision (a-h,o-z)
      double precision   r(nr)

*     ------------------------------------------------------------------
*     m6rcnd  finds the largest and smallest diagonals of the
*     upper triangular matrix  r, and returns the square of their ratio.
*     This is a lower bound on the condition number of  r(t)*r.
*     ------------------------------------------------------------------

      intrinsic          abs, max, min

      ncolr  = min( maxr, ns )
      dmax   = abs( r(1) )
      dmin   = dmax

      if (ncolr .gt. 1) then
         l   = 1
         do 100 j = 2, ncolr
            l     = l + j
            d     = abs( r(l) )
            dmax  = max( dmax, d )
            dmin  = min( dmin, d )
  100    continue
      end if

      cond   = (dmax/dmin)**2

*     end of m6rcnd
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6rdel( m, maxr, nr, ns, ms,
     $                   kb, bbl, bbu, grd, r, rg, x, jq, rset )

      implicit           double precision (a-h,o-z)
      integer            kb(ms)
      double precision   bbl(ms), bbu(ms), grd(ms)
      double precision   r(nr), rg(ns), x(ms)
      logical            rset

*     ------------------------------------------------------------------
*     m6rdel  deletes the  jq-th  superbasic variable from  r
*     and from various arrays  kb, bbl, bbu, grd, rg, x.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy

      intrinsic          abs, max, min, sqrt
      parameter        ( zero = 0.0d+0 )

      if (jq .eq. ns) return
      if (.not. rset) go to 500

*     Delete the  jq-th  column of  r  and triangularize the
*     remaining columns using a partial forward sweep of rotations.

      ncolr  = min( maxr, ns )
      k      = jq*(jq + 1)/2

      do  50 i  = jq + 1, ncolr
         k      = k + i
         t1     = r(k-1)
         t2     = r(k)
         d      = sqrt( t1*t1 + t2*t2 )
         r(k-1) = d
         if (i .eq. ncolr) go to 20
         sn     = t2/d
         cs     = t1/d
         if (abs( cs ) .le. eps0) cs = zero
         j1     = i + 1
         k1     = k + i
         do  10 j   = j1, ncolr
            t1      = r(k1-1)
            t2      = r(k1)
            r(k1-1) = cs*t1 + sn*t2
            r(k1)   = sn*t1 - cs*t2
            k1      = k1 + j
   10    continue
   20    k1     = i - 1
         j2     = k - i
         j1     = j2 - i + 2
         do 30 j = j1, j2
            r(j) = r(j+k1)
   30    continue
   50 continue

*     If necessary, clear out the last column of  r.

      if (ns .gt. maxr) then
         lastr  = maxr*(maxr + 1)/2
         l      = lastr - maxr + 1
         if (jq .le. maxr) call dload ( maxr, zero, r(l), 1 )

*        Shift surplus diagonals of  r  to the left.

         j1     = max( maxr, jq )
         j2     = ns - 1
         l      = lastr + (j1 - maxr)
         do 320 j = j1, j2
            r(l)  = r(l+1)
  320    continue
      end if

*     Shift all the arrays one place to the left.

  500 j2     = ns - 1
      do 550 j  = jq, j2
         rg(j)  = rg(j+1)
         k      = m + j
         kb(k)  = kb(k+1)
         bbl(k) = bbl(k+1)
         bbu(k) = bbu(k+1)
         grd(k) = grd(k+1)
         x(k)   = x(k+1)
  550 continue

*     end of m6rdel
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6rmod( n, nr, r, v, w, lastv, told, tolz, inform )

      implicit           double precision (a-h,o-z)
      double precision   r(nr), v(n), w(n)

*     ------------------------------------------------------------------
*     m6rmod  modifies the upper-triangular matrix  r  so that
*     q*(r + v*w(t))  is upper triangular, where  q  is orthogonal,
*     v  and  w  are vectors, and the new  r  overwrites the old.
*
*     q  is the product of two sweeps of plane rotations (not stored).
*     These affect the  lastv-th  row of  r,  which is temporarily held
*     in the vector  v.  Thus,  v  is overwritten.  w  is not altered.
*
*     lastv  points to the last nonzero of  v.  The value lastv = n
*            would always be ok, but sometimes it is known to be less
*            than  n,  so  q  reduces to two partial sweeps of
*            rotations.
*
*     told   is a tolerance on the lastv-th diagonal of  r.
*     tolz   is a tolerance for negligible elements in  v.
*
*     On exit,
*     inform = 1  if the diagonal of  r  is larger than  told,
*            = 2  if not (in which case it is reset to  told).
*     ------------------------------------------------------------------

      intrinsic          sqrt
      parameter        ( zero = 0.0d+0 )

      vlast  = v(lastv)
      lm1    = lastv - 1
      lastr  = lastv*(lastv + 1)/2

*     Copy the lastv-th row of  r  into the end of  v.

      l      = lastr
      do 100 j = lastv, n
         v(j)  = r(l)
         l     = l + j
  100 continue

*     Reduce  v  to a multiple of  e(lastv)
*     using a partial backward sweep of rotations.
*     This fills in the  lastv-th  row of  r  (held in  v).

      v2     = vlast**2
      ldiag  = lastr

      do 300 ii = 1, lm1
         i      = lastv - ii
         ldiag  = ldiag - i - 1
         s      = v(i)
         v(i)   = zero
         if (abs(s) .le. tolz) go to 300
         v2     = s**2 + v2
         root   = sqrt(v2)
         cs     = vlast/root
         sn     = s/root
         vlast  = root
         l      = ldiag

         do 200 j = i, n
            s     = v(j)
            t     = r(l)
            v(j)  = cs*s + sn*t
            r(l)  = sn*s - cs*t
            l     = l + j
  200    continue
  300 continue

*     Set  v  =  v  +  vlast*w.

      call daxpy ( n, vlast, w, 1, v, 1 )

*     Eliminate the front of the  lastv-th  row of  r  (held in  v)
*     using a partial forward sweep of rotations.

      ldiag  = 0

      do 700 i  = 1, lm1
         ldiag  = ldiag + i
         t      = v(i)
         if (abs(t) .le. tolz) go to 700
         s      = r(ldiag)
         root   = sqrt(s**2 + t**2)
         cs     = s/root
         sn     = t/root
         r(ldiag) = root
         l      = ldiag + i

         do 600 j = i + 1, n
            s     = r(l)
            t     = v(j)
            r(l)  = cs*s + sn*t
            v(j)  = sn*s - cs*t
            l     = l + j
  600    continue
  700 continue

*     Insert the new lastv-th row of  r.

      l      = lastr
      do 900 j = lastv, n
         r(l)  = v(j)
         l     = l + j
  900 continue

*     Test for (unlikely) singularity.

      inform = 1
      if (abs( r(lastr) ) .le. told) then
         inform   = 2
         r(lastr) = told
      end if

*     end of m6rmod
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6rset( maxr, nr, ns, r, cond )

      implicit           double precision (a-h,o-z)
      double precision   r(nr)

*     ------------------------------------------------------------------
*     m6rset  alters  r,  the upper-triangular factor of the
*     approximation to the reduced hessian.
*
*     If  r(1) = zero,  r  does not exist.
*     In this case,  r  is initialized to the identity matrix.
*
*     Otherwise,  r  already exists and we attempt to make it better
*     conditioned by scaling its columns by the square roots of its
*     current diagonals.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm

      intrinsic          abs, max, min, sqrt
      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

      cond   = one
      ncolr  = min( maxr, ns )
      if (ncolr .eq.  0  ) return

      dmax   = abs( r(1) )
      if (dmax  .eq. zero) then

*        Set  r = the identity.

         l      = ncolr*(ncolr + 1)/2
         call dload ( l, zero, r, 1 )
         ldiag  = 0

         do 100 k = 1, ncolr
            ldiag    = ldiag + k
            r(ldiag) = one
  100    continue

         do 120 k = maxr + 1, ns
            ldiag    = ldiag + 1
            r(ldiag) = one
  120    continue
      else

*        Scale the columns of  r.

         dmin   = dmax
         ldiag  = 0

         do 250 k = 1, ncolr
            l     = ldiag + 1
            ldiag = ldiag + k
            diag  = abs( r(ldiag) )
            dmax  = max( dmax, diag )
            dmin  = min( dmin, diag )
            s     = one / sqrt( diag )
            do 240 i = l, ldiag
               r(i)  =  r(i)*s
  240       continue
  250    continue

         cond   = dmax / dmin
         if (iprint .gt. 0) write(iprint, 2000) dmax, dmin, cond
      end if
      return

 2000 format(/ ' XXX  Hessian modified.    Diags =', 1p, 2e11.2,
     $         ',    Cond R(t)*R =', e11.2)

*     end of m6rset
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6rsol( mode, maxr, nr, ns, r, y )

      implicit           double precision (a-h,o-z)
      double precision   r(nr), y(ns)

*     ------------------------------------------------------------------
*     m6rsol  solves  R*x = y  or  R(t)*x = y,  where  R  is an
*     upper-triangular matrix stored by columns in  r(nr).
*     The solution  x  overwrites  y.
*     ------------------------------------------------------------------

      intrinsic          min

      ncolr  = min( maxr, ns )

      if (mode .eq. 1) then

*        mode = 1  --  solve  R*y = y.

         l    = ncolr*(ncolr + 1)/2
         do 120 i  = ncolr, 2, -1
            t      = y(i) / r(l)
            y(i)   = t
            l      = l - i
            call daxpy ( i-1, (-t), r(l+1), 1, y, 1 )
  120    continue
         y(1) = y(1) / r(1)
      else

*        mode = 2  --  solve  R(t)*y = y.

         y(1) = y(1) / r(1)
         l    = 1
         do 220 i = 2, ncolr
            t     = ddot  ( i-1, r(l+1), 1, y, 1 )
            l     = l + i
            y(i)  = (y(i) - t) / r(l)
  220    continue
      end if

*     Deal with surplus diagonals of  r.

      if (ns .gt. maxr) then
         l     = maxr*i/2
         do 320 j = maxr + 1, ns
            l     = l + 1
            y(j)  = y(j) / r(l)
  320    continue
      end if

*     end of m6rsol
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m6swap( m, maxr, nr, ns, ms,
     $                   kb, bbl, bbu, grd, r, rg, x )

      implicit           double precision (a-h,o-z)
      integer            kb(ms)
      double precision   bbl(ms), bbu(ms), grd(ms)
      double precision   r(nr), rg(ns), x(ms)

*     ------------------------------------------------------------------
*     m6swap  (superbasic swap)  finds the largest reduced gradient in
*     the range  rg(maxr+1), ..., rg(ns)  and swaps it into position
*     maxr + 1  (so we know where it is).
*     ------------------------------------------------------------------

      k1     = maxr + 1
      if (ns .gt. k1) then
         nz2    = ns - maxr
         k2     = maxr + idamax( nz2, rg(k1), 1 )
         if (k2 .gt. k1) then
            j      = m + k1
            k      = m + k2
            lastr  = maxr*k1/2
            ldiag1 = lastr + 1
            ldiag2 = lastr + (k2 - maxr)

            rdiag1    = r(ldiag1)
            rg1       = rg(k1)
            j1        = kb(j)
            bl1       = bbl(j)
            bu1       = bbu(j)
            grd1      = grd(j)
            x1        = x(j)

            r(ldiag1) = r(ldiag2)
            rg(k1)    = rg(k2)
            kb(j)     = kb(k)
            bbl(j)    = bbl(k)
            bbu(j)    = bbu(k)
            grd(j)    = grd(k)
            x(j)      = x(k)

            r(ldiag2) = rdiag1
            rg(k2)    = rg1
            kb(k)     = j1
            bbl(k)    = bl1
            bbu(k)    = bu1
            grd(k)    = grd1
            x(k)      = x1
         end if
      end if

*     end of m6swap
      end
************************************************************************
*
*     File  mi70nobj fortran.
*
*     m7bsg    m7chkd   m7chkg   m7chzq   m7fixb
*     m7rg     m7rgit   m7sdir   m7sscv
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m7bsg ( ms, nn, kb, gsub, grd )

      implicit           double precision (a-h,o-z)
      integer            kb(ms)
      double precision   gsub(nn), grd(ms)

*     ------------------------------------------------------------------
*     m7bsg   sets  grd = basic and superbasic components of  gsub.
*     ------------------------------------------------------------------

      common    /m3scal/ sclobj,scltol,lscale
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj

      parameter        ( zero = 0.0d+0 )

      do 20 k = 1, ms
         j    = kb(k)
         if (j .le. nn) then
            grd(k) = gsub(j)
         else
            grd(k) = zero
         end if
   20 continue

      if (iobj .ne. 0) grd(iobj) = - minimz * sclobj

*     end of m7bsg
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m7chkd( n, bl, bu, x, dx, d, nfeas )

      implicit           double precision (a-h,o-z)
      double precision   bl(n), bu(n), x(n), dx, d(n)

*     ------------------------------------------------------------------
*     m7chkd  checks that x + dx*d is feasible.
*     It is used by m7chkg and m8chkj for the cheap gradient checks.
*     Original:    Just looked at the sign of d for variables on a bound.
*     13 Mar 1992: dx added as a parameter to make certain that
*                  x + dx*d does not lie outside the bounds.
*                  d may be altered to achieve this.
*     ------------------------------------------------------------------

      parameter        ( zero  = 0.0d+0 )

      nfeas  = 0
      do 500 j = 1, n
         xj    = x(j)
         b1    = bl(j)
         b2    = bu(j)
         if (b1   .eq. b2  ) d(j) = zero

         if (d(j) .ne. zero) then

*           x(j) is not fixed, so there is room to move.
*           If xj + dx*dj is beyond one bound, reverse dj
*           and make sure it is not beyond the other.
*           Give up and use set dj = zero if both bounds are too close.

            dj     = d(j)
            xnew   = xj  +  dx*dj

            if (dj .gt. zero) then
               if (xnew .gt. b2) then
                  dj     = - dj
                  xnew   =   xj  +  dx*dj
                  if (xnew .lt. b1) dj = zero
               end if
            else
               if (xnew .lt. b1) then
                  dj     = - dj
                  xnew   =   xj  +  dx*dj
                  if (xnew .gt. b2) dj = zero
               end if
            end if

            d(j)   = dj
            if (dj .ne. zero) nfeas  = nfeas + 1
         end if
  500 continue

*     end of m7chkd
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m7chkg( n, bl, bu, g, g2,
     $                   x, da, db, z, nwcore )

      implicit           double precision (a-h,o-z)
      double precision   bl(n), bu(n), da(n), db(n),
     $                   g(n), g2(n), x(n), z(nwcore)

*     ------------------------------------------------------------------
*     This routine checks that the gradient of an n-dimensional
*     function has been defined and programmed correctly.
*
*     First, a cheap heuristic test is performed, as in
*     subroutine chkgrd by the following authors:
*     Philip E. Gill, Walter Murray, Susan M. Picken and Hazel M. Barber
*     D.N.A.C., National Physical Laboratory, England  (circa 1975).
*
*     Next, a more reliable test is performed on each component of the
*     gradient, for indices in the range  jverif(1)  thru  jverif(2).
*
*     lverif(1) is the verify level, which has the following meaning:
*
*     -1         do not perform any check.
*      0         do the cheap test only.
*      1 or 3    do both cheap and full test on objective gradients.
*      2 or 3    do both cheap and full test on the jacobian.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5log1/ idebug,ierr,lprint
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8veri/ jverif(4),lverif(2)

      intrinsic          abs, max, min

      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

      logical            cheap, nodd
      character*4        key
      character*4        lbad        , lgood
      data               lbad/'bad?'/, lgood/'ok  '/

      lvl    = lverif(1)
      if (lvl .lt. 0) return

      j1     = max( jverif(1), 1 )
      j2     = min( jverif(2), n )
      cheap  = lvl .eq. 0  .or.  lvl .eq. 2  .or.  j1 .gt. j2
      lssave = lscale
      lscale = 0

*     Evaluate the function f and gradient g at the base point x.

      call m6dmmy( n, g )
      call m6fobj( 2, n, f, g, x, z, nwcore )
      if (ierr      .ne. 0) go to 900
      if (knowng(1) .eq. 0) go to 900

      if (iprint .gt. 0) then
         if ( cheap ) then
            write(iprint, 1100)
         else
            write(iprint, 1000)
         end if
      end if

*     --------------------------
*     Cheap test.
*     --------------------------

*     If n is odd, u is set to 1/(n - 1).  Otherwise, u = 1/n.

      nodd = 2*(n/2) .ne. n
      rn   = n
      r    = one/rn
      u    = r
      if (nodd  .and.  n .ne. 1) u = one/(rn - one)

*     Set arrays da and db to be (almost) orthogonal.
*     We must not perturb x(j) if g(j) is unknown.

      call dload ( n, zero, da, 1 )
      call dload ( n, zero, db, 1 )
      do 20 j  = 1, n
         if (g(j) .eq. gdummy) go to 20
         db(j) = r
         da(j) = u
   20 continue

      do 30 j  = 1, n, 2
         if (da(j) .ne. zero) da(j) = - da(j)  -  j / rn
   30 continue
      if (nodd) da(n) = zero

*     Define a difference interval.
*     Make sure da and db are feasible directions.

      dx     = difint(1) * (one + dnorm1( n, x, 1 ))
      call m7chkd( n, bl, bu, x, dx, da, nfeas1 )
      call m7chkd( n, bl, bu, x, dx, db, nfeas2 )

      if (nfeas1 + nfeas2 .eq. 0) then
         if (iprint .gt. 0) write(iprint, 1200)
      else

*        Set w1 = g(t)*da and w2 = g(t)*db.

         w1     = ddot  ( n, da, 1, g, 1 )
         w2     = ddot  ( n, db, 1, g, 1 )

*        Make a forward-difference approximation to the gradient
*        along da and db.

         do 40 i  = 1, n
            da(i) = x(i) + dx*da(i)
   40    continue
         f1     = f
         if (n    .ne. 1) call m6fobj( 0, n, f1, g2, da, z, nwcore )
         if (ierr .ne. 0) go to 900
         v1     = (f1 - f)/dx

         do 50 i  = 1, n
            db(i) = x(i) + dx*db(i)
   50    continue
         call m6fobj( 0, n, f2, g2, db, z, nwcore )
         if (ierr .ne. 0) go to 900
         v2     = (f2 - f)/dx

*        c1 and c2 are the differences between approximated and
*        programmed gradient projected along da and db respectively.

         c1     = v1 - w1
         c2     = v2 - w2

*        Set an error indicator if c1 or c2 is too large.

         ifail  = 0
         if (c1*c1 .ge. dx*(w1*w1 + one)  .or.
     $       c2*c2 .ge. dx*(w2*w2 + one)) ifail = 2

         if (ifail .eq. 0) then
            if (iprint .gt. 0) write(iprint, 1400)
         else
            if (iprint .gt. 0) write(iprint, 1500)
            if (isumm  .gt. 0) write(isumm , 1500)
         end if
         if (iprint .gt. 0) write(iprint, 1600) w1,w2,v1,v2
      end if

*     ------------------------------------------------------------------
*     Check each component by differencing along
*     the coordinate directions.
*     Don't bother printing a line if it looks like an exact zero.
*     ------------------------------------------------------------------

      if (cheap) go to 900
      if (iprint .gt. 0) write(iprint, 2000)
      top    =  (one + abs( f ))*eps2
      emax   = - one
      jmax   = 0
      nwrong = 0
      ngood  = 0

      do 200 j = j1, j2
         xj    = x(j)
         gj    = g(j)
         if (gj .eq. gdummy) go to 200
         gabs  = one + abs( gj )
         dx    = top / gabs
         x(j)  = xj + dx
         call m6fobj( 0, n, fforwd, g2, x, z, nwcore )
         if (ierr .ne. 0) go to 900

         gdiff = (fforwd - f) / dx
         err   = abs( gdiff - gj ) / gabs

         if (emax .lt. err) then
            emax  = err
            jmax  = j
         end if

         key   = lgood
         if (err .gt. eps5) key = lbad
         if (key .eq. lbad) nwrong = nwrong + 1
         if (key .eq.lgood) ngood  = ngood  + 1
         if (abs( gj ) + err  .gt.  eps0) then
            if (iprint .gt. 0) write(iprint, 2100) j,xj,dx,gj,gdiff,key
         end if
         x(j)  = xj
  200 continue

      if (iprint .gt. 0) then
         if (nwrong .eq. 0) then
            write(iprint, 2500) ngood ,j1,j2
         else
            write(iprint, 2600) nwrong,j1,j2
         end if
         write(iprint, 2700) emax,jmax
      end if
      if (emax .lt. one) go to 900

*     Bad gradients in  funobj.

      ierr   = 7
      call m1envt( 1 )
      if (iprint .gt. 0) write(iprint, 3700)
      if (isumm  .gt. 0) write(isumm , 3700)

*     Exit.

  900 lscale = lssave
      return

 1000 format(/// ' Verification of objective gradients',
     $   ' returned by subroutine funobj.')
 1100 format(/ ' Cheap test on funobj...')
 1200 format(/ ' XXX  Can''t find a feasible step -',
     $   ' objective gradients not verified.')
 1400 format(  ' The objective gradients seem to be OK.')
 1500 format(  ' XXX  The objective gradients seem to be incorrect.')
 1600 format(  ' Gradient projected in two directions', 1p, 2e20.11,
     $       / ' Difference approximations           ', 2e20.11)
 2000 format(// 6x, 'j', 7x, 'x(j)', 8x, 'dx(j)',
     $   11x, 'g(j)', 9x, 'Difference approxn' /)
 2100 format(i7, 1p, e16.8, e10.2, 2e18.8, 2x, a4)
 2500 format(/ i7, '  objective gradients out of', i6, '  thru', i6,
     $         '  seem to be OK.')
 2600 format(/ ' XXX  There seem to be', i6,
     $   '  incorrect objective gradients in cols', i6, '  thru', i6)
 2700 format(/ ' XXX  The largest relative error was', 1p, e12.2,
     $   '   in column', i6 /)
 3700 format(// ' EXIT -- subroutine funobj appears to be',
     $   ' giving incorrect gradients')

*     end of m7chkg
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m7chzq( m, nb, ms, ns, Rset, jq, pivot,
     $                   ne, nka, a, ha, ka,
     $                   kb, bl, bu, x, y, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            Rset
      integer*4          ha(ne)
      integer            ka(ne), kb(ms)
      double precision   a(ne), bl(nb), bu(nb), x(ms), y(ms), z(nwcore)

*     ------------------------------------------------------------------
*     m7chzq  selects a superbasic to replace the jp-th basic variable.
*     On entry, y(1:m) contains the jp-th row of B(inverse).
*     On exit,  if Rset is true, y(S) = y(m1:ms)  holds the vector v
*               needed by m6bswp to update R following a basis change.
*
*     29 Nov 1991: qnewtn changed to Rset (should fix a bug).
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm

      intrinsic          abs, min
      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

*     Set  y(S) = 0 - S(t)*y.  Beware of the minus sign when using y(S).

      m1     = m + 1
      call dload ( ns, zero, y(m1), 1 )
      call m2aprd( 4, y, m, y(m1), ns,
     $             ne, nka, a, ha, ka,
     $             z, nwcore )
      jq     = m  +  idamax( ns, y(m1), 1 )
      pivot  = abs( y(jq) )

*     Exit if the pivot is too small.

      if (pivot .lt. tolpiv) then
         if (iprint .gt. 0) then
            write(iprint, '(/ a, 1p, e11.1)')
     $         ' XXX  m7chzq.  Max pivot is too small:', pivot
         end if
         jq     = - ms
      else

*        Choose one away from its bounds if possible.

         tol    =   0.1*pivot
         dmax   = - one

         do 200 k = m1, ms
            if (abs( y(k) ) .ge. tol) then
               j     = kb(k)
               xj    = x(k)
               d1    = xj - bl(j)
               d2    = bu(j) - xj
               d1    = min( abs( d1 ), abs( d2 ) )
               if (dmax .le. d1) then
                  dmax  = d1
                  jq    = k
               end if
            end if
  200    continue

         pivot = - y(jq)

*        Finish computing v, the vector needed to modify R.

         if ( Rset ) then
            y(jq) = - (one + pivot)
            call dscal ( ns, (one/pivot), y(m1), 1 )
         end if
      end if

*     end of m7chzq
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m7fixb( m, maxr, ms, n, nb, nr, ns, nx, inform,
     $                   ne, nka, a, ha, ka,
     $                   hs, kb, bl, bu, bbl, bbu,
     $                   r, x, y, y2, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hs(nb)
      integer            ka(ne), kb(ms)
      double precision   a(ne), bl(nb), bu(nb), bbl(ms), bbu(ms)
      double precision   r(nr), x(ms), y(ms), y2(nx), z(nwcore)

*     ------------------------------------------------------------------
*     m7fixb looks to see if the current basis B is ill-conditioned.
*     We assume that the factorization B = L*U has just been computed.
*     If the smallest diagonal of U seems too small, a superbasic is
*     chosen to be swapped with the corresponding column of B.
*     The reduced Hessian R is updated accordingly.
*
*     On exit,
*     inform = 0 means no error condition.
*     inform = 5 means the LU update failed.
*
*     15 Mar 1992: First version, derived from bits of m7chzq, m7rgit.
*     18 Mar 1992: If we call m7fixb only after m2bfac, we don't need
*                  to make it update grd, pi and rg.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2lu4 / parmlu(30),luparm(30)
      common    /m2parm/ dparm(30),iparm(30)
      common    /m5log3/ djq,theta,pivot,cond,nonopt,jp,jq,modr1,modr2
      common    /m5lp2 / invrq,invitn,invmod

      logical            Rset, swap
      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

*     LUSOL gives umax  = the largest element of U,
*                 dumin = the smallest diagonal of U,
*             and jumin = the corresponding column of B.

      inform = 0
      umax   = parmlu(12)
      dumin  = parmlu(14)
      jumin  = luparm(19)

*     Since modifying B is an expensive operation,
*     we don't do anything if dumin seems reasonably large.
*     The SPECS file gives swaptl = Swap tolerance = eps4 by default.
*     NOTE: If B has rank 0, dumin = umax = 0.
*           If B has rank 1, dumin = umax.
*           Hence we need both relative and absolute tests.

      swaptl = dparm(8)
      if (dumin .gt. swaptl * (umax + one)) return

*     U looks ill-conditioned.  We want to replace the jumin-th column.
*     Use jp = jumin for brevity, and for the final call to m2bsol.

      inform = 1
      jp     = jumin
      Rset   = r(1) .ne. zero
      m1     = m + 1
      nz1    = min( ns, maxr )

*     ------------------------------------------------------------------
*     Solve  B(t)*y = e(jp).
*     ------------------------------------------------------------------
      call dload ( m, zero, y2, 1 )
      y2(jp) = one       
      call m2bsol( 3, m, y2, y, z, nwcore )

*     ------------------------------------------------------------------
*     Use y to "price" the superbasics.
*     For each column of S, y(S) returns (minus) the pivot element
*     that would arise if that column replaced the jumin-th basic.
*     ------------------------------------------------------------------

*     Set  y(S) = 0 - S(t)*y.  Beware of the minus sign when using y(S).

      call dload ( ns, zero, y(m1), 1 )
      call m2aprd( 4, y, m, y(m1), ns,
     $             ne, nka, a, ha, ka,
     $             z, nwcore )
      kq     =   idamax( ns, y(m1), 1 )
      jq     =   m + kq
      pivot  = - y(jq)

*     Continue if pivot is noticeably larger than dumin.
*     It is not clear that this the best test, but it will do for now.

      cond0  = cond
      dtol   = 2.0d+0
      swap   = abs( pivot ) .ge. dtol*dumin

      if ( swap ) then
         if (Rset  .and.  kq .le. maxr) then

*           Finish computing v, the vector needed to modify R.
*           v is stored in y(S) for use in m6bswp.

            y(jq) = - (one + pivot)
            call dscal ( ns, (one/pivot), y(m1), 1 )

*           Modify R to account for the change in basis.
*           y2 is workspace.
*           Get cond(R) before and after to see how we're doing.

            call m6rcnd( maxr, nr, ns, r, dmax, dmin, cond0 )
            call m6bswp( nz1, nr, r, y2, y(m1), kq,
     $                   eps0, eps2, modr2 )
            call m6rcnd( maxr, nr, ns, r, dmax, dmin, cond )

*           We may want to cancel the swap if the new R
*           seems a lot worse than before.
*           At present we don't know enough about it, so let it go.

            worse  = 1.0d+3
            if (cond .gt. worse*cond0) then
*---           swap   = .false.
            end if

*           If cond is now pretty bad, set R = I.
*           (Alternatively we could set swap = .false.
*           and restore the old R by calling m6bswp.)

            if (cond .ge. one/eps1) then
               r(1) = zero
               call m6rset( maxr, nr, ns, r, cond )
            end if
         end if

*        Swap columns jp and jq of (B S).

         if ( swap ) then
            jr1     = kb(jp)
            jq1     = kb(jq)
            kb(jp)  = jq1
            kb(jq)  = jr1
            hs(jq1) = 3
            hs(jr1) = 2

            if (iprint .gt. 0)
     $      write(iprint, 1000) jr1, jq1, dumin, pivot, cond0, cond
            if (isumm  .gt. 0)
     $      write(isumm , 1000) jr1, jq1, dumin, pivot, cond0, cond

            t       = bbl(jp)
            bbl(jp) = bbl(jq)
            bbl(jq) = t

            t       = bbu(jp)
            bbu(jp) = bbu(jq)
            bbu(jq) = t

            t       = x(jp)
            x(jp)   = x(jq)
            x(jq)   = t

*           Set  y2  for modifying L and U.   L*y2 = a(jq1).
*           Then modify L and U, using jp in /m5log3/.

            call m2unpk( jq1, m, n, ne, nka, a, ha, ka, y2 )
            call m2bsol( 1, m, y2, y, z, nwcore )
            call m2bsol( 4, m, y2, y, z, nwcore )
            if (invrq .ne. 0) inform = 5
         end if
      end if

      return

 1000 format(1p, ' Swap', 2i6, '   Diag, pivot =', 2e9.1,
     $       '   cond(R) =', 2e9.1)

*     end of m7fixb
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m7rg  ( m, ms, ns, grd, pi, rg, rgnorm,
     $                   ne, nka, a, ha, ka,
     $                   z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne), grd(ms), pi(m), rg(ns), z(nwcore)

*     ------------------------------------------------------------------
*     m7rg    calculates the reduced gradient  rg = g(S) - S(t)*pi.
*     ------------------------------------------------------------------

      intrinsic          abs, min
      external           idamax

      call dcopy ( ns, grd(m+1), 1, rg, 1 )
      call m2aprd( 4, pi, m, rg, ns,
     $             ne, nka, a, ha, ka,
     $             z, nwcore )
      kmax   = idamax( ns, rg, 1 )
      rgnorm = abs( rg(kmax) )

*     end of m7rg
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m7rgit( m, maxr, maxs, mbs, n, nb, incres,
     $                   nn, nn0, nr, ns, nx, inform, nxtphs,
     $                   ne, nka, a, ha, ka,
     $                   hrtype, hs, kb, bl, bu, bbl, bbu,
     $                   fsub, gsub, grd, grd2,
     $                   pi, r, rg, rg2, x, xn, y, y2, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            incres
      integer*4          ha(ne), hrtype(mbs), hs(nb)
      integer            ka(nka), kb(mbs)
      double precision   a(ne), bl(nb), bu(nb), bbl(mbs), bbu(mbs),
     $                   gsub(nn0), grd(mbs), grd2(mbs),
     $                   pi(m), r(nr), rg(maxs), rg2(maxs),
     $                   x(mbs), xn(nb), y(nx), y2(nx), z(nwcore)

*     ------------------------------------------------------------------
*     m7rgit performs an iteration of the reduced-gradient algorithm.
*
*     incres in Phase 3 says if the new variable should increase or not.
*            It is used only in linear mode when m5pric is moving free
*            nonbasics toward their closest bound (and djq = zero).
*
*     Rset   says if a useful quasi-newton  r  exists.
*            It will be false if a feasible point has not yet been
*            found.
*            Note that Rset could be true even if the current itn
*            is infeasible.  This allows  r  to be updated in
*            certain ways during temporary loss of feasibility.
*
*     qnewtn in this version (Jan 1983) is the same as  nonlin.
*
*     parhes is true if  r  is not big enough to store a full
*            quasi-newton estimate of the projected hessian.
*            The null space is then  z = ( z1  z2 ),  and  r  estimates
*            the Hessian only in the subspace  z1.  A diagonal estimate
*            is used (also in  r) for the subspace  z2.
*
*     fullz  is true if the search direction for the superbasics is
*            computed using all of the reduced gradients.  Otherwise,
*            the components corresponding to  z2  are zero.
*
*     grd2   is not used in this version (Aug 1986).
*
*     xx Sep 1987: Anti-degeneracy m5chzr incorporated.
*     29 Sep 1991: a, ha, ka etc. passed in.  Argument list altered.
*     04 Oct 1991: switch and difint added as arguments to m6srch.
*     08 Apr 1992: hs(*) now has internal values.  We have to be more
*                  careful in setting jrstat, the state of a blocking
*                  variable.
*     15 Apr 1992: incres added as input parameter for linear Phase 3
*                  when djq = 0.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2parm/ dparm(30),iparm(30)
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m5log2/ jq1,jq2,jr1,jr2,lines1,lines2
      common    /m5log3/ djq,theta,pivot,cond,nonopt,jp,jq,modr1,modr2
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m5lp2 / invrq,invitn,invmod
      common    /m5prc / nparpr,nmulpr,kprc,newsb
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m7len / fobj  ,fobj2 ,nnobj ,nnobj0
      logical            conv
      common    /m7conv/ etash,etarg,lvltol,nfail,conv(4)
      common    /m7phes/ rgmin1,rgnrm1,rgnrm2,jz1,jz2,labz,nfullz,mfullz
      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8save/ vimax ,virel ,maxvi ,majits,minits,nssave
*     ------------------------------------------------------------------

      intrinsic          abs, max, min, sqrt

      logical            feasbl, infsbl, linear, nonlin,
     $                   fullz , parhes, qnewtn, Rset  ,
     $                   debug , fonly , grdcon, grdobj,
     $                   hitlow, move  , onbnd , switch, unbndd,
     $                   unbddf, unbddx, uncon , vertex

      parameter        ( zero   = 0.0d+0,  one    = 1.0d+0,
     $                   point1 = 0.1d+0,  point9 = 0.9d+0 )

      character*19       msg(4:8)
      data               msg/'max step too small.',
     $                       'step too small.    ',
     $                       'no minimizer.      ',
     $                       'too many functions.',
     $                       'uphill direction.  '/

      inform = 0
      m1     = m + 1
      tolz   = eps0
      feasbl = ninf .eq. 0
      infsbl = .not. feasbl
      linear = infsbl  .or.  nn .eq. 0
      nonlin = .not. linear
      qnewtn = nonlin
      Rset   = r(1) .ne. zero
      unbndd = .false.
      bplus  = point1 * plinfy
      toobig = sqrt(plinfy)
      oldfx  = fsub
      if ( qnewtn ) then
         if (.not. Rset) then
            call m6rset( maxr, nr, ns, r, cond )
            Rset = .true.
         end if
      end if

      if (nphs .eq. 3) then
*        ---------------------------------------------------------------
*        Phase 3.      First, make sure we dont want to stay in Phase 4.
*        Since price can select nonbasics floating free between their
*        bounds with zero reduced cost, we have to check that dqj is
*        not zero.
*        ---------------------------------------------------------------
         if (linear) then
            if (djq .eq. zero) then
               djq      = one
               if (incres) djq = - one
               rg(ns+1) = djq
            end if
         end if
               
         jq2    = jq
         djqmod = abs( djq )
         ratio  = rgnorm / djqmod

         if (      linear              ) go to 100
         if (ns     .eq. 0             ) go to 100
         if (nfail  .gt. 0             ) go to 100
         if (ratio  .le. point9        ) go to 100
         if (rgnorm .le. gtol(2)*pinorm) go to 100

            tolrg  =  point9 * rgnorm
            jq2    = -jq
            nphs   =  4
            go to 200

*        Add the superbasics selected during pricing.
*        The last newsb entries in kb and rg have been set by m5pric.

  100    nfullz = 0
         jqstat = hs(jq)
         rgnorm = max( rgnorm, djqmod )
         tolrg  = etarg * djqmod

*        NOTE.  The above line sets the level to which rgnorm
*        must be reduced in the current subspace (Phase 4)
*        before we consider moving off another constraint (Phase 3).
*        etarg between (0, 1) is set by the user at his/her own peril.
*        It is the Subspace tolerance in the SPECS file.

         do 130 j = 1, newsb
            ns         = ns + 1
            ms         = m + ns
            kq         = kb(ms)
            hs(kq)     = 2
            grd(ms)    = zero
            if (kq .le. nn  .and.  feasbl) grd(ms) = gsub(kq)
            hrtype(ms) = 0
            bbl(ms)    = bl(kq)
            bbu(ms)    = bu(kq)
            x(ms)      = xn(kq)
            if (Rset) call m6radd( maxr, nr, ns, r )
  130    continue

      else
*        ---------------------------------------------------------------
*        Phase 4.      Exit if rgnorm is already very small.
*        ---------------------------------------------------------------
         if (rgnorm  .le.  point1 * toldj(3) * pinorm) go to 910
      end if


*     ==================================================================
*     Get a search direction ys for the superbasics (in y(m1)...y(ms))
*     and then a search direction y for the basics.
*     ==================================================================
  200 ms     = m + ns
      nssave = ns
      vertex = ns .eq. 1
      fullz  = .true.
      parhes = nonlin  .and.  ns .gt. maxr
      nz1    = ns
      if (parhes) nz1 = maxr
      nz2    = ns - maxr
      lastr  = maxr*(maxr+1)/2

*     Compute the search direction for the superbasics.
*     rg2  is used for the search direction, but it is promptly
*     copied into  y(m1)  below.
*     If  r  is being updated,  m7sdir  saves w such that R(t)*w = rg.
*     w  is used later by  m6bfgs.

      mode   = 0
      if (nonlin) mode = 1
      call m7sdir( mode, maxr, nr, ns, r, rg, rg2, z(lw), z, nwcore )
      call dcopy ( ns, rg2, 1, y(m1), 1 )

      if ( parhes ) then

*        Partial Hessian.  Let all superbasics move for the first
*        mfullz iterations, but then move only the first set.

         mfullz = 3
         nfullz = nfullz + 1
         if (nfullz .gt. mfullz) then
            fullz  = .false.
            labz   = 2
            j      = m1 + maxr
            call dload ( nz2, zero, y(j), 1 )
         end if
      end if

*     ==================================================================
*     The search direction for the superbasics, ys, is now in
*     y(m+1), ..., y(m+ns).
*     Find norms of xs and ys.
*     ==================================================================
      xsnrm1 = dasum( nz1, x(m1), 1 )
      ysnrm1 = dasum( nz1, y(m1), 1 )
      if (parhes) then
         xsnrm2 = dasum( nz2, x(m1+maxr), 1 )
         ysnrm2 = dasum( nz2, y(m1+maxr), 1 )
      else
         xsnrm2 = zero
         ysnrm2 = zero
      end if
      xsnorm = xsnrm1 + xsnrm2
      ysnorm = ysnrm1 + ysnrm2

*     Compute  y2 = - S*ys and prepare to solve  B*y = y2
*     to get y, the search direction for the basics.
*     We first normalize y2 so the LU solver won't ignore
*     too many "small" elements while computing y.

      call dscal ( ns, (one / ysnorm), y(m1), 1 )
      call dload ( m, zero, y2, 1 )
      call m2aprd( 2, y(m1), ns, y2, m,
     $             ne, nka, a, ha, ka,
     $             z, nwcore )

*     Solve  B*y = y2  and then unnormalize all of y.

      call m2bsol( 2, m, y2, y, z, nwcore )
      call dscal ( ms, ysnorm, y, 1 )
      ynorm  = dasum( m, y, 1 )  +  ysnorm
      xnorm  = dasum( m, x, 1 )  +  xsnorm

*     ------------------------------------------------------------------
*     Find the nearest constraint in direction  x + theta*y  (theta>=0).
*     With the present version of m5chzr (Sep 1987), theta is always
*     positive.
*     Exact  is the step that takes x(jp) exactly onto bound.
*     It may be positive or slightly negative. (Not defined if unbndd.)
*
*     For the linesearch, theta becomes stepmx, the largest
*     step that the linesearch is allowed to take.
*
*     If exact is positive, or if we are at a vertex, we do a linesearch
*     to find a new step theta in the range 0 lt theta le stepmx.
*     Sometimes, this interval may be too small for the search to be
*     successful.  One thing to remember is that increasing the
*     feasibility tolerance has the effect of increasing stepmx,
*     and therefore increasing the chance of a successful search.
*
*     If exact isn't positive and we are not at a vertex, we change
*     theta and stepmx to zero and don't move.
*
*     If onbnd  is true, theta is a step that reaches a bound exactly.
*     x(jp) reaches the value bound.  If the linesearch says to
*     take a constrained step, bound is used to put the new nonbasic
*     variable xn(jr) exactly on its bound.
*
*     If unbndd is true, theta = stepmx.
*     ------------------------------------------------------------------

*xxx  stepmn = eps1    / (one + ynorm)  > 9 mar 1988: these are wrong if
*xxx  stepmx = 1.0d+12 / (one + ynorm)  > ynorm is small (john stone).
      stepmx = 1.0d+12 /  ynorm
      tolp   = tolpiv  *  ynorm

      call m5chzr( ms    , stepmx, plinfy, tolp  ,
     $             hrtype, bbl   , bbu   , x     , y,
     $             hitlow, move  , onbnd , unbndd,
     $             jp    , bound , exact , theta )

      if (.not. unbndd) then
         pivot  = - y(jp)
         stepmx =   theta
         jr     =   kb(jp)
      end if

      if (unbndd  .or.  vertex  .or.  exact .gt. zero) then
*        ===============================================================
*        A move looks possible.
*        If linear, do a normal update to  x  and skip the linesearch.
*        ===============================================================
         if ( linear ) then
            if ( unbndd ) go to 955
            call daxpy ( ms, theta, y, 1, x, 1 )
            call m5bsx ( 1, ms, nb, kb, x, xn )
            go to 500
         end if
      else
*        ===============================================================
*        Zero step.
*        The blocking variable x(jp) is currently on its bound or
*        slightly infeasible.  We shall make it nonbasic and not move.
*        The iteration proceeds as a constrained step with theta = zero
*        and again we skip the linesearch.
*
*        NOTE: If it is Phase 3 and the blocking variable is the one
*        we have just brought in, we would have to go back and try again
*        without it, to avoid "Cycling in Phase 3".
*        However, since r is expanded with a unit vector,
*        cycling of this kind can never happen.  Hence, we no longer
*        guard against it.
*        ===============================================================
         theta  = zero
         stepmx = zero
         onbnd  = .false.
         go to 500
      end if

*     ------------------------------------------------------------------
*     Perform a linesearch to find a downhill point  (x = x + theta*y).
*     switch tells m6srch whether there is an option to switch to
*     central differences to get a better search direction.
*
*     m6srch returns the following values:
*
*     inform =-1 (and ierr = 6) if the user wants to stop.
*     inform = 1 if the search is successful and theta < stepmx.
*            = 2 if the search is successful and theta = stepmx.
*            = 3 if a better point was found but too many functions
*                were needed (not sufficient decrease).
*            = 4 if stepmx < tolabs (too small to do a search).
*            = 5 if theta  < alfsml (srchq only -- maybe want to switch
*                to central differences to get a better direction).
*            = 6 if the search found that there is no useful step.
*                The interval of uncertainty is less than 2*tolabs.
*                The minimizer is very close to theta = zero
*                or the gradients are not sufficiently accurate.
*            = 7 if there were too many function calls.
*            = 8 if the input parameters were bad
*                (stepmx le toltny  or  uphill).
*     ------------------------------------------------------------------
      debug  = itn .ge. iparm(2)
      grdcon = nncon .eq. 0  .or.  lderiv .ge. 2
      grdobj = nnobj .eq. 0  .or.  lderiv .eq. 1  .or.  lderiv .eq. 3
      fonly  = .not. (grdcon  .and.  grdobj)
      switch = lvldif .eq. 1
     $           .and.
     $         ((.not. grdobj  .and.  knowng(1) .lt. nnobj)  .or.
     $          (.not. grdcon  .and.  knowng(2) .lt. njac )      )

      epsrf  = dparm(3)
      damp   = dparm(6) * (one + xnorm) / ynorm
      theta  = min( one, damp )

      call m6srch( ms, ns, n, nb, nn, itn, inform, 
     $             debug, fonly, switch,
     $             ne, nka, a, ha, ka,
     $             theta, stepmx, difint(1), eps , epsrf, etash,
     $             fsub , rgnorm, ynorm, xnorm,
     $             gsub , grd, y, x, y2, xn, z, nwcore )

      if (inform .lt. 0) return
      if (inform .le. 3) go to 500

      if (inform .eq. 4) then
*        ---------------------------------------------------------------
*        The linesearch says stepmx is too small.
*        (Function precision affects when this happens.)
*
*        If m5chzr set move false, stepmx is very small and we probably
*        should not have attempted the linesearch.  Rather than taking a
*        zero step (which might lead to cycling if vertex is true) we
*        force a step of stepmx regardless of the effect on the
*        objective.  The step will be constrained.
*        onbnd has the correct value.
*
*        Otherwise, we treat it as a linesearch failure
*        and try for a better direction, possibly with more superbasics.
*        ---------------------------------------------------------------
         if ( unbndd ) go to 440
         if ( move   ) go to 440

         theta  = stepmx
         modefg = 2
         call daxpy ( ms, theta, y, 1, x, 1 )
         call m6fun ( 0, modefg, n, nb, ms, fsub,
     $                ne, nka, a, ha, ka,
     $                x, xn, z, nwcore )
         call m6fun ( 1, modefg, n, nb, ms, fsub,
     $                ne, nka, a, ha, ka,
     $                x, xn, z, nwcore )
         if (ierr .ne. 0) return

         call m6grd ( ms, nb, nn, gsub, grd,
     $                ne, nka, a, ha, ka,
     $                xn, z, nwcore )
         t      = stepmx * ynorm
         if (iprint .gt. 0) write(iprint, 1075) t
         go to 500
      end if

*     ------------------------------------------------------------------
*     See if we should switch to central differences.
*     ------------------------------------------------------------------
      if (inform .eq. 5) then
         if (switch  .and.  lvltol .eq. 2) go to 920
      end if

*     ------------------------------------------------------------------
*     Trouble -- no function decrease.
*     Try resetting the Hessian, deleting constraints,
*     and finally refactorizing the basis.
*     Give up after 8 consecutive failures.
*     ------------------------------------------------------------------
  440 if (inform .ge. 4) then
         if (iprint .gt. 0)
     $      write(iprint, 1050) inform, msg(inform), itn, rgnorm
         if (isumm  .gt. 0)
     $      write(isumm , 1050) inform, msg(inform), itn, rgnorm
      end if
      if (inform .eq. 5  .and.  nfail .lt. 1) nfail = 1
      inform = 0

*     This is the top of the loop through various recovery procedures.

  450 nfail  = nfail + 1
      if ( prnt1 ) write(iprint, 1090) nfail

      if (nfail .eq. 1) then
         if ( qnewtn  .and.  cond .ge. one/eps1 ) then
            call m6rset( maxr, nr, ns, r, cond )
            go to 200
         end if

      else if (nfail .eq. 2) then

*        Switch to central differences if we have not yet done so
*        and if we're trying to minimize accurately.

         if (switch  .and.  lvltol .eq. 2) go to 920

      else if (nfail .le. 5) then

*        Ask for price (up to 3 times) if rgnorm is not too big.

         if (nfail  .ge. 5   .and.  nonopt .eq. 0) go to 475
         if (ns     .ge. maxs                    ) go to 475
         t      = 10.0
         if (lvltol .eq. 2) t = one
         if (rgnorm .gt. t * pinorm              ) go to 475
         go to 910

  475    nfail  = 5

      else if (nfail .eq. 6) then

*        Switch to central differences even if we're not trying to
*        minimize accurately.

         if (switch  .and.  lvltol .eq. 1) go to 920

      else if (nfail .eq. 7) then

*        Request refactorization of the basis (inform = 5).

         if (invitn .gt. 0) go to 925

      else if (nfail .eq. 8) then

*        Request a change of basis as a last resort (inform = 6).

         go to 930
      end if

      if (nfail .lt. 8) go to 450

*     Can't think what to do now other than stop.

      if (ns .eq. maxs) go to 960
      ierr   = 9
      return

*     ------------------------------------------------------------------
*     We got past the linesearch (or didn't have to do one).
*     See if the step is unbounded, unconstrained, zero, or otherwise.
*     ------------------------------------------------------------------
  500 inform = 0
      unbddf = abs( fsub )  .ge. dparm(1)
      unbddx = theta*ynorm  .ge. dparm(2)
      unbndd = unbddf       .or. unbddx
      uncon  = nonlin  .and.  theta .lt. stepmx
      if (unbndd) go to 950

*     ==================================================================
*     Get the new reduced gradient rg2.
*     ==================================================================
      modr1  = 0
      modr2  = 0
      if (linear  .or.  theta .eq. zero) then
         call dcopy ( ns, rg, 1, rg2, 1 )
      else
         call dcopy ( m, grd, 1, y, 1 )
         call m5setp( 1, m, y, pi, z, nwcore )
         call m7rg  ( m, ms, ns, grd, pi, rg2, rgnorm,
     $                ne, nka, a, ha, ka,
     $                z, nwcore )

*        Update the reduced Hessian.

         if (qnewtn) then
            nsmove = ns
            if (parhes  .and.  .not. fullz) nsmove = maxr
            call m6bfgs( maxr, nsmove, nr, r, rg, rg2, y(m1), z(lw),
     $                   theta, eps2, eps0, modr1 )
         end if
      end if

*     ------------------------------------------------------------------
*     Update the active constraint set if necessary, and modify R,
*     the Cholesky factorization of the approximate reduced Hessian.
*     ------------------------------------------------------------------
      if (  uncon  ) then
*        ===============================================================
*        The step is unconstrained.
*        ===============================================================
         pivot  = zero
         kmodlu = 0
         call dcopy ( ns, rg2, 1, rg, 1 )

      else
         if (jp .eq. 0) go to 955

*        ===============================================================
*        There is a blocking variable.
*        It could be a fixed variable, whose new state must be 4.
*        ===============================================================
         if (bbl(jp) .eq. bbu(jp)) then
            jrstat = 4
         else if (hitlow) then
            jrstat = 0
         else
            jrstat = 1
         end if
         if (  onbnd  ) xn(jr) = bound

         if (jp .le. m) then
*           ============================================================
*           A variable in B hit a bound.
*           Find a column in S to replace it.
*           ============================================================

*           Solve  B(t)*y = e(jp).

            call dload ( m, zero, y2, 1 )
            y2(jp) = one
            call m2bsol( 3, m, y2, y, z, nwcore )

*           Select a superbasic to become basic.
*           If Rset is true, y(S) is returned for use in m6bswp.

            call m7chzq( m, nb, ms, ns, Rset, jq, pivot,
     $                   ne, nka, a, ha, ka,
     $                   kb, bl, bu, x, y, z, nwcore )

            if (jq .le. 0) go to 940
            hs(jr)  = jrstat
            jr1     = jr
            jr2     = kb(jq)
            jq1     = jr2
            kb(jp)  = jr2
            bbl(jp) = bbl(jq)
            bbu(jp) = bbu(jq)
            grd(jp) = grd(jq)
            x(jp)   = x(jq)
            kq      = jq - m

*           Modify R to account for the change in basis.
*           y2 is workspace.

            if (Rset  .and.  ns .gt. 1  .and.  kq .le. maxr) then
               call m6bswp( nz1, nr, r, y2, y(m1), kq,
     $                      eps0, eps2, modr2 )
            end if

*           Modify  pi  using  y  where  B(t)*y = e(jp).

            t      = rg2(kq) / pivot
            call daxpy ( m, t, y, 1, pi, 1 )
            pinorm = dnorm1( m, pi, 1 )
            pinorm = max( pinorm, one )

*           Set  y2  for modifying L and U.

            hs(jq1) = 3
            call m2unpk( jq1, m, n, ne, nka, a, ha, ka, y2 )
            call m2bsol( 1, m, y2, y, z, nwcore )

         else
*           ============================================================
*           A variable in S hit a bound.
*           ============================================================
            hs(jr) = jrstat
            jr2    = jr
            kmodlu = 0
            kq     = jp - m
         end if

*        ===============================================================
*        If necessary, swap the largest reduced-gradient in  Z2  into
*        the front of  Z2,  so it will end up in the end of  Z1.
*        ===============================================================
         call m7rg  ( m, ms, ns, grd, pi, rg, rgnorm,
     $                ne, nka, a, ha, ka,
     $                z, nwcore )

         rgdel  = abs( rg(kq) )
         if ( parhes  .and.  kq .le. maxr ) then
            call m6swap( m, maxr, nr, ns, ms, kb, bbl, bbu,
     $                   grd, r, rg, x )
         end if

*        Delete the kq-th superbasic, updating R if it exists.

         call m6rdel( m, maxr, nr, ns, ms, kb, bbl, bbu,
     $                grd, r, rg, x, kq, Rset )
         ns     = ns - 1
         ms     = m + ns
         nssave = ns
         nfullz = 0

         if (rgnorm .le. rgdel) then
            rgnorm = zero
            if (ns  .gt.   0  ) then
               imax   = idamax( ns, rg, 1 )
               rgnorm = abs( rg(imax) )
            end if
         end if
      end if

*     ------------------------------------------------------------------
*     Estimate the condition of  R(t)*R  using the diagonals of  R.
*     ------------------------------------------------------------------
      if ( Rset  .and.  ns .gt. 0 ) then
         call m6rcnd( maxr, nr, ns, r, dmax, dmin, cond )

         if (infsbl) then
*           Infeasible.  If cond is pretty big, throw  R  away.

            if (cond .ge. one/eps2) r(1) = zero
         else
*           Feasible.
*           If cond is hoplessly big, try a basis change.
*           If cond is big but not quite that big, try modifying it.

            if (cond .ge. one/eps0) go to 930
            if (cond .ge. one/eps1) call m6rset( maxr, nr, ns, r, cond )
         end if
      end if

*     ==================================================================
*     Test for convergence in the current subspace.
*     ==================================================================
      nfail  = 0
      parhes = nonlin  .and.  ns .gt. maxr
      call m7sscv( m, maxr, maxs, ms, nr, ns, parhes, nxtphs,
     $             fsub, oldfx, theta, xsnorm, xsnrm1, ysnorm, ysnrm1,
     $             kb, bbl, bbu, grd, r, rg, x )

*     If no room for more superbasics, try to stay in phase 4.

      if (nxtphs .eq. 3  .and.  ns .eq. maxs) nxtphs = 4
      go to 990

*     ------------------------------------------------------------------
*     Various exits.
*     ------------------------------------------------------------------

*     rgnorm is small -- return to Phase 3.

  910 nphs   = 3
      inform = 1
      return

*     ==================================================================
*     Switch to central differences.
*     (Reset ninf to cause m5frmc to set dummy gradients.)
*     ==================================================================
  920 lvldif = 2
      nfail  = 0
      ninf   = 1
      inform = 4
      if (prnt1) write(iprint, 1040)
      return

*     ==================================================================
*     Request a basis factorization.
*     ==================================================================
  925 invrq  = 22
      inform = 5
      return

*     ==================================================================
*     Request a change of basis via m7fixb.
*     ==================================================================
  930 invrq  = 23
      inform = 6
      return

*     ==================================================================
*     m7chzq failed.
*     ==================================================================
  940 nfail  = nfail + 1
      if (nfail .lt. 5) then

*        Treat as an unconstrained step.  Then ask for price.

         jr1    = - jr1
         kmodlu = 0
         kmodpi = 0
         nxtphs = 3
         go to 990
      else

*        Fail.

         call m1page( 2 )
         if (iprint .gt. 0) write(iprint, 1200)
         ierr   = 11
         return
      end if

*     ==================================================================
*     Unbounded.
*     ==================================================================
  950 if (iprint .gt. 0) then
         if (unbddf) write(iprint, 2000) fsub
         if (unbddx) write(iprint, 2100) theta
      end if

  955 if (iprint .gt. 0) write(iprint, 2200) nphs, kb(ms), rg(ns)
      ierr   = 2
      return

*     ==================================================================
*     Too many superbasics.
*     ==================================================================
  960 call m1page( 2 )
      if (iprint .gt. 0) write(iprint, 1600) maxs
      if (isumm  .gt. 0) write(isumm , 1600) maxs
      ierr   = 5
      return

*     ------------------------------------------------------------------
*     Normal exit.
*     ------------------------------------------------------------------
  990 if (nphs .eq. 4) djq = rgnorm
      if (.not. fullz) djq = rgnrm1
      return

 1040 format(' Switch to central differences.')
 1050 format(' Search exit', i3, ' -- ', a,
     $    '   Itn =', i7, '   Norm rg =', 1p, e11.3)
 1075 format(' Forcing a small step.  alpha * norm(p) =', 1p, e9.1)
 1090 format(' Retry', i3)
 1200 format(' EXIT -- cannot find a superbasic to replace',
     $   ' basic variable')
 1600 format(' EXIT -- the superbasics limit is too small...', i7)
 2000 format(/ ' XXX  Linesearch has exceeded the Unbounded function',
     $   ' value.  fsub =', 1p, e15.5)
 2100 format(/ ' XXX  Linesearch has exceeded the Unbounded step size.',
     $   4x, ' step =', 1p, e13.3)
 2200 format(/ ' XXX  Unbounded in Phase', i3,
     $   '   Last SB =', i7, 1p, e13.3)

*     end of m7rgit
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m7sdir( mode, maxr, nr, ns, r, rg, p, w, z, nwcore )

      implicit           double precision (a-h,o-z)
      double precision   r(nr), rg(ns), p(ns), w(ns), z(nwcore)

*     ------------------------------------------------------------------
*     m7sdir  computes a search direction  p  for the superbasic
*     variables, using the current reduced gradient  rg.
*
*     mode       method
*       0     steepest descent:          p = - rg
*       1     quasi-Newton:       R(t)*R p = - rg
*     ------------------------------------------------------------------

      parameter        ( one = 1.0d+0 )

      if (mode .eq. 0) then

*        Steepest descent  (used when infeasible).

         do 20 j = 1, ns
            p(j) = - rg(j)
   20    continue
      else

*        Quasi-Newton.  We must save  w  satisfying  R(t)*w = rg.

         call dcopy ( ns, rg, 1, p, 1 )
         call m6rsol( 2, maxr, nr, ns, r, p )
         call dcopy ( ns, p, 1, w, 1 )
         call m6rsol( 1, maxr, nr, ns, r, p )
         call dscal ( ns, (- one), p, 1 )
      end if

*     end of m7sdir
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m7sscv( m, maxr, maxs, ms, nr, ns, parhes, nxtphs,
     $                   fx, oldfx, theta, xsnorm,xsnrm1, ysnorm,ysnrm1,
     $                   kb, bbl, bbu, grd, r, rg, x )

      implicit           double precision (a-h,o-z)
      logical            parhes
      integer            kb(ms)
      double precision   bbl(ms), bbu(ms)
      double precision   grd(ms), r(nr), rg(maxs), x(ms)

*     ------------------------------------------------------------------
*     m7sscv  (subspace convergence)  decides whether or not
*     optimization should continue on the current subspace.
*     On exit,  nxtphs = 4  means it should,
*               nxtphs = 3  means it should not.
*
*     parhes  is false if this is a linear iteration or if there
*     is sufficient storage in  r  for the full projected Hessian.
*
*     parhes  is true  if  r  is just a partial Hessian for the
*     first  maxr  superbasic variables.  The superbasics are then
*     in two sets  Z1  (containing   nz1 = maxr       variables)
*             and  Z2  (containing   nz2 = ns - maxr  variables).
*
*     The null-space matrix is similarly partitioned as  Z = ( Z1  Z2 ).
*
*     The normal convergence test is first applied to  Z1.  If it looks
*     like we are near enough to an optimum in that restricted
*     subspace, we find the largest reduced gradient in  Z2  (index k2).
*     If this is less than  tolrg  we exit with  nxtphs = 3  (to ask for
*     price).  Otherwise we make room in  Z1  for the corresponding
*     variable by the moving the superbasic with the smallest reduced
*     gradient in  Z1  (index  k1)  to the end of  Z2.
*     ------------------------------------------------------------------

      logical            conv
      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m7conv/ etash,etarg,lvltol,nfail,conv(4)
      common    /m7phes/ rgmin1,rgnrm1,rgnrm2,jz1,jz2,labz,nfullz,mfullz
      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg

      intrinsic          abs
      parameter        ( one = 1.0d+0 )

      rgnrm1 = rgnorm
      if (parhes) then
         xsnorm = xsnrm1
         ysnorm = ysnrm1
         k1     = idamax( maxr, rg, 1 )
         rgnrm1 = abs( rg(k1) )
      end if

      deltax  =  theta * ysnorm
      deltaf  =  abs( fx - oldfx )
      conv(1) =  deltax  .le.  xtol(lvltol) * (one +  xsnorm  )
      conv(2) =  deltaf  .le.  ftol(lvltol) * (one + abs( fx ))
      conv(3) =  rgnrm1  .le.  tolrg
      conv(4) =  rgnrm1  .le.  0.1 * tolrg     .or.
     $           rgnrm1  .le.  gtol(2) * pinorm

      nxtphs  =  4
      if (conv(1).and.conv(2).and.conv(3) .or. conv(4)) nxtphs = 3
      if (.not. parhes ) go to 900
      if (nxtphs .eq. 4) go to 900

*     Swap the largest reduced gradient in  Z2  into the front of  Z2
*     and see if it is significantly large.

      call m6swap( m, maxr, nr, ns, ms, kb, bbl, bbu, grd, r, rg, x )
      k2     = maxr + 1
      rgnrm2 = abs( rg(k2) )
      if (ns .lt. maxs  .and.  rgnrm2 .le. tolrg) go to 900

*     Find the smallest component of  Z1(t)*g.

      rgmin1 = abs( rg(1) )
      k1     = 1
      do 200 k  = 1, maxr
         if (rgmin1 .ge. abs( rg(k) )) then
             rgmin1   =  abs( rg(k) )
             k1       =  k
         end if
  200 continue

      if (rgmin1 .lt. rgnrm2) then

*        Save the relevant values.

         nxtphs = 4
         nfullz = 0
         lastr  = maxr*(maxr + 1)/2
         ldiag1 = k1 * (k1   + 1)/2
         rdiag1 = r(ldiag1)
         rg1    = rg(k1)
         k      = m + k1
         jz1    = kb(k)
         jz2    = kb(m + k2)
         bl1    = bbl(k)
         bu1    = bbu(k)
         grd1   = grd(k)
         x1     = x(k)

*        Delete the k1-th variable from  Z1,  and shift the remaining
*        superbasics in  Z1  and  Z2  one place to the left.

         call m6rdel( m, maxr, nr, ns, ms,
     $                kb, bbl, bbu, grd, r, rg, x, k1, .true. )

*        Put the old k1-th superbasic in at the very end.

         lr      = lastr + (ns - maxr)
         r(lr)   = rdiag1
         rg(ns)  = rg1
         kb(ms)  = jz1
         bbl(ms) = bl1
         bbu(ms) = bu1
         grd(ms) = grd1
         x(ms)   = x1
      end if

  900 return

*     end of m7sscv
      end
************************************************************************
*
*     File  mi80ncon fortran.
*
*     m8ajac   m8augl   m8aug1   m8chkj   m8prtj   m8sclj
*     m8setj   m8viol
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m8ajac( gotjac, nncon, nnjac, njac,
     $                   ne, nka, a, ha, ka,
     $                   fcon, fcon2, gcon, gcon2, xn, y, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            gotjac
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne)
      double precision   fcon(nncon), fcon2(nncon),
     $                   gcon(njac ), gcon2(njac ),
     $                   xn  (nnjac), y(nncon), z(nwcore)

*     ------------------------------------------------------------------
*     m8ajac  stores the Jacobian into  A.
*     If  gotjac  is true, the Jacobian is already in  gcon.
*     ------------------------------------------------------------------

      common    /m5log1/ idebug,ierr,lprint
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)

      if ( .not.  gotjac ) then
         if (lderiv .lt. 2)
     $      call m6dmmy( njac, gcon )

         call m6fcon( 2, nncon, nnjac, njac, fcon, gcon,
     $                ne, nka, ha, ka,
     $                xn, z, nwcore )

         if (lderiv .lt. 2)
     $      call m6dcon( nncon, nnjac, njac,
     $                   ne, nka, ha, ka,
     $                   fcon, fcon2, gcon, gcon2, xn, y, z, nwcore )
         if (ierr .ne. 0) return
      end if

      l     = 0
      do 400 j = 1, nnjac
         k1    = ka(j)
         k2    = ka(j+1) - 1
         do 300 k = k1, k2
            ir    = ha(k)
            if (ir .gt. nncon) go to 400
            l     = l + 1
            a(k)  = gcon(l)
  300    continue
  400 continue

*     end of m8ajac
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m8augl( mode, m, n, nb, ns, inform,
     $                   ne, nka, a, ha, ka,
     $                   hs, bl, bu, xn, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hs(nb)
      integer            ka(nka)
      double precision   a(ne), bl(nb), bu(nb), xn(nb), z(nwcore)

*     ------------------------------------------------------------------
*     m8augl  is a front-end for m8aug1.  It is called by  misolv  and
*     m5solv  at various stages of the augmented Lagrangian algorithm.
*     11 Oct 1991: a, ha, ka etc added as parameters.
*     05 Mar 1992: mode 3 now makes nonlinear rows free for first major.
*     11 Sep 1992: hs added as parameter.
*     ------------------------------------------------------------------

      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m5loc / lpi   ,lpi2  ,lw    ,lw2   ,
     $                   lx    ,lx2   ,ly    ,ly2   ,
     $                   lgsub ,lgsub2,lgrd  ,lgrd2 ,
     $                   lr    ,lrg   ,lrg2  ,lxn
      common    /m8len / njac  ,nncon ,nncon0,nnjac
      common    /m8loc / lfcon ,lfcon2,lfdif ,lfdif2,lfold ,
     $                   lblslk,lbuslk,lxlam ,lrhs  ,
     $                   lgcon ,lgcon2,lxdif ,lxold

      ms     = m + ns
      call       m8aug1( mode, ms, nncon, nnjac, njac, n, nb, inform,
     $                   ne, nka, a, ha, ka,
     $                   hs, z(lkb), bl, bu, z(lbbl), z(lbbu),
     $                   z(lblslk), z(lbuslk),
     $                   z(lgcon ), z(lgcon2), z(lxlam), xn )

*     end of m8augl
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m8aug1( mode, ms, nncon, nnjac, njac, n, nb, inform,
     $                   ne, nka, a, ha, ka,
     $                   hs, kb, bl, bu, bbl, bbu, blslk, buslk,
     $                   gcon, gcon2, xlam, xn )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hs(nb)
      integer            ka(nka), kb(ms)
      double precision   a(ne), bl(nb), bu(nb), bbl(ms), bbu(ms),
     $                   blslk(nncon), buslk(nncon),
     $                   gcon (njac ), gcon2(njac ),
     $                   xlam (nncon), xn(nb)

*     ------------------------------------------------------------------
*     m8aug1  does the work for  m8augl.
*     11 Sep 1992: hs added as parameter for mode 5, to allow
*                  nonbasic slacks to be moved onto the relaxed bounds.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /m8al2 / radius,rhsmod,modpen,modrhs

      intrinsic          max, min
      parameter        ( zero = 0.0d+0 )

      inform = 0
      bplus  = 0.9*plinfy
      bminus = - bplus

      if (mode .eq. 1) then
*        ---------------------------------------------------------------
*        Save the Jacobian from the initial a(*) in gcon, gcon2.
*        ---------------------------------------------------------------
         l      = 0
         do 140 j = 1, nnjac
            k1    = ka(j)
            k2    = ka(j+1) - 1
            do 120 k = k1, k2
               ir       = ha(k)
               if (ir .gt. nncon) go to 140
               l        = l + 1
               gcon (l) = a(k)
               gcon2(l) = a(k)
  120       continue
  140    continue

      else if (mode .eq. 2) then
*        ---------------------------------------------------------------
*        Make sure the initial xn values are feasible.
*        Do the slacks too, in case the RHS has been perturbed.
*        ---------------------------------------------------------------
         do 220 j = 1, nb
            xn(j) = max( xn(j), bl(j) )
            xn(j) = min( xn(j), bu(j) )
  220    continue

      else if (mode .eq. 3) then
*        ---------------------------------------------------------------
*        Start of the first major iteration.
*        1. Initialize  modpen, rhsmod.
*        2. Save the bounds on the nonlinear rows,
*        3. Make the nonlinear rows free (relax the bounds to infinity).
*        ---------------------------------------------------------------
         modpen = 0
         rhsmod = zero
         call dcopy ( nncon, bl(n+1), 1, blslk, 1 )
         call dcopy ( nncon, bu(n+1), 1, buslk, 1 )

         do 320 i = 1, nncon
            bl(n+i) = - plinfy
            bu(n+i) =   plinfy
  320    continue

      else if (mode .eq. 4) then
*        ---------------------------------------------------------------
*        Unbounded subproblem.
*        Increase the penalty parameter.
*        ---------------------------------------------------------------
         modpen = modpen + 1
         if (modpen .gt. 5  .or.  nlag .eq. 0) then
            inform = 1
         else
            t      = nncon
            if (penpar .le. zero) penpar = t / 100.0
            penpar = 10.0 * penpar
            if (iprint .gt. 0) write(iprint, 1400) penpar
            if (isumm  .gt. 0) write(isumm , 1400) penpar
         end if

      else if (mode .eq. 5) then
*        ---------------------------------------------------------------
*        Infeasible subproblem.
*        Relax the bounds on the linearized constraints.
*        Also, move nonbasic slacks onto the new bounds
*        so there won't be large numbers of them floating in between.
*        ---------------------------------------------------------------
         modrhs = modrhs + 1
         if (modrhs .gt. 5) then
            inform = 1
         else
            if (modrhs .eq. 1) rhsmod = tolx
            t      = sinf / nncon
            rhsmod = 10.0 * max( rhsmod, t )
            if (iprint .gt. 0) write(iprint, 1500) rhsmod
            if (isumm  .gt. 0) write(isumm , 1500) rhsmod

            do 520 i = 1, nncon
               j     = n + i
               if (bl(j) .gt. bminus) then
                   bl(j) = blslk(i) - rhsmod
                   if (hs(j) .eq. 0) xn(j) = bl(j)
               end if
               if (bu(j) .lt. bplus ) then
                   bu(j) = buslk(i) + rhsmod
                   if (hs(j) .eq. 1) xn(j) = bu(j)
               end if
  520       continue

            do 540 k  = 1, ms
               j      = kb(k)
               bbl(k) = bl(j)
               bbu(k) = bu(j)
  540       continue
         end if

      else if (mode .eq. 6) then
*        ---------------------------------------------------------------
*        Restore the bounds on the nonlinear constraints.
*        ---------------------------------------------------------------
         do 620 i = 1, nncon
            jslack     = n + i
            bl(jslack) = blslk(i)
            bu(jslack) = buslk(i)
  620    continue
      end if
      return

 1400 format(' Penalty parameter increased to', 1p, e12.2)
 1500 format(' XXX  Infeasible subproblem.  ',
     $   ' RHS perturbation increased to', 1p, e12.2)

*     end of m8aug1
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m8chkj( m, n, njac, nx,
     $                   ne, nka, ha, ka,
     $                   bl, bu, f, f2, g, g2,
     $                   x, y, y2, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   bl(n), bu(n), f(m), f2(m), g(njac), g2(njac),
     $                   x(n), y(n), y2(nx), z(nwcore)

*     ------------------------------------------------------------------
*     m8chkj  verifies the Jacobian  gcon  using
*     finite differences on the constraint functions  f.
*     The various verify levels are described in  m7chkg.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m3scal/ sclobj,scltol,lscale
      common    /m5log1/ idebug,ierr,lprint
      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8veri/ jverif(4),lverif(2)

      intrinsic          abs, max, min
      parameter        ( zero = 0.0d+0,  one = 1.0d+0,  ok = 0.1d+0 )

      logical            cheap, first
      character*4        key
      character*4        lbad         , lgood
      data               lbad /'bad?'/, lgood /'ok  '/

      lvl    = lverif(1)
      if (lvl .lt. 0) return
      j1     = max( jverif(3), 1 )
      j2     = min( jverif(4), n )
      cheap  = lvl .le. 1  .or.  j1 .gt. j2
      lssave = lscale
      lscale = 0

*     Evaluate f and g at the base point  x.

      if (lderiv .le. 1) call m6dmmy( njac, g )
      call m6fcon( 2, m, n, njac, f, g,
     $             ne, nka, ha, ka,
     $             x, z, nwcore )
      if (ierr      .ne. 0) go to 900
      if (knowng(2) .eq. 0) go to 900

      if (iprint .gt. 0) then
         if (cheap) then
            write(iprint, 1100)
         else
            write(iprint, 1000)
         end if
      end if

*     --------------------------
*     Cheap test.
*     --------------------------

*     Generate a direction in which to perturb  x.

      yj    = one/n
      do 10 j  =  1, n
         y(j)  =   yj
         y2(j) =   yj
         yj    = - yj * 0.99999
   10 continue

*     Define a difference interval.
*     If needed, alter y to ensure that it will be a feasible direction.
*     If this gives zero, go back to original y and forget feasibility.

      dx     = difint(1) * (one  +  dnorm1( n, x, 1 ))
      call m7chkd( n, bl, bu, x, dx, y, nfeas )
      if (nfeas .eq. 0) call dcopy ( n, y2, 1, y, 1 )

*     ------------------------------------------------------------------
*     We must not perturb x(j) if the j-th column of the Jacobian
*     contains any unknown elements.
*     ------------------------------------------------------------------
      nder   = 0
      l      = 0
      do 30 j = 1, n
         k1   = ka(j)
         k2   = ka(j+1) - 1

         do 20 k = k1, k2
            ir   = ha(k)
            if (ir .gt. m) go to 25
            l    = l + 1
            if (g(l) .eq. gdummy) y(j) = zero
   20    continue

   25    if (y(j) .ne. zero) nder = nder + 1
   30 continue

      if (nder .eq. 0) go to 900

*     Set f2 = constraint values at a short step along  y.

      do 40 j  = 1, n
         y2(j) = x(j) + dx*y(j)
   40 continue

      call m6fcon( 0, m, n, njac, f2, g2,
     $             ne, nka, ha, ka,
     $             y2, z, nwcore )
      if (ierr .ne. 0) go to 900

*     Set   y2  =  (f2 - f)/dx  -  Jacobian*y.  This should be small.
*     At the same time, find the first Jacobian element in column j1.

      do 60 i  = 1, m
         y2(i) = (f2(i) - f(i)) / dx
   60 continue

      l      = 0
      lsave  = 0
      do 100 j = 1, n
         yj    = y(j)
         k1    = ka(j)
         k2    = ka(j+1) - 1
         do 80 k = k1, k2
            ir   = ha(k)
            if (ir .gt. m) go to 100
            l    = l + 1
            if (j .lt. j1) lsave = l
            y2(ir) = y2(ir) - g(l)*yj
   80    continue
  100 continue

      imax   = idamax( m,y2,1 )
      gmax   = (f2(imax) - f(imax)) / dx
      emax   = abs( y2(imax) ) / (one + abs( gmax ))
      if (emax .le. ok) then
         if (iprint .gt. 0) write(iprint, 1400)
      else
         if (iprint .gt. 0) write(iprint, 1500)
         if (isumm  .gt. 0) write(isumm , 1500)
      end if
      if (iprint .gt. 0) write(iprint, 1600) emax, imax
      if (cheap) go to 900

*     ----------------------------------------------------
*     Proceed with the verification of columns j1 thru j2.
*     ----------------------------------------------------
      if (iprint .gt. 0) write(iprint, 2000)
      l      = lsave
      nwrong = 0
      ngood  = 0
      emax   = - one
      jmax   = 0

      do 200 j = j1, j2

*        See if there are any known gradients in this column.

         k1    = ka(j)
         k2    = ka(j+1) - 1
         lsave = l

         do 120 k = k1, k2
            ir    = ha(k)
            if (ir .gt. m) go to 200
            l     = l + 1
            if (g(l) .ne. gdummy) go to 140
  120    continue
         go to 200

*        Found one.

  140    xj     = x(j)
         dx     = difint(1) * (one + abs( xj ))
         if (bl(j) .lt. bu(j)  .and.  xj .ge. bu(j)) dx = -dx
         x(j)   = xj + dx
         call m6fcon( 0, m, n, njac, f2, g2,
     $                ne, nka, ha, ka,
     $                x, z, nwcore )
         if (ierr .ne. 0) go to 900

*        Check nonzeros in the jth column of the Jacobian.
*        Don't bother printing a line if it looks like an exact zero.

         l     = lsave
         first = .true.

         do 160 k = k1, k2
            ir    = ha(k)
            if (ir .gt. m) go to 180
            l     = l + 1
            gi    = g(l)
            if (gi .eq. gdummy) go to 160
            gdiff = (f2(ir) - f(ir))/dx
            err   = abs( gdiff - gi ) / (one + abs( gi ))

            if (emax .lt. err) then
               emax  = err
               imax  = ir
               jmax  = j
            end if

            key   = lgood
            if (err .gt. eps5) key = lbad
            if (key .eq. lbad) nwrong = nwrong + 1
            if (key .eq.lgood) ngood  = ngood  + 1
            if (abs( gi ) + err  .le.  eps0) go to 160
            if (iprint .gt. 0) then
               if (first) then
                  write(iprint, 2100) j,xj,dx,l,ir,gi,gdiff,key
               else
                  write(iprint, 2200)         l,ir,gi,gdiff,key
               end if
            end if
            first = .false.
  160    continue

  180    x(j)  = xj
  200 continue

      if (iprint .gt. 0) then
         if (nwrong .eq. 0) then
            write(iprint, 2500) ngood ,j1,j2
         else
            write(iprint, 2600) nwrong,j1,j2
         end if
         write(iprint, 2700) emax,imax,jmax
      end if

      if (emax .ge. one) then

*        Bad gradients in  funcon.

         ierr   = 8
         call m1envt( 1 )
         if (iprint .gt. 0) write(iprint, 3800)
         if (isumm  .gt. 0) write(isumm , 3800)
      end if

*     Exit.

  900 lscale = lssave
      return

 1000 format(/// ' Verification of constraint gradients',
     $   ' returned by subroutine funcon.')
 1100 format(/ ' Cheap test on funcon...')
 1400 format(  ' The Jacobian seems to be OK.')
 1500 format(  ' XXX  The Jacobian seems to be incorrect.')
 1600 format(  ' The largest discrepancy was', 1p, e12.2,
     $   '  in constraint', i6)
 2000 format(// ' Column       x(j)        dx(j)', 3x,
     $   ' Element no.    Row    Jacobian value    Difference approxn')
 2100 format(/ i7, 1p, e16.8, e10.2, 2i10, 2e18.8, 2x, a4)
 2200 format(           33x, 2i10, 1pe18.8, e18.8, 2x, a4)
 2500 format(/ i7, '  Jacobian elements in cols ', i6, '  thru', i6,
     $         '  seem to be OK.')
 2600 format(/ ' XXX  There seem to be', i6,
     $   '  incorrect Jacobian elements in cols', i6, '  thru', i6)
 2700 format(/ ' XXX  The largest relative error was', 1p, e12.2,
     $   '   in row', i6, ',  column', i6 /)
 3800 format(// ' EXIT -- subroutine funcon appears to be',
     $   ' giving incorrect gradients')

*     end of m8chkj
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m8prtj( n, nb, nncon, nnjac, lprint, majits, nlag,nscl,
     $                   ne, nka, a, ha, ka, hs,
     $                   ascale, bl, bu, fcon, xlam, xn )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne), hs(nnjac)
      integer            ka(nka)
      double precision   a(ne), ascale(nscl), bl(nb), bu(nb),
     $                   fcon(nncon), xlam(nncon), xn(nb)

*     ------------------------------------------------------------------
*     m8prtj  prints x, lambda, f(x) and/or the Jacobian
*     at the start of each major iteration.
*
*     08 Apr 1992: Internal values of hs(*) allow more values for key.
*     ------------------------------------------------------------------

      common    /m1file/ iread,iprint,isumm
      common    /m3scal/ sclobj,scltol,lscale

      intrinsic          min, mod

      logical            prtx, prtl, prtf, prtj, scaled
      character*2        key(-1:4)
      data               key/'FR', 'LO', 'UP', 'SB', 'BS', 'FX'/

      if (iprint .le. 0) return

*     Unscale everything if necessary.

      scaled = lscale .ge. 2
      if (scaled) call m2scla( 2, nncon, n, nb, ne, nka,
     $                         ha, ka, a, ascale, bl, bu, xlam, xn )
      if (scaled) call ddscl ( nncon, ascale(n+1), 1, fcon, 1 )

      nxn    = min( nnjac, 5 )
      nlam   = min( nncon, 5 )
      l      = lprint/10
      prtx   = mod( l,10 ) .gt. 0
      l      = l/10
      prtl   = mod( l,10 ) .gt. 0  .and.  majits .gt. 1
      l      = l/10
      prtf   = mod( l,10 ) .gt. 0
      l      = l/10
      prtj   = mod( l,10 ) .gt. 0

      if ( prtx ) write(iprint, 1100) (xn(j), j=1,nnjac)
      if ( prtl ) write(iprint, 1200) (xlam(i), i=1,nncon)
      if ( prtf ) write(iprint, 1300) (fcon(i), i=1,nncon)
      if ( prtj ) then
         write(iprint, 1400)
         do 100 j = 1, nnjac
            k1    = ka(j)
            k2    = ka(j+1) - 1
            do 50 k = k1, k2
               ir   = ha(k)
               if (ir .gt. nncon) go to 60
   50       continue
            k    = k2 + 1
   60       k2   = k  - 1
            l    = hs(j)
            write(iprint, 1410) j,xn(j),key(l), (ha(k),a(k), k=k1,k2)
  100    continue
      end if
   
*     Rescale if necessary.
         
      if (scaled) call m2scla( 1, nncon, n, nb, ne, nka,
     $                         ha, ka, a, ascale, bl, bu, xlam, xn )
      if (scaled) call dddiv ( nncon, ascale(n+1), 1, fcon, 1 )
      return

 1100 format(/ ' Jacobian variables'
     $       / ' ------------------'   / 1p, (5e16.7))
 1200 format(/ ' Multiplier estimates'
     $       / ' --------------------' / 1p, (5e16.7))
 1300 format(/ ' Constraint functions'
     $       / ' --------------------' / 1p, (5e16.7))
 1400 format(/ ' x  and  Jacobian' / ' ----------------')
 1410 format(i6, 1p, e13.5, 1x, a2, 1x, 4(i9, e13.5)
     $       / (22x, 4(i9, e13.5)))

*     end of m8prtj
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m8sclj( mode, nncon, nn, ng, n, nb, ne, nka,
     $                   ascale, ha, ka, g )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   ascale(nb), g(ng)

*     ------------------------------------------------------------------
*     If mode = 1,  m8sclj  scales the objective gradient.
*     If mode = 2,  m8sclj  scales the Jacobian.
*     nn, ng, g  are  nnobj, nnobj, gobj  or  nnjac, njac, gcon  resp.
*
*     m8sclj is called by m6fobj and m6fcon only if modefg = 2.
*     Hence, it is used to scale known gradient elements (if any),
*     but is not called when missing gradients are being estimated
*     by m6dobj or m6dcon.
*     ------------------------------------------------------------------

      common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)

      if (knowng(mode) .eq. 0) return

      if (mode .eq. 1) then
*        ---------------------------------------------------------------
*        Scale known objective gradients.
*        ---------------------------------------------------------------
         do 100 j = 1, nn
            grad  = g(j)
            if (grad .ne. gdummy) g(j) = grad * ascale(j)
  100    continue

      else
*        ---------------------------------------------------------------
*        Scale known Jacobian elements.
*        ---------------------------------------------------------------
         l     = 0

         do 300 j  = 1, nn
            k1     = ka(j)
            k2     = ka(j+1) - 1
            cscale = ascale(j)

            do 250 k = k1, k2
               ir    = ha(k)
               if (ir .gt. nncon) go to 300
               l     = l + 1
               grad  = g(l)
               if (grad .ne. gdummy)
     $         g(l)  = grad * cscale / ascale(n + ir)
  250       continue
  300    continue
      end if

*     end of m8sclj
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m8setj( lcstat, m, n, nb, nn, nncon, nnjac, njac, nscl,
     $                   lcrash, ns,
     $                   nconvg, nomove, nreduc, optsub, convgd,
     $                   ne, nka, a, ha, ka, hs,
     $                   ascale, bl, bu,
     $                   blslk, buslk, fcon, fcon2,
     $                   fdif , fold , gcon, gcon2, xlam,
     $                   rhs  , xdif , xold, pi, xn, y, z, nwcore )

      implicit           double precision (a-h,o-z)
      logical            optsub, convgd
      integer*4          ha(ne), hs(nb)
      integer            ka(nka)
      double precision   a(ne), ascale(nscl), bl(nb), bu(nb)
      double precision   blslk(nncon), buslk(nncon),
     $                   fcon (nncon), fcon2(nncon),
     $                   fdif (nncon), fold (nncon),
     $                   gcon (njac ), gcon2(njac ),
     $                   xlam (nncon), rhs  (nncon), xdif(nn), xold(nn),
     $                   pi(m), xn(nb), y(m), z(nwcore)

*     ------------------------------------------------------------------
*     m8setj  sets up the linearly constrained subproblem for the
*     next major iteration of the augmented Lagrangian algorithm.
*
*     lcrash is an input parameter.  If lcrash = 5, the nonlinear
*     constraints have been relaxed so far.  On major itn 2,
*     Crash is called to find a basis including them.
*
*     nconvg  counts no. of times optimality test has been satisfied.
*     nreduc  counts no. of times  penpar  has been reduced.
*     optsub  (input ) says if the previous subproblem was optimal.
*     convgd  (output) says if it is time to stop.
*
*     06 Mar 1992: The first major iteration now relaxes nonlinear rows
*                  and terminates as soon as the linear constraints are
*                  satisfied.  xold is not defined until Major 2.
*     08 Apr 1992: Internal values of hs(*) simplify setting xlam2.
*     22 May 1992: penpar reduced for more majors before setting to zero.
*                  Stops Steinke2 (and maybe OTIS?) from suddenly
*                  getting a huge x and lambda.
*     04 Jun 1992: lcrash added.
*     03 Sep 1992: Penalty parameter is now a multiple of the
*                  default value 100/nncon.
*     ------------------------------------------------------------------

      common    /m1eps / eps,eps0,eps1,eps2,eps3,eps4,eps5,plinfy
      common    /m1file/ iread,iprint,isumm
      common    /m2lu3 / lenl,lenu,ncp,lrow,lcol
      common    /m2parm/ dparm(30),iparm(30)
      common    /m3loc / lascal,lbl   ,lbu   ,lbbl  ,lbbu  ,
     $                   lhrtyp,lhs   ,lkb
      common    /m5lobj/ sinf,wtobj,minimz,ninf,iobj,jobj,kobj
      common    /m5log1/ idebug,ierr,lprint
      common    /m5log2/ jq1,jq2,jr1,jr2,lines1,lines2
      logical            prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5log4/ prnt0 ,prnt1 ,summ0 ,summ1 ,newhed
      common    /m5lp1 / itn,itnlim,nphs,kmodlu,kmodpi
      common    /m7tols/ xtol(2),ftol(2),gtol(2),pinorm,rgnorm,tolrg
      common    /m8al1 / penpar,rowtol,ncom,nden,nlag,nmajor,nminor
      common    /m8al2 / radius,rhsmod,modpen,modrhs
*-    common    /m8diff/ difint(2),gdummy,lderiv,lvldif,knowng(2)
      common    /m8func/ nfcon(4),nfobj(4),nprob,nstat1,nstat2
      common    /m8save/ vimax ,virel ,maxvi ,majits,minits,nssave
      common    /cycle2/ objtru,suminf,numinf

      intrinsic          abs, max, min

      parameter        ( zero   = 0.0d+0,  one    = 1.0d+0,
     $                   biglam = 1.0d+5,  dmaxlm = 1.0d+8 )

      logical            gotjac, feasbl, major1, major2, phead , zerlam
      double precision   objold
      save               objold

      character*8        line
      character*1        lstate(4), label
      data               line   /'--------'/
      data               lstate /' ','I','U','T'/

      rhsmod = zero
      modrhs = 0
      bplus  = 0.9*plinfy
      bminus = - bplus
      dlmax  = zero
      dlrel  = zero
      dxrel  = zero
      step   = one
      feasbl = ninf .eq. 0
      zerlam = .true.
      major1 = majits .eq. 1
      major2 = majits .eq. 2

*     Output heading for terse log.

      phead  = newhed .or.
     $         majits .gt. 2  .and.  mod( majits, 20 ) .eq. 2
      if (prnt0  .and.  phead) write(iprint, 1100)
      if (              phead) newhed = .false.
      phead  = major1 .or.
     $         majits .gt. 2  .and.  mod( majits, 10 ) .eq. 2
      if (summ0  .and.  phead) write(isumm , 1110)

*     Output heading for detailed log.

      if (prnt1  .and.  .not. major1) call m1page( 0 )
      if (prnt1) write(iprint, 2010) (line, j=1,16), majits, penpar
      if (summ1) write(isumm , 2020) (line, j=1, 8), majits, penpar

*     ------------------------------------------------------------------
*     Beginning of first major iteration.
*     misolv has already called m8augl( 2,... ) to make sure that
*     all nonlinear variables are inside their bounds.
*     ------------------------------------------------------------------
      if (major1) then

*        For interest, get the initial constraint violation.

         call m8viol( n, nb, nncon,
     $                ne, nka, a, ha, ka,
     $                bl, bu, fcon, xn, y, z, nwcore )

*        Save the nonlinear constraint bounds and relax them.
*        (The first major will just satisfy the linear constraints.)
*        Then set internal values for hs.

         call m8augl( 3, m, n, nb, ns, inform,
     $                ne, nka, a, ha, ka,
     $                hs, bl, bu, xn, z, nwcore )
         call m5hs  ( 'Internal', nb, bl, bu, hs, xn )

*        xlam has been initialized from pi in m4getb.
*        Initialize xold and fold, so they are defined when m5frmc
*        evaluates the augmented Lagrangian at the first feasible point.

         call dcopy ( nn   , xn  , 1, xold, 1 )
         call dcopy ( nncon, fcon, 1, fold, 1 )
         go to 500
      end if

*     ------------------------------------------------------------------
*     Beginning of later major iterations.
*     ------------------------------------------------------------------

*     Restore the bounds on the nonlinear constraints.
      
      call m8augl( 6, m, n, nb, ns, inform,
     $             ne, nka, a, ha, ka,
     $             hs, bl, bu, xn, z, nwcore )
      
*     Make sure all variables are inside their bounds.
*     Then set internal values for hs.
      
      call m8augl( 2, m, n, nb, ns, inform,
     $             ne, nka, a, ha, ka,
     $             hs, bl, bu, xn, z, nwcore )
      call m5hs  ( 'Internal', nb, bl, bu, hs, xn )

      if (major2) then

*        Initialize xold.
*        xlam still contains pi from m4getb.  Check it is OK.
*        fold is set later.

         call dcopy ( nn, xn, 1, xold, 1 )
         step   = one

         do 120 i = 1, nncon
            j     = n + i
            js    = hs(j)
            py    = xlam(i)
            xlam2 = py
            if (js .eq. 0  .and.  py .gt. zero) xlam2 = zero
            if (js .eq. 1  .and.  py .lt. zero) xlam2 = zero
            xlam2 = min( xlam2,    dmaxlm  )
            xlam2 = max( xlam2, (- dmaxlm) )
            xlam(i) = xlam2
  120    continue

         go to 400
      end if

*     ------------------------------------------------------------------
*     Major 3 and later.
*     If xn is feasible, reset the multiplier estimates.
*     Since the previous subproblem could have been stopped early,
*     the pi's for inequality constraints must be checked.
*     If constraint i is active and pi(i) has the wrong sign,
*     we just use zero.
*     22 May 1992: If the subproblem is optimal and a slack is
*                  superbasic, the pi(i) should again be zero.
*     ------------------------------------------------------------------
      if ( feasbl ) then
         do 150 i = 1, nncon
            j     = n + i
            js    = hs(j)
            py    = pi(i)
            xlam2 = py
            if (js .eq. 0  .and.  py .gt. zero) xlam2 = zero
            if (js .eq. 1  .and.  py .lt. zero) xlam2 = zero
            if (js .eq. 2  .and.     optsub   ) xlam2 = zero
            xlam2 = min( xlam2,    dmaxlm  )
            xlam2 = max( xlam2, (- dmaxlm) )
            pi(i) = xlam2
            y(i)  = xlam2 - xlam(i)
  150    continue
      
*        Find the relative change in  xlam.
*        If xlam is very small, it is probably still zero (never set).
*        We treat it specially to avoid taking a small step below.
      
         imax   = idamax( nncon, y   , 1 )
         dlmax  = abs   ( y(imax) )
         dlnorm = dasum ( nncon, y   , 1 )
         xlnorm = dasum ( nncon, xlam, 1 )
         zerlam = xlnorm .le. eps0
         dlrel  = dlnorm / (one + xlnorm)
      end if

*     ------------------------------------------------------------------
*     Find the relative change in the Jacobian variables.
*     Note that xdif and xold allow for all nonlinear variables,
*     in case "step" below turns out not to be one.
*     ------------------------------------------------------------------
      do 220  j  = 1, nn
         xdif(j) = xn(j) - xold(j)
  220 continue

      imax   = idamax( nnjac, xdif, 1 )
      dxmax  = abs   ( xdif(imax) )
      dxrel  = dasum ( nnjac, xdif, 1 ) / (one + dasum ( nnjac,xold,1 ))
      if (prnt1) write(iprint, 2400) dxmax, dxrel, dlmax, dlrel
      if (summ1) write(isumm , 2410) dxmax, dlmax

*     ------------------------------------------------------------------
*     Determine a step to be used as follows:
*        xlam  =  xlam + step*y,    xn  =  xold + step*xdif.
*
*     damp = 2.0 (for example) prevents  x  or  lambda  from changing
*     by more than 200 per cent.  This is an exceedingly crude attempt
*     to avoid trouble when the new subproblem solution  xn  is wildly
*     different from the previous solution  xold.
*
*     Note that the current active set is retained even if  step  ends
*     up being less than 1.0.  Some nonbasic variables may lie strictly
*     between their bounds (which is ok now that xn retains all values).
*     ------------------------------------------------------------------
      damp   = dparm(4)
      step   = one
      if (.not. zerlam) then
        step = min( step, damp/(dlrel + eps) )
      end if
      step   = min( step, damp/(dxrel + eps) )

      if (step .ge. 0.99) then
*        =====================
*        A unit step looks ok.
*        =====================
         step   = one
         if (feasbl) call dcopy ( nncon, pi, 1, xlam, 1 )
         call dcopy ( nn, xn, 1, xold, 1 )
      
*        see if  xn  is the same as  xold.
      
         if (dxrel .le. eps0) then
            nomove = nomove + 1
            if (prnt1) write(iprint, 2100)
            if (summ1) write(isumm , 2100)
            go to 450
         end if
      else
*        ====================
*        Take a shorter step.
*        ====================
         if (feasbl) call daxpy ( nncon, step, y, 1, xlam, 1 )
         call daxpy ( nn, step, xdif, 1, xold, 1 )
         call dcopy ( nn, xold, 1, xn, 1 )
         
         if (prnt1) write(iprint, 2500) step
         if (summ1) write(isumm , 2500) step
      end if

*     ------------------------------------------------------------------
*     Evaluate the nonlinear constraints and the Jacobian
*     and copy the Jacobian into  A  to form the next linearization.
*     ------------------------------------------------------------------
  400 nomove = 0
      gotjac = feasbl  .and.  nlag .eq. 1  .and.  step .eq. one

      call m8ajac( gotjac, nncon, nnjac, njac,
     $             ne, nka, a, ha, ka,
     $             fcon, fcon2, gcon, gcon2, xn, y, z, nwcore )
      if (ierr .ne. 0) go to 900

*     Save the function values in  fold.

      call dcopy ( nncon, fcon, 1, fold, 1 )

*     Find how well the new xn satisfies the nonlinear constraints.

  450 call m8viol( n, nb, nncon,
     $             ne, nka, a, ha, ka,
     $             bl, bu, fcon, xn, y, z, nwcore )

*     ------------------------------------------------------------------
*     Print a line.
*     ------------------------------------------------------------------
  500 label  = lstate(lcstat + 1)
      lu     = lenl + lenu
      if (prnt0) write(iprint, 1200) majits - 1, minits, label, itn,
     $           ninf, objtru, vimax, rgnorm, nssave,
     $           nfcon(1), dxrel, dlrel, step, lu, penpar
      if (summ0) write(isumm , 1210) majits - 1, minits, label,
     $           objtru, vimax, rgnorm, nssave,
     $           nfcon(1), dxrel, dlrel, step

      if (prnt1 .and. (major1 .or. major2)) write(iprint, 1000)
      if (prnt1) write(iprint, 2600) vimax, virel
      if (summ1) write(isumm , 2700) vimax

*     ------------------------------------------------------------------
*     Compute  rhs  =  fcon  -  Jacobian*xn.
*     This is minus the required  rhs  for the new subproblem.
*     ------------------------------------------------------------------
      if (lprint .gt. 1)
     $call m8prtj( n, nb, nncon, nnjac, lprint, majits, nlag, nscl,
     $             ne, nka, a, ha, ka, hs,
     $             ascale, bl, bu, fcon, xlam, xn )

      call dcopy ( nncon, fcon, 1, rhs, 1 )
      call m2aprd( 7, xn, nnjac, rhs, nncon,
     $             ne, nka, a, ha, ka,
     $             z, nwcore )
      call dscal ( nncon, (- one), rhs, 1 )

*     ------------------------------------------------------------------
*     Do Crash on nonlinear rows at start of Major 2
*     unless a basis is known already.
*     ------------------------------------------------------------------
      if (major2  .and.  lcrash .eq. 5) then

*        m8viol has set y to be the exact slacks on nonlinear rows.
*        Copy these into xn(n+i) and make sure they are feasible.
*        m2crsh uses them to decide which slacks to grab for the basis.

         do 550 i = 1, nncon
            j     = n + i
            xn(j) = max(  y(i), bl(j) )
            xn(j) = min( xn(j), bu(j) )
  550    continue

*        Set row types in hrtype.
*        m2crsh uses kb as workspace.  It may alter xn(n+i) for
*        nonlinear slacks.
*        Set internal values for hs again to match xn.

         call m2amat( 2, m, n, nb,
     $                ne, nka, a, ha, ka,
     $                bl, bu, z(lhrtyp) )
         call m2crsh( lcrash, m     , n     , nb    , nn,
     $                ne    , nka   , a     , ha    , ka,
     $                z(lkb), hs    , z(lhrtyp), bl , bu,
     $                xn    , z     , nwcore )
         lcrash = 0
         call m5hs  ( 'Internal', nb, bl, bu, hs, xn )
      end if

*     ------------------------------------------------------------------
*     Test for optimality.
*     For safety, we require the convergence test to be satisfied
*     for  2  subproblems in a row.
*     ------------------------------------------------------------------
      if (major1  .or.  major2) then
         objold = objtru
         convgd = .false.
         nconvg = 0
      else
         dfrel  = abs( objtru - objold ) / (one + objold)
         objold = objtru

         convgd = dxrel .le. 0.1      .and.   dlrel .le. 0.1  .and.
     $            virel .le. rowtol   .and.   optsub          .and.
     $            dfrel .le. 0.001
         nconvg = nconvg + 1
         if (.not. convgd) nconvg = 0
         convgd = nconvg .ge. 2

*        Change from Partial Completion to Full Completion if
*        the iterations seem to be converging.
*        (This may be begging the question!  More theory needed.)

         if (ncom .ne. 1) then      
            if (dxrel .le. 0.1  .and.  dlrel .le. 0.1  .and.
     $          virel .le. 0.1  .and.  optsub        ) then
               ncom = 1
               if (summ1) write(isumm , 2800)
               if (prnt1) write(iprint, 2800)
            end if
         end if
      
*        =============================
*        Adjust the penalty parameter.
*        =============================
         if (feasbl .and. optsub) then
      
            if (virel  .le. radius  .and.  dlrel .le. radius) then
               if (penpar .gt. zero) then
                  nreduc = nreduc + 1
                  penpar = penpar / 10.0

*                 The next line caused trouble on Steinke2
*                 (suddenly a huge search direction).
*                 For safety we should always reduce penpar gradually.
*------>          if (nconvg .ge. 1  .or.  nreduc .ge. 5) penpar = zero

                  if (penpar .le. 1.0d-6  .or.  nreduc .ge. 10) then
                     penpar = zero
                  end if
               end if
      
            else if (majits .ge. 5  .and.  dlrel .ge. 10.0) then
               nreduc = 0
               oldpen = penpar
               if (penpar .le. zero) penpar = 1.0d+0
*-04 Sep 1992: Raising the penalty parameter (below) seems to cause
*-             more trouble than it fixes.  Give it up for now.
*-             fac    = 1.5
*-             if (dlrel .ge. biglam) fac = 2.0
*-             penpar = penpar * fac
*-             penmax = 1.0d+3
*-             penpar = min( penpar, penmax )
               if (iprint .gt. 0  .and.  penpar .gt. oldpen) then
                  write(iprint, 2900) penpar
               end if
            end if
         end if
      end if

*     ------------------------------------------------------------------
*     Exit.
*     ------------------------------------------------------------------
  900 return

 1000 format(' ')
 1100 format(/ ' Major minor   total ninf  sinf,objective    viol',
     $         '    rg    nsb   ncon xchange lchange    step',
     $         '     LU penalty')
 1110 format(/ ' Major minor   sinf,objective    viol',
     $         '    rg    nsb   ncon xchange lchange    step')
 1200 format(1p, 2i6, a1, i7, i5, e16.8, 2e8.1, i5, i7, 3e8.1, i7, e8.1)
 1210 format(1p, 2i6, a1,         e16.8, 2e8.1, i5, i7, 3e8.1)
 2010 format(  1x, 16a8 / ' Start of major itn', i4,
     $   10x, ' Penalty parameter =', 1p, e12.2)
 2020 format(/ 1x,  8a8 / ' Start of major itn', i4,
     $   10x, ' Penalty parameter =', 1p, e12.2)
 2100 format(/ ' Jacobian variables have not changed --',
     $         ' old linearization kept')
 2400 format(/ ' Maximum change in Jacobian vars =', 1p, e12.4,
     $     4x,  ' ( =', e11.4, ' normalized)'
     $       / ' Maximum change in multipliers   =',   e12.4,
     $     4x,  ' ( =', e11.4, ' normalized)' )
 2410 format(' Change in Jacobn vars =', 1p, e12.4
     $     / ' Change in multipliers =', e12.4)
 2500 format(  ' Step size for  x  and  lamda  =', f12.5)
 2600 format(  ' Maximum constraint violation    =', 1p, e12.4,
     $     4x,  ' ( =', e11.4, ' normalized)' )
 2700 format(' Constraint violation  =', 1p, e12.4)
 2800 format(/ ' Completion  Full     requested as from now')
 2900 format(/ ' Penalty parameter increased to', 1p, e12.2 /)

*     end of m8setj
      end

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine m8viol( n, nb, nncon,
     $                   ne, nka, a, ha, ka,
     $                   bl, bu, fcon, xn, y, z, nwcore )

      implicit           double precision (a-h,o-z)
      integer*4          ha(ne)
      integer            ka(nka)
      double precision   a(ne), bl(nb), bu(nb), fcon(nncon),
     $                   xn(nb), y(nncon), z(nwcore)

*     ------------------------------------------------------------------
*     m8viol  finds how much xn violates the nonlinear constraints.
*     y(*)    is output as the vector of violations.
*     maxvi   points to the biggest violation.
*     vimax   is the biggest violation.
*     virel   is the biggest violation normalized by xnorm.
*     ------------------------------------------------------------------

      common    /m5tols/ toldj(3),tolx,tolpiv,tolrow,rowerr,xnorm
      common    /m8save/ vimax ,virel ,maxvi ,majits,minits,nssave

      intrinsic          max
      parameter        ( zero = 0.0d+0,  one = 1.0d+0 )

*     Set  y  =  - fcon - (linear A)*xn,   excluding slacks.

      do 520 i = 1, nncon
         y(i)  = - fcon(i)
  520 continue
      call m2aprd( 8, xn, n, y, nncon,
     $             ne, nka, a, ha, ka,
     $             z, nwcore )

*     See how much  y  violates the bounds on the nonlinear slacks.

      vimax  = zero
      maxvi  = 1

      do 550 i = 1, nncon
         j     = n + i
         slack = y(i)
         viol  = max( zero, bl(j) - slack, slack - bu(j) )
         if (vimax .lt. viol) then
             vimax   =  viol
             maxvi   =  i
         end if
  550 continue

      xnorm  = dnorm1( nb, xn, 1 )
      virel  = vimax / (one + xnorm)

*     end of m8viol
      end

